










package com.adacore.libadalang;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Objects;

import java.lang.StringBuilder;
import java.lang.Iterable;
import java.lang.reflect.Method;

import java.math.BigInteger;

import java.io.File;
import java.nio.ByteOrder;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.strings.TruffleString;

import org.graalvm.nativeimage.CurrentIsolate;
import org.graalvm.nativeimage.ImageInfo;
import org.graalvm.nativeimage.IsolateThread;
import org.graalvm.nativeimage.StackValue;
import org.graalvm.nativeimage.UnmanagedMemory;
import org.graalvm.nativeimage.c.CContext;
import org.graalvm.nativeimage.c.function.CEntryPoint;
import org.graalvm.nativeimage.c.function.CEntryPointLiteral;
import org.graalvm.nativeimage.c.function.CFunction;
import org.graalvm.nativeimage.c.function.CFunctionPointer;
import org.graalvm.nativeimage.c.function.InvokeCFunctionPointer;
import org.graalvm.nativeimage.c.struct.*;
import org.graalvm.nativeimage.c.type.*;
import org.graalvm.word.PointerBase;
import org.graalvm.word.Pointer;
import org.graalvm.word.UnsignedWord;
import org.graalvm.word.WordFactory;


/*
====================

This is the Java bindings of the libadalang API.
You can use all functionalities of the library in a Java environment.
Those bindings call the native library using JNI and Native-Image C API.

====================
*/
public final class Libadalang {

    // ==========
    // Native entry points
    // ==========

    /**
     * This method is the only valid callback to pass to a native event
     * handler for unit requests events.
     * This method will dispatch the execution according to the passed
     * analysis context.
     */
    @CEntryPoint
    static void unitRequested(
        final IsolateThread thread,
        final AnalysisContextNative contextNative,
        final TextNative nameNative,
        final AnalysisUnitNative fromNative,
        final byte foundNative,
        final byte isNotFoundErrorNative
    ) {
        try(
            final AnalysisContext context = AnalysisContext.wrap(
                contextNative
            );
            final Text text = Text.wrap(nameNative)
        ) {
            // Get the callback from the context event handler
            final EventHandler.UnitRequestedCallback callback = context
                .getEventHandler()
                .getUnitRequestCallback();

            // Call the callback
            if(callback != null) {
                callback.invoke(
                    context,
                    text.getContent(),
                    AnalysisUnit.wrap(fromNative),
                    foundNative != 0,
                    isNotFoundErrorNative != 0
                );
            }
        }
    }

    /**
     * This method is the only valid callback to pass to a native event
     * handler for unit parsing events.
     * This method will dispatch the execution according to the passed
     * analysis context.
     */
    @CEntryPoint
    static void unitParsed(
        final IsolateThread thread,
        final AnalysisContextNative contextNative,
        final AnalysisUnitNative unitNative,
        final byte reparsedNative
    ) {
        try(
            final AnalysisContext context = AnalysisContext.wrap(
                contextNative
            )
        ) {
            // Get the callback from the context event handler
            final EventHandler.UnitParsedCallback callback = context
                .getEventHandler()
                .getUnitParsedCallback();

            // Call the callback
            if(callback != null) {
                callback.invoke(
                    context,
                    AnalysisUnit.wrap(unitNative),
                    reparsedNative != 0
                );
            }
        }
    }

    // ==========
    // Static values
    // ==========

    /** The default grammar rule to parse the inputs. */
    private static final GrammarRule DEFAULT_GRAMMAR_RULE =
        GrammarRule.COMPILATION_RULE;

    /** The os name in lower case. */
    private static final String OS =
            System.getProperty("os.name").toLowerCase();

    /** The byte order of the system. */
    private static final ByteOrder BYTE_ORDER = ByteOrder.nativeOrder();

    /** The node to convert a Java string to a truffle string */
    private static final TruffleString.FromJavaStringNode fromJavaStringNode =
        TruffleString.FromJavaStringNode.create();

    /** The node to convert a truffle string to a Java string. */
    private static final TruffleString.ToJavaStringNode toJavaStringNode =
        TruffleString.ToJavaStringNode.create();

    /** The node to convert a byte array to a truffle string. */
    private static final TruffleString.FromByteArrayNode fromByteArrayNode =
        TruffleString.FromByteArrayNode.create();

    /** The node to convert a truffle string to a byte array. */
    private static final TruffleString.CopyToByteArrayNode toByteArrayNode =
        TruffleString.CopyToByteArrayNode.create();

    /** A map to store the nodes classes from their name. */
    public static final Map<String, Class<? extends AdaNode>>
        NODE_CLASS_MAP = new HashMap<>();

    static {
        // Populate the node class map
        NODE_CLASS_MAP.put(
            "AdaNode",
            AdaNode.class
        );
        NODE_CLASS_MAP.put(
            "AbortNode",
            AbortNode.class
        );
        NODE_CLASS_MAP.put(
            "AbortAbsent",
            AbortAbsent.class
        );
        NODE_CLASS_MAP.put(
            "AbortPresent",
            AbortPresent.class
        );
        NODE_CLASS_MAP.put(
            "AbstractNode",
            AbstractNode.class
        );
        NODE_CLASS_MAP.put(
            "AbstractAbsent",
            AbstractAbsent.class
        );
        NODE_CLASS_MAP.put(
            "AbstractPresent",
            AbstractPresent.class
        );
        NODE_CLASS_MAP.put(
            "AdaList",
            AdaList.class
        );
        NODE_CLASS_MAP.put(
            "AdaNodeList",
            AdaNodeList.class
        );
        NODE_CLASS_MAP.put(
            "AbstractStateDeclList",
            AbstractStateDeclList.class
        );
        NODE_CLASS_MAP.put(
            "AlternativesList",
            AlternativesList.class
        );
        NODE_CLASS_MAP.put(
            "ConstraintList",
            ConstraintList.class
        );
        NODE_CLASS_MAP.put(
            "DeclList",
            DeclList.class
        );
        NODE_CLASS_MAP.put(
            "StmtList",
            StmtList.class
        );
        NODE_CLASS_MAP.put(
            "AspectAssocList",
            AspectAssocList.class
        );
        NODE_CLASS_MAP.put(
            "BaseAssocList",
            BaseAssocList.class
        );
        NODE_CLASS_MAP.put(
            "BasicAssocList",
            BasicAssocList.class
        );
        NODE_CLASS_MAP.put(
            "AssocList",
            AssocList.class
        );
        NODE_CLASS_MAP.put(
            "BasicDeclList",
            BasicDeclList.class
        );
        NODE_CLASS_MAP.put(
            "CaseExprAlternativeList",
            CaseExprAlternativeList.class
        );
        NODE_CLASS_MAP.put(
            "CaseStmtAlternativeList",
            CaseStmtAlternativeList.class
        );
        NODE_CLASS_MAP.put(
            "CompilationUnitList",
            CompilationUnitList.class
        );
        NODE_CLASS_MAP.put(
            "ConcatOperandList",
            ConcatOperandList.class
        );
        NODE_CLASS_MAP.put(
            "ContractCaseAssocList",
            ContractCaseAssocList.class
        );
        NODE_CLASS_MAP.put(
            "DefiningNameList",
            DefiningNameList.class
        );
        NODE_CLASS_MAP.put(
            "DiscriminantSpecList",
            DiscriminantSpecList.class
        );
        NODE_CLASS_MAP.put(
            "ElsifExprPartList",
            ElsifExprPartList.class
        );
        NODE_CLASS_MAP.put(
            "ElsifStmtPartList",
            ElsifStmtPartList.class
        );
        NODE_CLASS_MAP.put(
            "EnumLiteralDeclList",
            EnumLiteralDeclList.class
        );
        NODE_CLASS_MAP.put(
            "ExprList",
            ExprList.class
        );
        NODE_CLASS_MAP.put(
            "ExprAlternativesList",
            ExprAlternativesList.class
        );
        NODE_CLASS_MAP.put(
            "IdentifierList",
            IdentifierList.class
        );
        NODE_CLASS_MAP.put(
            "DiscriminantChoiceList",
            DiscriminantChoiceList.class
        );
        NODE_CLASS_MAP.put(
            "NameList",
            NameList.class
        );
        NODE_CLASS_MAP.put(
            "ParentList",
            ParentList.class
        );
        NODE_CLASS_MAP.put(
            "ParamSpecList",
            ParamSpecList.class
        );
        NODE_CLASS_MAP.put(
            "PragmaNodeList",
            PragmaNodeList.class
        );
        NODE_CLASS_MAP.put(
            "SelectWhenPartList",
            SelectWhenPartList.class
        );
        NODE_CLASS_MAP.put(
            "UnconstrainedArrayIndexList",
            UnconstrainedArrayIndexList.class
        );
        NODE_CLASS_MAP.put(
            "VariantList",
            VariantList.class
        );
        NODE_CLASS_MAP.put(
            "AliasedNode",
            AliasedNode.class
        );
        NODE_CLASS_MAP.put(
            "AliasedAbsent",
            AliasedAbsent.class
        );
        NODE_CLASS_MAP.put(
            "AliasedPresent",
            AliasedPresent.class
        );
        NODE_CLASS_MAP.put(
            "AllNode",
            AllNode.class
        );
        NODE_CLASS_MAP.put(
            "AllAbsent",
            AllAbsent.class
        );
        NODE_CLASS_MAP.put(
            "AllPresent",
            AllPresent.class
        );
        NODE_CLASS_MAP.put(
            "ArrayIndices",
            ArrayIndices.class
        );
        NODE_CLASS_MAP.put(
            "ConstrainedArrayIndices",
            ConstrainedArrayIndices.class
        );
        NODE_CLASS_MAP.put(
            "UnconstrainedArrayIndices",
            UnconstrainedArrayIndices.class
        );
        NODE_CLASS_MAP.put(
            "AspectAssoc",
            AspectAssoc.class
        );
        NODE_CLASS_MAP.put(
            "AspectClause",
            AspectClause.class
        );
        NODE_CLASS_MAP.put(
            "AtClause",
            AtClause.class
        );
        NODE_CLASS_MAP.put(
            "AttributeDefClause",
            AttributeDefClause.class
        );
        NODE_CLASS_MAP.put(
            "EnumRepClause",
            EnumRepClause.class
        );
        NODE_CLASS_MAP.put(
            "RecordRepClause",
            RecordRepClause.class
        );
        NODE_CLASS_MAP.put(
            "AspectSpec",
            AspectSpec.class
        );
        NODE_CLASS_MAP.put(
            "BaseAssoc",
            BaseAssoc.class
        );
        NODE_CLASS_MAP.put(
            "ContractCaseAssoc",
            ContractCaseAssoc.class
        );
        NODE_CLASS_MAP.put(
            "PragmaArgumentAssoc",
            PragmaArgumentAssoc.class
        );
        NODE_CLASS_MAP.put(
            "BaseFormalParamHolder",
            BaseFormalParamHolder.class
        );
        NODE_CLASS_MAP.put(
            "BaseSubpSpec",
            BaseSubpSpec.class
        );
        NODE_CLASS_MAP.put(
            "EntrySpec",
            EntrySpec.class
        );
        NODE_CLASS_MAP.put(
            "EnumSubpSpec",
            EnumSubpSpec.class
        );
        NODE_CLASS_MAP.put(
            "SubpSpec",
            SubpSpec.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticBinarySpec",
            SyntheticBinarySpec.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticUnarySpec",
            SyntheticUnarySpec.class
        );
        NODE_CLASS_MAP.put(
            "ComponentList",
            ComponentList.class
        );
        NODE_CLASS_MAP.put(
            "DiscriminantPart",
            DiscriminantPart.class
        );
        NODE_CLASS_MAP.put(
            "KnownDiscriminantPart",
            KnownDiscriminantPart.class
        );
        NODE_CLASS_MAP.put(
            "UnknownDiscriminantPart",
            UnknownDiscriminantPart.class
        );
        NODE_CLASS_MAP.put(
            "EntryCompletionFormalParams",
            EntryCompletionFormalParams.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormalPart",
            GenericFormalPart.class
        );
        NODE_CLASS_MAP.put(
            "BaseRecordDef",
            BaseRecordDef.class
        );
        NODE_CLASS_MAP.put(
            "NullRecordDef",
            NullRecordDef.class
        );
        NODE_CLASS_MAP.put(
            "RecordDef",
            RecordDef.class
        );
        NODE_CLASS_MAP.put(
            "BasicAssoc",
            BasicAssoc.class
        );
        NODE_CLASS_MAP.put(
            "AggregateAssoc",
            AggregateAssoc.class
        );
        NODE_CLASS_MAP.put(
            "MultiDimArrayAssoc",
            MultiDimArrayAssoc.class
        );
        NODE_CLASS_MAP.put(
            "CompositeConstraintAssoc",
            CompositeConstraintAssoc.class
        );
        NODE_CLASS_MAP.put(
            "IteratedAssoc",
            IteratedAssoc.class
        );
        NODE_CLASS_MAP.put(
            "ParamAssoc",
            ParamAssoc.class
        );
        NODE_CLASS_MAP.put(
            "BasicDecl",
            BasicDecl.class
        );
        NODE_CLASS_MAP.put(
            "AbstractStateDecl",
            AbstractStateDecl.class
        );
        NODE_CLASS_MAP.put(
            "AnonymousExprDecl",
            AnonymousExprDecl.class
        );
        NODE_CLASS_MAP.put(
            "BaseFormalParamDecl",
            BaseFormalParamDecl.class
        );
        NODE_CLASS_MAP.put(
            "ComponentDecl",
            ComponentDecl.class
        );
        NODE_CLASS_MAP.put(
            "DiscriminantSpec",
            DiscriminantSpec.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormal",
            GenericFormal.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormalObjDecl",
            GenericFormalObjDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormalPackage",
            GenericFormalPackage.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormalSubpDecl",
            GenericFormalSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericFormalTypeDecl",
            GenericFormalTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "ParamSpec",
            ParamSpec.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticFormalParamDecl",
            SyntheticFormalParamDecl.class
        );
        NODE_CLASS_MAP.put(
            "BasePackageDecl",
            BasePackageDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericPackageInternal",
            GenericPackageInternal.class
        );
        NODE_CLASS_MAP.put(
            "PackageDecl",
            PackageDecl.class
        );
        NODE_CLASS_MAP.put(
            "BaseTypeDecl",
            BaseTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "BaseSubtypeDecl",
            BaseSubtypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "DiscreteBaseSubtypeDecl",
            DiscreteBaseSubtypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "SubtypeDecl",
            SubtypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "ClasswideTypeDecl",
            ClasswideTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "IncompleteTypeDecl",
            IncompleteTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "IncompleteFormalTypeDecl",
            IncompleteFormalTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "IncompleteTaggedTypeDecl",
            IncompleteTaggedTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedTypeDecl",
            ProtectedTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "TaskTypeDecl",
            TaskTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "SingleTaskTypeDecl",
            SingleTaskTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "TypeDecl",
            TypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "AnonymousTypeDecl",
            AnonymousTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "SynthAnonymousTypeDecl",
            SynthAnonymousTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "ConcreteTypeDecl",
            ConcreteTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "FormalTypeDecl",
            FormalTypeDecl.class
        );
        NODE_CLASS_MAP.put(
            "BasicSubpDecl",
            BasicSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "ClassicSubpDecl",
            ClassicSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "AbstractSubpDecl",
            AbstractSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "FormalSubpDecl",
            FormalSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "AbstractFormalSubpDecl",
            AbstractFormalSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "ConcreteFormalSubpDecl",
            ConcreteFormalSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "SubpDecl",
            SubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "EntryDecl",
            EntryDecl.class
        );
        NODE_CLASS_MAP.put(
            "EnumLiteralDecl",
            EnumLiteralDecl.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticCharEnumLit",
            SyntheticCharEnumLit.class
        );
        NODE_CLASS_MAP.put(
            "GenericSubpInternal",
            GenericSubpInternal.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticSubpDecl",
            SyntheticSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "BodyNode",
            BodyNode.class
        );
        NODE_CLASS_MAP.put(
            "AcceptStmtBody",
            AcceptStmtBody.class
        );
        NODE_CLASS_MAP.put(
            "BaseSubpBody",
            BaseSubpBody.class
        );
        NODE_CLASS_MAP.put(
            "ExprFunction",
            ExprFunction.class
        );
        NODE_CLASS_MAP.put(
            "NullSubpDecl",
            NullSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "SubpBody",
            SubpBody.class
        );
        NODE_CLASS_MAP.put(
            "SubpRenamingDecl",
            SubpRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "BodyStub",
            BodyStub.class
        );
        NODE_CLASS_MAP.put(
            "PackageBodyStub",
            PackageBodyStub.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedBodyStub",
            ProtectedBodyStub.class
        );
        NODE_CLASS_MAP.put(
            "SubpBodyStub",
            SubpBodyStub.class
        );
        NODE_CLASS_MAP.put(
            "TaskBodyStub",
            TaskBodyStub.class
        );
        NODE_CLASS_MAP.put(
            "EntryBody",
            EntryBody.class
        );
        NODE_CLASS_MAP.put(
            "PackageBody",
            PackageBody.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedBody",
            ProtectedBody.class
        );
        NODE_CLASS_MAP.put(
            "TaskBody",
            TaskBody.class
        );
        NODE_CLASS_MAP.put(
            "EntryIndexSpec",
            EntryIndexSpec.class
        );
        NODE_CLASS_MAP.put(
            "ErrorDecl",
            ErrorDecl.class
        );
        NODE_CLASS_MAP.put(
            "ExceptionDecl",
            ExceptionDecl.class
        );
        NODE_CLASS_MAP.put(
            "ExceptionHandler",
            ExceptionHandler.class
        );
        NODE_CLASS_MAP.put(
            "ForLoopVarDecl",
            ForLoopVarDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericDecl",
            GenericDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericPackageDecl",
            GenericPackageDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericSubpDecl",
            GenericSubpDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericInstantiation",
            GenericInstantiation.class
        );
        NODE_CLASS_MAP.put(
            "GenericPackageInstantiation",
            GenericPackageInstantiation.class
        );
        NODE_CLASS_MAP.put(
            "GenericSubpInstantiation",
            GenericSubpInstantiation.class
        );
        NODE_CLASS_MAP.put(
            "GenericRenamingDecl",
            GenericRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericPackageRenamingDecl",
            GenericPackageRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "GenericSubpRenamingDecl",
            GenericSubpRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "LabelDecl",
            LabelDecl.class
        );
        NODE_CLASS_MAP.put(
            "NamedStmtDecl",
            NamedStmtDecl.class
        );
        NODE_CLASS_MAP.put(
            "NumberDecl",
            NumberDecl.class
        );
        NODE_CLASS_MAP.put(
            "ObjectDecl",
            ObjectDecl.class
        );
        NODE_CLASS_MAP.put(
            "ExtendedReturnStmtObjectDecl",
            ExtendedReturnStmtObjectDecl.class
        );
        NODE_CLASS_MAP.put(
            "NoTypeObjectRenamingDecl",
            NoTypeObjectRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "PackageRenamingDecl",
            PackageRenamingDecl.class
        );
        NODE_CLASS_MAP.put(
            "SingleProtectedDecl",
            SingleProtectedDecl.class
        );
        NODE_CLASS_MAP.put(
            "SingleTaskDecl",
            SingleTaskDecl.class
        );
        NODE_CLASS_MAP.put(
            "CaseStmtAlternative",
            CaseStmtAlternative.class
        );
        NODE_CLASS_MAP.put(
            "CompilationUnit",
            CompilationUnit.class
        );
        NODE_CLASS_MAP.put(
            "ComponentClause",
            ComponentClause.class
        );
        NODE_CLASS_MAP.put(
            "ComponentDef",
            ComponentDef.class
        );
        NODE_CLASS_MAP.put(
            "ConstantNode",
            ConstantNode.class
        );
        NODE_CLASS_MAP.put(
            "ConstantAbsent",
            ConstantAbsent.class
        );
        NODE_CLASS_MAP.put(
            "ConstantPresent",
            ConstantPresent.class
        );
        NODE_CLASS_MAP.put(
            "Constraint",
            Constraint.class
        );
        NODE_CLASS_MAP.put(
            "CompositeConstraint",
            CompositeConstraint.class
        );
        NODE_CLASS_MAP.put(
            "DeltaConstraint",
            DeltaConstraint.class
        );
        NODE_CLASS_MAP.put(
            "DigitsConstraint",
            DigitsConstraint.class
        );
        NODE_CLASS_MAP.put(
            "RangeConstraint",
            RangeConstraint.class
        );
        NODE_CLASS_MAP.put(
            "DeclarativePart",
            DeclarativePart.class
        );
        NODE_CLASS_MAP.put(
            "PrivatePart",
            PrivatePart.class
        );
        NODE_CLASS_MAP.put(
            "PublicPart",
            PublicPart.class
        );
        NODE_CLASS_MAP.put(
            "ElsifExprPart",
            ElsifExprPart.class
        );
        NODE_CLASS_MAP.put(
            "ElsifStmtPart",
            ElsifStmtPart.class
        );
        NODE_CLASS_MAP.put(
            "Expr",
            Expr.class
        );
        NODE_CLASS_MAP.put(
            "AbstractStateDeclExpr",
            AbstractStateDeclExpr.class
        );
        NODE_CLASS_MAP.put(
            "Allocator",
            Allocator.class
        );
        NODE_CLASS_MAP.put(
            "BaseAggregate",
            BaseAggregate.class
        );
        NODE_CLASS_MAP.put(
            "Aggregate",
            Aggregate.class
        );
        NODE_CLASS_MAP.put(
            "BracketAggregate",
            BracketAggregate.class
        );
        NODE_CLASS_MAP.put(
            "DeltaAggregate",
            DeltaAggregate.class
        );
        NODE_CLASS_MAP.put(
            "BracketDeltaAggregate",
            BracketDeltaAggregate.class
        );
        NODE_CLASS_MAP.put(
            "NullRecordAggregate",
            NullRecordAggregate.class
        );
        NODE_CLASS_MAP.put(
            "BinOp",
            BinOp.class
        );
        NODE_CLASS_MAP.put(
            "RelationOp",
            RelationOp.class
        );
        NODE_CLASS_MAP.put(
            "BoxExpr",
            BoxExpr.class
        );
        NODE_CLASS_MAP.put(
            "CaseExprAlternative",
            CaseExprAlternative.class
        );
        NODE_CLASS_MAP.put(
            "ConcatOp",
            ConcatOp.class
        );
        NODE_CLASS_MAP.put(
            "ConcatOperand",
            ConcatOperand.class
        );
        NODE_CLASS_MAP.put(
            "CondExpr",
            CondExpr.class
        );
        NODE_CLASS_MAP.put(
            "CaseExpr",
            CaseExpr.class
        );
        NODE_CLASS_MAP.put(
            "IfExpr",
            IfExpr.class
        );
        NODE_CLASS_MAP.put(
            "ContractCases",
            ContractCases.class
        );
        NODE_CLASS_MAP.put(
            "DeclExpr",
            DeclExpr.class
        );
        NODE_CLASS_MAP.put(
            "MembershipExpr",
            MembershipExpr.class
        );
        NODE_CLASS_MAP.put(
            "Name",
            Name.class
        );
        NODE_CLASS_MAP.put(
            "AttributeRef",
            AttributeRef.class
        );
        NODE_CLASS_MAP.put(
            "CallExpr",
            CallExpr.class
        );
        NODE_CLASS_MAP.put(
            "DefiningName",
            DefiningName.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticDefiningName",
            SyntheticDefiningName.class
        );
        NODE_CLASS_MAP.put(
            "DiscreteSubtypeName",
            DiscreteSubtypeName.class
        );
        NODE_CLASS_MAP.put(
            "DottedName",
            DottedName.class
        );
        NODE_CLASS_MAP.put(
            "EndName",
            EndName.class
        );
        NODE_CLASS_MAP.put(
            "ExplicitDeref",
            ExplicitDeref.class
        );
        NODE_CLASS_MAP.put(
            "QualExpr",
            QualExpr.class
        );
        NODE_CLASS_MAP.put(
            "ReduceAttributeRef",
            ReduceAttributeRef.class
        );
        NODE_CLASS_MAP.put(
            "SingleTokNode",
            SingleTokNode.class
        );
        NODE_CLASS_MAP.put(
            "BaseId",
            BaseId.class
        );
        NODE_CLASS_MAP.put(
            "CharLiteral",
            CharLiteral.class
        );
        NODE_CLASS_MAP.put(
            "Identifier",
            Identifier.class
        );
        NODE_CLASS_MAP.put(
            "Op",
            Op.class
        );
        NODE_CLASS_MAP.put(
            "OpAbs",
            OpAbs.class
        );
        NODE_CLASS_MAP.put(
            "OpAnd",
            OpAnd.class
        );
        NODE_CLASS_MAP.put(
            "OpAndThen",
            OpAndThen.class
        );
        NODE_CLASS_MAP.put(
            "OpConcat",
            OpConcat.class
        );
        NODE_CLASS_MAP.put(
            "OpDiv",
            OpDiv.class
        );
        NODE_CLASS_MAP.put(
            "OpDoubleDot",
            OpDoubleDot.class
        );
        NODE_CLASS_MAP.put(
            "OpEq",
            OpEq.class
        );
        NODE_CLASS_MAP.put(
            "OpGt",
            OpGt.class
        );
        NODE_CLASS_MAP.put(
            "OpGte",
            OpGte.class
        );
        NODE_CLASS_MAP.put(
            "OpIn",
            OpIn.class
        );
        NODE_CLASS_MAP.put(
            "OpLt",
            OpLt.class
        );
        NODE_CLASS_MAP.put(
            "OpLte",
            OpLte.class
        );
        NODE_CLASS_MAP.put(
            "OpMinus",
            OpMinus.class
        );
        NODE_CLASS_MAP.put(
            "OpMod",
            OpMod.class
        );
        NODE_CLASS_MAP.put(
            "OpMult",
            OpMult.class
        );
        NODE_CLASS_MAP.put(
            "OpNeq",
            OpNeq.class
        );
        NODE_CLASS_MAP.put(
            "OpNot",
            OpNot.class
        );
        NODE_CLASS_MAP.put(
            "OpNotIn",
            OpNotIn.class
        );
        NODE_CLASS_MAP.put(
            "OpOr",
            OpOr.class
        );
        NODE_CLASS_MAP.put(
            "OpOrElse",
            OpOrElse.class
        );
        NODE_CLASS_MAP.put(
            "OpPlus",
            OpPlus.class
        );
        NODE_CLASS_MAP.put(
            "OpPow",
            OpPow.class
        );
        NODE_CLASS_MAP.put(
            "OpRem",
            OpRem.class
        );
        NODE_CLASS_MAP.put(
            "OpXor",
            OpXor.class
        );
        NODE_CLASS_MAP.put(
            "StringLiteral",
            StringLiteral.class
        );
        NODE_CLASS_MAP.put(
            "NullLiteral",
            NullLiteral.class
        );
        NODE_CLASS_MAP.put(
            "NumLiteral",
            NumLiteral.class
        );
        NODE_CLASS_MAP.put(
            "IntLiteral",
            IntLiteral.class
        );
        NODE_CLASS_MAP.put(
            "RealLiteral",
            RealLiteral.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticIdentifier",
            SyntheticIdentifier.class
        );
        NODE_CLASS_MAP.put(
            "TargetName",
            TargetName.class
        );
        NODE_CLASS_MAP.put(
            "UpdateAttributeRef",
            UpdateAttributeRef.class
        );
        NODE_CLASS_MAP.put(
            "ParenExpr",
            ParenExpr.class
        );
        NODE_CLASS_MAP.put(
            "QuantifiedExpr",
            QuantifiedExpr.class
        );
        NODE_CLASS_MAP.put(
            "RaiseExpr",
            RaiseExpr.class
        );
        NODE_CLASS_MAP.put(
            "UnOp",
            UnOp.class
        );
        NODE_CLASS_MAP.put(
            "HandledStmts",
            HandledStmts.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceKind",
            InterfaceKind.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceKindLimited",
            InterfaceKindLimited.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceKindProtected",
            InterfaceKindProtected.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceKindSynchronized",
            InterfaceKindSynchronized.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceKindTask",
            InterfaceKindTask.class
        );
        NODE_CLASS_MAP.put(
            "IterType",
            IterType.class
        );
        NODE_CLASS_MAP.put(
            "IterTypeIn",
            IterTypeIn.class
        );
        NODE_CLASS_MAP.put(
            "IterTypeOf",
            IterTypeOf.class
        );
        NODE_CLASS_MAP.put(
            "LibraryItem",
            LibraryItem.class
        );
        NODE_CLASS_MAP.put(
            "LimitedNode",
            LimitedNode.class
        );
        NODE_CLASS_MAP.put(
            "LimitedAbsent",
            LimitedAbsent.class
        );
        NODE_CLASS_MAP.put(
            "LimitedPresent",
            LimitedPresent.class
        );
        NODE_CLASS_MAP.put(
            "LoopSpec",
            LoopSpec.class
        );
        NODE_CLASS_MAP.put(
            "ForLoopSpec",
            ForLoopSpec.class
        );
        NODE_CLASS_MAP.put(
            "WhileLoopSpec",
            WhileLoopSpec.class
        );
        NODE_CLASS_MAP.put(
            "Mode",
            Mode.class
        );
        NODE_CLASS_MAP.put(
            "ModeDefault",
            ModeDefault.class
        );
        NODE_CLASS_MAP.put(
            "ModeIn",
            ModeIn.class
        );
        NODE_CLASS_MAP.put(
            "ModeInOut",
            ModeInOut.class
        );
        NODE_CLASS_MAP.put(
            "ModeOut",
            ModeOut.class
        );
        NODE_CLASS_MAP.put(
            "MultiAbstractStateDecl",
            MultiAbstractStateDecl.class
        );
        NODE_CLASS_MAP.put(
            "NotNull",
            NotNull.class
        );
        NODE_CLASS_MAP.put(
            "NotNullAbsent",
            NotNullAbsent.class
        );
        NODE_CLASS_MAP.put(
            "NotNullPresent",
            NotNullPresent.class
        );
        NODE_CLASS_MAP.put(
            "NullComponentDecl",
            NullComponentDecl.class
        );
        NODE_CLASS_MAP.put(
            "OthersDesignator",
            OthersDesignator.class
        );
        NODE_CLASS_MAP.put(
            "OverridingNode",
            OverridingNode.class
        );
        NODE_CLASS_MAP.put(
            "OverridingNotOverriding",
            OverridingNotOverriding.class
        );
        NODE_CLASS_MAP.put(
            "OverridingOverriding",
            OverridingOverriding.class
        );
        NODE_CLASS_MAP.put(
            "OverridingUnspecified",
            OverridingUnspecified.class
        );
        NODE_CLASS_MAP.put(
            "Params",
            Params.class
        );
        NODE_CLASS_MAP.put(
            "ParenAbstractStateDecl",
            ParenAbstractStateDecl.class
        );
        NODE_CLASS_MAP.put(
            "PpDirective",
            PpDirective.class
        );
        NODE_CLASS_MAP.put(
            "PpElseDirective",
            PpElseDirective.class
        );
        NODE_CLASS_MAP.put(
            "PpElsifDirective",
            PpElsifDirective.class
        );
        NODE_CLASS_MAP.put(
            "PpEndIfDirective",
            PpEndIfDirective.class
        );
        NODE_CLASS_MAP.put(
            "PpIfDirective",
            PpIfDirective.class
        );
        NODE_CLASS_MAP.put(
            "PpThenKw",
            PpThenKw.class
        );
        NODE_CLASS_MAP.put(
            "PragmaNode",
            PragmaNode.class
        );
        NODE_CLASS_MAP.put(
            "PrivateNode",
            PrivateNode.class
        );
        NODE_CLASS_MAP.put(
            "PrivateAbsent",
            PrivateAbsent.class
        );
        NODE_CLASS_MAP.put(
            "PrivatePresent",
            PrivatePresent.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedDef",
            ProtectedDef.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedNode",
            ProtectedNode.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedAbsent",
            ProtectedAbsent.class
        );
        NODE_CLASS_MAP.put(
            "ProtectedPresent",
            ProtectedPresent.class
        );
        NODE_CLASS_MAP.put(
            "Quantifier",
            Quantifier.class
        );
        NODE_CLASS_MAP.put(
            "QuantifierAll",
            QuantifierAll.class
        );
        NODE_CLASS_MAP.put(
            "QuantifierSome",
            QuantifierSome.class
        );
        NODE_CLASS_MAP.put(
            "RangeSpec",
            RangeSpec.class
        );
        NODE_CLASS_MAP.put(
            "RenamingClause",
            RenamingClause.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticRenamingClause",
            SyntheticRenamingClause.class
        );
        NODE_CLASS_MAP.put(
            "ReverseNode",
            ReverseNode.class
        );
        NODE_CLASS_MAP.put(
            "ReverseAbsent",
            ReverseAbsent.class
        );
        NODE_CLASS_MAP.put(
            "ReversePresent",
            ReversePresent.class
        );
        NODE_CLASS_MAP.put(
            "SelectWhenPart",
            SelectWhenPart.class
        );
        NODE_CLASS_MAP.put(
            "Stmt",
            Stmt.class
        );
        NODE_CLASS_MAP.put(
            "CompositeStmt",
            CompositeStmt.class
        );
        NODE_CLASS_MAP.put(
            "AcceptStmt",
            AcceptStmt.class
        );
        NODE_CLASS_MAP.put(
            "AcceptStmtWithStmts",
            AcceptStmtWithStmts.class
        );
        NODE_CLASS_MAP.put(
            "BaseLoopStmt",
            BaseLoopStmt.class
        );
        NODE_CLASS_MAP.put(
            "ForLoopStmt",
            ForLoopStmt.class
        );
        NODE_CLASS_MAP.put(
            "LoopStmt",
            LoopStmt.class
        );
        NODE_CLASS_MAP.put(
            "WhileLoopStmt",
            WhileLoopStmt.class
        );
        NODE_CLASS_MAP.put(
            "BlockStmt",
            BlockStmt.class
        );
        NODE_CLASS_MAP.put(
            "BeginBlock",
            BeginBlock.class
        );
        NODE_CLASS_MAP.put(
            "DeclBlock",
            DeclBlock.class
        );
        NODE_CLASS_MAP.put(
            "CaseStmt",
            CaseStmt.class
        );
        NODE_CLASS_MAP.put(
            "ExtendedReturnStmt",
            ExtendedReturnStmt.class
        );
        NODE_CLASS_MAP.put(
            "IfStmt",
            IfStmt.class
        );
        NODE_CLASS_MAP.put(
            "NamedStmt",
            NamedStmt.class
        );
        NODE_CLASS_MAP.put(
            "SelectStmt",
            SelectStmt.class
        );
        NODE_CLASS_MAP.put(
            "ErrorStmt",
            ErrorStmt.class
        );
        NODE_CLASS_MAP.put(
            "SimpleStmt",
            SimpleStmt.class
        );
        NODE_CLASS_MAP.put(
            "AbortStmt",
            AbortStmt.class
        );
        NODE_CLASS_MAP.put(
            "AssignStmt",
            AssignStmt.class
        );
        NODE_CLASS_MAP.put(
            "CallStmt",
            CallStmt.class
        );
        NODE_CLASS_MAP.put(
            "DelayStmt",
            DelayStmt.class
        );
        NODE_CLASS_MAP.put(
            "ExitStmt",
            ExitStmt.class
        );
        NODE_CLASS_MAP.put(
            "GotoStmt",
            GotoStmt.class
        );
        NODE_CLASS_MAP.put(
            "Label",
            Label.class
        );
        NODE_CLASS_MAP.put(
            "NullStmt",
            NullStmt.class
        );
        NODE_CLASS_MAP.put(
            "RaiseStmt",
            RaiseStmt.class
        );
        NODE_CLASS_MAP.put(
            "RequeueStmt",
            RequeueStmt.class
        );
        NODE_CLASS_MAP.put(
            "ReturnStmt",
            ReturnStmt.class
        );
        NODE_CLASS_MAP.put(
            "TerminateAlternative",
            TerminateAlternative.class
        );
        NODE_CLASS_MAP.put(
            "SubpKind",
            SubpKind.class
        );
        NODE_CLASS_MAP.put(
            "SubpKindFunction",
            SubpKindFunction.class
        );
        NODE_CLASS_MAP.put(
            "SubpKindProcedure",
            SubpKindProcedure.class
        );
        NODE_CLASS_MAP.put(
            "Subunit",
            Subunit.class
        );
        NODE_CLASS_MAP.put(
            "SynchronizedNode",
            SynchronizedNode.class
        );
        NODE_CLASS_MAP.put(
            "SynchronizedAbsent",
            SynchronizedAbsent.class
        );
        NODE_CLASS_MAP.put(
            "SynchronizedPresent",
            SynchronizedPresent.class
        );
        NODE_CLASS_MAP.put(
            "TaggedNode",
            TaggedNode.class
        );
        NODE_CLASS_MAP.put(
            "TaggedAbsent",
            TaggedAbsent.class
        );
        NODE_CLASS_MAP.put(
            "TaggedPresent",
            TaggedPresent.class
        );
        NODE_CLASS_MAP.put(
            "TaskDef",
            TaskDef.class
        );
        NODE_CLASS_MAP.put(
            "TypeAttributesRepository",
            TypeAttributesRepository.class
        );
        NODE_CLASS_MAP.put(
            "TypeDef",
            TypeDef.class
        );
        NODE_CLASS_MAP.put(
            "AccessDef",
            AccessDef.class
        );
        NODE_CLASS_MAP.put(
            "AccessToSubpDef",
            AccessToSubpDef.class
        );
        NODE_CLASS_MAP.put(
            "BaseTypeAccessDef",
            BaseTypeAccessDef.class
        );
        NODE_CLASS_MAP.put(
            "AnonymousTypeAccessDef",
            AnonymousTypeAccessDef.class
        );
        NODE_CLASS_MAP.put(
            "TypeAccessDef",
            TypeAccessDef.class
        );
        NODE_CLASS_MAP.put(
            "ArrayTypeDef",
            ArrayTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "DerivedTypeDef",
            DerivedTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "EnumTypeDef",
            EnumTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "FormalDiscreteTypeDef",
            FormalDiscreteTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "InterfaceTypeDef",
            InterfaceTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "ModIntTypeDef",
            ModIntTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "PrivateTypeDef",
            PrivateTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "RealTypeDef",
            RealTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "DecimalFixedPointDef",
            DecimalFixedPointDef.class
        );
        NODE_CLASS_MAP.put(
            "FloatingPointDef",
            FloatingPointDef.class
        );
        NODE_CLASS_MAP.put(
            "OrdinaryFixedPointDef",
            OrdinaryFixedPointDef.class
        );
        NODE_CLASS_MAP.put(
            "RecordTypeDef",
            RecordTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "SignedIntTypeDef",
            SignedIntTypeDef.class
        );
        NODE_CLASS_MAP.put(
            "TypeExpr",
            TypeExpr.class
        );
        NODE_CLASS_MAP.put(
            "AnonymousType",
            AnonymousType.class
        );
        NODE_CLASS_MAP.put(
            "EnumLitSynthTypeExpr",
            EnumLitSynthTypeExpr.class
        );
        NODE_CLASS_MAP.put(
            "SubtypeIndication",
            SubtypeIndication.class
        );
        NODE_CLASS_MAP.put(
            "ConstrainedSubtypeIndication",
            ConstrainedSubtypeIndication.class
        );
        NODE_CLASS_MAP.put(
            "DiscreteSubtypeIndication",
            DiscreteSubtypeIndication.class
        );
        NODE_CLASS_MAP.put(
            "SyntheticTypeExpr",
            SyntheticTypeExpr.class
        );
        NODE_CLASS_MAP.put(
            "UnconstrainedArrayIndex",
            UnconstrainedArrayIndex.class
        );
        NODE_CLASS_MAP.put(
            "UntilNode",
            UntilNode.class
        );
        NODE_CLASS_MAP.put(
            "UntilAbsent",
            UntilAbsent.class
        );
        NODE_CLASS_MAP.put(
            "UntilPresent",
            UntilPresent.class
        );
        NODE_CLASS_MAP.put(
            "UseClause",
            UseClause.class
        );
        NODE_CLASS_MAP.put(
            "UsePackageClause",
            UsePackageClause.class
        );
        NODE_CLASS_MAP.put(
            "UseTypeClause",
            UseTypeClause.class
        );
        NODE_CLASS_MAP.put(
            "ValueSequence",
            ValueSequence.class
        );
        NODE_CLASS_MAP.put(
            "Variant",
            Variant.class
        );
        NODE_CLASS_MAP.put(
            "VariantPart",
            VariantPart.class
        );
        NODE_CLASS_MAP.put(
            "WithClause",
            WithClause.class
        );
        NODE_CLASS_MAP.put(
            "WithPrivate",
            WithPrivate.class
        );
        NODE_CLASS_MAP.put(
            "WithPrivateAbsent",
            WithPrivateAbsent.class
        );
        NODE_CLASS_MAP.put(
            "WithPrivatePresent",
            WithPrivatePresent.class
        );
    }

    // ==========
    // Util functions
    // ==========

    /**
     * Get the string representing the memory.
     *
     * @param pointer The pointer to start displaying the memory from.
     * @param count The number of bytes to display from the pointer.
     * @return The string representing the memory as hex bytes.
     */
    private static String dumpMemory(
        final Pointer pointer,
        final long count
    ) {
        final StringBuilder res = new StringBuilder();
        for(int i = 0 ; i < count ; i++) {
            final byte toDump = pointer.readByte(i);
            res.append(String.format("%02x ", toDump));
        }
        return res.toString();
    }

    /**
     * Convert a Java string to a C string by allocating memory.
     *
     * @param jString The Java string to convert.
     * @return The native C char pointer. This pointer MUST be freed.
     */
    @CompilerDirectives.TruffleBoundary
    private static CCharPointer toCString(
        final String jString
    ) {
        final UnsignedWord size = WordFactory.unsigned(jString.length() + 1);
        final CCharPointer res = UnmanagedMemory.calloc(size);
        if(jString.length() > 0) {
            CTypeConversion.toCString(
                jString,
                StandardCharsets.UTF_8,
                res,
                size
            );
        }

        return res;
    }

    /**
     * Convert a Java byte array in a native byte array.
     *
     * @param bytes The Java bytes.
     * @return The native C char pointer. This pointer MUST be freed.
     */
    private static CCharPointer toCBytes(
        final byte[] bytes
    ) {
        final UnsignedWord size = WordFactory.unsigned(bytes.length);
        final CCharPointer res = UnmanagedMemory.malloc(size);
        for(int i = 0 ; i < bytes.length ; i++) {
            res.write(i, bytes[i]);
        }
        return res;
    }

    /**
     * Convert a native-image C string to a Java string.
     *
     * @param pointer The char pointer to convert to a Java string.
     * @return The Java string.
     */
    @CompilerDirectives.TruffleBoundary
    private static String toJString(
        final CCharPointer pointer
    ) {
        return CTypeConversion.toJavaString(pointer);
    }

    /**
     * This function decode a utf 32 int array in a
     * string without calling Java charset.
     *
     * @param chars The int array to decode.
     * @return The resulting string.
     */
    @CompilerDirectives.TruffleBoundary
    private static String decodeUTF32(
        final byte[] toDecode
    ) {
        return toJavaStringNode.execute(
            fromByteArrayNode.execute(toDecode, TruffleString.Encoding.UTF_32)
        );
    }

    /**
     * This function encode a given string to a int array
     * according to the utf 32 standard.
     *
     * @param toEncode The string to encode.
     * @return The encoded string in an int array.
     */
    @CompilerDirectives.TruffleBoundary
    private static byte[] encodeUTF32(
        final String toEncode
    ) {
        return toByteArrayNode.execute(
            fromJavaStringNode.execute(
                toEncode,
                TruffleString.Encoding.UTF_32
            ),
            TruffleString.Encoding.UTF_32
        );
    }

    /**
     * Get the string representation of the given string.
     * This function escaped needed chars and format the string.
     *
     * @param source The source string to get the representation for.
     * @return The representation of the string.
     */
    private static String stringRepresentation(
        final String source
    ) {
        return source
            .replace("\"", "\\\"")
            .replace("\n", "\\x0a");
    }

    /**
     * Convert a C Langkit exception to the LangkitException class.

     * @param
     */
    private static LangkitException wrapException(
        final LangkitExceptionNative exc
    ) {
        return new LangkitException(
            exc.get_kind(),
            toJString(exc.get_information())
        );
    }

    /**
      * Return the exception raised by the last C API call, or null if the last
      * call was successful.
      */
    private static LangkitException getLastException() {
        LangkitException result = null;

        if(ImageInfo.inImageCode()) {
            final LangkitExceptionNative exceptionNative =
                NI_LIB.ada_get_last_exception();
            if(exceptionNative.isNonNull()) {
                result = wrapException(exceptionNative);
            }
        } else {
            result = JNI_LIB.ada_get_last_exception();
        }
        return result;
    }

    /**
     * Check the last exception raised by langkit and throw it.
     *
     * @throws The last langkit exception if there is one.
     */
    private static void checkException() throws LangkitException {
        LangkitException exc = getLastException();
        if(exc != null)
            throw exc;
    }

    // ==========
    // Util interfaces
    // ==========

    /**
     * Interface to visit the AST.
     */
    public static interface BasicVisitor<T> {
        T visit(AdaNode node);
        T visit(AbortAbsent node);
        T visit(AbortPresent node);
        T visit(AbstractAbsent node);
        T visit(AbstractPresent node);
        T visit(AdaNodeList node);
        T visit(AbstractStateDeclList node);
        T visit(AlternativesList node);
        T visit(ConstraintList node);
        T visit(DeclList node);
        T visit(StmtList node);
        T visit(AspectAssocList node);
        T visit(BaseAssocList node);
        T visit(AssocList node);
        T visit(BasicDeclList node);
        T visit(CaseExprAlternativeList node);
        T visit(CaseStmtAlternativeList node);
        T visit(CompilationUnitList node);
        T visit(ConcatOperandList node);
        T visit(ContractCaseAssocList node);
        T visit(DefiningNameList node);
        T visit(DiscriminantSpecList node);
        T visit(ElsifExprPartList node);
        T visit(ElsifStmtPartList node);
        T visit(EnumLiteralDeclList node);
        T visit(ExprAlternativesList node);
        T visit(DiscriminantChoiceList node);
        T visit(NameList node);
        T visit(ParentList node);
        T visit(ParamSpecList node);
        T visit(PragmaNodeList node);
        T visit(SelectWhenPartList node);
        T visit(UnconstrainedArrayIndexList node);
        T visit(VariantList node);
        T visit(AliasedAbsent node);
        T visit(AliasedPresent node);
        T visit(AllAbsent node);
        T visit(AllPresent node);
        T visit(ConstrainedArrayIndices node);
        T visit(UnconstrainedArrayIndices node);
        T visit(AspectAssoc node);
        T visit(AtClause node);
        T visit(AttributeDefClause node);
        T visit(EnumRepClause node);
        T visit(RecordRepClause node);
        T visit(AspectSpec node);
        T visit(ContractCaseAssoc node);
        T visit(PragmaArgumentAssoc node);
        T visit(EntrySpec node);
        T visit(EnumSubpSpec node);
        T visit(SubpSpec node);
        T visit(SyntheticBinarySpec node);
        T visit(SyntheticUnarySpec node);
        T visit(ComponentList node);
        T visit(KnownDiscriminantPart node);
        T visit(UnknownDiscriminantPart node);
        T visit(EntryCompletionFormalParams node);
        T visit(GenericFormalPart node);
        T visit(NullRecordDef node);
        T visit(RecordDef node);
        T visit(AggregateAssoc node);
        T visit(MultiDimArrayAssoc node);
        T visit(CompositeConstraintAssoc node);
        T visit(IteratedAssoc node);
        T visit(ParamAssoc node);
        T visit(AbstractStateDecl node);
        T visit(AnonymousExprDecl node);
        T visit(ComponentDecl node);
        T visit(DiscriminantSpec node);
        T visit(GenericFormalObjDecl node);
        T visit(GenericFormalPackage node);
        T visit(GenericFormalSubpDecl node);
        T visit(GenericFormalTypeDecl node);
        T visit(ParamSpec node);
        T visit(SyntheticFormalParamDecl node);
        T visit(GenericPackageInternal node);
        T visit(PackageDecl node);
        T visit(DiscreteBaseSubtypeDecl node);
        T visit(SubtypeDecl node);
        T visit(ClasswideTypeDecl node);
        T visit(IncompleteTypeDecl node);
        T visit(IncompleteFormalTypeDecl node);
        T visit(IncompleteTaggedTypeDecl node);
        T visit(ProtectedTypeDecl node);
        T visit(TaskTypeDecl node);
        T visit(SingleTaskTypeDecl node);
        T visit(AnonymousTypeDecl node);
        T visit(SynthAnonymousTypeDecl node);
        T visit(ConcreteTypeDecl node);
        T visit(FormalTypeDecl node);
        T visit(AbstractSubpDecl node);
        T visit(AbstractFormalSubpDecl node);
        T visit(ConcreteFormalSubpDecl node);
        T visit(SubpDecl node);
        T visit(EntryDecl node);
        T visit(EnumLiteralDecl node);
        T visit(SyntheticCharEnumLit node);
        T visit(GenericSubpInternal node);
        T visit(SyntheticSubpDecl node);
        T visit(AcceptStmtBody node);
        T visit(ExprFunction node);
        T visit(NullSubpDecl node);
        T visit(SubpBody node);
        T visit(SubpRenamingDecl node);
        T visit(PackageBodyStub node);
        T visit(ProtectedBodyStub node);
        T visit(SubpBodyStub node);
        T visit(TaskBodyStub node);
        T visit(EntryBody node);
        T visit(PackageBody node);
        T visit(ProtectedBody node);
        T visit(TaskBody node);
        T visit(EntryIndexSpec node);
        T visit(ErrorDecl node);
        T visit(ExceptionDecl node);
        T visit(ExceptionHandler node);
        T visit(ForLoopVarDecl node);
        T visit(GenericPackageDecl node);
        T visit(GenericSubpDecl node);
        T visit(GenericPackageInstantiation node);
        T visit(GenericSubpInstantiation node);
        T visit(GenericPackageRenamingDecl node);
        T visit(GenericSubpRenamingDecl node);
        T visit(LabelDecl node);
        T visit(NamedStmtDecl node);
        T visit(NumberDecl node);
        T visit(ObjectDecl node);
        T visit(ExtendedReturnStmtObjectDecl node);
        T visit(NoTypeObjectRenamingDecl node);
        T visit(PackageRenamingDecl node);
        T visit(SingleProtectedDecl node);
        T visit(SingleTaskDecl node);
        T visit(CaseStmtAlternative node);
        T visit(CompilationUnit node);
        T visit(ComponentClause node);
        T visit(ComponentDef node);
        T visit(ConstantAbsent node);
        T visit(ConstantPresent node);
        T visit(CompositeConstraint node);
        T visit(DeltaConstraint node);
        T visit(DigitsConstraint node);
        T visit(RangeConstraint node);
        T visit(DeclarativePart node);
        T visit(PrivatePart node);
        T visit(PublicPart node);
        T visit(ElsifExprPart node);
        T visit(ElsifStmtPart node);
        T visit(AbstractStateDeclExpr node);
        T visit(Allocator node);
        T visit(Aggregate node);
        T visit(BracketAggregate node);
        T visit(DeltaAggregate node);
        T visit(BracketDeltaAggregate node);
        T visit(NullRecordAggregate node);
        T visit(BinOp node);
        T visit(RelationOp node);
        T visit(BoxExpr node);
        T visit(CaseExprAlternative node);
        T visit(ConcatOp node);
        T visit(ConcatOperand node);
        T visit(CaseExpr node);
        T visit(IfExpr node);
        T visit(ContractCases node);
        T visit(DeclExpr node);
        T visit(MembershipExpr node);
        T visit(AttributeRef node);
        T visit(CallExpr node);
        T visit(DefiningName node);
        T visit(SyntheticDefiningName node);
        T visit(DiscreteSubtypeName node);
        T visit(DottedName node);
        T visit(EndName node);
        T visit(ExplicitDeref node);
        T visit(QualExpr node);
        T visit(ReduceAttributeRef node);
        T visit(CharLiteral node);
        T visit(Identifier node);
        T visit(OpAbs node);
        T visit(OpAnd node);
        T visit(OpAndThen node);
        T visit(OpConcat node);
        T visit(OpDiv node);
        T visit(OpDoubleDot node);
        T visit(OpEq node);
        T visit(OpGt node);
        T visit(OpGte node);
        T visit(OpIn node);
        T visit(OpLt node);
        T visit(OpLte node);
        T visit(OpMinus node);
        T visit(OpMod node);
        T visit(OpMult node);
        T visit(OpNeq node);
        T visit(OpNot node);
        T visit(OpNotIn node);
        T visit(OpOr node);
        T visit(OpOrElse node);
        T visit(OpPlus node);
        T visit(OpPow node);
        T visit(OpRem node);
        T visit(OpXor node);
        T visit(StringLiteral node);
        T visit(NullLiteral node);
        T visit(IntLiteral node);
        T visit(RealLiteral node);
        T visit(SyntheticIdentifier node);
        T visit(TargetName node);
        T visit(UpdateAttributeRef node);
        T visit(ParenExpr node);
        T visit(QuantifiedExpr node);
        T visit(RaiseExpr node);
        T visit(UnOp node);
        T visit(HandledStmts node);
        T visit(InterfaceKindLimited node);
        T visit(InterfaceKindProtected node);
        T visit(InterfaceKindSynchronized node);
        T visit(InterfaceKindTask node);
        T visit(IterTypeIn node);
        T visit(IterTypeOf node);
        T visit(LibraryItem node);
        T visit(LimitedAbsent node);
        T visit(LimitedPresent node);
        T visit(ForLoopSpec node);
        T visit(WhileLoopSpec node);
        T visit(ModeDefault node);
        T visit(ModeIn node);
        T visit(ModeInOut node);
        T visit(ModeOut node);
        T visit(MultiAbstractStateDecl node);
        T visit(NotNullAbsent node);
        T visit(NotNullPresent node);
        T visit(NullComponentDecl node);
        T visit(OthersDesignator node);
        T visit(OverridingNotOverriding node);
        T visit(OverridingOverriding node);
        T visit(OverridingUnspecified node);
        T visit(Params node);
        T visit(ParenAbstractStateDecl node);
        T visit(PpElseDirective node);
        T visit(PpElsifDirective node);
        T visit(PpEndIfDirective node);
        T visit(PpIfDirective node);
        T visit(PpThenKw node);
        T visit(PragmaNode node);
        T visit(PrivateAbsent node);
        T visit(PrivatePresent node);
        T visit(ProtectedDef node);
        T visit(ProtectedAbsent node);
        T visit(ProtectedPresent node);
        T visit(QuantifierAll node);
        T visit(QuantifierSome node);
        T visit(RangeSpec node);
        T visit(RenamingClause node);
        T visit(SyntheticRenamingClause node);
        T visit(ReverseAbsent node);
        T visit(ReversePresent node);
        T visit(SelectWhenPart node);
        T visit(AcceptStmt node);
        T visit(AcceptStmtWithStmts node);
        T visit(ForLoopStmt node);
        T visit(LoopStmt node);
        T visit(WhileLoopStmt node);
        T visit(BeginBlock node);
        T visit(DeclBlock node);
        T visit(CaseStmt node);
        T visit(ExtendedReturnStmt node);
        T visit(IfStmt node);
        T visit(NamedStmt node);
        T visit(SelectStmt node);
        T visit(ErrorStmt node);
        T visit(AbortStmt node);
        T visit(AssignStmt node);
        T visit(CallStmt node);
        T visit(DelayStmt node);
        T visit(ExitStmt node);
        T visit(GotoStmt node);
        T visit(Label node);
        T visit(NullStmt node);
        T visit(RaiseStmt node);
        T visit(RequeueStmt node);
        T visit(ReturnStmt node);
        T visit(TerminateAlternative node);
        T visit(SubpKindFunction node);
        T visit(SubpKindProcedure node);
        T visit(Subunit node);
        T visit(SynchronizedAbsent node);
        T visit(SynchronizedPresent node);
        T visit(TaggedAbsent node);
        T visit(TaggedPresent node);
        T visit(TaskDef node);
        T visit(TypeAttributesRepository node);
        T visit(AccessToSubpDef node);
        T visit(AnonymousTypeAccessDef node);
        T visit(TypeAccessDef node);
        T visit(ArrayTypeDef node);
        T visit(DerivedTypeDef node);
        T visit(EnumTypeDef node);
        T visit(FormalDiscreteTypeDef node);
        T visit(InterfaceTypeDef node);
        T visit(ModIntTypeDef node);
        T visit(PrivateTypeDef node);
        T visit(DecimalFixedPointDef node);
        T visit(FloatingPointDef node);
        T visit(OrdinaryFixedPointDef node);
        T visit(RecordTypeDef node);
        T visit(SignedIntTypeDef node);
        T visit(AnonymousType node);
        T visit(EnumLitSynthTypeExpr node);
        T visit(SubtypeIndication node);
        T visit(ConstrainedSubtypeIndication node);
        T visit(DiscreteSubtypeIndication node);
        T visit(SyntheticTypeExpr node);
        T visit(UnconstrainedArrayIndex node);
        T visit(UntilAbsent node);
        T visit(UntilPresent node);
        T visit(UsePackageClause node);
        T visit(UseTypeClause node);
        T visit(ValueSequence node);
        T visit(Variant node);
        T visit(VariantPart node);
        T visit(WithClause node);
        T visit(WithPrivateAbsent node);
        T visit(WithPrivatePresent node);
    }

    /**
     * Interface to visit the AST with a parameter.
     */
    public static interface ParamVisitor<T, P> {
        T visit(AdaNode node, P param);
        T visit(AbortAbsent node, P param);
        T visit(AbortPresent node, P param);
        T visit(AbstractAbsent node, P param);
        T visit(AbstractPresent node, P param);
        T visit(AdaNodeList node, P param);
        T visit(AbstractStateDeclList node, P param);
        T visit(AlternativesList node, P param);
        T visit(ConstraintList node, P param);
        T visit(DeclList node, P param);
        T visit(StmtList node, P param);
        T visit(AspectAssocList node, P param);
        T visit(BaseAssocList node, P param);
        T visit(AssocList node, P param);
        T visit(BasicDeclList node, P param);
        T visit(CaseExprAlternativeList node, P param);
        T visit(CaseStmtAlternativeList node, P param);
        T visit(CompilationUnitList node, P param);
        T visit(ConcatOperandList node, P param);
        T visit(ContractCaseAssocList node, P param);
        T visit(DefiningNameList node, P param);
        T visit(DiscriminantSpecList node, P param);
        T visit(ElsifExprPartList node, P param);
        T visit(ElsifStmtPartList node, P param);
        T visit(EnumLiteralDeclList node, P param);
        T visit(ExprAlternativesList node, P param);
        T visit(DiscriminantChoiceList node, P param);
        T visit(NameList node, P param);
        T visit(ParentList node, P param);
        T visit(ParamSpecList node, P param);
        T visit(PragmaNodeList node, P param);
        T visit(SelectWhenPartList node, P param);
        T visit(UnconstrainedArrayIndexList node, P param);
        T visit(VariantList node, P param);
        T visit(AliasedAbsent node, P param);
        T visit(AliasedPresent node, P param);
        T visit(AllAbsent node, P param);
        T visit(AllPresent node, P param);
        T visit(ConstrainedArrayIndices node, P param);
        T visit(UnconstrainedArrayIndices node, P param);
        T visit(AspectAssoc node, P param);
        T visit(AtClause node, P param);
        T visit(AttributeDefClause node, P param);
        T visit(EnumRepClause node, P param);
        T visit(RecordRepClause node, P param);
        T visit(AspectSpec node, P param);
        T visit(ContractCaseAssoc node, P param);
        T visit(PragmaArgumentAssoc node, P param);
        T visit(EntrySpec node, P param);
        T visit(EnumSubpSpec node, P param);
        T visit(SubpSpec node, P param);
        T visit(SyntheticBinarySpec node, P param);
        T visit(SyntheticUnarySpec node, P param);
        T visit(ComponentList node, P param);
        T visit(KnownDiscriminantPart node, P param);
        T visit(UnknownDiscriminantPart node, P param);
        T visit(EntryCompletionFormalParams node, P param);
        T visit(GenericFormalPart node, P param);
        T visit(NullRecordDef node, P param);
        T visit(RecordDef node, P param);
        T visit(AggregateAssoc node, P param);
        T visit(MultiDimArrayAssoc node, P param);
        T visit(CompositeConstraintAssoc node, P param);
        T visit(IteratedAssoc node, P param);
        T visit(ParamAssoc node, P param);
        T visit(AbstractStateDecl node, P param);
        T visit(AnonymousExprDecl node, P param);
        T visit(ComponentDecl node, P param);
        T visit(DiscriminantSpec node, P param);
        T visit(GenericFormalObjDecl node, P param);
        T visit(GenericFormalPackage node, P param);
        T visit(GenericFormalSubpDecl node, P param);
        T visit(GenericFormalTypeDecl node, P param);
        T visit(ParamSpec node, P param);
        T visit(SyntheticFormalParamDecl node, P param);
        T visit(GenericPackageInternal node, P param);
        T visit(PackageDecl node, P param);
        T visit(DiscreteBaseSubtypeDecl node, P param);
        T visit(SubtypeDecl node, P param);
        T visit(ClasswideTypeDecl node, P param);
        T visit(IncompleteTypeDecl node, P param);
        T visit(IncompleteFormalTypeDecl node, P param);
        T visit(IncompleteTaggedTypeDecl node, P param);
        T visit(ProtectedTypeDecl node, P param);
        T visit(TaskTypeDecl node, P param);
        T visit(SingleTaskTypeDecl node, P param);
        T visit(AnonymousTypeDecl node, P param);
        T visit(SynthAnonymousTypeDecl node, P param);
        T visit(ConcreteTypeDecl node, P param);
        T visit(FormalTypeDecl node, P param);
        T visit(AbstractSubpDecl node, P param);
        T visit(AbstractFormalSubpDecl node, P param);
        T visit(ConcreteFormalSubpDecl node, P param);
        T visit(SubpDecl node, P param);
        T visit(EntryDecl node, P param);
        T visit(EnumLiteralDecl node, P param);
        T visit(SyntheticCharEnumLit node, P param);
        T visit(GenericSubpInternal node, P param);
        T visit(SyntheticSubpDecl node, P param);
        T visit(AcceptStmtBody node, P param);
        T visit(ExprFunction node, P param);
        T visit(NullSubpDecl node, P param);
        T visit(SubpBody node, P param);
        T visit(SubpRenamingDecl node, P param);
        T visit(PackageBodyStub node, P param);
        T visit(ProtectedBodyStub node, P param);
        T visit(SubpBodyStub node, P param);
        T visit(TaskBodyStub node, P param);
        T visit(EntryBody node, P param);
        T visit(PackageBody node, P param);
        T visit(ProtectedBody node, P param);
        T visit(TaskBody node, P param);
        T visit(EntryIndexSpec node, P param);
        T visit(ErrorDecl node, P param);
        T visit(ExceptionDecl node, P param);
        T visit(ExceptionHandler node, P param);
        T visit(ForLoopVarDecl node, P param);
        T visit(GenericPackageDecl node, P param);
        T visit(GenericSubpDecl node, P param);
        T visit(GenericPackageInstantiation node, P param);
        T visit(GenericSubpInstantiation node, P param);
        T visit(GenericPackageRenamingDecl node, P param);
        T visit(GenericSubpRenamingDecl node, P param);
        T visit(LabelDecl node, P param);
        T visit(NamedStmtDecl node, P param);
        T visit(NumberDecl node, P param);
        T visit(ObjectDecl node, P param);
        T visit(ExtendedReturnStmtObjectDecl node, P param);
        T visit(NoTypeObjectRenamingDecl node, P param);
        T visit(PackageRenamingDecl node, P param);
        T visit(SingleProtectedDecl node, P param);
        T visit(SingleTaskDecl node, P param);
        T visit(CaseStmtAlternative node, P param);
        T visit(CompilationUnit node, P param);
        T visit(ComponentClause node, P param);
        T visit(ComponentDef node, P param);
        T visit(ConstantAbsent node, P param);
        T visit(ConstantPresent node, P param);
        T visit(CompositeConstraint node, P param);
        T visit(DeltaConstraint node, P param);
        T visit(DigitsConstraint node, P param);
        T visit(RangeConstraint node, P param);
        T visit(DeclarativePart node, P param);
        T visit(PrivatePart node, P param);
        T visit(PublicPart node, P param);
        T visit(ElsifExprPart node, P param);
        T visit(ElsifStmtPart node, P param);
        T visit(AbstractStateDeclExpr node, P param);
        T visit(Allocator node, P param);
        T visit(Aggregate node, P param);
        T visit(BracketAggregate node, P param);
        T visit(DeltaAggregate node, P param);
        T visit(BracketDeltaAggregate node, P param);
        T visit(NullRecordAggregate node, P param);
        T visit(BinOp node, P param);
        T visit(RelationOp node, P param);
        T visit(BoxExpr node, P param);
        T visit(CaseExprAlternative node, P param);
        T visit(ConcatOp node, P param);
        T visit(ConcatOperand node, P param);
        T visit(CaseExpr node, P param);
        T visit(IfExpr node, P param);
        T visit(ContractCases node, P param);
        T visit(DeclExpr node, P param);
        T visit(MembershipExpr node, P param);
        T visit(AttributeRef node, P param);
        T visit(CallExpr node, P param);
        T visit(DefiningName node, P param);
        T visit(SyntheticDefiningName node, P param);
        T visit(DiscreteSubtypeName node, P param);
        T visit(DottedName node, P param);
        T visit(EndName node, P param);
        T visit(ExplicitDeref node, P param);
        T visit(QualExpr node, P param);
        T visit(ReduceAttributeRef node, P param);
        T visit(CharLiteral node, P param);
        T visit(Identifier node, P param);
        T visit(OpAbs node, P param);
        T visit(OpAnd node, P param);
        T visit(OpAndThen node, P param);
        T visit(OpConcat node, P param);
        T visit(OpDiv node, P param);
        T visit(OpDoubleDot node, P param);
        T visit(OpEq node, P param);
        T visit(OpGt node, P param);
        T visit(OpGte node, P param);
        T visit(OpIn node, P param);
        T visit(OpLt node, P param);
        T visit(OpLte node, P param);
        T visit(OpMinus node, P param);
        T visit(OpMod node, P param);
        T visit(OpMult node, P param);
        T visit(OpNeq node, P param);
        T visit(OpNot node, P param);
        T visit(OpNotIn node, P param);
        T visit(OpOr node, P param);
        T visit(OpOrElse node, P param);
        T visit(OpPlus node, P param);
        T visit(OpPow node, P param);
        T visit(OpRem node, P param);
        T visit(OpXor node, P param);
        T visit(StringLiteral node, P param);
        T visit(NullLiteral node, P param);
        T visit(IntLiteral node, P param);
        T visit(RealLiteral node, P param);
        T visit(SyntheticIdentifier node, P param);
        T visit(TargetName node, P param);
        T visit(UpdateAttributeRef node, P param);
        T visit(ParenExpr node, P param);
        T visit(QuantifiedExpr node, P param);
        T visit(RaiseExpr node, P param);
        T visit(UnOp node, P param);
        T visit(HandledStmts node, P param);
        T visit(InterfaceKindLimited node, P param);
        T visit(InterfaceKindProtected node, P param);
        T visit(InterfaceKindSynchronized node, P param);
        T visit(InterfaceKindTask node, P param);
        T visit(IterTypeIn node, P param);
        T visit(IterTypeOf node, P param);
        T visit(LibraryItem node, P param);
        T visit(LimitedAbsent node, P param);
        T visit(LimitedPresent node, P param);
        T visit(ForLoopSpec node, P param);
        T visit(WhileLoopSpec node, P param);
        T visit(ModeDefault node, P param);
        T visit(ModeIn node, P param);
        T visit(ModeInOut node, P param);
        T visit(ModeOut node, P param);
        T visit(MultiAbstractStateDecl node, P param);
        T visit(NotNullAbsent node, P param);
        T visit(NotNullPresent node, P param);
        T visit(NullComponentDecl node, P param);
        T visit(OthersDesignator node, P param);
        T visit(OverridingNotOverriding node, P param);
        T visit(OverridingOverriding node, P param);
        T visit(OverridingUnspecified node, P param);
        T visit(Params node, P param);
        T visit(ParenAbstractStateDecl node, P param);
        T visit(PpElseDirective node, P param);
        T visit(PpElsifDirective node, P param);
        T visit(PpEndIfDirective node, P param);
        T visit(PpIfDirective node, P param);
        T visit(PpThenKw node, P param);
        T visit(PragmaNode node, P param);
        T visit(PrivateAbsent node, P param);
        T visit(PrivatePresent node, P param);
        T visit(ProtectedDef node, P param);
        T visit(ProtectedAbsent node, P param);
        T visit(ProtectedPresent node, P param);
        T visit(QuantifierAll node, P param);
        T visit(QuantifierSome node, P param);
        T visit(RangeSpec node, P param);
        T visit(RenamingClause node, P param);
        T visit(SyntheticRenamingClause node, P param);
        T visit(ReverseAbsent node, P param);
        T visit(ReversePresent node, P param);
        T visit(SelectWhenPart node, P param);
        T visit(AcceptStmt node, P param);
        T visit(AcceptStmtWithStmts node, P param);
        T visit(ForLoopStmt node, P param);
        T visit(LoopStmt node, P param);
        T visit(WhileLoopStmt node, P param);
        T visit(BeginBlock node, P param);
        T visit(DeclBlock node, P param);
        T visit(CaseStmt node, P param);
        T visit(ExtendedReturnStmt node, P param);
        T visit(IfStmt node, P param);
        T visit(NamedStmt node, P param);
        T visit(SelectStmt node, P param);
        T visit(ErrorStmt node, P param);
        T visit(AbortStmt node, P param);
        T visit(AssignStmt node, P param);
        T visit(CallStmt node, P param);
        T visit(DelayStmt node, P param);
        T visit(ExitStmt node, P param);
        T visit(GotoStmt node, P param);
        T visit(Label node, P param);
        T visit(NullStmt node, P param);
        T visit(RaiseStmt node, P param);
        T visit(RequeueStmt node, P param);
        T visit(ReturnStmt node, P param);
        T visit(TerminateAlternative node, P param);
        T visit(SubpKindFunction node, P param);
        T visit(SubpKindProcedure node, P param);
        T visit(Subunit node, P param);
        T visit(SynchronizedAbsent node, P param);
        T visit(SynchronizedPresent node, P param);
        T visit(TaggedAbsent node, P param);
        T visit(TaggedPresent node, P param);
        T visit(TaskDef node, P param);
        T visit(TypeAttributesRepository node, P param);
        T visit(AccessToSubpDef node, P param);
        T visit(AnonymousTypeAccessDef node, P param);
        T visit(TypeAccessDef node, P param);
        T visit(ArrayTypeDef node, P param);
        T visit(DerivedTypeDef node, P param);
        T visit(EnumTypeDef node, P param);
        T visit(FormalDiscreteTypeDef node, P param);
        T visit(InterfaceTypeDef node, P param);
        T visit(ModIntTypeDef node, P param);
        T visit(PrivateTypeDef node, P param);
        T visit(DecimalFixedPointDef node, P param);
        T visit(FloatingPointDef node, P param);
        T visit(OrdinaryFixedPointDef node, P param);
        T visit(RecordTypeDef node, P param);
        T visit(SignedIntTypeDef node, P param);
        T visit(AnonymousType node, P param);
        T visit(EnumLitSynthTypeExpr node, P param);
        T visit(SubtypeIndication node, P param);
        T visit(ConstrainedSubtypeIndication node, P param);
        T visit(DiscreteSubtypeIndication node, P param);
        T visit(SyntheticTypeExpr node, P param);
        T visit(UnconstrainedArrayIndex node, P param);
        T visit(UntilAbsent node, P param);
        T visit(UntilPresent node, P param);
        T visit(UsePackageClause node, P param);
        T visit(UseTypeClause node, P param);
        T visit(ValueSequence node, P param);
        T visit(Variant node, P param);
        T visit(VariantPart node, P param);
        T visit(WithClause node, P param);
        T visit(WithPrivateAbsent node, P param);
        T visit(WithPrivatePresent node, P param);
    }

    // ==========
    // Util classes
    // ==========

    /**
     * This class represents a pointer and can hold NI and JNI addresses.
     */
    public static final class PointerWrapper {

        // ----- Instance attributes -----

        /** The pointer NI value. */
        private PointerBase ni;

        /** The pointer JNI value. */
        private final long jni;

        // ----- Constructors -----

        /**
         * Create a new custom pointer from a NI pointer based value.
         *
         * @param niPointer The pointer based value.
         */
        PointerWrapper(
            final PointerBase niPointer
        ) {
            this.ni = niPointer;
            this.jni = -1;
        }

        /**
         * Create a new custom pointer from a long value.
         *
         * @param jniPointer The pointer in a long value.
         */
        PointerWrapper(
            final long jniPointer
        ) {
            this.jni = jniPointer;
        }

        /**
         * Wrap the given NI pointer in the Java class.
         *
         * @param niPointer The NI pointer to wrap.
         * @return The wrapped pointer.
         */
        static PointerWrapper wrap(
            final PointerBase niPointer
        ) {
            return new PointerWrapper(niPointer);
        }

        /**
         * Get the null pointer according to the execution mode.
         *
         * @return The null custom pointer.
         */
        public static PointerWrapper nullPointer() {

            if(ImageInfo.inImageCode()) {
                return new PointerWrapper(WordFactory.nullPointer());
            } else {
                return new PointerWrapper(0L);
            }

        }

        // ----- Instance methods -----

        /**
         * Get the pointer as an NI pointer based value.
         *
         * @return The pointer based value for NI.
         */
        public <T extends PointerBase> T ni() {
            return (T) this.ni;
        }

        /**
         * Get the pointer as a long Java value.
         *
         * @return The pointer as a long value for JNI.
         */
        public long jni() {
            return this.jni;
        }

        /**
         * Get if the pointer is null.
         *
         * @return True if the pointer is null, false else.
         */
        public boolean isNull() {

            if(ImageInfo.inImageCode()) {
                return this.ni.isNull();
            } else {
                return this.jni == 0;
            }

        }

        // ----- Override methods -----

        @Override
        public String toString() {

            if(ImageInfo.inImageCode()) {
                return "PointerWrapper{"
                    + this.ni.rawValue()
                    + "}";
            } else {
                return "PointerWrapper{"
                    + this.jni
                    + "}";
            }

        }

        @Override
        public boolean equals(Object o) {
            if(o == this) return true;
            if(!(o instanceof PointerWrapper)) return false;
            final PointerWrapper other = (PointerWrapper) o;
            if(ImageInfo.inImageCode()) {
                return this.ni.equal(other.ni);
            } else {
                return this.jni == other.jni;
            }
        }

        @Override
        public int hashCode() {

            if(ImageInfo.inImageCode()) {
                return (int) this.ni.rawValue();
            } else {
                return (int) this.jni;
            }

        }

    }

    /**
     * This class represents the description of a node field.
     */
    public static final class LibadalangField {

        // ----- Instance attributes -----

        /** The Java method for the field */
        public final Method javaMethod;

        /** The parameters of the method */
        public final List<Param> params;

        // ----- Constructors -----

        /**
         * Create a new field description.
         *
         * @param method The Java method to access the field.
         * @param params The parameters of the field call.
         */
        public LibadalangField(
            final Method javaMethod,
            final List<Param> params
        ) {
            this.javaMethod = javaMethod;
            this.params = params;
        }

    }

    /**
     * This class represents a parameter description.
     */
    public static class Param {

        // ----- Instance attributes -----

        /** The type of the argument */
        public final Class<?> type;

        /** The name of the parameter */
        public final String name;

        // ----- Constructors -----

        /**
         * Create a new langkit parameter.
         *
         * @param type The type of the parameter.
         * @param name The name of the parameter.
         */
        public Param(
            final Class<?> type,
            final String name
        ) {
            this.type = type;
            this.name = name;
        }

    }

    /**
     * This class represents an parameter in a langkit function description
     * which has a default value.
     */
    public static final class ParamWithDefaultValue extends Param {

        // ----- Instance attributes -----

        /** The default value of the parameter */
        public final Object defaultValue;

        // ----- Constructors -----

        /**
         * Create a new langkit parameter.
         *
         * @param type The type of the parameter.
         * @param name The name of the parameter.
         * @param defaultValue The default value of the parameter.
         */
        public ParamWithDefaultValue(
            final Class<?> type,
            final String name,
            final Object defaultValue
        ) {
            super(type, name);
            this.defaultValue = defaultValue;
        }

    }

    // ==========
    // Language specific extensions
    // ==========

    
      


    // ==========
    // Define classes to handle project loading
    // ==========

    /**
     * Exception to represent errors during project manipulation
     */
    public static final class ProjectManagerException extends RuntimeException {
        ProjectManagerException(
            final String message
        ) {
            super(message);
        }
    }

    /**
     * Enum to represent the source file mode for a GPR project
     */
    public static enum SourceFileMode {

        // ----- Enum values -----

        DEFAULT(0),
        ROOT_PROJECT(1),
        WHOLE_PROJECT(2),
        WHOLE_PROJECT_WITH_RUNTIME(3),
        ;

        // ----- Class attributes -----

        /** Singleton that represents the none source file mode */
        public static final SourceFileMode NONE = DEFAULT;

        /** The map from int to enum values */
        private static final Map<Integer, SourceFileMode> map = new HashMap<>();

        // ----- Instance attributes -----

        /** The value of the instance */
        private final int value;

        // ----- Constructors -----

        /**
         * The private constructor
         */
        private SourceFileMode(
            final int value
        ) {
            this.value = value;
        }

        static {
            for(SourceFileMode elem : SourceFileMode.values()) {
                map.put(elem.value, elem);
            }
        }

        // ----- Enum methods -----

        /**
         * Get a source file mode from a native integer value.
         *
         * @param cValue The native value of the enum.
         * @return The Java source file mode.
         * @throws EnumException If the given native value doesn't correspond
         * to an actual enum value.
         */
        public static SourceFileMode fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get SourceFileMode from " + cValue
                );
            return (SourceFileMode) map.get(cValue);
        }

        /**
         * Get the native integer value of the enum instance.
         *
         * @return The native C value.
         */
        public int toC() {
            return this.value;
        }

    }

    /**
     * This class represents a scenario variable for a GPR project file.
     */
    public static final class ScenarioVariable {

        // ----- Class attributes -----

        /** Singleton that represents the none scenario variable. */
        public static final ScenarioVariable NONE =
            new ScenarioVariable(null, null);

        // ----- Instance attributes -----

        /** The name of the variable. */
        public final String name;

        /** The value of the variable. */
        public final String value;

        // ----- Constructors -----

        /**
         * Create a new scenario variable with its name and value.
         *
         * @param name Name of the variable.
         * @param value Value of the variable.
         */
        ScenarioVariable(
            final String name,
            final String value
        ) {
            this.name = name;
            this.value = value;
        }

        /**
         * Public method to create scenario variable.
         * This method raise a runtime exception if name or value is null.
         *
         * @param name Name of the variable.
         * @param value Value of the variable.
         */
        public static ScenarioVariable create(
            final String name,
            final String value
        ) {
            if(name == null) throw new RuntimeException("Scenario variable name cannot be null");
            if(value == null) throw new RuntimeException("Scenario variable value cannot be null");
            return new ScenarioVariable(name, value);
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given pointer to a native scenario variable.
         *
         * @param pointer The pointer to the native scenario variable.
         * @return The wrapped scenario variable.
         */
        public static ScenarioVariable wrap(
            final Pointer pointer
        ) {
            return wrap((ScenarioVariableNative) pointer.readWord(0));
        }

        /**
         * Wrap the given native scenario variable.
         *
         * @param scenarioVariableNative The native scenario variable.
         * @return The wrapped scenario variable.
         */
        public static ScenarioVariable wrap(
            final ScenarioVariableNative scenarioVariableNative
        ) {
            final CCharPointer nameNative = scenarioVariableNative.get_name();
            final CCharPointer valueNative = scenarioVariableNative.get_value();
            return new ScenarioVariable(
                nameNative.isNull() ?
                    null :
                    toJString(nameNative),
                valueNative.isNull() ?
                    null :
                    toJString(valueNative)
            );
        }

        /**
         * Unwrap the scenario variable in the given native value.
         *
         * @param scenarioVariableNative The native value to fill.
         */
        public void unwrap(
            final ScenarioVariableNative scenarioVariableNative
        ) {
            final CCharPointer nameNative = this.name == null ?
                WordFactory.nullPointer() :
                toCString(this.name);
            final CCharPointer valueNative = this.value == null ?
                WordFactory.nullPointer() :
                toCString(this.value);
            scenarioVariableNative.set_name(nameNative);
            scenarioVariableNative.set_value(valueNative);
        }

        /**
         * Release the given native scenario variable allocated strings.
         *
         * @param scenarioVariableNative The native scenario variable to release.
         */
        public static void release(
            final ScenarioVariableNative scenarioVariableNative
        ) {
            if(scenarioVariableNative.get_name().isNonNull()) {
                UnmanagedMemory.free(scenarioVariableNative.get_name());
            }
            if(scenarioVariableNative.get_value().isNonNull()) {
                UnmanagedMemory.free(scenarioVariableNative.get_value());
            }
        }

        // ----- Override methods -----

        /** @see java.lang.Object#toString() */
        public String toString() {
            return "ScenarioVariable(" +
                this.name + " = " + this.value +
                ")";
        }

    }

    /**
     * This class is used for the GPR project loading.
     */
    public static final class ProjectManager implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none project manager. */
        public static final ProjectManager NONE =
            new ProjectManager(PointerWrapper.nullPointer());

        // ----- Instance attributes -----

        /** Reference to the native value. */
        private final PointerWrapper reference;

        // ----- Constructors -----

        /**
         * Create a new project manager from its native reference.
         *
         * @param reference The reference to the native project manager.
         */
        ProjectManager(
            final PointerWrapper reference
        ) {
            this.reference = reference;
        }

        /**
         * Create a project manager for the given project file.
         *
         * @param projectFile The GPR project file to load.
         * @return The newly created project manager.
         */
        public static ProjectManager create(
            final String projectFile
        ) {
            return create(
                projectFile,
                null,
                "",
                ""
            );
        }

        /**
         * Create a project manager from a project file, target and runtime.
         *
         * @param projectFile The GPR file to load.
         * @param scenarioVariables The scenario variables for the project, it can be null.
         * @param target The target to load.
         * @param runtime The runtime to load.
         * @return The newly created project manager.
         */
        public static ProjectManager create(
            final String projectFile,
            final ScenarioVariable[] scenarioVariables,
            final String target,
            final String runtime
        ) {

            if(ImageInfo.inImageCode()) {
                // Create the scenario variable array
                final Pointer scenarioVariablesNative;
                final int scenarioVariableNativeSize = SizeOf.get(ScenarioVariableNative.class);

                if(scenarioVariables != null && scenarioVariables.length > 0) {
                    final int size = scenarioVariables.length + 1;
                    scenarioVariablesNative = UnmanagedMemory.calloc(
                        size * scenarioVariableNativeSize
                    );
                    for(int i = 0 ; i < scenarioVariables.length ; i++) {
                        final ScenarioVariableNative scenarioVariableNative = (ScenarioVariableNative)
                            scenarioVariablesNative.add(i * scenarioVariableNativeSize);
                        scenarioVariables[i].unwrap(scenarioVariableNative);
                    }
                } else {
                    scenarioVariablesNative = WordFactory.nullPointer();
                }

                // Call the native project loading function
                final CCharPointer projectFileNative = toCString(projectFile);
                final CCharPointer targetNative = toCString(target);
                final CCharPointer runtimeNative = toCString(runtime);
                final Pointer projectPointer = StackValue.get(SizeOf.get(VoidPointer.class));
                projectPointer.writeWord(0, WordFactory.nullPointer());
                final Pointer errorsPointer = StackValue.get(SizeOf.get(VoidPointer.class));
                errorsPointer.writeWord(0, WordFactory.nullPointer());
                NI_LIB.ada_gpr_project_load(
                    projectFileNative,
                    scenarioVariablesNative,
                    targetNative,
                    runtimeNative,
                    projectPointer,
                    errorsPointer
                );

                // Check the langkit exception and cast it into a project manager error
                try {
                    checkException();
                } catch (LangkitException e) {
                    throw new ProjectManagerException(e.getMessage());
                }

                // Get the result of modified values
                final ProjectManagerNative projectManagerNative = (ProjectManagerNative) projectPointer.readWord(0);
                final StringArrayNative errorArrayNative = (StringArrayNative) errorsPointer.readWord(0);

                // Free the scenario variables
                if(scenarioVariablesNative.isNonNull()) {
                    for(int i = 0 ; i < scenarioVariables.length ; i++) {
                        final ScenarioVariableNative scenarioVariableNative = (ScenarioVariableNative)
                            scenarioVariablesNative.add(i * scenarioVariableNativeSize);
                        ScenarioVariable.release(scenarioVariableNative);
                    }
                    UnmanagedMemory.free(scenarioVariablesNative);
                }

                // Free the allocated strings
                UnmanagedMemory.free(projectFileNative);
                UnmanagedMemory.free(targetNative);
                UnmanagedMemory.free(runtimeNative);

                // `errorsPointer` is not allocated if an exception was raised during project file loading
                if (errorArrayNative.isNonNull()) {
                    // Translate the error native array into a Java array
                    final String[] errors = toJStringArray(errorArrayNative);
                    for (String error : errors) {
                        System.err.println("Error during project opening: " + error);
                    }

                    // Free the error array
                    NI_LIB.ada_free_string_array(errorArrayNative);
                }

                // Check the langkit exception and cast it into a project manager error
                try {
                    checkException();
                } catch (LangkitException e) {
                    throw new ProjectManagerException(e.getMessage());
                }

                // Return the Java wrapped project manager
                return wrap(projectManagerNative);
            } else {
                final PointerWrapper reference = JNI_LIB.ada_gpr_project_load(
                    projectFile,
                    scenarioVariables,
                    target,
                    runtime
                );

                // Check the langkit exceptions
                try {
                    checkException();
                } catch (LangkitException e) {
                    throw new ProjectManagerException(e.getMessage());
                }

                // Return the project manager
                return new ProjectManager(reference);
            }

        }

        // ----- Graal C API methods -----

        /**
         * Wrap a native project manager in the Java class.
         *
         * @param pointer The pointer to the native project manager.
         * @return The newly wrapped project manager.
         */
        static ProjectManager wrap(
            final Pointer pointer
        ) {
            return wrap((ProjectManagerNative) pointer.readWord(0));
        }

        /**
         * Wrap a native project manager in the Java class.
         *
         * @param projectManagerNative The native project manager to wrap.
         * @return The newly wrapped project manager.
         */
        static ProjectManager wrap(
            final ProjectManagerNative projectManagerNative
        ) {
            return new ProjectManager(new PointerWrapper(projectManagerNative));
        }

        /**
         * Unwrap the project manager inside the given pointer.
         *
         * @param pointer The pointer to write in.
         */
        public void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Get the native value of the project manager.
         *
         * @return The native project manager.
         */
        public ProjectManagerNative unwrap() {
            return (ProjectManagerNative) this.reference.ni();
        }

        // ----- Class methods -----

        /**
         * Translate a native string array structure into a Java string
         * array.
         *
         * @param stringArrayNative The native string array structure.
         * @return The Java string array.
         */
        private static String[] toJStringArray(
            final StringArrayNative stringArrayNative
        ) {
            final String[] res = new String[stringArrayNative.get_length()];
            final CCharPointerPointer nativeFilesPointer = stringArrayNative.get_c_ptr();
            for(int i = 0 ; i < res.length ; i++) {
                final CCharPointer nativeFile = nativeFilesPointer.read(i);
                res[i] = toJString(nativeFile);
            }
            return res;
        }

        // -----  Instance methods -----

        /**
         * Create a unit provider for the given subproject.
         *
         * @param subproject The subproject for which to create a unit provider.
         * @return The unit provider for the project manager.
         */
        public UnitProvider getProvider(final String subproject) {
            final UnitProvider result;
            if(ImageInfo.inImageCode()) {
                final CCharPointer subprojectNative =
                    subproject == null ?
                    WordFactory.nullPointer() :
                    toCString(subproject);

                UnitProviderNative unitProviderNative = NI_LIB.ada_gpr_project_create_unit_provider(
                    this.reference.ni(),
                    subprojectNative
                );
                result = UnitProvider.wrap(unitProviderNative);
                if (subproject != null) {
                    UnmanagedMemory.free(subprojectNative);
                }
            } else {
                result = JNI_LIB.ada_gpr_project_create_unit_provider(
                    this,
                    subproject
                );
            }
            return result;
        }

        /**
         * Create a unit provider for root project.
         */
        public UnitProvider getProvider() {
            return this.getProvider(null);
        }

        /**
         * Create a new analysis context from a GPR project.
         *
         * The unit provider, file reader, config pragmas and default charset
         * are inferred from the designated project: see the ``getProvider``
         * method for the semantics of the ``project`` argument.
         *
         * See the ``AnalysisContext`` class constructor for the semantics of
         * the other arguments.
         *
         * .. TODO: For now, the returned ``AnalysisContext`` instance has a
         *    weak reference to the project manager: make sure the
         *    ``ProjectManager`` instance lives at least as long as the
         *    ``AnalysisContext`` one.
         */
        public AnalysisContext createContext(
            String subproject,
            EventHandler eventHandler,
            boolean withTrivia,
            int tabStop
        ) {
            if(ImageInfo.inImageCode()) {
                // Prepare C values for native calls
                final CCharPointer subproject_c =
                  subproject == null
                  ? WordFactory.nullPointer()
                  : toCString(subproject);
                EventHandlerNative eventHandler_c =
                  eventHandler == null
                  ? WordFactory.nullPointer()
                  : eventHandler.reference.ni();

                // Manually allocate a C-level analysis context so that we can
                // initialize it ourselves.
                final PointerWrapper context = new PointerWrapper(
                    NI_LIB.ada_allocate_analysis_context()
                );

                // Create the Java wrapper, so that we have one ready for
                // event handler callbacks triggered during context
                // initialization.
                AnalysisContext result =
                    AnalysisContext.fromReference(context, eventHandler, true);

                // "result" has its own ownership share: release ours
                NI_LIB.ada_context_decref(context.ni());

                // TODO: attach "this" to "result" so that the former lives at
                // least as long as the former.

                // Finally, initialize the analysis context. Note that this
                // step may raise an exception: in that case, the analysis
                // context is considered not initialized: release it.
                NI_LIB.ada_gpr_project_initialize_context(
                    this.reference.ni(),
                    context.ni(),
                    subproject_c,
                    eventHandler_c,
                    withTrivia ? 1 : 0,
                    tabStop
                );
                UnmanagedMemory.free(subproject_c);
                final LangkitExceptionNative exc_c =
                    NI_LIB.ada_get_last_exception();
                if (exc_c.isNonNull()) {
                    LangkitException exc = wrapException(exc_c);
                    NI_LIB.ada_context_decref(context.ni());
                    throw exc;
                }

                return result;
            } else {
                return JNI_LIB.ada_gpr_project_create_context(
                    this,
                    subproject,
                    eventHandler,
                    withTrivia,
                    tabStop
                );
            }
        }

        /**
         * Get the files for the given subprojects in a string array.
         *
         * @param mode The file getting mode.
         * @param subprojects The subprojects to consider.
         * @return The array that contains the project files.
         */
        public String[] getFiles(
            final SourceFileMode mode,
            final String[] subprojects
        ) {
            // Verify if the project is null
            if(this.reference.isNull())
                return new String[0];

            String[] result = null;
            LangkitException exc;

            if(ImageInfo.inImageCode()) {
                // Convert the Java array of subprojects ("subprojects"
                // argument) into the required C array
                // (subprojectCount/subprojectsNative).
                final int subprojectCount;
                final CCharPointerPointer subprojectsNative;

                if (subprojects != null && subprojects.length > 0) {
                    subprojectCount = subprojects.length;
                    subprojectsNative =
                        UnmanagedMemory.calloc(
                            subprojectCount * SizeOf.get(CCharPointerPointer.class)
                        );
                } else {
                    subprojectCount = 0;
                    subprojectsNative = WordFactory.nullPointer();
                }

                for (int i = 0; i < subprojectCount; ++i) {
                    subprojectsNative.write(i, toCString(subprojects[i]));
                }

                // Call the C API. Keep track of a potential native exception.
                StringArrayNative sourceFileArray =
                    NI_LIB.ada_gpr_project_source_files(
                        this.reference.ni(),
                        mode.toC(),
                        subprojectsNative,
                        subprojectCount
                    );
                exc = getLastException();

                // If the call was successful, create the Java array result and
                // initialize it, and free the C array result.
                if(exc == null) {
                    result = toJStringArray(sourceFileArray);
                    NI_LIB.ada_free_string_array(sourceFileArray);
                }

                // Release subprojectsNative
                for (int i = 0; i < subprojectCount; ++i) {
                    UnmanagedMemory.free(subprojectsNative.read(i));
                }
                if (subprojectCount > 0) {
                    UnmanagedMemory.free(subprojectsNative);
                }
            } else {
                result = JNI_LIB.ada_gpr_project_source_files(
                    this,
                    mode.toC(),
                    subprojects
                );
                exc = getLastException();
            }

            // If we got an exception, turn it into a ProjectManagerException
            // one.
            if(exc != null) {
                throw new ProjectManagerException(exc.getMessage());
            }

            return result;

        }

        /**
         * Get the files of the root project in a string array.
         *
         * @param mode The file getting mode.
         * @return The array that contains the project files.
         */
        public String[] getFiles(final SourceFileMode mode) {
            return this.getFiles(mode, null);
        }

        /** @see java.lang.AutoCloseable#close() */
        @Override
        public void close() {

            if(ImageInfo.inImageCode()) {
                NI_LIB.ada_gpr_project_free(this.reference.ni());
            } else {
                JNI_LIB.ada_gpr_project_free(this);
            }

        }

    }

    /**
     * Return a unit provider that knows which compilation units are to be
     * found in the given list of source files.
     *
     * This knowledge is built trying to parse all given input files as Ada
     * source files and listing the compilation units found there. Files that
     * cannot be parsed properly are discarded. If two compilation units are
     * found for the same unit, the first that is found in the given input
     * files is taken and the other ones are discarded.
     *
     * Source files are decoded using the given charset. If it is ``null``, the
     * default charset (ISO-8859-1) is used.
     *
     * .. TODO: Find a way to report discarded source files/compilation units.
     */
    public static UnitProvider createAutoProvider(
        final String[] sourceFiles,
        final String charset
    ) {
        if (ImageInfo.inImageCode()) {
            final CCharPointer charsetNative =
                charset == null ?
                WordFactory.nullPointer() :
                toCString(charset);

            // Allocate the C array of C strings that will contain decoded
            // source file names. Make room for one additional null pointer
            // to mark the end of the array.
            final CCharPointerPointer sourceFilesNative =
                UnmanagedMemory.calloc(
                    (sourceFiles.length + 1) * SizeOf.get(CCharPointerPointer.class)
                );

            for (int i = 0; i < sourceFiles.length; ++i) {
                sourceFilesNative.write(i, toCString(sourceFiles[i]));
            }

            // Create the auto provider
            final UnitProviderNative unitProviderNative =
                NI_LIB.ada_create_auto_provider(
                    sourceFilesNative,
                    charsetNative
                );

            // Release all temporarily allocated memory
            for (int i = 0; i < sourceFiles.length; ++i) {
                UnmanagedMemory.free(sourceFilesNative.read(i));
            }

            UnmanagedMemory.free(sourceFilesNative);

            if (charset != null) {
                UnmanagedMemory.free(charsetNative);
            }

            return UnitProvider.wrap(unitProviderNative);
        } else {
            return JNI_LIB.ada_create_auto_provider(
                sourceFiles,
                charset
            );
        }
    }



    // ==========
    // Defining the JNI bindings library
    // ==========

    
    
    
    
    
    
    

    /** This class contains all native function definitions for JNI */
    public static final class JNI_LIB {

        // ----- Static initializer -----

        static {
            if(!ImageInfo.inImageCode()) {
                // Load the needed libraries
                if(OS.indexOf("win") < 0) {
                    System.loadLibrary("langkit_sigsegv_handler");
                }
                System.loadLibrary("adalang_jni");

                // Initialize the JNI library
                ada_initialize();

                // Register the library finalizer
                Runtime.getRuntime().addShutdownHook(
                    new Thread(JNI_LIB::ada_finalize)
                );
            }
        }

        // ----- Language specific functions -----

        
      


    // --- Project manager functions

    /** Function to load a GPR project */
    @CompilerDirectives.TruffleBoundary
    public static native PointerWrapper ada_gpr_project_load(
        String projectFile,
        ScenarioVariable[] scenarioVariables,
        String target,
        String runtime
    );

    /** Free the given project */
    @CompilerDirectives.TruffleBoundary
    public static native void ada_gpr_project_free(
        ProjectManager project
    );

    /** Get the unit provider for the given project */
    @CompilerDirectives.TruffleBoundary
    public static native UnitProvider ada_gpr_project_create_unit_provider(
        ProjectManager project,
        String subproject
    );

    /** Get the source file array for the given project */
    @CompilerDirectives.TruffleBoundary
    public static native String[] ada_gpr_project_source_files(
        ProjectManager project,
        int mode,
        String[] subprojects
    );

    /**
     * Create a new analysis context from a GPR project.
     *
     * The unit provider, file reader, config pragmas and default charset are
     * inferred from the designated project: see the ``getProvider`` method for
     * the semantics of the ``project`` argument.
     *
     * See the ``AnalysisContext`` class constructor for the semantics of the
     * other arguments.
     *
     * .. TODO: For now, the returned ``AnalysisContext`` instance has a weak
     *    reference to the project manager: make sure the ``ProjectManager``
     *    instance lives at least as long as the ``AnalysisContext`` one.
     */
    public static native AnalysisContext ada_gpr_project_create_context(
        ProjectManager project,
        String subproject,
        EventHandler eventHandler,
        boolean withTrivia,
        int tabStop
    );

    /** Create an auto provider reference */
    @CompilerDirectives.TruffleBoundary
    public static native UnitProvider ada_create_auto_provider(
        String[] sourceFiles,
        String charset
    );

    // --- Config pragmas

    @CompilerDirectives.TruffleBoundary
    public static native void ada_set_config_pragmas_mapping(
        AnalysisContext context,
        AnalysisUnit globalPragmas,
        AnalysisUnit[] localPragmas
    );



        // ----- Lifecycle functions ------

        /** Function to initialize the JNI library */
        public static native void ada_initialize();

        /** Function to finalize the JNI library */
        public static native void ada_finalize();

        // ----- Exception functions ------

        /** Get the last langkit exception */
        @CompilerDirectives.TruffleBoundary
        public static native LangkitException ada_get_last_exception();

        // ----- Text functions -----

        /** Create a new text from its content */
        @CompilerDirectives.TruffleBoundary
        public static native Text ada_create_text(
            byte[] utf32Content
        );

        /** Destroy the given text */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_destroy_text(
            Text text
        );

        // ----- File reader functions -----

        /** Decrease the reference counter of the given file reader */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_dec_ref_file_reader(
            FileReader fileReader
        );

        // ----- Unit provider functions -----

        /** Decrease the ref counter of the unit provider */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_dec_ref_unit_provider(
            UnitProvider unitProvider
        );

        // ----- Event handler functions -----

        /** Create a new event handler */
        @CompilerDirectives.TruffleBoundary
        public static native PointerWrapper ada_create_event_handler(
            EventHandler.UnitRequestedCallback unitRequestedCallback,
            EventHandler.UnitParsedCallback unitParsedCallback
        );

        /** Decrease the ref counter of the event handler */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_dec_ref_event_handler(
            EventHandler eventHandler
        );

        // ----- Token functions -----

        /** Get the next token */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_token_next(
            Token token
        );

        /** Get the previous token */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_token_previous(
            Token token
        );

        /** Get if the tokens are equivalent */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_token_is_equivalent(
            Token left,
            Token right
        );

        /** Get text between the two tokens */
        @CompilerDirectives.TruffleBoundary
        public static native Text ada_token_range_text(
            Token start,
            Token end
        );

        // ----- Analysis context functions -----

        /** Create a new analysis context */
        @CompilerDirectives.TruffleBoundary
        public static native PointerWrapper ada_create_analysis_context(
            String charset,
            FileReader fileReader,
            UnitProvider unitProvider,
            EventHandler eventHandler,
            boolean withTrivia,
            int tabstop
        );

        /** Increase the reference counter of a context */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_context_incref(
            long context
        );

        /** Decrease the reference counter of a context */
        @CompilerDirectives.TruffleBoundary
        public static native void ada_context_decref(
            long context
        );

        // ----- Analysis unit functions -----

        /** Get the analysis unit from a file */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit
        ada_get_analysis_unit_from_file(
            AnalysisContext context,
            String fileName,
            String charset,
            boolean reparse,
            int grammarRule
        );

        /** Get the analysis unit from a buffer */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit
        ada_get_analysis_unit_from_buffer(
            AnalysisContext context,
            String fileName,
            String charset,
            String buffer,
            long bufferSize,
            int grammarRule
        );

        /** Get the analysis unit from the unit provider. */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit
        ada_get_analysis_unit_from_provider(
            AnalysisContext context,
            Text name,
            int kind,
            String charset,
            boolean reparse
        );

        /** Get the root of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_unit_root(
            AnalysisUnit unit
        );

        /** Get the file name of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_unit_filename(
            AnalysisUnit unit
        );

        /** Get the token count of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_unit_token_count(
            AnalysisUnit unit
        );

        /** Get the trivia count of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_unit_trivia_count(
            AnalysisUnit unit
        );

        /** Get the first token of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_unit_first_token(
            AnalysisUnit unit
        );

        /** Get the last token of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_unit_last_token(
            AnalysisUnit unit
        );

        /** Get the context of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisContext ada_unit_context(
            AnalysisUnit unit
        );

        /** Get the number of diagnostic in the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_unit_diagnostic_count(
            AnalysisUnit unit
        );

        /** Get the nth diagnostic of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        public static native Diagnostic ada_unit_diagnostic(
            AnalysisUnit unit,
            int n
        );

        // ----- Node functions -----

        /** Return whether the two given entities are equal */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_node_is_equivalent(
            Entity entity_left,
            Entity entity_right
        );

        /** Get the hash of a node */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_node_hash(
            Entity entity
        );

        /** Get the node kind */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_node_kind(
            Entity entity
        );

        /** Get the node text */
        @CompilerDirectives.TruffleBoundary
        public static native Text ada_node_text(
            Entity entity
        );

        /** Get the node source location range */
        @CompilerDirectives.TruffleBoundary
        public static native SourceLocationRange ada_node_sloc_range(
            Entity entity
        );

        /** Get the node children count */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_node_children_count(
            Entity entity
        );

        /** Get the node nth child */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_node_child(
            Entity entity,
            int n
        );

        /** Get if the node is a token node */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_node_is_token_node(
            Entity entity
        );

        /** Get the unit of the node */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit ada_node_unit(
            Entity entity
        );

        /** Get the entity image of the node */
        @CompilerDirectives.TruffleBoundary
        public static native Text ada_node_image(
            Entity entity
        );

        // ----- Node fields accessors and properties -----

        
    

            

        /** Isomethod of ada_ada_node_p_declarative_scope langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_declarative_scope(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_enclosing_compilation_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_enclosing_compilation_unit(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_get_uninstantiated_node langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_get_uninstantiated_node(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_complete langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native CompletionItemIterator ada_ada_node_p_complete(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_valid_keywords langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_ada_node_p_valid_keywords(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_generic_instantiations langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_ada_node_p_generic_instantiations(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_semantic_parent langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_semantic_parent(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_parent_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_parent_basic_decl(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_filter_is_imported_by langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnitArray ada_ada_node_p_filter_is_imported_by(
            AnalysisUnitArray units,
            boolean transitive,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_xref_entry_point langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_ada_node_p_xref_entry_point(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_resolve_names langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_ada_node_p_resolve_names(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_standard_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit ada_ada_node_p_standard_unit(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_std_entity langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_std_entity(
            Symbol sym,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_bool_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_bool_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_int_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_universal_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_universal_int_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_universal_real_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_universal_real_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_std_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_std_char_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_std_wide_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_std_wide_char_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_std_wide_wide_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_std_wide_wide_char_type(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_top_level_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_top_level_decl(
            AnalysisUnit unit,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_choice_match langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_ada_node_p_choice_match(
            BigInteger value,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_p_gnat_xref langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_p_gnat_xref(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_parent langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_parent(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_parents langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_ada_node_parents(
            boolean with_self,
            Entity node
        );
            

        /** Isomethod of ada_ada_node_children langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_ada_node_children(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_token_start langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_ada_node_token_start(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_token_end langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Token ada_ada_node_token_end(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_child_index langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_ada_node_child_index(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_previous_sibling langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_previous_sibling(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_next_sibling langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ada_node_next_sibling(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnit ada_ada_node_unit(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_is_ghost langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_ada_node_is_ghost(
            Entity node
        );
            

        /** Isomethod of ada_ada_node_full_sloc_image langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_ada_node_full_sloc_image(
            Entity node
        );

        
    

            

        /** Isomethod of ada_abort_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_abort_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_abstract_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_abstract_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_assoc_list_p_zip_with_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_assoc_list_p_zip_with_params(
            boolean imprecise_fallback,
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_aliased_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_aliased_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_all_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_all_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_constrained_array_indices_f_list langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_constrained_array_indices_f_list(
            Entity node
        );

        
    

            

        /** Isomethod of ada_unconstrained_array_indices_f_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_unconstrained_array_indices_f_types(
            Entity node
        );

        
    

            

        /** Isomethod of ada_aspect_assoc_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_aspect_assoc_f_id(
            Entity node
        );
            

        /** Isomethod of ada_aspect_assoc_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_aspect_assoc_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_aspect_assoc_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_aspect_assoc_p_is_ghost_code(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_at_clause_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_at_clause_f_name(
            Entity node
        );
            

        /** Isomethod of ada_at_clause_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_at_clause_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_attribute_def_clause_f_attribute_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_attribute_def_clause_f_attribute_expr(
            Entity node
        );
            

        /** Isomethod of ada_attribute_def_clause_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_attribute_def_clause_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_enum_rep_clause_f_type_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_enum_rep_clause_f_type_name(
            Entity node
        );
            

        /** Isomethod of ada_enum_rep_clause_f_aggregate langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_enum_rep_clause_f_aggregate(
            Entity node
        );
            

        /** Isomethod of ada_enum_rep_clause_p_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_enum_rep_clause_p_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_record_rep_clause_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_rep_clause_f_name(
            Entity node
        );
            

        /** Isomethod of ada_record_rep_clause_f_at_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_rep_clause_f_at_expr(
            Entity node
        );
            

        /** Isomethod of ada_record_rep_clause_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_rep_clause_f_components(
            Entity node
        );

        
    

            

        /** Isomethod of ada_aspect_spec_f_aspect_assocs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_aspect_spec_f_aspect_assocs(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_assoc_p_assoc_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_assoc_p_assoc_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_contract_case_assoc_f_guard langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_contract_case_assoc_f_guard(
            Entity node
        );
            

        /** Isomethod of ada_contract_case_assoc_f_consequence langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_contract_case_assoc_f_consequence(
            Entity node
        );

        
    

            

        /** Isomethod of ada_pragma_argument_assoc_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pragma_argument_assoc_f_name(
            Entity node
        );
            

        /** Isomethod of ada_pragma_argument_assoc_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pragma_argument_assoc_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_formal_param_holder_p_abstract_formal_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_formal_param_holder_p_abstract_formal_params(
            Entity node
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_formal_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_formal_param_holder_p_formal_params(
            Entity node
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_nb_min_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_base_formal_param_holder_p_nb_min_params(
            Entity node
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_nb_max_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native int ada_base_formal_param_holder_p_nb_max_params(
            Entity node
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_param_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_formal_param_holder_p_param_types(
            Entity origin,
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_subp_spec_p_returns langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_spec_p_returns(
            Entity node
        );
            

        /** Isomethod of ada_base_subp_spec_p_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_subp_spec_p_params(
            Entity node
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_subp_spec_p_primitive_subp_types(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_first_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_spec_p_primitive_subp_first_type(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_tagged_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_spec_p_primitive_subp_tagged_type(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_subp_spec_p_return_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_spec_p_return_type(
            Entity origin,
            Entity node
        );

        
    

            

        /** Isomethod of ada_entry_spec_f_entry_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_spec_f_entry_name(
            Entity node
        );
            

        /** Isomethod of ada_entry_spec_f_family_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_spec_f_family_type(
            Entity node
        );
            

        /** Isomethod of ada_entry_spec_f_entry_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_spec_f_entry_params(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_subp_spec_f_subp_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_spec_f_subp_kind(
            Entity node
        );
            

        /** Isomethod of ada_subp_spec_f_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_spec_f_subp_name(
            Entity node
        );
            

        /** Isomethod of ada_subp_spec_f_subp_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_spec_f_subp_params(
            Entity node
        );
            

        /** Isomethod of ada_subp_spec_f_subp_returns langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_spec_f_subp_returns(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synthetic_binary_spec_f_left_param langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_binary_spec_f_left_param(
            Entity node
        );
            

        /** Isomethod of ada_synthetic_binary_spec_f_right_param langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_binary_spec_f_right_param(
            Entity node
        );
            

        /** Isomethod of ada_synthetic_binary_spec_f_return_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_binary_spec_f_return_type_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synthetic_unary_spec_f_right_param langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_unary_spec_f_right_param(
            Entity node
        );
            

        /** Isomethod of ada_synthetic_unary_spec_f_return_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_unary_spec_f_return_type_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_component_list_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_list_f_components(
            Entity node
        );
            

        /** Isomethod of ada_component_list_f_variant_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_list_f_variant_part(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_known_discriminant_part_f_discr_specs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_known_discriminant_part_f_discr_specs(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_entry_completion_formal_params_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_completion_formal_params_f_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_formal_part_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_formal_part_f_decls(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_record_def_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_record_def_f_components(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_basic_assoc_p_get_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_assoc_p_get_params(
            boolean imprecise_fallback,
            Entity node
        );

        
    

            

        /** Isomethod of ada_aggregate_assoc_f_designators langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_aggregate_assoc_f_designators(
            Entity node
        );
            

        /** Isomethod of ada_aggregate_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_aggregate_assoc_f_r_expr(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_composite_constraint_assoc_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_composite_constraint_assoc_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_composite_constraint_assoc_f_constraint_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_composite_constraint_assoc_f_constraint_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_iterated_assoc_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_iterated_assoc_f_spec(
            Entity node
        );
            

        /** Isomethod of ada_iterated_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_iterated_assoc_f_r_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_param_assoc_f_designator langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_assoc_f_designator(
            Entity node
        );
            

        /** Isomethod of ada_param_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_assoc_f_r_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_basic_decl_p_is_formal langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_formal(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_doc_annotations langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native DocAnnotationArray ada_basic_decl_p_doc_annotations(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_doc langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_basic_decl_p_doc(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_previous_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_previous_part_for_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_canonical_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_canonical_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_all_parts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_decl_p_all_parts(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_static_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_static_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_f_aspects langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_f_aspects(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect_assoc langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_get_aspect_assoc(
            Symbol name,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect_spec_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_get_aspect_spec_expr(
            Symbol name,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Aspect ada_basic_decl_p_get_aspect(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_has_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_has_aspect(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_pragma langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_get_pragma(
            Symbol name,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_get_representation_clause(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_get_at_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_get_at_clause(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_imported langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_imported(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_ghost_code(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_compilation_unit_root langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_compilation_unit_root(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_visible langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_visible(
            Entity from_node,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_base_subp_declarations langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_decl_p_base_subp_declarations(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_root_subp_declarations langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_decl_p_root_subp_declarations(
            Entity origin,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_find_all_overrides langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_decl_p_find_all_overrides(
            AnalysisUnitArray units,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_defining_names langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_basic_decl_p_defining_names(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_defining_name(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_type_expression langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_type_expression(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_subp_spec_or_null langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_subp_spec_or_null(
            boolean follow_generic,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_subprogram langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_subprogram(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_relative_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_relative_name(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_relative_name_text langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Symbol ada_basic_decl_p_relative_name_text(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_next_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_next_part_for_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_body_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_body_part_for_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_most_visible_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_decl_p_most_visible_part(
            Entity origin,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_fully_qualified_name_array langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_basic_decl_p_fully_qualified_name_array(
            boolean include_profile,
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_basic_decl_p_fully_qualified_name(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_canonical_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_basic_decl_p_canonical_fully_qualified_name(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_unique_identifying_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_basic_decl_p_unique_identifying_name(
            Entity node
        );
            

        /** Isomethod of ada_basic_decl_p_is_constant_object langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_decl_p_is_constant_object(
            Entity node
        );

        
    

            

        /** Isomethod of ada_abstract_state_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_abstract_state_decl_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_anonymous_expr_decl_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_anonymous_expr_decl_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_anonymous_expr_decl_p_get_formal langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_anonymous_expr_decl_p_get_formal(
            boolean imprecise_fallback,
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_formal_param_decl_p_formal_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_formal_param_decl_p_formal_type(
            Entity origin,
            Entity node
        );

        
    

            

        /** Isomethod of ada_component_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_decl_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_component_decl_f_component_def langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_decl_f_component_def(
            Entity node
        );
            

        /** Isomethod of ada_component_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_decl_f_default_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_discriminant_spec_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_discriminant_spec_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_discriminant_spec_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_discriminant_spec_f_type_expr(
            Entity node
        );
            

        /** Isomethod of ada_discriminant_spec_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_discriminant_spec_f_default_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_formal_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_formal_f_decl(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_param_spec_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_spec_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_param_spec_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_spec_f_has_aliased(
            Entity node
        );
            

        /** Isomethod of ada_param_spec_f_mode langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_spec_f_mode(
            Entity node
        );
            

        /** Isomethod of ada_param_spec_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_spec_f_type_expr(
            Entity node
        );
            

        /** Isomethod of ada_param_spec_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_param_spec_f_default_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synthetic_formal_param_decl_f_param_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_formal_param_decl_f_param_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_package_decl_f_package_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_package_decl_f_package_name(
            Entity node
        );
            

        /** Isomethod of ada_base_package_decl_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_package_decl_f_public_part(
            Entity node
        );
            

        /** Isomethod of ada_base_package_decl_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_package_decl_f_private_part(
            Entity node
        );
            

        /** Isomethod of ada_base_package_decl_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_package_decl_f_end_name(
            Entity node
        );
            

        /** Isomethod of ada_base_package_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_package_decl_p_body_part(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_base_type_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_base_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_base_subtype(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_private_completion langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_private_completion(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_inherited_primitive langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_inherited_primitive(
            Entity p,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_get_record_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_get_record_representation_clause(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_get_enum_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_get_enum_representation_clause(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_record_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_record_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_array_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_array_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_find_derived_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_type_decl_p_find_derived_types(
            Entity root,
            Entity origin,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_real_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_real_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_float_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_float_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_fixed_point langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_fixed_point(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_scalar_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_scalar_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_enum_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_enum_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_access_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_access_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_specific_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_specific_type(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_char_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_classwide_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_classwide_type(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_discrete_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native DiscreteRange ada_base_type_decl_p_discrete_range(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_discrete_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_discrete_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_int_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_accessed_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_accessed_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_tagged_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_tagged_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_base_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_base_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_base_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_type_decl_p_base_types(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_find_all_derived_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_type_decl_p_find_all_derived_types(
            AnalysisUnitArray units,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_comp_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_comp_type(
            boolean is_subscript,
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_index_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_index_type(
            int dim,
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_derived_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_derived_type(
            Entity other_type,
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_interface_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_interface_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_matching_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_matching_type(
            Entity expected_type,
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_canonical_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_canonical_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_previous_part(
            boolean go_to_incomplete,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_next_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_next_part(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_full_view langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_full_view(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_definite_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_definite_subtype(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_is_private langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_type_decl_p_is_private(
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_discriminants_list langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_base_type_decl_p_discriminants_list(
            Entity stop_recurse_at,
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_root_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_type_decl_p_root_type(
            Entity origin,
            Entity node
        );
            

        /** Isomethod of ada_base_type_decl_p_shapes langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ShapeArray ada_base_type_decl_p_shapes(
            boolean include_discriminants,
            Entity origin,
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_subtype_decl_p_get_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subtype_decl_p_get_type(
            Entity origin,
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_subtype_decl_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subtype_decl_f_subtype(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_incomplete_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_incomplete_type_decl_f_discriminants(
            Entity node
        );

        
    

            

        /** Isomethod of ada_incomplete_formal_type_decl_f_is_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_incomplete_formal_type_decl_f_is_tagged(
            Entity node
        );
            

        /** Isomethod of ada_incomplete_formal_type_decl_f_default_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_incomplete_formal_type_decl_f_default_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_incomplete_tagged_type_decl_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_incomplete_tagged_type_decl_f_has_abstract(
            Entity node
        );

        
    

            

        /** Isomethod of ada_protected_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_type_decl_f_discriminants(
            Entity node
        );
            

        /** Isomethod of ada_protected_type_decl_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_type_decl_f_interfaces(
            Entity node
        );
            

        /** Isomethod of ada_protected_type_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_type_decl_f_definition(
            Entity node
        );

        
    

            

        /** Isomethod of ada_task_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_type_decl_f_discriminants(
            Entity node
        );
            

        /** Isomethod of ada_task_type_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_type_decl_f_definition(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_decl_f_discriminants(
            Entity node
        );
            

        /** Isomethod of ada_type_decl_f_type_def langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_decl_f_type_def(
            Entity node
        );
            

        /** Isomethod of ada_type_decl_p_get_primitives langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_type_decl_p_get_primitives(
            boolean only_inherited,
            boolean include_predefined_operators,
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_formal_type_decl_f_default_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_formal_type_decl_f_default_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_basic_subp_decl_p_has_spark_mode_on langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_basic_subp_decl_p_has_spark_mode_on(
            Entity node
        );
            

        /** Isomethod of ada_basic_subp_decl_p_subp_decl_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_basic_subp_decl_p_subp_decl_spec(
            Entity node
        );

        
    

            

        /** Isomethod of ada_classic_subp_decl_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_classic_subp_decl_f_overriding(
            Entity node
        );
            

        /** Isomethod of ada_classic_subp_decl_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_classic_subp_decl_f_subp_spec(
            Entity node
        );
            

        /** Isomethod of ada_classic_subp_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_classic_subp_decl_p_body_part(
            boolean imprecise_fallback,
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_formal_subp_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_formal_subp_decl_f_default_expr(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_entry_decl_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_decl_f_overriding(
            Entity node
        );
            

        /** Isomethod of ada_entry_decl_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_decl_f_spec(
            Entity node
        );
            

        /** Isomethod of ada_entry_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_decl_p_body_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_entry_decl_p_accept_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_entry_decl_p_accept_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_enum_literal_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_enum_literal_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_enum_literal_decl_p_enum_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_enum_literal_decl_p_enum_type(
            Entity node
        );
            

        /** Isomethod of ada_enum_literal_decl_p_enum_rep langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native BigInteger ada_enum_literal_decl_p_enum_rep(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synthetic_char_enum_lit_p_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_char_enum_lit_p_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_subp_internal_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_internal_f_subp_spec(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synthetic_subp_decl_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_subp_decl_f_spec(
            Entity node
        );

        
    

            

        /** Isomethod of ada_body_node_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_body_node_p_previous_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_body_node_p_decl_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_body_node_p_decl_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_body_node_p_subunit_root langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_body_node_p_subunit_root(
            Entity node
        );

        
    

            

        /** Isomethod of ada_accept_stmt_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_body_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_subp_body_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_body_f_overriding(
            Entity node
        );
            

        /** Isomethod of ada_base_subp_body_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_subp_body_f_subp_spec(
            Entity node
        );
            

        /** Isomethod of ada_base_subp_body_p_is_subject_to_proof langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_subp_body_p_is_subject_to_proof(
            Entity node
        );
            

        /** Isomethod of ada_base_subp_body_p_has_spark_mode_on langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_subp_body_p_has_spark_mode_on(
            Entity node
        );

        
    

            

        /** Isomethod of ada_expr_function_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_expr_function_f_expr(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_subp_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_body_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_subp_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_body_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_subp_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_body_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_subp_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_renaming_decl_f_renames(
            Entity node
        );

        
    

            

        /** Isomethod of ada_body_stub_p_syntactic_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_body_stub_p_syntactic_fully_qualified_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_package_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_body_stub_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_protected_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_body_stub_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_subp_body_stub_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_body_stub_f_overriding(
            Entity node
        );
            

        /** Isomethod of ada_subp_body_stub_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subp_body_stub_f_subp_spec(
            Entity node
        );

        
    

            

        /** Isomethod of ada_task_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_body_stub_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_entry_body_f_entry_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_entry_name(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_index_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_index_spec(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_params(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_barrier langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_barrier(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_entry_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_body_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_package_body_f_package_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_body_f_package_name(
            Entity node
        );
            

        /** Isomethod of ada_package_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_body_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_package_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_body_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_package_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_body_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_protected_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_body_f_name(
            Entity node
        );
            

        /** Isomethod of ada_protected_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_body_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_protected_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_body_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_task_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_body_f_name(
            Entity node
        );
            

        /** Isomethod of ada_task_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_body_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_task_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_body_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_task_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_body_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_entry_index_spec_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_index_spec_f_id(
            Entity node
        );
            

        /** Isomethod of ada_entry_index_spec_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_entry_index_spec_f_subtype(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_exception_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exception_decl_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_exception_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exception_decl_f_renames(
            Entity node
        );

        
    

            

        /** Isomethod of ada_exception_handler_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exception_handler_f_exception_name(
            Entity node
        );
            

        /** Isomethod of ada_exception_handler_f_handled_exceptions langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exception_handler_f_handled_exceptions(
            Entity node
        );
            

        /** Isomethod of ada_exception_handler_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exception_handler_f_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_for_loop_var_decl_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_var_decl_f_id(
            Entity node
        );
            

        /** Isomethod of ada_for_loop_var_decl_f_id_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_var_decl_f_id_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_decl_f_formal_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_decl_f_formal_part(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_package_decl_f_package_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_decl_f_package_decl(
            Entity node
        );
            

        /** Isomethod of ada_generic_package_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_decl_p_body_part(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_subp_decl_f_subp_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_decl_f_subp_decl(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_decl_p_body_part(
            boolean imprecise_fallback,
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_instantiation_p_designated_generic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_instantiation_p_designated_generic_decl(
            Entity node
        );
            

        /** Isomethod of ada_generic_instantiation_p_inst_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_generic_instantiation_p_inst_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_package_instantiation_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_instantiation_f_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_package_instantiation_f_generic_pkg_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_instantiation_f_generic_pkg_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_package_instantiation_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_instantiation_f_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_subp_instantiation_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_f_overriding(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_f_kind(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_f_subp_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_generic_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_f_generic_subp_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_f_params(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_instantiation_p_designated_subp langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_instantiation_p_designated_subp(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_generic_package_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_renaming_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_package_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_package_renaming_decl_f_renames(
            Entity node
        );

        
    

            

        /** Isomethod of ada_generic_subp_renaming_decl_f_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_renaming_decl_f_kind(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_renaming_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_generic_subp_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_generic_subp_renaming_decl_f_renames(
            Entity node
        );

        
    

            

        /** Isomethod of ada_label_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_label_decl_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_named_stmt_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_named_stmt_decl_f_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_number_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_number_decl_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_number_decl_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_number_decl_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_object_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_ids(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_has_aliased(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_has_constant(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_mode langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_mode(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_type_expr(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_default_expr(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_f_renaming_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_f_renaming_clause(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_p_private_part_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_p_private_part_decl(
            Entity node
        );
            

        /** Isomethod of ada_object_decl_p_public_part_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_object_decl_p_public_part_decl(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_package_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_renaming_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_package_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_renaming_decl_f_renames(
            Entity node
        );
            

        /** Isomethod of ada_package_renaming_decl_p_renamed_package langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_renaming_decl_p_renamed_package(
            Entity node
        );
            

        /** Isomethod of ada_package_renaming_decl_p_final_renamed_package langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_package_renaming_decl_p_final_renamed_package(
            Entity node
        );

        
    

            

        /** Isomethod of ada_single_protected_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_single_protected_decl_f_name(
            Entity node
        );
            

        /** Isomethod of ada_single_protected_decl_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_single_protected_decl_f_interfaces(
            Entity node
        );
            

        /** Isomethod of ada_single_protected_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_single_protected_decl_f_definition(
            Entity node
        );

        
    

            

        /** Isomethod of ada_single_task_decl_f_task_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_single_task_decl_f_task_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_case_stmt_alternative_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_stmt_alternative_f_choices(
            Entity node
        );
            

        /** Isomethod of ada_case_stmt_alternative_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_stmt_alternative_f_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_compilation_unit_f_prelude langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_compilation_unit_f_prelude(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_f_body langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_compilation_unit_f_body(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_f_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_compilation_unit_f_pragmas(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_syntactic_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_compilation_unit_p_syntactic_fully_qualified_name(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_unit_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AnalysisUnitKind ada_compilation_unit_p_unit_kind(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_withed_units langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_compilation_unit_p_withed_units(
            boolean include_privates,
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_imported_units langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_compilation_unit_p_imported_units(
            boolean include_privates,
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_unit_dependencies langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_compilation_unit_p_unit_dependencies(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_compilation_unit_p_decl(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_is_preelaborable langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_compilation_unit_p_is_preelaborable(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_other_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_compilation_unit_p_other_part(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_has_restriction langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_compilation_unit_p_has_restriction(
            Symbol name,
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_all_config_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_compilation_unit_p_all_config_pragmas(
            Entity node
        );
            

        /** Isomethod of ada_compilation_unit_p_config_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_compilation_unit_p_config_pragmas(
            Symbol name,
            Entity node
        );

        
    

            

        /** Isomethod of ada_component_clause_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_clause_f_id(
            Entity node
        );
            

        /** Isomethod of ada_component_clause_f_position langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_clause_f_position(
            Entity node
        );
            

        /** Isomethod of ada_component_clause_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_clause_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_component_def_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_def_f_has_aliased(
            Entity node
        );
            

        /** Isomethod of ada_component_def_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_def_f_has_constant(
            Entity node
        );
            

        /** Isomethod of ada_component_def_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_component_def_f_type_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_constant_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_constant_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_composite_constraint_f_constraints langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_composite_constraint_f_constraints(
            Entity node
        );
            

        /** Isomethod of ada_composite_constraint_p_is_index_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_composite_constraint_p_is_index_constraint(
            Entity node
        );
            

        /** Isomethod of ada_composite_constraint_p_is_discriminant_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_composite_constraint_p_is_discriminant_constraint(
            Entity node
        );

        
    

            

        /** Isomethod of ada_delta_constraint_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_delta_constraint_f_digits(
            Entity node
        );
            

        /** Isomethod of ada_delta_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_delta_constraint_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_digits_constraint_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_digits_constraint_f_digits(
            Entity node
        );
            

        /** Isomethod of ada_digits_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_digits_constraint_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_range_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_range_constraint_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_declarative_part_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_declarative_part_f_decls(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_elsif_expr_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_elsif_expr_part_f_cond_expr(
            Entity node
        );
            

        /** Isomethod of ada_elsif_expr_part_f_then_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_elsif_expr_part_f_then_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_elsif_stmt_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_elsif_stmt_part_f_cond_expr(
            Entity node
        );
            

        /** Isomethod of ada_elsif_stmt_part_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_elsif_stmt_part_f_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_expr_p_expression_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_expr_p_expression_type(
            Entity node
        );
            

        /** Isomethod of ada_expr_p_expected_expression_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_expr_p_expected_expression_type(
            Entity node
        );
            

        /** Isomethod of ada_expr_p_is_dynamically_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_expr_p_is_dynamically_tagged(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_expr_p_is_dispatching_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_expr_p_is_dispatching_call(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_expr_p_is_static_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_expr_p_is_static_expr(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_expr_p_first_corresponding_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_expr_p_first_corresponding_decl(
            Entity node
        );
            

        /** Isomethod of ada_expr_p_eval_as_int langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native BigInteger ada_expr_p_eval_as_int(
            Entity node
        );
            

        /** Isomethod of ada_expr_p_eval_as_int_in_env langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native BigInteger ada_expr_p_eval_as_int_in_env(
            SubstitutionArray env,
            Entity node
        );
            

        /** Isomethod of ada_expr_p_eval_as_string langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_expr_p_eval_as_string(
            Entity node
        );
            

        /** Isomethod of ada_expr_p_eval_as_string_in_env langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_expr_p_eval_as_string_in_env(
            SubstitutionArray env,
            Entity node
        );
            

        /** Isomethod of ada_expr_p_matching_nodes langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_expr_p_matching_nodes(
            Entity node
        );

        
    

            

        /** Isomethod of ada_abstract_state_decl_expr_f_state_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_abstract_state_decl_expr_f_state_decl(
            Entity node
        );

        
    

            

        /** Isomethod of ada_allocator_f_subpool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_allocator_f_subpool(
            Entity node
        );
            

        /** Isomethod of ada_allocator_f_type_or_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_allocator_f_type_or_expr(
            Entity node
        );
            

        /** Isomethod of ada_allocator_p_get_allocated_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_allocator_p_get_allocated_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_aggregate_f_ancestor_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_aggregate_f_ancestor_expr(
            Entity node
        );
            

        /** Isomethod of ada_base_aggregate_f_assocs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_aggregate_f_assocs(
            Entity node
        );
            

        /** Isomethod of ada_base_aggregate_p_aggregate_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_base_aggregate_p_aggregate_params(
            Entity node
        );
            

        /** Isomethod of ada_base_aggregate_p_is_subaggregate langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_base_aggregate_p_is_subaggregate(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_bin_op_f_left langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_bin_op_f_left(
            Entity node
        );
            

        /** Isomethod of ada_bin_op_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_bin_op_f_op(
            Entity node
        );
            

        /** Isomethod of ada_bin_op_f_right langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_bin_op_f_right(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_case_expr_alternative_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_expr_alternative_f_choices(
            Entity node
        );
            

        /** Isomethod of ada_case_expr_alternative_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_expr_alternative_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_concat_op_f_first_operand langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_concat_op_f_first_operand(
            Entity node
        );
            

        /** Isomethod of ada_concat_op_f_other_operands langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_concat_op_f_other_operands(
            Entity node
        );
            

        /** Isomethod of ada_concat_op_p_operands langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_concat_op_p_operands(
            Entity node
        );

        
    

            

        /** Isomethod of ada_concat_operand_f_operator langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_concat_operand_f_operator(
            Entity node
        );
            

        /** Isomethod of ada_concat_operand_f_operand langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_concat_operand_f_operand(
            Entity node
        );

        
    

            

        /** Isomethod of ada_cond_expr_p_dependent_exprs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_cond_expr_p_dependent_exprs(
            Entity node
        );

        
    

            

        /** Isomethod of ada_case_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_expr_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_case_expr_f_cases langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_expr_f_cases(
            Entity node
        );

        
    

            

        /** Isomethod of ada_if_expr_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_expr_f_cond_expr(
            Entity node
        );
            

        /** Isomethod of ada_if_expr_f_then_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_expr_f_then_expr(
            Entity node
        );
            

        /** Isomethod of ada_if_expr_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_expr_f_alternatives(
            Entity node
        );
            

        /** Isomethod of ada_if_expr_f_else_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_expr_f_else_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_contract_cases_f_contract_cases langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_contract_cases_f_contract_cases(
            Entity node
        );

        
    

            

        /** Isomethod of ada_decl_expr_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decl_expr_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_decl_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decl_expr_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_membership_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_membership_expr_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_membership_expr_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_membership_expr_f_op(
            Entity node
        );
            

        /** Isomethod of ada_membership_expr_f_membership_exprs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_membership_expr_f_membership_exprs(
            Entity node
        );

        
    

            

        /** Isomethod of ada_name_p_enclosing_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_enclosing_defining_name(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_defining langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_defining(
            Entity node
        );
            

        /** Isomethod of ada_name_p_name_is langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_name_is(
            Symbol sym,
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_direct_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_direct_call(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_access_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_access_call(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_call(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_dot_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_dot_call(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_failsafe_referenced_def_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native RefdDef ada_name_p_failsafe_referenced_def_name(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_referenced_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_referenced_defining_name(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_all_env_elements langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_name_p_all_env_elements(
            boolean seq,
            Entity seq_from,
            Entity node
        );
            

        /** Isomethod of ada_name_p_called_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_called_subp_spec(
            Entity node
        );
            

        /** Isomethod of ada_name_p_referenced_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_referenced_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_failsafe_referenced_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native RefdDecl ada_name_p_failsafe_referenced_decl(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_name_designated_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_name_designated_type(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_static_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_static_subtype(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_name_matches langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_name_matches(
            Entity n,
            Entity node
        );
            

        /** Isomethod of ada_name_p_relative_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_name_p_relative_name(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_operator_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_operator_name(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_write_reference langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_write_reference(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_static_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_static_call(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_name_p_as_symbol_array langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_name_p_as_symbol_array(
            Entity node
        );
            

        /** Isomethod of ada_name_p_canonical_text langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Symbol ada_name_p_canonical_text(
            Entity node
        );
            

        /** Isomethod of ada_name_p_is_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_name_p_is_constant(
            Entity node
        );
            

        /** Isomethod of ada_name_p_call_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_name_p_call_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_attribute_ref_f_prefix(
            Entity node
        );
            

        /** Isomethod of ada_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_attribute_ref_f_attribute(
            Entity node
        );
            

        /** Isomethod of ada_attribute_ref_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_attribute_ref_f_args(
            Entity node
        );

        
    

            

        /** Isomethod of ada_call_expr_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_call_expr_f_name(
            Entity node
        );
            

        /** Isomethod of ada_call_expr_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_call_expr_f_suffix(
            Entity node
        );
            

        /** Isomethod of ada_call_expr_p_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native CallExprKind ada_call_expr_p_kind(
            Entity node
        );
            

        /** Isomethod of ada_call_expr_p_is_array_slice langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_call_expr_p_is_array_slice(
            Entity node
        );

        
    

            

        /** Isomethod of ada_defining_name_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_f_name(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_canonical_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_defining_name_p_canonical_fully_qualified_name(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_unique_identifying_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_defining_name_p_unique_identifying_name(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_fully_qualified_name_array langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native UnboundedTextTypeArray ada_defining_name_p_fully_qualified_name_array(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_defining_name_p_fully_qualified_name(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_basic_decl(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_find_refs langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native RefResultArray ada_defining_name_p_find_refs(
            Entity root,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_find_all_references langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native RefResultArray ada_defining_name_p_find_all_references(
            AnalysisUnitArray units,
            boolean follow_renamings,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_find_all_calls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native RefResultArray ada_defining_name_p_find_all_calls(
            AnalysisUnitArray units,
            boolean follow_renamings,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_next_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_next_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_previous_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_canonical_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_canonical_part(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_most_visible_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_most_visible_part(
            Entity origin,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_all_parts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_defining_name_p_all_parts(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_get_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Aspect ada_defining_name_p_get_aspect(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_has_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_defining_name_p_has_aspect(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_get_pragma langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_get_pragma(
            Symbol name,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_get_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_get_representation_clause(
            Symbol name,
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_get_at_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_defining_name_p_get_at_clause(
            boolean imprecise_fallback,
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_is_imported langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_defining_name_p_is_imported(
            Entity node
        );
            

        /** Isomethod of ada_defining_name_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_defining_name_p_is_ghost_code(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_discrete_subtype_name_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_discrete_subtype_name_f_subtype(
            Entity node
        );

        
    

            

        /** Isomethod of ada_dotted_name_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_dotted_name_f_prefix(
            Entity node
        );
            

        /** Isomethod of ada_dotted_name_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_dotted_name_f_suffix(
            Entity node
        );

        
    

            

        /** Isomethod of ada_end_name_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_end_name_f_name(
            Entity node
        );
            

        /** Isomethod of ada_end_name_p_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_end_name_p_basic_decl(
            Entity node
        );

        
    

            

        /** Isomethod of ada_explicit_deref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_explicit_deref_f_prefix(
            Entity node
        );

        
    

            

        /** Isomethod of ada_qual_expr_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_qual_expr_f_prefix(
            Entity node
        );
            

        /** Isomethod of ada_qual_expr_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_qual_expr_f_suffix(
            Entity node
        );

        
    

            

        /** Isomethod of ada_reduce_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_reduce_attribute_ref_f_prefix(
            Entity node
        );
            

        /** Isomethod of ada_reduce_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_reduce_attribute_ref_f_attribute(
            Entity node
        );
            

        /** Isomethod of ada_reduce_attribute_ref_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_reduce_attribute_ref_f_args(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_char_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Char ada_char_literal_p_denoted_value(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_string_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native String ada_string_literal_p_denoted_value(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_int_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native BigInteger ada_int_literal_p_denoted_value(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_update_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_update_attribute_ref_f_prefix(
            Entity node
        );
            

        /** Isomethod of ada_update_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_update_attribute_ref_f_attribute(
            Entity node
        );
            

        /** Isomethod of ada_update_attribute_ref_f_values langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_update_attribute_ref_f_values(
            Entity node
        );

        
    

            

        /** Isomethod of ada_paren_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_paren_expr_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_quantified_expr_f_quantifier langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_quantified_expr_f_quantifier(
            Entity node
        );
            

        /** Isomethod of ada_quantified_expr_f_loop_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_quantified_expr_f_loop_spec(
            Entity node
        );
            

        /** Isomethod of ada_quantified_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_quantified_expr_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_raise_expr_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_raise_expr_f_exception_name(
            Entity node
        );
            

        /** Isomethod of ada_raise_expr_f_error_message langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_raise_expr_f_error_message(
            Entity node
        );

        
    

            

        /** Isomethod of ada_un_op_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_un_op_f_op(
            Entity node
        );
            

        /** Isomethod of ada_un_op_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_un_op_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_handled_stmts_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_handled_stmts_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_handled_stmts_f_exceptions langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_handled_stmts_f_exceptions(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_library_item_f_has_private langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_library_item_f_has_private(
            Entity node
        );
            

        /** Isomethod of ada_library_item_f_item langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_library_item_f_item(
            Entity node
        );

        
    

            

        /** Isomethod of ada_limited_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_limited_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_for_loop_spec_f_var_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_spec_f_var_decl(
            Entity node
        );
            

        /** Isomethod of ada_for_loop_spec_f_loop_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_spec_f_loop_type(
            Entity node
        );
            

        /** Isomethod of ada_for_loop_spec_f_has_reverse langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_spec_f_has_reverse(
            Entity node
        );
            

        /** Isomethod of ada_for_loop_spec_f_iter_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_spec_f_iter_expr(
            Entity node
        );
            

        /** Isomethod of ada_for_loop_spec_f_iter_filter langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_for_loop_spec_f_iter_filter(
            Entity node
        );

        
    

            

        /** Isomethod of ada_while_loop_spec_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_while_loop_spec_f_expr(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_multi_abstract_state_decl_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_multi_abstract_state_decl_f_decls(
            Entity node
        );

        
    

            

        /** Isomethod of ada_not_null_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_not_null_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_params_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_params_f_params(
            Entity node
        );

        
    

            

        /** Isomethod of ada_paren_abstract_state_decl_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_paren_abstract_state_decl_f_decl(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_pp_elsif_directive_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pp_elsif_directive_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_pp_elsif_directive_f_then_kw langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pp_elsif_directive_f_then_kw(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_pp_if_directive_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pp_if_directive_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_pp_if_directive_f_then_kw langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pp_if_directive_f_then_kw(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_pragma_node_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pragma_node_f_id(
            Entity node
        );
            

        /** Isomethod of ada_pragma_node_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_pragma_node_f_args(
            Entity node
        );
            

        /** Isomethod of ada_pragma_node_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_pragma_node_p_is_ghost_code(
            Entity node
        );
            

        /** Isomethod of ada_pragma_node_p_associated_entities langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native AdaNodeArray ada_pragma_node_p_associated_entities(
            Entity node
        );

        
    

            

        /** Isomethod of ada_private_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_private_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_protected_def_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_def_f_public_part(
            Entity node
        );
            

        /** Isomethod of ada_protected_def_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_def_f_private_part(
            Entity node
        );
            

        /** Isomethod of ada_protected_def_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_protected_def_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_protected_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_protected_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_range_spec_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_range_spec_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_renaming_clause_f_renamed_object langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_renaming_clause_f_renamed_object(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_reverse_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_reverse_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_select_when_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_select_when_part_f_cond_expr(
            Entity node
        );
            

        /** Isomethod of ada_select_when_part_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_select_when_part_f_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_stmt_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_stmt_p_is_ghost_code(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_accept_stmt_f_body_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_f_body_decl(
            Entity node
        );
            

        /** Isomethod of ada_accept_stmt_f_entry_index_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_f_entry_index_expr(
            Entity node
        );
            

        /** Isomethod of ada_accept_stmt_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_f_params(
            Entity node
        );
            

        /** Isomethod of ada_accept_stmt_p_corresponding_entry langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_p_corresponding_entry(
            Entity origin,
            Entity node
        );

        
    

            

        /** Isomethod of ada_accept_stmt_with_stmts_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_with_stmts_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_accept_stmt_with_stmts_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_accept_stmt_with_stmts_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_base_loop_stmt_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_loop_stmt_f_spec(
            Entity node
        );
            

        /** Isomethod of ada_base_loop_stmt_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_loop_stmt_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_base_loop_stmt_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_base_loop_stmt_f_end_name(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_begin_block_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_begin_block_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_begin_block_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_begin_block_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_decl_block_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decl_block_f_decls(
            Entity node
        );
            

        /** Isomethod of ada_decl_block_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decl_block_f_stmts(
            Entity node
        );
            

        /** Isomethod of ada_decl_block_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decl_block_f_end_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_case_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_stmt_f_expr(
            Entity node
        );
            

        /** Isomethod of ada_case_stmt_f_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_stmt_f_pragmas(
            Entity node
        );
            

        /** Isomethod of ada_case_stmt_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_case_stmt_f_alternatives(
            Entity node
        );

        
    

            

        /** Isomethod of ada_extended_return_stmt_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_extended_return_stmt_f_decl(
            Entity node
        );
            

        /** Isomethod of ada_extended_return_stmt_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_extended_return_stmt_f_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_if_stmt_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_stmt_f_cond_expr(
            Entity node
        );
            

        /** Isomethod of ada_if_stmt_f_then_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_stmt_f_then_stmts(
            Entity node
        );
            

        /** Isomethod of ada_if_stmt_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_stmt_f_alternatives(
            Entity node
        );
            

        /** Isomethod of ada_if_stmt_f_else_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_if_stmt_f_else_stmts(
            Entity node
        );

        
    

            

        /** Isomethod of ada_named_stmt_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_named_stmt_f_decl(
            Entity node
        );
            

        /** Isomethod of ada_named_stmt_f_stmt langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_named_stmt_f_stmt(
            Entity node
        );

        
    

            

        /** Isomethod of ada_select_stmt_f_guards langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_select_stmt_f_guards(
            Entity node
        );
            

        /** Isomethod of ada_select_stmt_f_else_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_select_stmt_f_else_stmts(
            Entity node
        );
            

        /** Isomethod of ada_select_stmt_f_abort_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_select_stmt_f_abort_stmts(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_abort_stmt_f_names langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_abort_stmt_f_names(
            Entity node
        );

        
    

            

        /** Isomethod of ada_assign_stmt_f_dest langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_assign_stmt_f_dest(
            Entity node
        );
            

        /** Isomethod of ada_assign_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_assign_stmt_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_call_stmt_f_call langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_call_stmt_f_call(
            Entity node
        );

        
    

            

        /** Isomethod of ada_delay_stmt_f_has_until langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_delay_stmt_f_has_until(
            Entity node
        );
            

        /** Isomethod of ada_delay_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_delay_stmt_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_exit_stmt_f_loop_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exit_stmt_f_loop_name(
            Entity node
        );
            

        /** Isomethod of ada_exit_stmt_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_exit_stmt_f_cond_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_goto_stmt_f_label_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_goto_stmt_f_label_name(
            Entity node
        );

        
    

            

        /** Isomethod of ada_label_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_label_f_decl(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_raise_stmt_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_raise_stmt_f_exception_name(
            Entity node
        );
            

        /** Isomethod of ada_raise_stmt_f_error_message langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_raise_stmt_f_error_message(
            Entity node
        );

        
    

            

        /** Isomethod of ada_requeue_stmt_f_call_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_requeue_stmt_f_call_name(
            Entity node
        );
            

        /** Isomethod of ada_requeue_stmt_f_has_abort langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_requeue_stmt_f_has_abort(
            Entity node
        );

        
    

            

        /** Isomethod of ada_return_stmt_f_return_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_return_stmt_f_return_expr(
            Entity node
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_subunit_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subunit_f_name(
            Entity node
        );
            

        /** Isomethod of ada_subunit_f_body langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subunit_f_body(
            Entity node
        );
            

        /** Isomethod of ada_subunit_p_body_root langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subunit_p_body_root(
            Entity node
        );

        
    

            

        /** Isomethod of ada_synchronized_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_synchronized_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_tagged_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_tagged_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_task_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_def_f_interfaces(
            Entity node
        );
            

        /** Isomethod of ada_task_def_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_def_f_public_part(
            Entity node
        );
            

        /** Isomethod of ada_task_def_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_def_f_private_part(
            Entity node
        );
            

        /** Isomethod of ada_task_def_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_task_def_f_end_name(
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_access_def_f_has_not_null langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_access_def_f_has_not_null(
            Entity node
        );

        
    

            

        /** Isomethod of ada_access_to_subp_def_f_has_protected langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_access_to_subp_def_f_has_protected(
            Entity node
        );
            

        /** Isomethod of ada_access_to_subp_def_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_access_to_subp_def_f_subp_spec(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_anonymous_type_access_def_f_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_anonymous_type_access_def_f_type_decl(
            Entity node
        );

        
    

            

        /** Isomethod of ada_type_access_def_f_has_all langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_access_def_f_has_all(
            Entity node
        );
            

        /** Isomethod of ada_type_access_def_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_access_def_f_has_constant(
            Entity node
        );
            

        /** Isomethod of ada_type_access_def_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_access_def_f_subtype_indication(
            Entity node
        );

        
    

            

        /** Isomethod of ada_array_type_def_f_indices langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_array_type_def_f_indices(
            Entity node
        );
            

        /** Isomethod of ada_array_type_def_f_component_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_array_type_def_f_component_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_derived_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_has_abstract(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_has_limited(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_has_synchronized langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_has_synchronized(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_subtype_indication(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_interfaces(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_record_extension langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_record_extension(
            Entity node
        );
            

        /** Isomethod of ada_derived_type_def_f_has_with_private langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_derived_type_def_f_has_with_private(
            Entity node
        );

        
    

            

        /** Isomethod of ada_enum_type_def_f_enum_literals langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_enum_type_def_f_enum_literals(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_interface_type_def_f_interface_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_interface_type_def_f_interface_kind(
            Entity node
        );
            

        /** Isomethod of ada_interface_type_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_interface_type_def_f_interfaces(
            Entity node
        );

        
    

            

        /** Isomethod of ada_mod_int_type_def_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_mod_int_type_def_f_expr(
            Entity node
        );

        
    

            

        /** Isomethod of ada_private_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_private_type_def_f_has_abstract(
            Entity node
        );
            

        /** Isomethod of ada_private_type_def_f_has_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_private_type_def_f_has_tagged(
            Entity node
        );
            

        /** Isomethod of ada_private_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_private_type_def_f_has_limited(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_decimal_fixed_point_def_f_delta langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decimal_fixed_point_def_f_delta(
            Entity node
        );
            

        /** Isomethod of ada_decimal_fixed_point_def_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decimal_fixed_point_def_f_digits(
            Entity node
        );
            

        /** Isomethod of ada_decimal_fixed_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_decimal_fixed_point_def_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_floating_point_def_f_num_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_floating_point_def_f_num_digits(
            Entity node
        );
            

        /** Isomethod of ada_floating_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_floating_point_def_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_ordinary_fixed_point_def_f_delta langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ordinary_fixed_point_def_f_delta(
            Entity node
        );
            

        /** Isomethod of ada_ordinary_fixed_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_ordinary_fixed_point_def_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_record_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_type_def_f_has_abstract(
            Entity node
        );
            

        /** Isomethod of ada_record_type_def_f_has_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_type_def_f_has_tagged(
            Entity node
        );
            

        /** Isomethod of ada_record_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_type_def_f_has_limited(
            Entity node
        );
            

        /** Isomethod of ada_record_type_def_f_record_def langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_record_type_def_f_record_def(
            Entity node
        );

        
    

            

        /** Isomethod of ada_signed_int_type_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_signed_int_type_def_f_range(
            Entity node
        );

        
    

            

        /** Isomethod of ada_type_expr_p_type_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_expr_p_type_name(
            Entity node
        );
            

        /** Isomethod of ada_type_expr_p_designated_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_expr_p_designated_type_decl(
            Entity node
        );
            

        /** Isomethod of ada_type_expr_p_designated_type_decl_from langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_type_expr_p_designated_type_decl_from(
            Entity origin_node,
            Entity node
        );

        
    

            

        /** Isomethod of ada_anonymous_type_f_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_anonymous_type_f_type_decl(
            Entity node
        );

        
    


        
    

            

        /** Isomethod of ada_subtype_indication_f_has_not_null langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subtype_indication_f_has_not_null(
            Entity node
        );
            

        /** Isomethod of ada_subtype_indication_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subtype_indication_f_name(
            Entity node
        );
            

        /** Isomethod of ada_subtype_indication_f_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_subtype_indication_f_constraint(
            Entity node
        );
            

        /** Isomethod of ada_subtype_indication_p_subtype_constraints langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native ParamActualArray ada_subtype_indication_p_subtype_constraints(
            Entity node
        );
            

        /** Isomethod of ada_subtype_indication_p_is_static_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_subtype_indication_p_is_static_subtype(
            boolean imprecise_fallback,
            Entity node
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_synthetic_type_expr_f_target_type langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_synthetic_type_expr_f_target_type(
            Entity node
        );

        
    

            

        /** Isomethod of ada_unconstrained_array_index_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_unconstrained_array_index_f_subtype_indication(
            Entity node
        );

        
    

            

        /** Isomethod of ada_until_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_until_node_p_as_bool(
            Entity node
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_use_package_clause_f_packages langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_use_package_clause_f_packages(
            Entity node
        );

        
    

            

        /** Isomethod of ada_use_type_clause_f_has_all langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_use_type_clause_f_has_all(
            Entity node
        );
            

        /** Isomethod of ada_use_type_clause_f_types langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_use_type_clause_f_types(
            Entity node
        );

        
    

            

        /** Isomethod of ada_value_sequence_f_iter_assoc langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_value_sequence_f_iter_assoc(
            Entity node
        );

        
    

            

        /** Isomethod of ada_variant_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_variant_f_choices(
            Entity node
        );
            

        /** Isomethod of ada_variant_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_variant_f_components(
            Entity node
        );

        
    

            

        /** Isomethod of ada_variant_part_f_discr_name langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_variant_part_f_discr_name(
            Entity node
        );
            

        /** Isomethod of ada_variant_part_f_variant langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_variant_part_f_variant(
            Entity node
        );

        
    

            

        /** Isomethod of ada_with_clause_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_with_clause_f_has_limited(
            Entity node
        );
            

        /** Isomethod of ada_with_clause_f_has_private langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_with_clause_f_has_private(
            Entity node
        );
            

        /** Isomethod of ada_with_clause_f_packages langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native Entity ada_with_clause_f_packages(
            Entity node
        );

        
    

            

        /** Isomethod of ada_with_private_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        public static native boolean ada_with_private_p_as_bool(
            Entity node
        );

        
    


        
    



    }


    // ==========
    // Defining the Native-Image bindings library
    // ==========

    
    
    
    
    
    
    

    /** This class contains the directives for the shared lib loading */
    public static final class LibDirectives implements CContext.Directives {
        @Override
        public List<String> getHeaderFiles() {
            List<String> res = new ArrayList<>();
            res.add("<libadalang.h>");
            res.add("<stdlib.h>");
            return res;
        }

        @Override
        public List<String> getLibraries() {
            List<String> res = new ArrayList<>();
            res.add("adalang");
            return res;
        }
    }

    // ===== Language specific structures =====

    
      


    // ===== Native structures =====

    /** Project manager native is just a pointer. */
    public interface ProjectManagerNative extends Pointer {}

    /** Structure for a string array. */
    @CContext(LibDirectives.class)
    @CStruct("ada_string_array_ptr_struct")
    public interface StringArrayNative extends PointerBase {
        @CField("length") public int get_length();
        @CField("length") public void set_length(int length);

        @CField("c_ptr") public CCharPointerPointer get_c_ptr();
        @CField("c_ptr") public void set_c_ptr(CCharPointerPointer c_ptr);
    }

    /** Structure for a scenrio variable. */
    @CContext(LibDirectives.class)
    @CStruct("ada_gpr_project_scenario_variable")
    public interface ScenarioVariableNative extends PointerBase {
        @CField("name") public CCharPointer get_name();
        @CField("name") public void set_name(CCharPointer name);

        @CField("value") public CCharPointer get_value();
        @CField("value") public void set_value(CCharPointer value);
    }



    // ===== Constant structures =====

    /** The structure for the langkit exceptions */
    @CContext(LibDirectives.class)
    @CStruct("ada_exception")
    public interface LangkitExceptionNative extends PointerBase {
        @CField("kind") public int get_kind();
        @CField("kind") public void set_kind(
            int kind
        );

        @CField("information") public CCharPointer get_information();
        @CField("information") public void set_information(
            CCharPointer information
        );
    }

    /** The big integers are just pointers */
    public interface BigIntegerNative extends Pointer {}

    /** The structure for the symbols */
    @CContext(LibDirectives.class)
    @CStruct("ada_symbol_type")
    public interface SymbolNative extends PointerBase {
        @CField("data") public VoidPointer get_data();
        @CField("data") public void set_data(
            VoidPointer data
        );

        @CField("bounds") public VoidPointer get_bounds();
        @CField("bounds") public void set_bounds(
            VoidPointer bounds
        );
    }

    /** The string wrappers are just pointers */
    public interface StringNative extends Pointer {}

    /** The structure for the text */
    @CContext(LibDirectives.class)
    @CStruct("ada_text")
    public interface TextNative extends PointerBase {
        @CField("chars") public CIntPointer get_chars();
        @CField("chars") public void set_chars(
            CIntPointer chars
        );

        @CField("length") public long get_length();
        @CField("length") public void set_length(
            long length
        );

        @CField("is_allocated") public int get_is_allocated();
        @CField("is_allocated") public void set_is_allocated(
            int is_allocated
        );
    }

    /** The structure for the source locations */
    @CContext(LibDirectives.class)
    @CStruct("ada_source_location")
    public interface SourceLocationNative extends PointerBase {
        @CField("line") public int get_line();
        @CField("line") public void set_line(
            int line
        );

        @CField("column") public short get_column();
        @CField("column") public void set_column(
            short column
        );
    }

    /** The structure for the source location ranges */
    @CContext(LibDirectives.class)
    @CStruct("ada_source_location_range")
    public interface SourceLocationRangeNative extends PointerBase {
        @CField("start.line") public int get_start_line();
        @CField("start.line") public void set_start_line(
            int start_line
        );

        @CField("start.column") public short get_start_column();
        @CField("start.column") public void set_start_column(
            short start_column
        );

        @CField("end.line") public int get_end_line();
        @CField("end.line") public void set_end_line(
            int end_line
        );

        @CField("end.column") public short get_end_column();
        @CField("end.column") public void set_end_column(
            short end_column
        );
    }

    /** The structure for the diagnostic */
    @CContext(LibDirectives.class)
    @CStruct("ada_diagnostic")
    public interface DiagnosticNative extends PointerBase {
        @CField("sloc_range.start.line") public int get_start_line();
        @CField("sloc_range.start.line") public void set_start_line(
            int start_line
        );

        @CField("sloc_range.start.column") public short get_start_column();
        @CField("sloc_range.start.column") public void set_start_column(
            short start_column
        );

        @CField("sloc_range.end.line") public int get_end_line();
        @CField("sloc_range.end.line") public void set_end_line(
            int end_line
        );

        @CField("sloc_range.end.column") public short get_end_column();
        @CField("sloc_range.end.column") public void set_end_column(
            short end_column
        );

        @CField("message.chars") public CIntPointer get_message_chars();
        @CField("message.chars") public void set_message_chars(
            CIntPointer chars
        );

        @CField("message.length") public long get_message_length();
        @CField("message.length") public void set_message_length(
            long length
        );

        @CField("message.is_allocated") public int get_message_is_allocated();
        @CField("message.is_allocated") public void set_message_is_allocated(
            int is_allocated
        );
    }


    /** The file reader is just a pointer */
    public interface FileReaderNative extends Pointer {}

    /** The unit provider is just a pointer */
    public interface UnitProviderNative extends Pointer {}

    /** The event handler is just a pointer */
    public interface EventHandlerNative extends Pointer {}

    /** The event handler unit requested callback type */
    public interface UnitRequestedFunctionPointer extends CFunctionPointer {
        @InvokeCFunctionPointer
        void invoke(
            VoidPointer data,
            AnalysisContextNative context,
            TextNative name,
            AnalysisUnitNative from,
            boolean found,
            boolean is_not_found_error
        );
    }

    /** The event handler unit parsed callback type */
    public interface UnitParsedFunctionPointer extends CFunctionPointer {
        @InvokeCFunctionPointer
        void invoke(
            VoidPointer data,
            AnalysisContextNative context,
            AnalysisUnitNative unit,
            boolean reparsed
        );
    }

    /** Anonymous structure for the token data handler */
    @RawStructure
    public interface TokenDataHandlerNative extends PointerBase {
        @RawField public long version();
    }

    /** The structure representing a token */
    @CContext(LibDirectives.class)
    @CStruct("ada_token")
    public interface TokenNative extends PointerBase {
        @CField("context") public AnalysisContextNative get_context();
        @CField("context") public void set_context(
            AnalysisContextNative context
        );

        @CField("token_data") public TokenDataHandlerNative get_data();
        @CField("token_data") public void set_data(
            TokenDataHandlerNative data
        );

        @CField("token_index") public int get_token_index();
        @CField("token_index") public void set_token_index(
            int token_index
        );

        @CField("trivia_index") public int get_trivia_index();
        @CField("trivia_index") public void set_trivia_index(
            int trivia_index
        );
    }

    /** Anonymous strucutre for analysis context */
    @RawStructure
    public interface AnalysisContextNative extends PointerBase {
        @RawField public long serial_number();
    }

    /** Anonymous strucutre for analysis unit */
    @RawStructure
    public interface AnalysisUnitNative extends PointerBase {
        @RawField public long version_number();
    }

    // ===== Generated structures =====

        
    
    

    /** The structure for the langkit ada_internal_metadata */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_metadata")
    public interface MetadataNative extends PointerBase {
        @CField("dottable_subp")
        public byte
        get_dottable_subp();

        @CField("dottable_subp")
        public void
        set_dottable_subp(
            byte val
        );

        @CFieldAddress("dottable_subp")
        public <T extends PointerBase> T address_dottable_subp();
        @CField("primitive")
        public Pointer
        get_primitive();

        @CField("primitive")
        public void
        set_primitive(
            Pointer val
        );

        @CFieldAddress("primitive")
        public <T extends PointerBase> T address_primitive();
        @CField("primitive_real_type")
        public Pointer
        get_primitive_real_type();

        @CField("primitive_real_type")
        public void
        set_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("primitive_real_type")
        public <T extends PointerBase> T address_primitive_real_type();
    }

        
    
    

    /** The structure for the langkit ada_internal_entity_info */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_entity_info")
    public interface EntityInfoNative extends PointerBase {
        @CField("md.dottable_subp")
        public byte
        get_md_dottable_subp();

        @CField("md.dottable_subp")
        public void
        set_md_dottable_subp(
            byte val
        );

        @CFieldAddress("md.dottable_subp")
        public <T extends PointerBase> T address_md_dottable_subp();
        @CField("md.primitive")
        public Pointer
        get_md_primitive();

        @CField("md.primitive")
        public void
        set_md_primitive(
            Pointer val
        );

        @CFieldAddress("md.primitive")
        public <T extends PointerBase> T address_md_primitive();
        @CField("md.primitive_real_type")
        public Pointer
        get_md_primitive_real_type();

        @CField("md.primitive_real_type")
        public void
        set_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("md.primitive_real_type")
        public <T extends PointerBase> T address_md_primitive_real_type();
        @CField("rebindings")
        public Pointer
        get_rebindings();

        @CField("rebindings")
        public void
        set_rebindings(
            Pointer val
        );

        @CFieldAddress("rebindings")
        public <T extends PointerBase> T address_rebindings();
        @CField("from_rebound")
        public byte
        get_from_rebound();

        @CField("from_rebound")
        public void
        set_from_rebound(
            byte val
        );

        @CFieldAddress("from_rebound")
        public <T extends PointerBase> T address_from_rebound();
    }

    
    

    /** The structure for the langkit ada_base_entity */
    @CContext(LibDirectives.class)
    @CStruct("ada_base_entity")
    public interface EntityNative extends PointerBase {
        @CField("node")
        public Pointer
        get_node();

        @CField("node")
        public void
        set_node(
            Pointer val
        );

        @CFieldAddress("node")
        public <T extends PointerBase> T address_node();
        @CField("info.md.dottable_subp")
        public byte
        get_info_md_dottable_subp();

        @CField("info.md.dottable_subp")
        public void
        set_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("info.md.dottable_subp")
        public <T extends PointerBase> T address_info_md_dottable_subp();
        @CField("info.md.primitive")
        public Pointer
        get_info_md_primitive();

        @CField("info.md.primitive")
        public void
        set_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("info.md.primitive")
        public <T extends PointerBase> T address_info_md_primitive();
        @CField("info.md.primitive_real_type")
        public Pointer
        get_info_md_primitive_real_type();

        @CField("info.md.primitive_real_type")
        public void
        set_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("info.md.primitive_real_type")
        public <T extends PointerBase> T address_info_md_primitive_real_type();
        @CField("info.rebindings")
        public Pointer
        get_info_rebindings();

        @CField("info.rebindings")
        public void
        set_info_rebindings(
            Pointer val
        );

        @CFieldAddress("info.rebindings")
        public <T extends PointerBase> T address_info_rebindings();
        @CField("info.from_rebound")
        public byte
        get_info_from_rebound();

        @CField("info.from_rebound")
        public void
        set_info_from_rebound(
            byte val
        );

        @CFieldAddress("info.from_rebound")
        public <T extends PointerBase> T address_info_from_rebound();
    }

        
    
    

    /** The structure for the langkit ada_internal_aspect */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_aspect")
    public interface AspectNative extends PointerBase {
        @CField("exists")
        public byte
        get_exists();

        @CField("exists")
        public void
        set_exists(
            byte val
        );

        @CFieldAddress("exists")
        public <T extends PointerBase> T address_exists();
        @CField("node.node")
        public Pointer
        get_node_node();

        @CField("node.node")
        public void
        set_node_node(
            Pointer val
        );

        @CFieldAddress("node.node")
        public <T extends PointerBase> T address_node_node();
        @CField("node.info.md.dottable_subp")
        public byte
        get_node_info_md_dottable_subp();

        @CField("node.info.md.dottable_subp")
        public void
        set_node_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("node.info.md.dottable_subp")
        public <T extends PointerBase> T address_node_info_md_dottable_subp();
        @CField("node.info.md.primitive")
        public Pointer
        get_node_info_md_primitive();

        @CField("node.info.md.primitive")
        public void
        set_node_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("node.info.md.primitive")
        public <T extends PointerBase> T address_node_info_md_primitive();
        @CField("node.info.md.primitive_real_type")
        public Pointer
        get_node_info_md_primitive_real_type();

        @CField("node.info.md.primitive_real_type")
        public void
        set_node_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("node.info.md.primitive_real_type")
        public <T extends PointerBase> T address_node_info_md_primitive_real_type();
        @CField("node.info.rebindings")
        public Pointer
        get_node_info_rebindings();

        @CField("node.info.rebindings")
        public void
        set_node_info_rebindings(
            Pointer val
        );

        @CFieldAddress("node.info.rebindings")
        public <T extends PointerBase> T address_node_info_rebindings();
        @CField("node.info.from_rebound")
        public byte
        get_node_info_from_rebound();

        @CField("node.info.from_rebound")
        public void
        set_node_info_from_rebound(
            byte val
        );

        @CFieldAddress("node.info.from_rebound")
        public <T extends PointerBase> T address_node_info_from_rebound();
        @CField("value.node")
        public Pointer
        get_value_node();

        @CField("value.node")
        public void
        set_value_node(
            Pointer val
        );

        @CFieldAddress("value.node")
        public <T extends PointerBase> T address_value_node();
        @CField("value.info.md.dottable_subp")
        public byte
        get_value_info_md_dottable_subp();

        @CField("value.info.md.dottable_subp")
        public void
        set_value_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("value.info.md.dottable_subp")
        public <T extends PointerBase> T address_value_info_md_dottable_subp();
        @CField("value.info.md.primitive")
        public Pointer
        get_value_info_md_primitive();

        @CField("value.info.md.primitive")
        public void
        set_value_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("value.info.md.primitive")
        public <T extends PointerBase> T address_value_info_md_primitive();
        @CField("value.info.md.primitive_real_type")
        public Pointer
        get_value_info_md_primitive_real_type();

        @CField("value.info.md.primitive_real_type")
        public void
        set_value_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("value.info.md.primitive_real_type")
        public <T extends PointerBase> T address_value_info_md_primitive_real_type();
        @CField("value.info.rebindings")
        public Pointer
        get_value_info_rebindings();

        @CField("value.info.rebindings")
        public void
        set_value_info_rebindings(
            Pointer val
        );

        @CFieldAddress("value.info.rebindings")
        public <T extends PointerBase> T address_value_info_rebindings();
        @CField("value.info.from_rebound")
        public byte
        get_value_info_from_rebound();

        @CField("value.info.from_rebound")
        public void
        set_value_info_from_rebound(
            byte val
        );

        @CFieldAddress("value.info.from_rebound")
        public <T extends PointerBase> T address_value_info_from_rebound();
        @CField("inherited")
        public byte
        get_inherited();

        @CField("inherited")
        public void
        set_inherited(
            byte val
        );

        @CFieldAddress("inherited")
        public <T extends PointerBase> T address_inherited();
    }

        
    
    

    /** The structure for the langkit ada_internal_completion_item */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_completion_item")
    public interface CompletionItemNative extends PointerBase {
        @CField("decl.node")
        public Pointer
        get_decl_node();

        @CField("decl.node")
        public void
        set_decl_node(
            Pointer val
        );

        @CFieldAddress("decl.node")
        public <T extends PointerBase> T address_decl_node();
        @CField("decl.info.md.dottable_subp")
        public byte
        get_decl_info_md_dottable_subp();

        @CField("decl.info.md.dottable_subp")
        public void
        set_decl_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("decl.info.md.dottable_subp")
        public <T extends PointerBase> T address_decl_info_md_dottable_subp();
        @CField("decl.info.md.primitive")
        public Pointer
        get_decl_info_md_primitive();

        @CField("decl.info.md.primitive")
        public void
        set_decl_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("decl.info.md.primitive")
        public <T extends PointerBase> T address_decl_info_md_primitive();
        @CField("decl.info.md.primitive_real_type")
        public Pointer
        get_decl_info_md_primitive_real_type();

        @CField("decl.info.md.primitive_real_type")
        public void
        set_decl_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("decl.info.md.primitive_real_type")
        public <T extends PointerBase> T address_decl_info_md_primitive_real_type();
        @CField("decl.info.rebindings")
        public Pointer
        get_decl_info_rebindings();

        @CField("decl.info.rebindings")
        public void
        set_decl_info_rebindings(
            Pointer val
        );

        @CFieldAddress("decl.info.rebindings")
        public <T extends PointerBase> T address_decl_info_rebindings();
        @CField("decl.info.from_rebound")
        public byte
        get_decl_info_from_rebound();

        @CField("decl.info.from_rebound")
        public void
        set_decl_info_from_rebound(
            byte val
        );

        @CFieldAddress("decl.info.from_rebound")
        public <T extends PointerBase> T address_decl_info_from_rebound();
        @CField("is_dot_call")
        public byte
        get_is_dot_call();

        @CField("is_dot_call")
        public void
        set_is_dot_call(
            byte val
        );

        @CFieldAddress("is_dot_call")
        public <T extends PointerBase> T address_is_dot_call();
        @CField("is_visible")
        public byte
        get_is_visible();

        @CField("is_visible")
        public void
        set_is_visible(
            byte val
        );

        @CFieldAddress("is_visible")
        public <T extends PointerBase> T address_is_visible();
        @CField("weight")
        public int
        get_weight();

        @CField("weight")
        public void
        set_weight(
            int val
        );

        @CFieldAddress("weight")
        public <T extends PointerBase> T address_weight();
    }

        
        
    
    

    /** The structure for the langkit ada_internal_discrete_range */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_discrete_range")
    public interface DiscreteRangeNative extends PointerBase {
        @CField("low_bound.node")
        public Pointer
        get_low_bound_node();

        @CField("low_bound.node")
        public void
        set_low_bound_node(
            Pointer val
        );

        @CFieldAddress("low_bound.node")
        public <T extends PointerBase> T address_low_bound_node();
        @CField("low_bound.info.md.dottable_subp")
        public byte
        get_low_bound_info_md_dottable_subp();

        @CField("low_bound.info.md.dottable_subp")
        public void
        set_low_bound_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("low_bound.info.md.dottable_subp")
        public <T extends PointerBase> T address_low_bound_info_md_dottable_subp();
        @CField("low_bound.info.md.primitive")
        public Pointer
        get_low_bound_info_md_primitive();

        @CField("low_bound.info.md.primitive")
        public void
        set_low_bound_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("low_bound.info.md.primitive")
        public <T extends PointerBase> T address_low_bound_info_md_primitive();
        @CField("low_bound.info.md.primitive_real_type")
        public Pointer
        get_low_bound_info_md_primitive_real_type();

        @CField("low_bound.info.md.primitive_real_type")
        public void
        set_low_bound_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("low_bound.info.md.primitive_real_type")
        public <T extends PointerBase> T address_low_bound_info_md_primitive_real_type();
        @CField("low_bound.info.rebindings")
        public Pointer
        get_low_bound_info_rebindings();

        @CField("low_bound.info.rebindings")
        public void
        set_low_bound_info_rebindings(
            Pointer val
        );

        @CFieldAddress("low_bound.info.rebindings")
        public <T extends PointerBase> T address_low_bound_info_rebindings();
        @CField("low_bound.info.from_rebound")
        public byte
        get_low_bound_info_from_rebound();

        @CField("low_bound.info.from_rebound")
        public void
        set_low_bound_info_from_rebound(
            byte val
        );

        @CFieldAddress("low_bound.info.from_rebound")
        public <T extends PointerBase> T address_low_bound_info_from_rebound();
        @CField("high_bound.node")
        public Pointer
        get_high_bound_node();

        @CField("high_bound.node")
        public void
        set_high_bound_node(
            Pointer val
        );

        @CFieldAddress("high_bound.node")
        public <T extends PointerBase> T address_high_bound_node();
        @CField("high_bound.info.md.dottable_subp")
        public byte
        get_high_bound_info_md_dottable_subp();

        @CField("high_bound.info.md.dottable_subp")
        public void
        set_high_bound_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("high_bound.info.md.dottable_subp")
        public <T extends PointerBase> T address_high_bound_info_md_dottable_subp();
        @CField("high_bound.info.md.primitive")
        public Pointer
        get_high_bound_info_md_primitive();

        @CField("high_bound.info.md.primitive")
        public void
        set_high_bound_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("high_bound.info.md.primitive")
        public <T extends PointerBase> T address_high_bound_info_md_primitive();
        @CField("high_bound.info.md.primitive_real_type")
        public Pointer
        get_high_bound_info_md_primitive_real_type();

        @CField("high_bound.info.md.primitive_real_type")
        public void
        set_high_bound_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("high_bound.info.md.primitive_real_type")
        public <T extends PointerBase> T address_high_bound_info_md_primitive_real_type();
        @CField("high_bound.info.rebindings")
        public Pointer
        get_high_bound_info_rebindings();

        @CField("high_bound.info.rebindings")
        public void
        set_high_bound_info_rebindings(
            Pointer val
        );

        @CFieldAddress("high_bound.info.rebindings")
        public <T extends PointerBase> T address_high_bound_info_rebindings();
        @CField("high_bound.info.from_rebound")
        public byte
        get_high_bound_info_from_rebound();

        @CField("high_bound.info.from_rebound")
        public void
        set_high_bound_info_from_rebound(
            byte val
        );

        @CFieldAddress("high_bound.info.from_rebound")
        public <T extends PointerBase> T address_high_bound_info_from_rebound();
    }

        
    
    

    /** The structure for the langkit ada_internal_discriminant_values */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_discriminant_values")
    public interface DiscriminantValuesNative extends PointerBase {
        @CField("discriminant.node")
        public Pointer
        get_discriminant_node();

        @CField("discriminant.node")
        public void
        set_discriminant_node(
            Pointer val
        );

        @CFieldAddress("discriminant.node")
        public <T extends PointerBase> T address_discriminant_node();
        @CField("discriminant.info.md.dottable_subp")
        public byte
        get_discriminant_info_md_dottable_subp();

        @CField("discriminant.info.md.dottable_subp")
        public void
        set_discriminant_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("discriminant.info.md.dottable_subp")
        public <T extends PointerBase> T address_discriminant_info_md_dottable_subp();
        @CField("discriminant.info.md.primitive")
        public Pointer
        get_discriminant_info_md_primitive();

        @CField("discriminant.info.md.primitive")
        public void
        set_discriminant_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("discriminant.info.md.primitive")
        public <T extends PointerBase> T address_discriminant_info_md_primitive();
        @CField("discriminant.info.md.primitive_real_type")
        public Pointer
        get_discriminant_info_md_primitive_real_type();

        @CField("discriminant.info.md.primitive_real_type")
        public void
        set_discriminant_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("discriminant.info.md.primitive_real_type")
        public <T extends PointerBase> T address_discriminant_info_md_primitive_real_type();
        @CField("discriminant.info.rebindings")
        public Pointer
        get_discriminant_info_rebindings();

        @CField("discriminant.info.rebindings")
        public void
        set_discriminant_info_rebindings(
            Pointer val
        );

        @CFieldAddress("discriminant.info.rebindings")
        public <T extends PointerBase> T address_discriminant_info_rebindings();
        @CField("discriminant.info.from_rebound")
        public byte
        get_discriminant_info_from_rebound();

        @CField("discriminant.info.from_rebound")
        public void
        set_discriminant_info_from_rebound(
            byte val
        );

        @CFieldAddress("discriminant.info.from_rebound")
        public <T extends PointerBase> T address_discriminant_info_from_rebound();
        @CField("values.node")
        public Pointer
        get_values_node();

        @CField("values.node")
        public void
        set_values_node(
            Pointer val
        );

        @CFieldAddress("values.node")
        public <T extends PointerBase> T address_values_node();
        @CField("values.info.md.dottable_subp")
        public byte
        get_values_info_md_dottable_subp();

        @CField("values.info.md.dottable_subp")
        public void
        set_values_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("values.info.md.dottable_subp")
        public <T extends PointerBase> T address_values_info_md_dottable_subp();
        @CField("values.info.md.primitive")
        public Pointer
        get_values_info_md_primitive();

        @CField("values.info.md.primitive")
        public void
        set_values_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("values.info.md.primitive")
        public <T extends PointerBase> T address_values_info_md_primitive();
        @CField("values.info.md.primitive_real_type")
        public Pointer
        get_values_info_md_primitive_real_type();

        @CField("values.info.md.primitive_real_type")
        public void
        set_values_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("values.info.md.primitive_real_type")
        public <T extends PointerBase> T address_values_info_md_primitive_real_type();
        @CField("values.info.rebindings")
        public Pointer
        get_values_info_rebindings();

        @CField("values.info.rebindings")
        public void
        set_values_info_rebindings(
            Pointer val
        );

        @CFieldAddress("values.info.rebindings")
        public <T extends PointerBase> T address_values_info_rebindings();
        @CField("values.info.from_rebound")
        public byte
        get_values_info_from_rebound();

        @CField("values.info.from_rebound")
        public void
        set_values_info_from_rebound(
            byte val
        );

        @CFieldAddress("values.info.from_rebound")
        public <T extends PointerBase> T address_values_info_from_rebound();
    }

        
    
    

    /** The structure for the langkit ada_internal_doc_annotation */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_doc_annotation")
    public interface DocAnnotationNative extends PointerBase {
        @CField("key")
        public StringNative
        get_key();

        @CField("key")
        public void
        set_key(
            StringNative val
        );

        @CFieldAddress("key")
        public <T extends PointerBase> T address_key();
        @CField("value")
        public StringNative
        get_value();

        @CField("value")
        public void
        set_value(
            StringNative val
        );

        @CFieldAddress("value")
        public <T extends PointerBase> T address_value();
    }

        
        
        
        
        
        
        
    
    

    /** The structure for the langkit ada_internal_param_actual */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_param_actual")
    public interface ParamActualNative extends PointerBase {
        @CField("param.node")
        public Pointer
        get_param_node();

        @CField("param.node")
        public void
        set_param_node(
            Pointer val
        );

        @CFieldAddress("param.node")
        public <T extends PointerBase> T address_param_node();
        @CField("param.info.md.dottable_subp")
        public byte
        get_param_info_md_dottable_subp();

        @CField("param.info.md.dottable_subp")
        public void
        set_param_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("param.info.md.dottable_subp")
        public <T extends PointerBase> T address_param_info_md_dottable_subp();
        @CField("param.info.md.primitive")
        public Pointer
        get_param_info_md_primitive();

        @CField("param.info.md.primitive")
        public void
        set_param_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("param.info.md.primitive")
        public <T extends PointerBase> T address_param_info_md_primitive();
        @CField("param.info.md.primitive_real_type")
        public Pointer
        get_param_info_md_primitive_real_type();

        @CField("param.info.md.primitive_real_type")
        public void
        set_param_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("param.info.md.primitive_real_type")
        public <T extends PointerBase> T address_param_info_md_primitive_real_type();
        @CField("param.info.rebindings")
        public Pointer
        get_param_info_rebindings();

        @CField("param.info.rebindings")
        public void
        set_param_info_rebindings(
            Pointer val
        );

        @CFieldAddress("param.info.rebindings")
        public <T extends PointerBase> T address_param_info_rebindings();
        @CField("param.info.from_rebound")
        public byte
        get_param_info_from_rebound();

        @CField("param.info.from_rebound")
        public void
        set_param_info_from_rebound(
            byte val
        );

        @CFieldAddress("param.info.from_rebound")
        public <T extends PointerBase> T address_param_info_from_rebound();
        @CField("actual.node")
        public Pointer
        get_actual_node();

        @CField("actual.node")
        public void
        set_actual_node(
            Pointer val
        );

        @CFieldAddress("actual.node")
        public <T extends PointerBase> T address_actual_node();
        @CField("actual.info.md.dottable_subp")
        public byte
        get_actual_info_md_dottable_subp();

        @CField("actual.info.md.dottable_subp")
        public void
        set_actual_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("actual.info.md.dottable_subp")
        public <T extends PointerBase> T address_actual_info_md_dottable_subp();
        @CField("actual.info.md.primitive")
        public Pointer
        get_actual_info_md_primitive();

        @CField("actual.info.md.primitive")
        public void
        set_actual_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("actual.info.md.primitive")
        public <T extends PointerBase> T address_actual_info_md_primitive();
        @CField("actual.info.md.primitive_real_type")
        public Pointer
        get_actual_info_md_primitive_real_type();

        @CField("actual.info.md.primitive_real_type")
        public void
        set_actual_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("actual.info.md.primitive_real_type")
        public <T extends PointerBase> T address_actual_info_md_primitive_real_type();
        @CField("actual.info.rebindings")
        public Pointer
        get_actual_info_rebindings();

        @CField("actual.info.rebindings")
        public void
        set_actual_info_rebindings(
            Pointer val
        );

        @CFieldAddress("actual.info.rebindings")
        public <T extends PointerBase> T address_actual_info_rebindings();
        @CField("actual.info.from_rebound")
        public byte
        get_actual_info_from_rebound();

        @CField("actual.info.from_rebound")
        public void
        set_actual_info_from_rebound(
            byte val
        );

        @CFieldAddress("actual.info.from_rebound")
        public <T extends PointerBase> T address_actual_info_from_rebound();
    }

        
        
        
    
    

    /** The structure for the langkit ada_internal_ref_result */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_ref_result")
    public interface RefResultNative extends PointerBase {
        @CField("ref.node")
        public Pointer
        get_ref_node();

        @CField("ref.node")
        public void
        set_ref_node(
            Pointer val
        );

        @CFieldAddress("ref.node")
        public <T extends PointerBase> T address_ref_node();
        @CField("ref.info.md.dottable_subp")
        public byte
        get_ref_info_md_dottable_subp();

        @CField("ref.info.md.dottable_subp")
        public void
        set_ref_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("ref.info.md.dottable_subp")
        public <T extends PointerBase> T address_ref_info_md_dottable_subp();
        @CField("ref.info.md.primitive")
        public Pointer
        get_ref_info_md_primitive();

        @CField("ref.info.md.primitive")
        public void
        set_ref_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("ref.info.md.primitive")
        public <T extends PointerBase> T address_ref_info_md_primitive();
        @CField("ref.info.md.primitive_real_type")
        public Pointer
        get_ref_info_md_primitive_real_type();

        @CField("ref.info.md.primitive_real_type")
        public void
        set_ref_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("ref.info.md.primitive_real_type")
        public <T extends PointerBase> T address_ref_info_md_primitive_real_type();
        @CField("ref.info.rebindings")
        public Pointer
        get_ref_info_rebindings();

        @CField("ref.info.rebindings")
        public void
        set_ref_info_rebindings(
            Pointer val
        );

        @CFieldAddress("ref.info.rebindings")
        public <T extends PointerBase> T address_ref_info_rebindings();
        @CField("ref.info.from_rebound")
        public byte
        get_ref_info_from_rebound();

        @CField("ref.info.from_rebound")
        public void
        set_ref_info_from_rebound(
            byte val
        );

        @CFieldAddress("ref.info.from_rebound")
        public <T extends PointerBase> T address_ref_info_from_rebound();
        @CField("kind")
        public int
        get_kind();

        @CField("kind")
        public void
        set_kind(
            int val
        );

        @CFieldAddress("kind")
        public <T extends PointerBase> T address_kind();
    }

        
    
    

    /** The structure for the langkit ada_internal_refd_decl */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_refd_decl")
    public interface RefdDeclNative extends PointerBase {
        @CField("decl.node")
        public Pointer
        get_decl_node();

        @CField("decl.node")
        public void
        set_decl_node(
            Pointer val
        );

        @CFieldAddress("decl.node")
        public <T extends PointerBase> T address_decl_node();
        @CField("decl.info.md.dottable_subp")
        public byte
        get_decl_info_md_dottable_subp();

        @CField("decl.info.md.dottable_subp")
        public void
        set_decl_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("decl.info.md.dottable_subp")
        public <T extends PointerBase> T address_decl_info_md_dottable_subp();
        @CField("decl.info.md.primitive")
        public Pointer
        get_decl_info_md_primitive();

        @CField("decl.info.md.primitive")
        public void
        set_decl_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("decl.info.md.primitive")
        public <T extends PointerBase> T address_decl_info_md_primitive();
        @CField("decl.info.md.primitive_real_type")
        public Pointer
        get_decl_info_md_primitive_real_type();

        @CField("decl.info.md.primitive_real_type")
        public void
        set_decl_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("decl.info.md.primitive_real_type")
        public <T extends PointerBase> T address_decl_info_md_primitive_real_type();
        @CField("decl.info.rebindings")
        public Pointer
        get_decl_info_rebindings();

        @CField("decl.info.rebindings")
        public void
        set_decl_info_rebindings(
            Pointer val
        );

        @CFieldAddress("decl.info.rebindings")
        public <T extends PointerBase> T address_decl_info_rebindings();
        @CField("decl.info.from_rebound")
        public byte
        get_decl_info_from_rebound();

        @CField("decl.info.from_rebound")
        public void
        set_decl_info_from_rebound(
            byte val
        );

        @CFieldAddress("decl.info.from_rebound")
        public <T extends PointerBase> T address_decl_info_from_rebound();
        @CField("kind")
        public int
        get_kind();

        @CField("kind")
        public void
        set_kind(
            int val
        );

        @CFieldAddress("kind")
        public <T extends PointerBase> T address_kind();
    }

        
    
    

    /** The structure for the langkit ada_internal_refd_def */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_refd_def")
    public interface RefdDefNative extends PointerBase {
        @CField("def_name.node")
        public Pointer
        get_def_name_node();

        @CField("def_name.node")
        public void
        set_def_name_node(
            Pointer val
        );

        @CFieldAddress("def_name.node")
        public <T extends PointerBase> T address_def_name_node();
        @CField("def_name.info.md.dottable_subp")
        public byte
        get_def_name_info_md_dottable_subp();

        @CField("def_name.info.md.dottable_subp")
        public void
        set_def_name_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("def_name.info.md.dottable_subp")
        public <T extends PointerBase> T address_def_name_info_md_dottable_subp();
        @CField("def_name.info.md.primitive")
        public Pointer
        get_def_name_info_md_primitive();

        @CField("def_name.info.md.primitive")
        public void
        set_def_name_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("def_name.info.md.primitive")
        public <T extends PointerBase> T address_def_name_info_md_primitive();
        @CField("def_name.info.md.primitive_real_type")
        public Pointer
        get_def_name_info_md_primitive_real_type();

        @CField("def_name.info.md.primitive_real_type")
        public void
        set_def_name_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("def_name.info.md.primitive_real_type")
        public <T extends PointerBase> T address_def_name_info_md_primitive_real_type();
        @CField("def_name.info.rebindings")
        public Pointer
        get_def_name_info_rebindings();

        @CField("def_name.info.rebindings")
        public void
        set_def_name_info_rebindings(
            Pointer val
        );

        @CFieldAddress("def_name.info.rebindings")
        public <T extends PointerBase> T address_def_name_info_rebindings();
        @CField("def_name.info.from_rebound")
        public byte
        get_def_name_info_from_rebound();

        @CField("def_name.info.from_rebound")
        public void
        set_def_name_info_from_rebound(
            byte val
        );

        @CFieldAddress("def_name.info.from_rebound")
        public <T extends PointerBase> T address_def_name_info_from_rebound();
        @CField("kind")
        public int
        get_kind();

        @CField("kind")
        public void
        set_kind(
            int val
        );

        @CFieldAddress("kind")
        public <T extends PointerBase> T address_kind();
    }

        
    
    

    /** The structure for the langkit ada_internal_shape */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_shape")
    public interface ShapeNative extends PointerBase {
        @CField("components")
        public AdaNodeArrayNative
        get_components();

        @CField("components")
        public void
        set_components(
            AdaNodeArrayNative val
        );

        @CFieldAddress("components")
        public <T extends PointerBase> T address_components();
        @CField("discriminants_values")
        public DiscriminantValuesArrayNative
        get_discriminants_values();

        @CField("discriminants_values")
        public void
        set_discriminants_values(
            DiscriminantValuesArrayNative val
        );

        @CFieldAddress("discriminants_values")
        public <T extends PointerBase> T address_discriminants_values();
    }

        
    
    

    /** The structure for the langkit ada_internal_substitution */
    @CContext(LibDirectives.class)
    @CStruct("ada_internal_substitution")
    public interface SubstitutionNative extends PointerBase {
        @CField("from_decl.node")
        public Pointer
        get_from_decl_node();

        @CField("from_decl.node")
        public void
        set_from_decl_node(
            Pointer val
        );

        @CFieldAddress("from_decl.node")
        public <T extends PointerBase> T address_from_decl_node();
        @CField("from_decl.info.md.dottable_subp")
        public byte
        get_from_decl_info_md_dottable_subp();

        @CField("from_decl.info.md.dottable_subp")
        public void
        set_from_decl_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("from_decl.info.md.dottable_subp")
        public <T extends PointerBase> T address_from_decl_info_md_dottable_subp();
        @CField("from_decl.info.md.primitive")
        public Pointer
        get_from_decl_info_md_primitive();

        @CField("from_decl.info.md.primitive")
        public void
        set_from_decl_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("from_decl.info.md.primitive")
        public <T extends PointerBase> T address_from_decl_info_md_primitive();
        @CField("from_decl.info.md.primitive_real_type")
        public Pointer
        get_from_decl_info_md_primitive_real_type();

        @CField("from_decl.info.md.primitive_real_type")
        public void
        set_from_decl_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("from_decl.info.md.primitive_real_type")
        public <T extends PointerBase> T address_from_decl_info_md_primitive_real_type();
        @CField("from_decl.info.rebindings")
        public Pointer
        get_from_decl_info_rebindings();

        @CField("from_decl.info.rebindings")
        public void
        set_from_decl_info_rebindings(
            Pointer val
        );

        @CFieldAddress("from_decl.info.rebindings")
        public <T extends PointerBase> T address_from_decl_info_rebindings();
        @CField("from_decl.info.from_rebound")
        public byte
        get_from_decl_info_from_rebound();

        @CField("from_decl.info.from_rebound")
        public void
        set_from_decl_info_from_rebound(
            byte val
        );

        @CFieldAddress("from_decl.info.from_rebound")
        public <T extends PointerBase> T address_from_decl_info_from_rebound();
        @CField("to_value")
        public BigIntegerNative
        get_to_value();

        @CField("to_value")
        public void
        set_to_value(
            BigIntegerNative val
        );

        @CFieldAddress("to_value")
        public <T extends PointerBase> T address_to_value();
        @CField("value_type.node")
        public Pointer
        get_value_type_node();

        @CField("value_type.node")
        public void
        set_value_type_node(
            Pointer val
        );

        @CFieldAddress("value_type.node")
        public <T extends PointerBase> T address_value_type_node();
        @CField("value_type.info.md.dottable_subp")
        public byte
        get_value_type_info_md_dottable_subp();

        @CField("value_type.info.md.dottable_subp")
        public void
        set_value_type_info_md_dottable_subp(
            byte val
        );

        @CFieldAddress("value_type.info.md.dottable_subp")
        public <T extends PointerBase> T address_value_type_info_md_dottable_subp();
        @CField("value_type.info.md.primitive")
        public Pointer
        get_value_type_info_md_primitive();

        @CField("value_type.info.md.primitive")
        public void
        set_value_type_info_md_primitive(
            Pointer val
        );

        @CFieldAddress("value_type.info.md.primitive")
        public <T extends PointerBase> T address_value_type_info_md_primitive();
        @CField("value_type.info.md.primitive_real_type")
        public Pointer
        get_value_type_info_md_primitive_real_type();

        @CField("value_type.info.md.primitive_real_type")
        public void
        set_value_type_info_md_primitive_real_type(
            Pointer val
        );

        @CFieldAddress("value_type.info.md.primitive_real_type")
        public <T extends PointerBase> T address_value_type_info_md_primitive_real_type();
        @CField("value_type.info.rebindings")
        public Pointer
        get_value_type_info_rebindings();

        @CField("value_type.info.rebindings")
        public void
        set_value_type_info_rebindings(
            Pointer val
        );

        @CFieldAddress("value_type.info.rebindings")
        public <T extends PointerBase> T address_value_type_info_rebindings();
        @CField("value_type.info.from_rebound")
        public byte
        get_value_type_info_from_rebound();

        @CField("value_type.info.from_rebound")
        public void
        set_value_type_info_from_rebound(
            byte val
        );

        @CFieldAddress("value_type.info.from_rebound")
        public <T extends PointerBase> T address_value_type_info_from_rebound();
    }


    // ===== Generated arrays =====

    
    

    /**
     * The native structure of the ada_discriminant_values_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_discriminant_values_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface DiscriminantValuesArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_doc_annotation_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_doc_annotation_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface DocAnnotationArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_ada_node_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_ada_node_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface AdaNodeArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_param_actual_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_param_actual_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface ParamActualArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_ref_result_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_ref_result_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface RefResultArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_shape_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_shape_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface ShapeArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_substitution_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_substitution_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface SubstitutionArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_analysis_unit_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_analysis_unit_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface AnalysisUnitArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }

    
    

    /**
     * The native structure of the ada_unbounded_text_type_array langkit array.
     */
    @CContext(LibDirectives.class)
    @CStruct(
        value = "ada_unbounded_text_type_array_record",
        addStructKeyword = true,
        isIncomplete = true
    )
    public interface UnboundedTextTypeArrayNative extends PointerBase {
        @CField("n") public int get_n();
        @CField("ref_count") public int get_ref_count();
        @CFieldAddress("items")
        public <T extends PointerBase> T address_items();
    }


    // ===== Generated iterators =====

    
    

    /** This interface represents the langkit ada_completion_item_iterator iterator */
    public interface CompletionItemIteratorNative extends Pointer {}


    // ===== Native function definitions =====

    /** This class contains all native function definitions for NI */
    @CContext(LibDirectives.class)
    public static final class NI_LIB {

        // ----- Language specific functions -----

        
      


    // --- Project manager functions

    /** Function to load a GPR project */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native void ada_gpr_project_load(
        CCharPointer projectFile,
        Pointer scenarioVariable,
        CCharPointer target,
        CCharPointer runtime,
        Pointer project,
        Pointer errors
    );

    /** Free the given project */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native void ada_gpr_project_free(
        ProjectManagerNative project
    );

    /** Get the unit provider for the given project */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native UnitProviderNative ada_gpr_project_create_unit_provider(
        ProjectManagerNative project,
        CCharPointer subProject
    );

    /** Get the source file array for the given project */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native StringArrayNative ada_gpr_project_source_files(
        ProjectManagerNative project,
        int mode,
        CCharPointerPointer projectsData,
        int projectsLength
    );

    /**
     * Wrapper around ``Initialize_Context_From_Project`` to initialize
     * ``Context`` (an already allocated but not yet initialized analysis
     * context) from ``Self``.
     */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native void ada_gpr_project_initialize_context(
        ProjectManagerNative project,
        AnalysisContextNative context,
        CCharPointer subproject,
        EventHandlerNative eventHandler,
        int withTrivia,
        int tabStop
    );

    /** Free the given source file array */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native void ada_free_string_array(
        StringArrayNative sourceFileArray
    );

    /** Create an auto provider reference */
    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native UnitProviderNative ada_create_auto_provider(
        CCharPointerPointer sourceFiles,
        CCharPointer charset
    );


    // --- Config pragmas

    @CompilerDirectives.TruffleBoundary
    @CFunction
    public static native void ada_set_config_pragmas_mapping(
        AnalysisContextNative context,
        AnalysisUnitNative globalPragmas,
        Pointer localPragmas
    );



        // ----- Entry point literals -----

        /**
         * This entry point literal provide a pointer to the unit requested
         * callback.
         */
        public static final CEntryPointLiteral<UnitRequestedFunctionPointer>
            unitRequestedFunction = CEntryPointLiteral.create(
                Libadalang.class,
                "unitRequested",
                IsolateThread.class,
                AnalysisContextNative.class,
                TextNative.class,
                AnalysisUnitNative.class,
                byte.class,
                byte.class
            );

        /**
         * This entry point literal provide a pointer to the unit parsed
         * callback.
         */
        public static final CEntryPointLiteral<UnitParsedFunctionPointer>
            unitParsedFunction = CEntryPointLiteral.create(
                Libadalang.class,
                "unitParsed",
                IsolateThread.class,
                AnalysisContextNative.class,
                AnalysisUnitNative.class,
                byte.class
            );

        // ----- Util functions -----

        /** Util function to free langkit side allocated memory */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_free(
            PointerBase pointer
        );

        // ----- Exception functions -----

        /** Get the last exception raised by langkit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native LangkitExceptionNative
        ada_get_last_exception();

        // ----- Big integer functions -----

        /** Create a big integer from a text */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native BigIntegerNative ada_create_big_integer(
            TextNative text
        );

        /** Get the text representation of a big integer */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_big_integer_text(
            BigIntegerNative big_integer,
            TextNative text
        );

        /** Decrease the reference counter of the big integer */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_big_integer_decref(
            BigIntegerNative big_integer
        );

        // ----- Symbol functions -----

        /** Create a new symbol in the given context */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_context_symbol(
            AnalysisContextNative context,
            TextNative text,
            SymbolNative res
        );

        /** Get the text of a given symbol */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_symbol_text(
            SymbolNative symbol,
            TextNative text
        );

        // ----- String functions -----

        /** Create a new string wrapper in langkit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native StringNative ada_create_string(
            CIntPointer content,
            int length
        );

        /** Decrease the reference counter of a string */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_string_dec_ref(
            StringNative string
        );

        // ----- Text functions -----

        /** Destroy a text in the memory */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_destroy_text(
            TextNative text
        );

        // ----- File reader functions -----

        /** Decrease the reference counter of the given file reader */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_dec_ref_file_reader(
            FileReaderNative fileReader
        );

        // ----- Unit provider functions -----

        /** Decrease the ref counter of the unit provider */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_dec_ref_unit_provider(
            UnitProviderNative unitProvider
        );

        // ----- Event handler functions -----

        /** Create a new event handler */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native EventHandlerNative ada_create_event_handler(
            VoidPointer data,
            VoidPointer destroy_callback,
            UnitRequestedFunctionPointer unit_requested_func,
            UnitParsedFunctionPointer unit_parsed_func
        );

        /** Decrease the ref counter of the event handler */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_dec_ref_event_handler(
            EventHandlerNative eventHandler
        );

        // ----- Token functions -----

        /**
         * Kind for this token.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_token_get_kind(
            TokenNative token
        );

        /**
         * Return the source location range of the given token.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_token_sloc_range(
            TokenNative token,
            SourceLocationRangeNative result
        );

        /** Get the next token */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_token_next(
            TokenNative token,
            TokenNative res
        );

        /** Get the previous token */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_token_previous(
            TokenNative token,
            TokenNative res
        );

        /** Get if two tokens are equivalent */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native byte ada_token_is_equivalent(
            TokenNative left,
            TokenNative right
        );

        /** Get the text in a token range */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_token_range_text(
            TokenNative start,
            TokenNative end,
            TextNative res
        );

        // ----- Analysis context functions -----

        /** Allocate a new analysis context */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisContextNative
        ada_allocate_analysis_context();

        /** Create a new analysis context */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void
        ada_initialize_analysis_context(
            AnalysisContextNative context,
            CCharPointer charset,
            FileReaderNative file_reader,
            UnitProviderNative unit_provider,
            EventHandlerNative event_handler,
            int with_trivia,
            int tab_stop
        );

        /** Increase the reference counter of a context */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_context_incref(
            AnalysisContextNative context
        );

        /** Decrease the reference counter of a context */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_context_decref(
            AnalysisContextNative context
        );

        // ----- Analysis unit functions -----

        /** Get a unit from a file */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisUnitNative
        ada_get_analysis_unit_from_file(
            AnalysisContextNative context,
            CCharPointer file_name,
            CCharPointer charset,
            int reparse,
            int rule
        );

        /** Get a unit from a buffer */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisUnitNative
        ada_get_analysis_unit_from_buffer(
            AnalysisContextNative context,
            CCharPointer file_name,
            CCharPointer charset,
            CCharPointer buffer,
            long buffer_size,
            int rule
        );

        /** Get a unit from the unit provider. */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisUnitNative
        ada_get_analysis_unit_from_provider(
            AnalysisContextNative context,
            TextNative name,
            int kind,
            CCharPointer charset,
            int reparse
        );

        /** Get the root of an analysis unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_unit_root(
            AnalysisUnitNative unit,
            EntityNative res
        );

        /** Get the file name for a given unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native CCharPointer ada_unit_filename(
            AnalysisUnitNative unit
        );

        /** Get the token count of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_unit_token_count(
            AnalysisUnitNative unit
        );

        /** Get the trivia count of the analysis unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_unit_trivia_count(
            AnalysisUnitNative unit
        );

        /** Get the first token of an analysis unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_unit_first_token(
            AnalysisUnitNative unit,
            TokenNative res
        );

        /** Get the last token of an analysis unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_unit_last_token (
            AnalysisUnitNative unit,
            TokenNative res
        );

        /** Get the context for a given unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisContextNative ada_unit_context(
            AnalysisUnitNative unit
        );

        /** Get the diagnostic count of the unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_unit_diagnostic_count(
            AnalysisUnitNative unit
        );

        /** Get the nth diagnostic for the unit */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_unit_diagnostic(
            AnalysisUnitNative unit,
            int n,
            DiagnosticNative diagnostic
        );

        // ----- Array functions -----

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native DiscriminantValuesArrayNative ada_discriminant_values_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_discriminant_values_array_dec_ref(DiscriminantValuesArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native DocAnnotationArrayNative ada_doc_annotation_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_doc_annotation_array_dec_ref(DocAnnotationArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AdaNodeArrayNative ada_ada_node_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_ada_node_array_dec_ref(AdaNodeArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native ParamActualArrayNative ada_param_actual_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_param_actual_array_dec_ref(ParamActualArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native RefResultArrayNative ada_ref_result_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_ref_result_array_dec_ref(RefResultArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native ShapeArrayNative ada_shape_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_shape_array_dec_ref(ShapeArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native SubstitutionArrayNative ada_substitution_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_substitution_array_dec_ref(SubstitutionArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisUnitArrayNative ada_analysis_unit_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_analysis_unit_array_dec_ref(AnalysisUnitArrayNative array);

        
    

        /**
         * Create a new sized array.
         *
         * @param size The size of the array to create.
         * @return The native pointer to the created array.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native UnboundedTextTypeArrayNative ada_unbounded_text_type_array_create(int size);

        /**
         * Decrease reference counter of the given array
         *
         * @param array The array to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_unbounded_text_type_array_dec_ref(UnboundedTextTypeArrayNative array);


        // ----- Structure functions -----

            
        
    


            
        
    


        
    


            
        
    


            
        
    


            
            
        
    


            
        
    


            
        
    

        /**
         * Decreate the reference counter of the given struct.
         *
         * @param structNative The structure to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_internal_doc_annotation_dec_ref(
            DocAnnotationNative structNative
        );

            
            
            
            
            
            
            
        
    


            
            
            
        
    


            
        
    


            
        
    


            
        
    

        /**
         * Decreate the reference counter of the given struct.
         *
         * @param structNative The structure to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_internal_shape_dec_ref(
            ShapeNative structNative
        );

            
        
    

        /**
         * Decreate the reference counter of the given struct.
         *
         * @param structNative The structure to decrease the reference counter.
         */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_internal_substitution_dec_ref(
            SubstitutionNative structNative
        );


        // ----- Node functions -----

        /** Return whether the two given entities are equal */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_node_is_equivalent(
            EntityNative entity_left,
            EntityNative entity_right
        );

        /** Get the hash of a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_node_hash(
            EntityNative entity
        );

        /** Get the type of a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_node_kind(
            EntityNative entity
        );

        /** Get the text from a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_node_text(
            EntityNative entity,
            TextNative text
        );

        /** Get the source location range for a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_node_sloc_range(
            EntityNative entity,
            SourceLocationRangeNative slocr
        );

        /** Get the number of children for a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_node_children_count(
            EntityNative entity
        );

        /** Get the nth child for the node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_node_child(
            EntityNative entity,
            int n,
            EntityNative res
        );

        /** Get if the node is a token node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_node_is_token_node(
            EntityNative entity
        );

        /** Get the unit of the node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native AnalysisUnitNative ada_node_unit(
            EntityNative entity
        );

        /** Get the image of a node */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native void ada_node_image(
            EntityNative entity,
            TextNative text
        );

        // ----- Node fields accessors and properties -----

        
    

            

        /** Isomethod of ada_ada_node_p_declarative_scope langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_declarative_scope(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_enclosing_compilation_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_enclosing_compilation_unit(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_get_uninstantiated_node langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_get_uninstantiated_node(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_complete langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_complete(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_p_valid_keywords langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_valid_keywords(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_p_generic_instantiations langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_generic_instantiations(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_p_semantic_parent langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_semantic_parent(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_parent_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_parent_basic_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_filter_is_imported_by langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_filter_is_imported_by(
            EntityNative node,
            AnalysisUnitArrayNative units,
            byte transitive,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_p_xref_entry_point langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_xref_entry_point(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_ada_node_p_resolve_names langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_resolve_names(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_ada_node_p_standard_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_standard_unit(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_p_std_entity langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_std_entity(
            EntityNative node,
            SymbolNative sym,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_bool_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_bool_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_int_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_universal_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_universal_int_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_universal_real_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_universal_real_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_std_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_std_char_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_std_wide_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_std_wide_char_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_std_wide_wide_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_std_wide_wide_char_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_top_level_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_top_level_decl(
            EntityNative node,
            AnalysisUnitNative unit,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_p_choice_match langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_choice_match(
            EntityNative node,
            BigIntegerNative value,
            CCharPointer result
        );
            

        /** Isomethod of ada_ada_node_p_gnat_xref langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_p_gnat_xref(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_parent langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_parent(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_parents langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_parents(
            EntityNative node,
            byte with_self,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_children langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_children(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_token_start langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_token_start(
            EntityNative node,
            TokenNative result
        );
            

        /** Isomethod of ada_ada_node_token_end langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_token_end(
            EntityNative node,
            TokenNative result
        );
            

        /** Isomethod of ada_ada_node_child_index langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_child_index(
            EntityNative node,
            CIntPointer result
        );
            

        /** Isomethod of ada_ada_node_previous_sibling langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_previous_sibling(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_next_sibling langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_next_sibling(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ada_node_unit langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_unit(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_ada_node_is_ghost langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_is_ghost(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_ada_node_full_sloc_image langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ada_node_full_sloc_image(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_abort_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_abort_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_abstract_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_abstract_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_assoc_list_p_zip_with_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_assoc_list_p_zip_with_params(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_aliased_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aliased_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_all_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_all_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_constrained_array_indices_f_list langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_constrained_array_indices_f_list(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_unconstrained_array_indices_f_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_unconstrained_array_indices_f_types(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_aspect_assoc_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aspect_assoc_f_id(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_aspect_assoc_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aspect_assoc_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_aspect_assoc_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aspect_assoc_p_is_ghost_code(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    

            

        /** Isomethod of ada_at_clause_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_at_clause_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_at_clause_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_at_clause_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_attribute_def_clause_f_attribute_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_attribute_def_clause_f_attribute_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_attribute_def_clause_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_attribute_def_clause_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_enum_rep_clause_f_type_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_rep_clause_f_type_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_enum_rep_clause_f_aggregate langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_rep_clause_f_aggregate(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_enum_rep_clause_p_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_rep_clause_p_params(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_record_rep_clause_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_rep_clause_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_record_rep_clause_f_at_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_rep_clause_f_at_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_record_rep_clause_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_rep_clause_f_components(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_aspect_spec_f_aspect_assocs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aspect_spec_f_aspect_assocs(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_assoc_p_assoc_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_assoc_p_assoc_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_contract_case_assoc_f_guard langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_contract_case_assoc_f_guard(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_contract_case_assoc_f_consequence langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_contract_case_assoc_f_consequence(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_pragma_argument_assoc_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_argument_assoc_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_pragma_argument_assoc_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_argument_assoc_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_formal_param_holder_p_abstract_formal_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_holder_p_abstract_formal_params(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_formal_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_holder_p_formal_params(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_nb_min_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_holder_p_nb_min_params(
            EntityNative node,
            CIntPointer result
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_nb_max_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_holder_p_nb_max_params(
            EntityNative node,
            CIntPointer result
        );
            

        /** Isomethod of ada_base_formal_param_holder_p_param_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_holder_p_param_types(
            EntityNative node,
            EntityNative origin,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_base_subp_spec_p_returns langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_returns(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_subp_spec_p_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_params(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_primitive_subp_types(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_first_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_primitive_subp_first_type(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_base_subp_spec_p_primitive_subp_tagged_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_primitive_subp_tagged_type(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_base_subp_spec_p_return_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_spec_p_return_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_entry_spec_f_entry_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_spec_f_entry_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_spec_f_family_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_spec_f_family_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_spec_f_entry_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_spec_f_entry_params(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_subp_spec_f_subp_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_spec_f_subp_kind(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_spec_f_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_spec_f_subp_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_spec_f_subp_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_spec_f_subp_params(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_spec_f_subp_returns langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_spec_f_subp_returns(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_synthetic_binary_spec_f_left_param langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_binary_spec_f_left_param(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_synthetic_binary_spec_f_right_param langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_binary_spec_f_right_param(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_synthetic_binary_spec_f_return_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_binary_spec_f_return_type_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_synthetic_unary_spec_f_right_param langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_unary_spec_f_right_param(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_synthetic_unary_spec_f_return_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_unary_spec_f_return_type_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_component_list_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_list_f_components(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_list_f_variant_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_list_f_variant_part(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_known_discriminant_part_f_discr_specs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_known_discriminant_part_f_discr_specs(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_entry_completion_formal_params_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_completion_formal_params_f_params(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_formal_part_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_formal_part_f_decls(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_record_def_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_record_def_f_components(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_basic_assoc_p_get_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_assoc_p_get_params(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_aggregate_assoc_f_designators langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aggregate_assoc_f_designators(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_aggregate_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_aggregate_assoc_f_r_expr(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_composite_constraint_assoc_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_composite_constraint_assoc_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_composite_constraint_assoc_f_constraint_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_composite_constraint_assoc_f_constraint_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_iterated_assoc_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_iterated_assoc_f_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_iterated_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_iterated_assoc_f_r_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_param_assoc_f_designator langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_assoc_f_designator(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_param_assoc_f_r_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_assoc_f_r_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_basic_decl_p_is_formal langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_formal(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_doc_annotations langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_doc_annotations(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_doc langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_doc(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_previous_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_previous_part_for_decl(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_canonical_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_canonical_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_all_parts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_all_parts(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_is_static_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_static_decl(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_f_aspects langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_f_aspects(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect_assoc langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_aspect_assoc(
            EntityNative node,
            SymbolNative name,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect_spec_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_aspect_spec_expr(
            EntityNative node,
            SymbolNative name,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_get_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_aspect(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            AspectNative result
        );
            

        /** Isomethod of ada_basic_decl_p_has_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_has_aspect(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_get_pragma langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_pragma(
            EntityNative node,
            SymbolNative name,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_get_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_representation_clause(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_get_at_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_get_at_clause(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_is_imported langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_imported(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_ghost_code(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_is_compilation_unit_root langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_compilation_unit_root(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_is_visible langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_visible(
            EntityNative node,
            EntityNative from_node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_base_subp_declarations langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_base_subp_declarations(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_root_subp_declarations langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_root_subp_declarations(
            EntityNative node,
            EntityNative origin,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_find_all_overrides langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_find_all_overrides(
            EntityNative node,
            AnalysisUnitArrayNative units,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_defining_names langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_defining_names(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_defining_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_type_expression langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_type_expression(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_subp_spec_or_null langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_subp_spec_or_null(
            EntityNative node,
            byte follow_generic,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_is_subprogram langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_subprogram(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_decl_p_relative_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_relative_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_relative_name_text langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_relative_name_text(
            EntityNative node,
            SymbolNative result
        );
            

        /** Isomethod of ada_basic_decl_p_next_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_next_part_for_decl(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_body_part_for_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_body_part_for_decl(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_most_visible_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_most_visible_part(
            EntityNative node,
            EntityNative origin,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_basic_decl_p_fully_qualified_name_array langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_fully_qualified_name_array(
            EntityNative node,
            byte include_profile,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_fully_qualified_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_canonical_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_canonical_fully_qualified_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_unique_identifying_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_unique_identifying_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_basic_decl_p_is_constant_object langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_decl_p_is_constant_object(
            EntityNative node,
            CCharPointer result
        );

        
    

            

        /** Isomethod of ada_abstract_state_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_abstract_state_decl_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_anonymous_expr_decl_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_anonymous_expr_decl_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_anonymous_expr_decl_p_get_formal langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_anonymous_expr_decl_p_get_formal(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_formal_param_decl_p_formal_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_formal_param_decl_p_formal_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_component_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_decl_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_decl_f_component_def langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_decl_f_component_def(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_decl_f_default_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_discriminant_spec_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_discriminant_spec_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_discriminant_spec_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_discriminant_spec_f_type_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_discriminant_spec_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_discriminant_spec_f_default_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_formal_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_formal_f_decl(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_param_spec_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_spec_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_param_spec_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_spec_f_has_aliased(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_param_spec_f_mode langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_spec_f_mode(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_param_spec_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_spec_f_type_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_param_spec_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_param_spec_f_default_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_synthetic_formal_param_decl_f_param_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_formal_param_decl_f_param_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_package_decl_f_package_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_package_decl_f_package_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_package_decl_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_package_decl_f_public_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_package_decl_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_package_decl_f_private_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_package_decl_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_package_decl_f_end_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_package_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_package_decl_p_body_part(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_base_type_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_base_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_base_subtype(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_private_completion langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_private_completion(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_inherited_primitive langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_inherited_primitive(
            EntityNative node,
            EntityNative p,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_get_record_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_get_record_representation_clause(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_get_enum_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_get_enum_representation_clause(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_record_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_record_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_array_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_array_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_find_derived_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_find_derived_types(
            EntityNative node,
            EntityNative root,
            EntityNative origin,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_real_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_real_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_float_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_float_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_fixed_point langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_fixed_point(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_scalar_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_scalar_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_enum_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_enum_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_access_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_access_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_specific_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_specific_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_char_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_char_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_classwide_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_classwide_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_discrete_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_discrete_range(
            EntityNative node,
            DiscreteRangeNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_discrete_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_discrete_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_int_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_int_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_accessed_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_accessed_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_tagged_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_tagged_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_base_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_base_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_base_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_base_types(
            EntityNative node,
            EntityNative origin,
            Pointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_find_all_derived_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_find_all_derived_types(
            EntityNative node,
            AnalysisUnitArrayNative units,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_comp_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_comp_type(
            EntityNative node,
            byte is_subscript,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_index_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_index_type(
            EntityNative node,
            int dim,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_derived_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_derived_type(
            EntityNative node,
            EntityNative other_type,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_interface_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_interface_type(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_matching_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_matching_type(
            EntityNative node,
            EntityNative expected_type,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_canonical_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_canonical_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_previous_part(
            EntityNative node,
            byte go_to_incomplete,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_next_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_next_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_full_view langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_full_view(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_definite_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_definite_subtype(
            EntityNative node,
            EntityNative origin,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_is_private langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_is_private(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_discriminants_list langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_discriminants_list(
            EntityNative node,
            EntityNative stop_recurse_at,
            EntityNative origin,
            Pointer result
        );
            

        /** Isomethod of ada_base_type_decl_p_root_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_root_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );
            

        /** Isomethod of ada_base_type_decl_p_shapes langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_type_decl_p_shapes(
            EntityNative node,
            byte include_discriminants,
            EntityNative origin,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_base_subtype_decl_p_get_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subtype_decl_p_get_type(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_subtype_decl_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_decl_f_subtype(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_incomplete_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_incomplete_type_decl_f_discriminants(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_incomplete_formal_type_decl_f_is_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_incomplete_formal_type_decl_f_is_tagged(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_incomplete_formal_type_decl_f_default_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_incomplete_formal_type_decl_f_default_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_incomplete_tagged_type_decl_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_incomplete_tagged_type_decl_f_has_abstract(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_protected_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_type_decl_f_discriminants(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_type_decl_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_type_decl_f_interfaces(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_type_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_type_decl_f_definition(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_task_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_type_decl_f_discriminants(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_type_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_type_decl_f_definition(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_type_decl_f_discriminants langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_decl_f_discriminants(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_decl_f_type_def langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_decl_f_type_def(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_decl_p_get_primitives langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_decl_p_get_primitives(
            EntityNative node,
            byte only_inherited,
            byte include_predefined_operators,
            Pointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_formal_type_decl_f_default_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_formal_type_decl_f_default_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_basic_subp_decl_p_has_spark_mode_on langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_subp_decl_p_has_spark_mode_on(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_basic_subp_decl_p_subp_decl_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_basic_subp_decl_p_subp_decl_spec(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_classic_subp_decl_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_classic_subp_decl_f_overriding(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_classic_subp_decl_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_classic_subp_decl_f_subp_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_classic_subp_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_classic_subp_decl_p_body_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_formal_subp_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_formal_subp_decl_f_default_expr(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_entry_decl_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_decl_f_overriding(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_decl_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_decl_f_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_decl_p_body_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_decl_p_accept_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_decl_p_accept_stmts(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_enum_literal_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_literal_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_enum_literal_decl_p_enum_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_literal_decl_p_enum_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_enum_literal_decl_p_enum_rep langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_literal_decl_p_enum_rep(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_synthetic_char_enum_lit_p_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_char_enum_lit_p_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_subp_internal_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_internal_f_subp_spec(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_synthetic_subp_decl_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_subp_decl_f_spec(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_body_node_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_body_node_p_previous_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_body_node_p_decl_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_body_node_p_decl_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_body_node_p_subunit_root langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_body_node_p_subunit_root(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_accept_stmt_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_body_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_subp_body_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_body_f_overriding(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_subp_body_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_body_f_subp_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_subp_body_p_is_subject_to_proof langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_body_p_is_subject_to_proof(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_base_subp_body_p_has_spark_mode_on langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_subp_body_p_has_spark_mode_on(
            EntityNative node,
            CCharPointer result
        );

        
    

            

        /** Isomethod of ada_expr_function_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_function_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_subp_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_body_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_body_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_body_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_subp_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_renaming_decl_f_renames(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_body_stub_p_syntactic_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_body_stub_p_syntactic_fully_qualified_name(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_package_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_body_stub_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_protected_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_body_stub_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_subp_body_stub_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_body_stub_f_overriding(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subp_body_stub_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subp_body_stub_f_subp_spec(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_task_body_stub_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_body_stub_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_entry_body_f_entry_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_entry_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_index_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_index_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_params(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_barrier langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_barrier(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_body_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_package_body_f_package_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_body_f_package_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_body_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_body_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_body_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_protected_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_body_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_body_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_body_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_task_body_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_body_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_body_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_body_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_body_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_body_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_body_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_body_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_entry_index_spec_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_index_spec_f_id(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_entry_index_spec_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_entry_index_spec_f_subtype(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_exception_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exception_decl_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_exception_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exception_decl_f_renames(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_exception_handler_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exception_handler_f_exception_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_exception_handler_f_handled_exceptions langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exception_handler_f_handled_exceptions(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_exception_handler_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exception_handler_f_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_for_loop_var_decl_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_var_decl_f_id(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_for_loop_var_decl_f_id_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_var_decl_f_id_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_decl_f_formal_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_decl_f_formal_part(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_package_decl_f_package_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_decl_f_package_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_package_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_decl_p_body_part(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_subp_decl_f_subp_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_decl_f_subp_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_decl_p_body_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_decl_p_body_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_instantiation_p_designated_generic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_instantiation_p_designated_generic_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_instantiation_p_inst_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_instantiation_p_inst_params(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_generic_package_instantiation_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_instantiation_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_package_instantiation_f_generic_pkg_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_instantiation_f_generic_pkg_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_package_instantiation_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_instantiation_f_params(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_subp_instantiation_f_overriding langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_f_overriding(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_f_kind(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_f_subp_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_generic_subp_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_f_generic_subp_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_instantiation_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_f_params(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_instantiation_p_designated_subp langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_instantiation_p_designated_subp(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_generic_package_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_renaming_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_package_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_package_renaming_decl_f_renames(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_generic_subp_renaming_decl_f_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_renaming_decl_f_kind(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_renaming_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_generic_subp_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_generic_subp_renaming_decl_f_renames(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_label_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_label_decl_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_named_stmt_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_named_stmt_decl_f_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_number_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_number_decl_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_number_decl_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_number_decl_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_object_decl_f_ids langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_ids(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_has_aliased(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_has_constant(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_mode langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_mode(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_type_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_default_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_default_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_f_renaming_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_f_renaming_clause(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_p_private_part_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_p_private_part_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_object_decl_p_public_part_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_object_decl_p_public_part_decl(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_package_renaming_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_renaming_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_renaming_decl_f_renames langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_renaming_decl_f_renames(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_renaming_decl_p_renamed_package langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_renaming_decl_p_renamed_package(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_package_renaming_decl_p_final_renamed_package langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_package_renaming_decl_p_final_renamed_package(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_single_protected_decl_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_single_protected_decl_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_single_protected_decl_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_single_protected_decl_f_interfaces(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_single_protected_decl_f_definition langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_single_protected_decl_f_definition(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_single_task_decl_f_task_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_single_task_decl_f_task_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_case_stmt_alternative_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_stmt_alternative_f_choices(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_case_stmt_alternative_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_stmt_alternative_f_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_compilation_unit_f_prelude langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_f_prelude(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_compilation_unit_f_body langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_f_body(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_compilation_unit_f_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_f_pragmas(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_compilation_unit_p_syntactic_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_syntactic_fully_qualified_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_unit_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_unit_kind(
            EntityNative node,
            CIntPointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_withed_units langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_withed_units(
            EntityNative node,
            byte include_privates,
            Pointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_imported_units langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_imported_units(
            EntityNative node,
            byte include_privates,
            Pointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_unit_dependencies langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_unit_dependencies(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_compilation_unit_p_is_preelaborable langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_is_preelaborable(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_other_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_other_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_compilation_unit_p_has_restriction langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_has_restriction(
            EntityNative node,
            SymbolNative name,
            CCharPointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_all_config_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_all_config_pragmas(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_compilation_unit_p_config_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_compilation_unit_p_config_pragmas(
            EntityNative node,
            SymbolNative name,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_component_clause_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_clause_f_id(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_clause_f_position langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_clause_f_position(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_clause_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_clause_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_component_def_f_has_aliased langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_def_f_has_aliased(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_def_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_def_f_has_constant(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_component_def_f_type_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_component_def_f_type_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_constant_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_constant_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_composite_constraint_f_constraints langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_composite_constraint_f_constraints(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_composite_constraint_p_is_index_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_composite_constraint_p_is_index_constraint(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_composite_constraint_p_is_discriminant_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_composite_constraint_p_is_discriminant_constraint(
            EntityNative node,
            CCharPointer result
        );

        
    

            

        /** Isomethod of ada_delta_constraint_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_delta_constraint_f_digits(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_delta_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_delta_constraint_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_digits_constraint_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_digits_constraint_f_digits(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_digits_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_digits_constraint_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_range_constraint_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_range_constraint_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_declarative_part_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_declarative_part_f_decls(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_elsif_expr_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_elsif_expr_part_f_cond_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_elsif_expr_part_f_then_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_elsif_expr_part_f_then_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_elsif_stmt_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_elsif_stmt_part_f_cond_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_elsif_stmt_part_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_elsif_stmt_part_f_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_expr_p_expression_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_expression_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_expr_p_expected_expression_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_expected_expression_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_expr_p_is_dynamically_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_is_dynamically_tagged(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_expr_p_is_dispatching_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_is_dispatching_call(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_expr_p_is_static_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_is_static_expr(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_expr_p_first_corresponding_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_first_corresponding_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_expr_p_eval_as_int langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_eval_as_int(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_expr_p_eval_as_int_in_env langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_eval_as_int_in_env(
            EntityNative node,
            SubstitutionArrayNative env,
            Pointer result
        );
            

        /** Isomethod of ada_expr_p_eval_as_string langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_eval_as_string(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_expr_p_eval_as_string_in_env langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_eval_as_string_in_env(
            EntityNative node,
            SubstitutionArrayNative env,
            Pointer result
        );
            

        /** Isomethod of ada_expr_p_matching_nodes langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_expr_p_matching_nodes(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_abstract_state_decl_expr_f_state_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_abstract_state_decl_expr_f_state_decl(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_allocator_f_subpool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_allocator_f_subpool(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_allocator_f_type_or_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_allocator_f_type_or_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_allocator_p_get_allocated_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_allocator_p_get_allocated_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_aggregate_f_ancestor_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_aggregate_f_ancestor_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_aggregate_f_assocs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_aggregate_f_assocs(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_aggregate_p_aggregate_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_aggregate_p_aggregate_params(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_base_aggregate_p_is_subaggregate langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_aggregate_p_is_subaggregate(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_bin_op_f_left langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_bin_op_f_left(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_bin_op_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_bin_op_f_op(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_bin_op_f_right langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_bin_op_f_right(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_case_expr_alternative_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_expr_alternative_f_choices(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_case_expr_alternative_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_expr_alternative_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_concat_op_f_first_operand langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_concat_op_f_first_operand(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_concat_op_f_other_operands langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_concat_op_f_other_operands(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_concat_op_p_operands langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_concat_op_p_operands(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_concat_operand_f_operator langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_concat_operand_f_operator(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_concat_operand_f_operand langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_concat_operand_f_operand(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_cond_expr_p_dependent_exprs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_cond_expr_p_dependent_exprs(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_case_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_expr_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_case_expr_f_cases langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_expr_f_cases(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_if_expr_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_expr_f_cond_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_expr_f_then_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_expr_f_then_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_expr_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_expr_f_alternatives(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_expr_f_else_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_expr_f_else_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_contract_cases_f_contract_cases langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_contract_cases_f_contract_cases(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_decl_expr_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decl_expr_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_decl_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decl_expr_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_membership_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_membership_expr_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_membership_expr_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_membership_expr_f_op(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_membership_expr_f_membership_exprs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_membership_expr_f_membership_exprs(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_name_p_enclosing_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_enclosing_defining_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_is_defining langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_defining(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_name_is langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_name_is(
            EntityNative node,
            SymbolNative sym,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_direct_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_direct_call(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_access_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_access_call(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_call(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_dot_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_dot_call(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_failsafe_referenced_def_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_failsafe_referenced_def_name(
            EntityNative node,
            byte imprecise_fallback,
            RefdDefNative result
        );
            

        /** Isomethod of ada_name_p_referenced_defining_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_referenced_defining_name(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_all_env_elements langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_all_env_elements(
            EntityNative node,
            byte seq,
            EntityNative seq_from,
            Pointer result
        );
            

        /** Isomethod of ada_name_p_called_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_called_subp_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_referenced_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_referenced_decl(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_failsafe_referenced_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_failsafe_referenced_decl(
            EntityNative node,
            byte imprecise_fallback,
            RefdDeclNative result
        );
            

        /** Isomethod of ada_name_p_name_designated_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_name_designated_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_is_static_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_static_subtype(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_name_matches langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_name_matches(
            EntityNative node,
            EntityNative n,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_relative_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_relative_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_name_p_is_operator_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_operator_name(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_write_reference langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_write_reference(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_is_static_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_static_call(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_as_symbol_array langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_as_symbol_array(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_name_p_canonical_text langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_canonical_text(
            EntityNative node,
            SymbolNative result
        );
            

        /** Isomethod of ada_name_p_is_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_is_constant(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_name_p_call_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_name_p_call_params(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_attribute_ref_f_prefix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_attribute_ref_f_attribute(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_attribute_ref_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_attribute_ref_f_args(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_call_expr_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_call_expr_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_call_expr_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_call_expr_f_suffix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_call_expr_p_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_call_expr_p_kind(
            EntityNative node,
            CIntPointer result
        );
            

        /** Isomethod of ada_call_expr_p_is_array_slice langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_call_expr_p_is_array_slice(
            EntityNative node,
            CCharPointer result
        );

        
    

            

        /** Isomethod of ada_defining_name_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_canonical_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_canonical_fully_qualified_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_unique_identifying_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_unique_identifying_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_fully_qualified_name_array langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_fully_qualified_name_array(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_fully_qualified_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_fully_qualified_name(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_basic_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_find_refs langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_find_refs(
            EntityNative node,
            EntityNative root,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_find_all_references langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_find_all_references(
            EntityNative node,
            AnalysisUnitArrayNative units,
            byte follow_renamings,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_find_all_calls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_find_all_calls(
            EntityNative node,
            AnalysisUnitArrayNative units,
            byte follow_renamings,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_next_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_next_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_previous_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_previous_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_canonical_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_canonical_part(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_most_visible_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_most_visible_part(
            EntityNative node,
            EntityNative origin,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_all_parts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_all_parts(
            EntityNative node,
            byte imprecise_fallback,
            Pointer result
        );
            

        /** Isomethod of ada_defining_name_p_get_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_get_aspect(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            AspectNative result
        );
            

        /** Isomethod of ada_defining_name_p_has_aspect langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_has_aspect(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            CCharPointer result
        );
            

        /** Isomethod of ada_defining_name_p_get_pragma langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_get_pragma(
            EntityNative node,
            SymbolNative name,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_get_representation_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_get_representation_clause(
            EntityNative node,
            SymbolNative name,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_get_at_clause langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_get_at_clause(
            EntityNative node,
            byte imprecise_fallback,
            EntityNative result
        );
            

        /** Isomethod of ada_defining_name_p_is_imported langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_is_imported(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_defining_name_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_defining_name_p_is_ghost_code(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    

            

        /** Isomethod of ada_discrete_subtype_name_f_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_discrete_subtype_name_f_subtype(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_dotted_name_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_dotted_name_f_prefix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_dotted_name_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_dotted_name_f_suffix(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_end_name_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_end_name_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_end_name_p_basic_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_end_name_p_basic_decl(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_explicit_deref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_explicit_deref_f_prefix(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_qual_expr_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_qual_expr_f_prefix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_qual_expr_f_suffix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_qual_expr_f_suffix(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_reduce_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_reduce_attribute_ref_f_prefix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_reduce_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_reduce_attribute_ref_f_attribute(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_reduce_attribute_ref_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_reduce_attribute_ref_f_args(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_char_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_char_literal_p_denoted_value(
            EntityNative node,
            CIntPointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_string_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_string_literal_p_denoted_value(
            EntityNative node,
            Pointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_int_literal_p_denoted_value langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_int_literal_p_denoted_value(
            EntityNative node,
            Pointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_update_attribute_ref_f_prefix langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_update_attribute_ref_f_prefix(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_update_attribute_ref_f_attribute langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_update_attribute_ref_f_attribute(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_update_attribute_ref_f_values langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_update_attribute_ref_f_values(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_paren_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_paren_expr_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_quantified_expr_f_quantifier langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_quantified_expr_f_quantifier(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_quantified_expr_f_loop_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_quantified_expr_f_loop_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_quantified_expr_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_quantified_expr_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_raise_expr_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_raise_expr_f_exception_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_raise_expr_f_error_message langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_raise_expr_f_error_message(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_un_op_f_op langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_un_op_f_op(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_un_op_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_un_op_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_handled_stmts_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_handled_stmts_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_handled_stmts_f_exceptions langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_handled_stmts_f_exceptions(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_library_item_f_has_private langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_library_item_f_has_private(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_library_item_f_item langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_library_item_f_item(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_limited_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_limited_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_for_loop_spec_f_var_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_spec_f_var_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_for_loop_spec_f_loop_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_spec_f_loop_type(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_for_loop_spec_f_has_reverse langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_spec_f_has_reverse(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_for_loop_spec_f_iter_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_spec_f_iter_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_for_loop_spec_f_iter_filter langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_for_loop_spec_f_iter_filter(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_while_loop_spec_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_while_loop_spec_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_multi_abstract_state_decl_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_multi_abstract_state_decl_f_decls(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_not_null_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_not_null_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_params_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_params_f_params(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_paren_abstract_state_decl_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_paren_abstract_state_decl_f_decl(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_pp_elsif_directive_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pp_elsif_directive_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_pp_elsif_directive_f_then_kw langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pp_elsif_directive_f_then_kw(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_pp_if_directive_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pp_if_directive_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_pp_if_directive_f_then_kw langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pp_if_directive_f_then_kw(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_pragma_node_f_id langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_node_f_id(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_pragma_node_f_args langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_node_f_args(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_pragma_node_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_node_p_is_ghost_code(
            EntityNative node,
            CCharPointer result
        );
            

        /** Isomethod of ada_pragma_node_p_associated_entities langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_pragma_node_p_associated_entities(
            EntityNative node,
            Pointer result
        );

        
    

            

        /** Isomethod of ada_private_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_private_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_protected_def_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_def_f_public_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_def_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_def_f_private_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_protected_def_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_def_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_protected_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_protected_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_range_spec_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_range_spec_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_renaming_clause_f_renamed_object langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_renaming_clause_f_renamed_object(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_reverse_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_reverse_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_select_when_part_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_select_when_part_f_cond_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_select_when_part_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_select_when_part_f_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_stmt_p_is_ghost_code langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_stmt_p_is_ghost_code(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    

            

        /** Isomethod of ada_accept_stmt_f_body_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_f_body_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_accept_stmt_f_entry_index_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_f_entry_index_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_accept_stmt_f_params langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_f_params(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_accept_stmt_p_corresponding_entry langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_p_corresponding_entry(
            EntityNative node,
            EntityNative origin,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_accept_stmt_with_stmts_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_with_stmts_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_accept_stmt_with_stmts_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_accept_stmt_with_stmts_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_base_loop_stmt_f_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_loop_stmt_f_spec(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_loop_stmt_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_loop_stmt_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_base_loop_stmt_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_base_loop_stmt_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_begin_block_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_begin_block_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_begin_block_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_begin_block_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_decl_block_f_decls langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decl_block_f_decls(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_decl_block_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decl_block_f_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_decl_block_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decl_block_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_case_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_stmt_f_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_case_stmt_f_pragmas langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_stmt_f_pragmas(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_case_stmt_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_case_stmt_f_alternatives(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_extended_return_stmt_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_extended_return_stmt_f_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_extended_return_stmt_f_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_extended_return_stmt_f_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_if_stmt_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_stmt_f_cond_expr(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_stmt_f_then_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_stmt_f_then_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_stmt_f_alternatives langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_stmt_f_alternatives(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_if_stmt_f_else_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_if_stmt_f_else_stmts(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_named_stmt_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_named_stmt_f_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_named_stmt_f_stmt langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_named_stmt_f_stmt(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_select_stmt_f_guards langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_select_stmt_f_guards(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_select_stmt_f_else_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_select_stmt_f_else_stmts(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_select_stmt_f_abort_stmts langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_select_stmt_f_abort_stmts(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_abort_stmt_f_names langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_abort_stmt_f_names(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_assign_stmt_f_dest langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_assign_stmt_f_dest(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_assign_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_assign_stmt_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_call_stmt_f_call langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_call_stmt_f_call(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_delay_stmt_f_has_until langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_delay_stmt_f_has_until(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_delay_stmt_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_delay_stmt_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_exit_stmt_f_loop_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exit_stmt_f_loop_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_exit_stmt_f_cond_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_exit_stmt_f_cond_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_goto_stmt_f_label_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_goto_stmt_f_label_name(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_label_f_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_label_f_decl(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_raise_stmt_f_exception_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_raise_stmt_f_exception_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_raise_stmt_f_error_message langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_raise_stmt_f_error_message(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_requeue_stmt_f_call_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_requeue_stmt_f_call_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_requeue_stmt_f_has_abort langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_requeue_stmt_f_has_abort(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_return_stmt_f_return_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_return_stmt_f_return_expr(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_subunit_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subunit_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subunit_f_body langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subunit_f_body(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subunit_p_body_root langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subunit_p_body_root(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_synchronized_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synchronized_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_tagged_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_tagged_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_task_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_def_f_interfaces(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_def_f_public_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_def_f_public_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_def_f_private_part langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_def_f_private_part(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_task_def_f_end_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_task_def_f_end_name(
            EntityNative node,
            EntityNative result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_access_def_f_has_not_null langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_access_def_f_has_not_null(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_access_to_subp_def_f_has_protected langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_access_to_subp_def_f_has_protected(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_access_to_subp_def_f_subp_spec langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_access_to_subp_def_f_subp_spec(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_anonymous_type_access_def_f_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_anonymous_type_access_def_f_type_decl(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_type_access_def_f_has_all langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_access_def_f_has_all(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_access_def_f_has_constant langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_access_def_f_has_constant(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_access_def_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_access_def_f_subtype_indication(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_array_type_def_f_indices langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_array_type_def_f_indices(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_array_type_def_f_component_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_array_type_def_f_component_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_derived_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_has_abstract(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_has_limited(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_has_synchronized langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_has_synchronized(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_subtype_indication(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_interfaces(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_record_extension langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_record_extension(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_derived_type_def_f_has_with_private langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_derived_type_def_f_has_with_private(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_enum_type_def_f_enum_literals langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_enum_type_def_f_enum_literals(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_interface_type_def_f_interface_kind langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_interface_type_def_f_interface_kind(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_interface_type_def_f_interfaces langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_interface_type_def_f_interfaces(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_mod_int_type_def_f_expr langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_mod_int_type_def_f_expr(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_private_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_private_type_def_f_has_abstract(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_private_type_def_f_has_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_private_type_def_f_has_tagged(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_private_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_private_type_def_f_has_limited(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_decimal_fixed_point_def_f_delta langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decimal_fixed_point_def_f_delta(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_decimal_fixed_point_def_f_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decimal_fixed_point_def_f_digits(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_decimal_fixed_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_decimal_fixed_point_def_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_floating_point_def_f_num_digits langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_floating_point_def_f_num_digits(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_floating_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_floating_point_def_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_ordinary_fixed_point_def_f_delta langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ordinary_fixed_point_def_f_delta(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_ordinary_fixed_point_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_ordinary_fixed_point_def_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_record_type_def_f_has_abstract langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_type_def_f_has_abstract(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_record_type_def_f_has_tagged langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_type_def_f_has_tagged(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_record_type_def_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_type_def_f_has_limited(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_record_type_def_f_record_def langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_record_type_def_f_record_def(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_signed_int_type_def_f_range langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_signed_int_type_def_f_range(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_type_expr_p_type_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_expr_p_type_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_expr_p_designated_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_expr_p_designated_type_decl(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_type_expr_p_designated_type_decl_from langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_type_expr_p_designated_type_decl_from(
            EntityNative node,
            EntityNative origin_node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_anonymous_type_f_type_decl langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_anonymous_type_f_type_decl(
            EntityNative node,
            EntityNative result
        );

        
    


        
    

            

        /** Isomethod of ada_subtype_indication_f_has_not_null langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_indication_f_has_not_null(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subtype_indication_f_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_indication_f_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subtype_indication_f_constraint langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_indication_f_constraint(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_subtype_indication_p_subtype_constraints langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_indication_p_subtype_constraints(
            EntityNative node,
            Pointer result
        );
            

        /** Isomethod of ada_subtype_indication_p_is_static_subtype langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_subtype_indication_p_is_static_subtype(
            EntityNative node,
            byte imprecise_fallback,
            CCharPointer result
        );

        
    


        
    


        
    

            

        /** Isomethod of ada_synthetic_type_expr_f_target_type langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_synthetic_type_expr_f_target_type(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_unconstrained_array_index_f_subtype_indication langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_unconstrained_array_index_f_subtype_indication(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_until_node_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_until_node_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    


        
    


        
    

            

        /** Isomethod of ada_use_package_clause_f_packages langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_use_package_clause_f_packages(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_use_type_clause_f_has_all langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_use_type_clause_f_has_all(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_use_type_clause_f_types langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_use_type_clause_f_types(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_value_sequence_f_iter_assoc langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_value_sequence_f_iter_assoc(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_variant_f_choices langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_variant_f_choices(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_variant_f_components langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_variant_f_components(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_variant_part_f_discr_name langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_variant_part_f_discr_name(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_variant_part_f_variant langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_variant_part_f_variant(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_with_clause_f_has_limited langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_with_clause_f_has_limited(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_with_clause_f_has_private langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_with_clause_f_has_private(
            EntityNative node,
            EntityNative result
        );
            

        /** Isomethod of ada_with_clause_f_packages langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_with_clause_f_packages(
            EntityNative node,
            EntityNative result
        );

        
    

            

        /** Isomethod of ada_with_private_p_as_bool langkit function */
        @CompilerDirectives.TruffleBoundary
        @CFunction
        public static native int ada_with_private_p_as_bool(
            EntityNative node,
            CCharPointer result
        );

        
    


        
    



    }


    // ==========
    // Exceptions
    // ==========

    /**
     * This class represents exception during symbol manipulation.
     */
    public static final class SymbolException extends RuntimeException {
        public SymbolException(
            final String symbol
        ) {
            super("Invalid symbol : '" + symbol + "'");
        }
    }

    /**
     * This class reprsents exception during enum manipulation.
     */
    public static final class EnumException extends RuntimeException {
        public EnumException(
            final String msg
        ) {
            super(msg);
        }
    }

    /**
     * This class represents an exception in the references manipulation.
     */
    public static final class ReferenceException extends RuntimeException {
        public ReferenceException(
            final String msg
        ) {
            super(msg);
        }
    }

    /**
     * This class wraps the exceptions from the langkit native library.
     */
    public static class LangkitException extends RuntimeException {

        // ----- Instance attributes -----

        /** The kind of the langkit exception. */
        public final ExceptionKind kind;

        // ----- Constructors -----

        /**
         * Create a new langkit exception.
         *
         * @param kind The kind of the exception represented by an integer
         * which will be mapped to an enum value.
         * @param message The message of the exception.
         */
        public LangkitException(
            final int kind,
            final String message
        ) {
            super(message);
            this.kind = ExceptionKind.fromC(kind);
        }

    }

    // ==========
    // Enum definitions
    // ==========

    // ===== Constants enumeration =====

    /**
     * Kind for this token.
     */
    public enum TokenKind {

        // ----- Enum values -----

        NO_TOKEN(-1, "No_Token"),
        ADA_TERMINATION(0, "Termination"),
        ADA_LEXING_FAILURE(1, "Lexing_Failure"),
        ADA_IDENTIFIER(2, "Identifier"),
        ADA_ALL(3, "All"),
        ADA_ABORT(4, "Abort"),
        ADA_ELSE(5, "Else"),
        ADA_NEW(6, "New"),
        ADA_RETURN(7, "Return"),
        ADA_ABS(8, "Abs"),
        ADA_ELSIF(9, "Elsif"),
        ADA_NOT(10, "Not"),
        ADA_REVERSE(11, "Reverse"),
        ADA_END(12, "End"),
        ADA_NULL(13, "Null"),
        ADA_ACCEPT(14, "Accept"),
        ADA_ENTRY(15, "Entry"),
        ADA_SELECT(16, "Select"),
        ADA_ACCESS(17, "Access"),
        ADA_EXCEPTION(18, "Exception"),
        ADA_OF(19, "Of"),
        ADA_SEPARATE(20, "Separate"),
        ADA_EXIT(21, "Exit"),
        ADA_OR(22, "Or"),
        ADA_OTHERS(23, "Others"),
        ADA_SUBTYPE(24, "Subtype"),
        ADA_AND(25, "And"),
        ADA_FOR(26, "For"),
        ADA_OUT(27, "Out"),
        ADA_ARRAY(28, "Array"),
        ADA_FUNCTION(29, "Function"),
        ADA_AT(30, "At"),
        ADA_GENERIC(31, "Generic"),
        ADA_PACKAGE(32, "Package"),
        ADA_TASK(33, "Task"),
        ADA_BEGIN(34, "Begin"),
        ADA_GOTO(35, "Goto"),
        ADA_PRAGMA(36, "Pragma"),
        ADA_TERMINATE(37, "Terminate"),
        ADA_BODY(38, "Body"),
        ADA_PRIVATE(39, "Private"),
        ADA_THEN(40, "Then"),
        ADA_IF(41, "If"),
        ADA_PROCEDURE(42, "Procedure"),
        ADA_TYPE(43, "Type"),
        ADA_CASE(44, "Case"),
        ADA_IN(45, "In"),
        ADA_CONSTANT(46, "Constant"),
        ADA_IS(47, "Is"),
        ADA_RAISE(48, "Raise"),
        ADA_USE(49, "Use"),
        ADA_DECLARE(50, "Declare"),
        ADA_RANGE(51, "Range"),
        ADA_DELAY(52, "Delay"),
        ADA_LIMITED(53, "Limited"),
        ADA_RECORD(54, "Record"),
        ADA_WHEN(55, "When"),
        ADA_DELTA(56, "Delta"),
        ADA_LOOP(57, "Loop"),
        ADA_REM(58, "Rem"),
        ADA_WHILE(59, "While"),
        ADA_DIGITS(60, "Digits"),
        ADA_RENAMES(61, "Renames"),
        ADA_DO(62, "Do"),
        ADA_MOD(63, "Mod"),
        ADA_XOR(64, "Xor"),
        ADA_PAR_CLOSE(65, "Par_Close"),
        ADA_PAR_OPEN(66, "Par_Open"),
        ADA_BRACK_CLOSE(67, "Brack_Close"),
        ADA_BRACK_OPEN(68, "Brack_Open"),
        ADA_SEMICOLON(69, "Semicolon"),
        ADA_COLON(70, "Colon"),
        ADA_COMMA(71, "Comma"),
        ADA_DOUBLEDOT(72, "Doubledot"),
        ADA_DOT(73, "Dot"),
        ADA_DIAMOND(74, "Diamond"),
        ADA_LTE(75, "Lte"),
        ADA_GTE(76, "Gte"),
        ADA_ARROW(77, "Arrow"),
        ADA_EQUAL(78, "Equal"),
        ADA_LT(79, "Lt"),
        ADA_GT(80, "Gt"),
        ADA_PLUS(81, "Plus"),
        ADA_MINUS(82, "Minus"),
        ADA_POWER(83, "Power"),
        ADA_MULT(84, "Mult"),
        ADA_AMP(85, "Amp"),
        ADA_NOTEQUAL(86, "Notequal"),
        ADA_DIVIDE(87, "Divide"),
        ADA_TICK(88, "Tick"),
        ADA_PIPE(89, "Pipe"),
        ADA_ASSIGN(90, "Assign"),
        ADA_LABEL_START(91, "Label_Start"),
        ADA_LABEL_END(92, "Label_End"),
        ADA_TARGET(93, "Target"),
        ADA_STRING(94, "String"),
        ADA_CHAR(95, "Char"),
        ADA_WITH(96, "With"),
        ADA_DECIMAL(97, "Decimal"),
        ADA_INTEGER(98, "Integer"),
        ADA_COMMENT(99, "Comment"),
        ADA_PREP_LINE(100, "Prep_Line"),
        ADA_WHITESPACE(101, "Whitespace"),
        ;

        // ----- Class attributes -----

        /** Singleton that represents the none token kind. */
        public static final TokenKind NONE = NO_TOKEN;

        /** The map from int to enum values. */
        private static final Map<Integer, TokenKind> map = new HashMap<>();

        // ----- Instance attributes -----

        /** The value of the enum instance. */
        private final int value;

        /** The name of the enum instance in the Langkit DSL. */
        public final String name;

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(TokenKind elem : TokenKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private TokenKind(
            final int value,
            final String name
        ) {
            this.value = value;
            this.name = name;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static TokenKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get TokenKind from " + cValue
                );
            return (TokenKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    /**
     * Enumerated type describing all possible exceptions that need to be
     * handled in the C bindings.
     */
    public enum ExceptionKind {

        // ----- Enum values -----

        FILE_READ_ERROR(0),
        BAD_TYPE_ERROR(1),
        OUT_OF_BOUNDS_ERROR(2),
        INVALID_INPUT(3),
        INVALID_SYMBOL_ERROR(4),
        INVALID_UNIT_NAME_ERROR(5),
        NATIVE_EXCEPTION(6),
        PRECONDITION_FAILURE(7),
        PROPERTY_ERROR(8),
        TEMPLATE_ARGS_ERROR(9),
        TEMPLATE_FORMAT_ERROR(10),
        TEMPLATE_INSTANTIATION_ERROR(11),
        STALE_REFERENCE_ERROR(12),
        SYNTAX_ERROR(13),
        UNKNOWN_CHARSET(14),
        MALFORMED_TREE_ERROR(15),
        INVALID_PROJECT(16),
        UNSUPPORTED_VIEW_ERROR(17),
        ;

        // ----- Class attributes

        /** Singleton that represents the none expcetion kind. */
        public static final ExceptionKind NONE =
            FILE_READ_ERROR;

        /** The map from int to enum values. */
        private static final Map<Integer, ExceptionKind> map =
            new HashMap<>();

        // ----- Instance ttributes -----

        /** The value of the enum instance. */
        private final int value;

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(ExceptionKind elem : ExceptionKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private ExceptionKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static ExceptionKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get ExceptionKind from " + cValue
                );
            return (ExceptionKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    // ===== Generated enums =====

    
    

    /**
     * Specify a kind of analysis unit. Specification units provide an
     * interface to the outer world while body units provide an implementation
     * for the corresponding interface.
     */
    public enum AnalysisUnitKind {

        // ----- Enum values -----

        UNIT_SPECIFICATION(0),
        UNIT_BODY(1),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final AnalysisUnitKind NONE =
            UNIT_SPECIFICATION;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, AnalysisUnitKind> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(AnalysisUnitKind elem : AnalysisUnitKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private AnalysisUnitKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static AnalysisUnitKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get AnalysisUnitKind from " + cValue
                );
            return (AnalysisUnitKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    
    

    /**

     */
    public enum LookupKind {

        // ----- Enum values -----

        RECURSIVE(0),
        FLAT(1),
        MINIMAL(2),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final LookupKind NONE =
            RECURSIVE;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, LookupKind> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(LookupKind elem : LookupKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private LookupKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static LookupKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get LookupKind from " + cValue
                );
            return (LookupKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    
    

    /**
     * Discriminant for DesignatedEnv structures.
     */
    public enum DesignatedEnvKind {

        // ----- Enum values -----

        NONE_ENUM(0),
        CURRENT_ENV(1),
        NAMED_ENV(2),
        DIRECT_ENV(3),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final DesignatedEnvKind NONE =
            NONE_ENUM;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, DesignatedEnvKind> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(DesignatedEnvKind elem : DesignatedEnvKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private DesignatedEnvKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static DesignatedEnvKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get DesignatedEnvKind from " + cValue
                );
            return (DesignatedEnvKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    
    

    /**
     * Kind for the result of a cross reference operation.
     *
     * * ``no_ref`` is for no reference, it is the null value for this enum.
     *
     * * ``precise`` is when the reference result is precise.
     *
     * * ``imprecise`` is when there was an error computing the precise result,
     *   and a result was gotten in an imprecise fashion.
     *
     * * ``error`` is for unrecoverable errors (either there is no imprecise
     *   path for the request you made, or the imprecise path errored out too).
     */
    public enum RefResultKind {

        // ----- Enum values -----

        NO_REF(0),
        PRECISE(1),
        IMPRECISE(2),
        ERROR(3),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final RefResultKind NONE =
            NO_REF;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, RefResultKind> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(RefResultKind elem : RefResultKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private RefResultKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static RefResultKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get RefResultKind from " + cValue
                );
            return (RefResultKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    
    

    /**
     * Kind of CallExpr type.
     *
     * * ``call`` is when the CallExpr is a procedure or function call.
     *
     * * ``array_slice``, ``array_index`` is when the CallExpr is in fact an
     *   array slice or an array subcomponent access expression, respectively.
     *
     * * ``type_conversion`` is when the CallExpr is a type conversion.
     */
    public enum CallExprKind {

        // ----- Enum values -----

        CALL(0),
        ARRAY_SLICE(1),
        ARRAY_INDEX(2),
        TYPE_CONVERSION(3),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final CallExprKind NONE =
            CALL;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, CallExprKind> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(CallExprKind elem : CallExprKind.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private CallExprKind(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static CallExprKind fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get CallExprKind from " + cValue
                );
            return (CallExprKind) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }

    
    

    /**
     * Gramar rule to use for parsing.
     */
    public enum GrammarRule {

        // ----- Enum values -----

        PARENT_LIST_RULE(0),
        PROTECTED_TYPE_DECL_RULE(1),
        PROTECTED_OP_RULE(2),
        PROTECTED_EL_RULE(3),
        PROTECTED_DEF_RULE(4),
        PROTECTED_DECL_RULE(5),
        TASK_ITEM_RULE(6),
        TASK_DEF_RULE(7),
        TASK_TYPE_DECL_RULE(8),
        SUBTYPE_DECL_RULE(9),
        INTERFACE_TYPE_DEF_RULE(10),
        UNCONSTRAINED_INDEX_RULE(11),
        ARRAY_TYPE_DEF_RULE(12),
        DISCRETE_SUBTYPE_DEFINITION_RULE(13),
        CONSTRAINT_LIST_RULE(14),
        SIGNED_INT_TYPE_DEF_RULE(15),
        MOD_INT_TYPE_DEF_RULE(16),
        DERIVED_TYPE_DEF_RULE(17),
        COMPOSITE_CONSTRAINT_ASSOC_RULE(18),
        COMPOSITE_CONSTRAINT_RULE(19),
        DIGITS_CONSTRAINT_RULE(20),
        DELTA_CONSTRAINT_RULE(21),
        RANGE_CONSTRAINT_RULE(22),
        CONSTRAINT_RULE(23),
        DISCRIMINANT_SPEC_RULE(24),
        DISCR_SPEC_LIST_RULE(25),
        DISCRIMINANT_PART_RULE(26),
        ENUM_LITERAL_DECL_RULE(27),
        FORMAL_DISCRETE_TYPE_DEF_RULE(28),
        RECORD_DEF_RULE(29),
        RANGE_SPEC_RULE(30),
        REAL_TYPE_DEF_RULE(31),
        SEXPR_OR_BOX_RULE(32),
        ORDINARY_FIXED_POINT_DEF_RULE(33),
        DECIMAL_FIXED_POINT_DEF_RULE(34),
        FLOATING_POINT_DEF_RULE(35),
        RECORD_TYPE_DEF_RULE(36),
        ACCESS_DEF_RULE(37),
        ENUM_TYPE_DEF_RULE(38),
        TYPE_DEF_RULE(39),
        VARIANT_RULE(40),
        ANONYMOUS_TYPE_DECL_RULE(41),
        INCOMPLETE_TYPE_DECL_RULE(42),
        TYPE_DECL_RULE(43),
        VARIANT_PART_RULE(44),
        COMPONENT_DEF_RULE(45),
        COMPONENT_ITEM_RULE(46),
        COMPONENT_DECL_RULE(47),
        COMPONENT_LIST_RULE(48),
        GENERIC_DECL_RULE(49),
        GENERIC_FORMAL_PART_RULE(50),
        GENERIC_FORMAL_DECL_RULE(51),
        FORMAL_TYPE_DECL_RULE(52),
        FORMAL_SUBP_DECL_RULE(53),
        RENAMING_CLAUSE_RULE(54),
        GENERIC_RENAMING_DECL_RULE(55),
        GENERIC_INSTANTIATION_RULE(56),
        EXCEPTION_DECL_RULE(57),
        BASIC_DECLS_RULE(58),
        PACKAGE_RENAMING_DECL_RULE(59),
        PACKAGE_DECL_RULE(60),
        BASIC_DECL_RULE(61),
        OBJECT_DECL_RULE(62),
        SUB_OBJECT_DECL_RULE(63),
        NO_TYPE_OBJECT_RENAMING_DECL_RULE(64),
        EXT_RET_STMT_OBJECT_DECL_RULE(65),
        DEFINING_ID_LIST_RULE(66),
        NUMBER_DECL_RULE(67),
        CONTRACT_CASE_ASSOC_RULE(68),
        CONTRACT_CASES_EXPR_RULE(69),
        ABSTRACT_STATE_DECL_RULE(70),
        MULTI_ABSTRACT_STATE_DECL_RULE(71),
        ASPECT_ASSOC_RULE(72),
        ASPECT_SPEC_RULE(73),
        SINGLE_TASK_DECL_RULE(74),
        OVERRIDING_INDICATOR_RULE(75),
        ENTRY_DECL_RULE(76),
        COMPONENT_CLAUSE_RULE(77),
        ASPECT_CLAUSE_RULE(78),
        PARAM_SPEC_RULE(79),
        PARAM_SPECS_RULE(80),
        SUBP_SPEC_RULE(81),
        EXPR_FN_RULE(82),
        NULL_SUBP_DECL_RULE(83),
        ABSTRACT_SUBP_DECL_RULE(84),
        SUBP_RENAMING_DECL_RULE(85),
        SIMPLE_SUBP_DECL_RULE(86),
        SUBP_DECL_RULE(87),
        WITH_CLAUSE_RULE(88),
        CONTEXT_ITEM_RULE(89),
        USE_CLAUSE_RULE(90),
        USE_PACKAGE_CLAUSE_RULE(91),
        USE_TYPE_CLAUSE_RULE(92),
        SUBTYPE_INDICATION_RULE(93),
        DISCRETE_SUBTYPE_INDICATION_RULE(94),
        CONSTRAINED_SUBTYPE_INDICATION_RULE(95),
        TYPE_EXPR_RULE(96),
        ANONYMOUS_TYPE_RULE(97),
        MODE_RULE(98),
        PRAGMA_ARGUMENT_RULE(99),
        PRAGMA_RULE(100),
        SUBUNIT_RULE(101),
        LIBRARY_UNIT_BODY_RULE(102),
        LIBRARY_UNIT_RENAMING_DECL_RULE(103),
        LIBRARY_ITEM_RULE(104),
        COMPILATION_UNIT_RULE(105),
        COMPILATION_RULE(106),
        DECL_PART_RULE(107),
        ENTRY_BODY_RULE(108),
        PROTECTED_BODY_RULE(109),
        PROTECTED_BODY_STUB_RULE(110),
        TASK_BODY_RULE(111),
        TASK_BODY_STUB_RULE(112),
        PACKAGE_BODY_STUB_RULE(113),
        PACKAGE_BODY_RULE(114),
        TERMINATE_ALTERNATIVE_RULE(115),
        SELECT_STMT_RULE(116),
        ACCEPT_STMT_RULE(117),
        CASE_ALT_RULE(118),
        CASE_STMT_RULE(119),
        EXT_RETURN_STMT_RULE(120),
        IBLOCK_STMT_RULE(121),
        BLOCK_STMT_RULE(122),
        WHILE_LOOP_SPEC_RULE(123),
        ILOOP_STMT_RULE(124),
        LOOP_STMT_RULE(125),
        COMPOUND_STMT_RULE(126),
        ELSIF_PART_RULE(127),
        IF_STMT_RULE(128),
        RAISE_STMT_RULE(129),
        DELAY_STMT_RULE(130),
        ABORT_STMT_RULE(131),
        BODY_RULE(132),
        BODY_STUB_RULE(133),
        SUBP_BODY_STUB_RULE(134),
        RECOV_DECL_PART_RULE(135),
        SUBP_BODY_RULE(136),
        HANDLED_STMTS_RULE(137),
        EXCEPTION_HANDLER_RULE(138),
        STMTS_RULE(139),
        LABEL_RULE(140),
        STMT_RULE(141),
        CALL_STMT_RULE(142),
        SIMPLE_STMT_RULE(143),
        NULL_STMT_RULE(144),
        ASSIGNMENT_STMT_RULE(145),
        GOTO_STMT_RULE(146),
        EXIT_STMT_RULE(147),
        RETURN_STMT_RULE(148),
        REQUEUE_STMT_RULE(149),
        IDENTIFIER_RULE(150),
        CHAR_LITERAL_RULE(151),
        STRING_LITERAL_RULE(152),
        DEFINING_ID_RULE(153),
        DEC_LITERAL_RULE(154),
        INT_LITERAL_RULE(155),
        NUM_LITERAL_RULE(156),
        NULL_LITERAL_RULE(157),
        ALLOCATOR_RULE(158),
        FOR_LOOP_PARAM_SPEC_RULE(159),
        QUANTIFIED_EXPR_RULE(160),
        CASE_EXPR_RULE(161),
        CASE_EXPR_ALT_RULE(162),
        RAISE_EXPR_RULE(163),
        IF_EXPR_RULE(164),
        CONDITIONAL_EXPR_RULE(165),
        BOX_EXPR_RULE(166),
        OTHERS_DESIGNATOR_RULE(167),
        ITERATED_ASSOC_RULE(168),
        AGGREGATE_ASSOC_RULE(169),
        REGULAR_AGGREGATE_RULE(170),
        BRACKET_AGGREGATE_RULE(171),
        AGGREGATE_RULE(172),
        DIRECT_NAME_RULE(173),
        PARAM_ASSOC_RULE(174),
        CALL_SUFFIX_RULE(175),
        ATTR_SUFFIX_RULE(176),
        QUALIFIED_NAME_RULE(177),
        QUAL_NAME_INTERNAL_RULE(178),
        VALUE_SEQUENCE_RULE(179),
        NAME_RULE(180),
        DEFINING_NAME_RULE(181),
        DIRECT_NAME_OR_TARGET_NAME_RULE(182),
        TARGET_NAME_RULE(183),
        UPDATE_ATTR_AGGREGATE_RULE(184),
        UPDATE_ATTR_CONTENT_RULE(185),
        MULTIDIM_ARRAY_ASSOC_RULE(186),
        SUBTYPE_NAME_RULE(187),
        STATIC_NAME_RULE(188),
        PRIMARY_RULE(189),
        PAREN_EXPR_RULE(190),
        DECLARE_EXPR_RULE(191),
        FACTOR_RULE(192),
        TERM_RULE(193),
        UNOP_TERM_RULE(194),
        SIMPLE_EXPR_RULE(195),
        BOOLEAN_OP_RULE(196),
        DISCRETE_RANGE_RULE(197),
        CHOICE_RULE(198),
        CHOICE_LIST_RULE(199),
        REL_OP_RULE(200),
        MEMBERSHIP_CHOICE_RULE(201),
        MEMBERSHIP_CHOICE_LIST_RULE(202),
        RELATION_RULE(203),
        EXPR_RULE(204),
        PP_DIRECTIVE_RULE(205),
        PP_THEN_RULE(206),
        PP_EXPR_RULE(207),
        PP_TERM_RULE(208),
        ;

        // ----- Attributes -----

        /** Singleton that represents the none enum value. */
        public static final GrammarRule NONE =
            PARENT_LIST_RULE;

        /** The value of the enum instance. */
        private final int value;

        /** The map from int to enum values. */
        private static final Map<Integer, GrammarRule> map = new HashMap<>();

        // ----- Constructors -----

        static {
            // Initialise the lookup map
            for(GrammarRule elem : GrammarRule.values()) {
                map.put(elem.value, elem);
            }
        }

        /** Private constructor. */
        private GrammarRule(
            final int value
        ) {
            this.value = value;
        }

        // ----- Enum methods -----

        /**
         * Get the enum instance for the given C value.
         *
         * @param cValue The C int value to get the enum instance from.
         * @return The enum instance which correspond to the int value.
         * @throws EnumException When the int value doesn't map to any enum
         * instance.
         */
        public static GrammarRule fromC(
            final int cValue
        ) throws EnumException {
            if(!map.containsKey(cValue))
                throw new EnumException(
                    "Cannot get GrammarRule from " + cValue
                );
            return (GrammarRule) map.get(cValue);
        }

        /**
         * Get the C value from the enum instance.
         *
         * @return The int C value of the enum instance.
         */
        public int toC() {
            return this.value;
        }

    }


    // ==========
    // Java wrapping classes
    // ==========

    // ===== Constant structure wrapping classes =====

    /**
     * This class wraps the langkit characters which are 32 bit wide.
     */
    public static final class Char {

        // ----- Class attributes -----

        /** Singleton that represents the none char. */
        public static final Char NONE = new Char(0);

        // ----- Instance attributes -----

        /** The value of the character. */
        public final int value;

        // ----- Constructors -----

        /**
         * Create a new character from its value. In langkit characters are
         * 32 bit wide so represented by Java integer.
         *
         * @param value The value of the character.
         */
        Char(
            final int value
        ) {
            this.value = value;
        }

        /**
         * Create a character from its integer value.
         *
         * @param value The character value.
         * @return The newly created character.
         */
        public static Char create(
            final int value
        ) {
            return new Char(value);
        }

        /**
         * Create a character from a Java character.
         *
         * @param value The source value of the character.
         * @return The newly created character.
         */
        public static Char create(
            final char value
        ) {
            return new Char((int) value);
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given NI pointer in a Java class.
         *
         * @param pointer The NI pointer to wrap.
         * @return The wrapped character.
         */
        static Char wrap(
            final CIntPointer pointer
        ) {
            return wrap(pointer.read());
        }

        /**
         * Wrap an integer to a character.
         *
         * @param value The value of the character in an integer.
         * @return The newly created character.
         */
        static Char wrap(
            final int value
        ) {
            return new Char(value);
        }

        /**
         * Unwrap the character in the given int pointer.
         *
         * @param pointer The pointer to unwrap the character in.
         */
        void unwrap(
            final CIntPointer pointer
        ) {
            pointer.write(this.value);
        }

        /**
         * Unwrap the character in a Java integer.
         *
         * @return The character value in a Java integer.
         */
        int unwrap() {
            return this.value;
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            final ByteBuffer buffer = ByteBuffer.allocate(4);
            buffer.order(BYTE_ORDER);
            buffer.putInt(this.value);
            return decodeUTF32(buffer.array());
        }

    }

    /**
     * Arbitrarily large integer.
     */
    static final class BigIntegerWrapper {

        // ----- Class attributes -----

        /** Singleton that represents the none big integer. */
        public static final BigInteger NONE = BigInteger.ZERO;

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer which points to a native big integer.
         *
         * @param pointer The pointer to the native big integer.
         * @return The Java big integer.
         */
        static BigInteger wrap(
            final Pointer pointer
        ) {
            return wrap((BigIntegerNative) pointer.readWord(0));
        }

        /**
         * Wrap the given native big integer in a Java big integer.
         *
         * @param bigIntegerNative The big integer native value.
         * @return The Java big integer.
         */
        static BigInteger wrap(
            final BigIntegerNative bigIntegerNative
        ) {
            final String representation = getRepresentation(bigIntegerNative);
            return new BigInteger(representation);
        }

        /**
         * Unwrap the given big integer in the given pointer as a native
         * big integer.
         *
         * @param bigInteger The big integer to unwrap.
         * @param pointer The pointer to place the big integer in.
         */
        static void unwrap(
            final BigInteger bigInteger,
            final Pointer pointer
        ) {
            pointer.writeWord(0, unwrap(bigInteger));
        }

        /**
         * Unwrap the given big integer.
         *
         * @param bigInteger The big integer to unwrap.
         * @return The native big integer newly allocated.
         */
        static BigIntegerNative unwrap(
            final BigInteger bigInteger
        ) {
            // Create the representation of the big integer
            final String representation = bigInteger.toString();
            try(final Text bigIntegerText = Text.create(representation)) {
                TextNative bigIntegerTextNative = StackValue.get(
                    TextNative.class
                );
                bigIntegerText.unwrap(bigIntegerTextNative);

                // Create the big intger by calling the native function
                return NI_LIB.ada_create_big_integer(
                    bigIntegerTextNative
                );
            }
        }

        /**
         * Release the big integer pointed by the given pointer.
         *
         * @param pointer The pointer to the big integer to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((BigIntegerNative) pointer.readWord(0));
        }

        /**
         * Release the given native big integer.
         *
         * @param bigIntegerNative The native big integer to release.
         */
        static void release(
            final BigIntegerNative bigIntegerNative
        ) {
            NI_LIB.ada_big_integer_decref(bigIntegerNative);
        }

        // ----- Class methods -----

        /**
         * Get the string representation of the given native big integer.
         *
         * @param bigIntegerNative The native big integer to get the
         * representation from.
         */
        private static String getRepresentation(
            final BigIntegerNative bigIntegerNative
        ) {
            // Allocate the stack value for the text
            final TextNative bigIntegerTextNative = StackValue.get(
                TextNative.class
            );
            Text.NONE.unwrap(bigIntegerTextNative);

            // Call the native function
            NI_LIB.ada_big_integer_text(
                bigIntegerNative,
                bigIntegerTextNative
            );

            // Wrap the text and return the result
            try(final Text bigIntegerText = Text.wrap(bigIntegerTextNative)) {
                return bigIntegerText.getContent();
            }
        }

    }

    /**
     * Reference to a symbol. Symbols are owned by analysis contexts, so they
     * must not outlive them. This type exists only in the C API, and roughly
     * wraps the corresponding Ada type (an array fat pointer).
     */
    public static final class Symbol {

        // ----- Class attributes -----

        /** Singleton that represents the none symbol. */
        public static final Symbol NONE = new Symbol("");

        // ----- Instance attributes

        /** The text of the symbol. */
        public final String text;

        // ----- Constructors -----

        /**
         * Create a new symbol from its text.
         *
         * @param text The symbol text.
         */
        Symbol(
            final String text
        ) {
            this.text = text;
        }

        /**
         * Public access to the symbol creation.
         *
         * @param text The text of the symbol.
         */
        public static Symbol create(
            final String text
        ) {
            return new Symbol(text);
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given pointer to a native symbol.
         *
         * @param pointer The pointer to the native symbol.
         * @return The wrapped symbol.
         */
        static Symbol wrap(
            final Pointer pointer
        ) {
            return wrap((SymbolNative) pointer.readWord(0));
        }

        /**
         * Wrap the given symbol native value in a Java value.
         *
         * @param symbolNative The symbol native value.
         * @return The wrapped symbol.
         */
        static Symbol wrap(
            final SymbolNative symbolNative
        ) {
            // Get the symbol text
            final TextNative textNative = StackValue.get(TextNative.class);
            Text.NONE.unwrap(textNative);
            NI_LIB.ada_symbol_text(
                symbolNative,
                textNative
            );

            // Return the new symbol
            try(final Text text = Text.wrap(textNative)) {
                return new Symbol(text.getContent());
            }
        }

        /**
         * Unwrap the symbol in the given native structure.
         *
         * @param symbolNative The native structure to unwrap in.
         */
        void unwrap(
            final SymbolNative symbolNative,
            final AnalysisContext context
        ) {
            // Unwrap the symbol text
            try(Text text = Text.create(this.text)) {
                final TextNative textNative = StackValue.get(
                    TextNative.class
                );
                text.unwrap(textNative);

                // Call the symbol creation
                final int resCode = NI_LIB.ada_context_symbol(
                    context.reference.ni(),
                    textNative,
                    symbolNative
                );

                // Check the symbol creation success
                if(resCode == 0) {
                    throw new SymbolException(this.text);
                }
            }
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return this.text;
        }

    }

    /**
     * Type to contain Unicode text data.
     */
    static final class StringWrapper {

        // ----- Class attributes -----

        /** Singleton that represents the none string. */
        public static final String NONE = "";

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer that points to a native string.
         *
         * @param pointer The pointer to the native string.
         * @return The Java string wrapped from the native string.
         */
        static String wrap(
            final Pointer pointer
        ) {
            return wrap((StringNative) pointer.readWord(0));
        }

        /**
         * Wrap a native string wrapper in a Java string.
         *
         * @param stringNative The native string to wrap.
         * @return The Java string created from the native one.
         */
        static String wrap(
            final StringNative stringNative
        ) {
            return getNativeStringContent(stringNative);
        }

        /**
         * Unwrap the string in the given pointer.
         *
         * @param string The string to unwrap.
         * @param pointer The pointer to place the native string in.
         */
        static void unwrap(
            final String string,
            final Pointer pointer
        ) {
            pointer.writeWord(0, unwrap(string));
        }

        /**
         * Unwrap the given string in a native one.
         *
         * @param string The string to unwrap.
         * @return The native string unwrapped.
         */
        static StringNative unwrap(
            final String string
        ) {
            return nativeStringFromContent(string);
        }

        /**
         * Release the native string at the given pointer.
         *
         * @param pointer The pointer to the native string.
         */
        static void release(
            final Pointer pointer
        ) {
            release((StringNative) pointer.readWord(0));
        }

        /**
         * Release the given native string.
         *
         * @param stringNative The native string to release.
         */
        static void release(
            final StringNative stringNative
        ) {
            NI_LIB.ada_string_dec_ref(stringNative);
        }

        // ----- Class methods -----

        /**
         * Get the content of the given native string in a Java one.
         *
         * @param stringNative The native string to get the content from.
         * @return The Java string.
         */
        private static String getNativeStringContent(
            final StringNative stringNative
        ) {
            // Prepare the working variable
            final Pointer pointer = (Pointer) stringNative;
            final int length = pointer.readInt(0);
            final byte[] contentArray = new byte[length * 4];

            // Get the content from the native string
            for(int i = 0 ; i < contentArray.length ; i++) {
                contentArray[i] = pointer.readByte(i + 8);
            }

            // Return the decoded string
            return decodeUTF32(contentArray);
        }

        /**
         * Create a native string from a Java string.
         *
         * @param string The Java string to create the native string from.
         * @return The native string.
         */
        private static StringNative nativeStringFromContent(
            final String string
        ) {
            // Encode the string in UTF-32
            final byte[] contentArray = encodeUTF32(string);
            final int length = string.length();

            // Create the native array
            final Pointer contentArrayNative = UnmanagedMemory.malloc(
                contentArray.length
            );
            for(int i = 0 ; i < contentArray.length ; i++) {
                contentArrayNative.writeByte(i, contentArray[i]);
            }

            // Create the native string
            final StringNative res = NI_LIB.ada_create_string(
                (CIntPointer) contentArrayNative,
                length
            );

            // Free the memory
            UnmanagedMemory.free(contentArrayNative);

            // Return the result
            return res;
        }

    }

    /**
     * String encoded in UTF-32 (native endianness).
     */
    public static final class Text implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none text. */
        public static final Text NONE = new Text(
            PointerWrapper.nullPointer(),
            0,
            false
        );

        // ----- Instance attributes -----

        /** The pointer to the characters. */
        private final PointerWrapper charPointer;

        /** The size of the text. */
        private final long length;

        /** If the text is allocated. */
        private final boolean isAllocated;

        /** If the text object is the owner of its buffer. */
        private final boolean isOwner;

        /** The content of the text in a Java string. */
        private String content;

        // ----- Constructors -----

        /**
         * Create a new text from its content.
         *
         * @param charPointer The pointer to the characters of the text.
         * @param length The length of the text in character.
         * @param isAllocated If the text is allocated in the memory.
         */
        Text(
            final PointerWrapper charPointer,
            final long length,
            final boolean isAllocated
        ) {
            this(
                charPointer,
                length,
                isAllocated,
                false,
                null
            );
        }

        /**
         * Create a new text from its content and buffer.
         *
         * @param charPointer The pointer to the characters of the text.
         * @param length The length of the text in character.
         * @param isAllocated If the text is allocated in the memory.
         * @param contentArray The characters of the text (for JNI)
         */
        Text(
            final PointerWrapper charPointer,
            final long length,
            final boolean isAllocated,
            final byte[] contentArray
        ) {
            this(
                charPointer,
                length,
                isAllocated,
                false,
                contentArray
            );
        }

        /**
         * Create a new text from its content and buffer.
         *
         * @param charPointer The pointer to the characters of the text.
         * @param length The length of the text in character.
         * @param isAllocated If the text is allocated in the memory.
         * @param isOwner If the Java object owns the text buffer.
         * @param contentArray The characters of the text
         * (as strings, this is only used in JNI mode).
         */
        private Text(
            final PointerWrapper charPointer,
            final long length,
            final boolean isAllocated,
            final boolean isOwner,
            final byte[] contentArray
        ) {
            this.charPointer = charPointer;
            this.length = length;
            this.isAllocated = isAllocated;
            this.isOwner = isOwner;

            if(contentArray != null) {
                this.content = decodeUTF32(contentArray);
            } else {
                this.content = null;
            }
        }

        /**
         * Create a new langkit text from its string content.
         *
         * @param content The content of the text in a Java string.
         * @return The newly created text.
         */
        public static Text create(
            final String content
        ) {
            final byte[] contentArray = encodeUTF32(content);

            if(ImageInfo.inImageCode()) {
                final PointerWrapper charPointer = new PointerWrapper(
                    toCBytes(contentArray)
                );
                return new Text(
                    charPointer,
                    (long) content.length(),
                    false,
                    true,
                    contentArray
                );
            } else {
                return JNI_LIB.ada_create_text(contentArray);
            }
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the native text pointed by the given pointer.
         *
         * @param pointer The pointer to the native text to wrap.
         * @return The wrapped text.
         */
        static Text wrap(
            final Pointer pointer
        ) {
            return wrap((TextNative) pointer.readWord(0));
        }

        /**
         * Wrap a text native value in the Java class.
         *
         * @param textNative The text native NI value.
         * @return The newly wrapped text.
         */
        static Text wrap(
            final TextNative textNative
        ) {
            return new Text(
                new PointerWrapper(textNative.get_chars()),
                textNative.get_length(),
                textNative.get_is_allocated() != 0
            );
        }

        /**
         * Unwrap the text in the given NI pointer.
         *
         * @param textNative The NI pointer to the native text structure.
         */
        void unwrap(
            final TextNative textNative
        ) {
            textNative.set_chars(this.charPointer.ni());
            textNative.set_length(this.length);
            textNative.set_is_allocated(this.isAllocated ? 1 : 0);
        }

        // ----- Instance methods -----

        /**
         * Get the content of the text in a Java string.
         *
         * @return the content of the text.
         */
        public String getContent() {
            // Content is null only when using the Graal C API.
            if(this.content == null) {
                final byte[] contentArray = new byte[(int) this.length * 4];
                for(int i = 0 ; i < contentArray.length ; i++) {
                    contentArray[i] = (
                        (CCharPointer) this.charPointer.ni()
                    ).read(i);
                }
                this.content = decodeUTF32(contentArray);
            }

            return this.content;
        }

        /** @see java.lang.AutoCloseable#close() */
        @Override
        public void close() {

            if(ImageInfo.inImageCode()) {
                if(this.isOwner) {
                    UnmanagedMemory.free(this.charPointer.ni());
                } else {
                    final TextNative textNative = StackValue.get(
                        TextNative.class
                    );
                    this.unwrap(textNative);
                    NI_LIB.ada_destroy_text(textNative);
                }
            } else {
                JNI_LIB.ada_destroy_text(this);
            }
            checkException();

        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return this.getContent();
        }

    }

    /**
     * Location in a source file. Line and column numbers are one-based.
     */
    public static final class SourceLocation {

        // ----- Class attributes -----

        /** Singleton that represents the none source location. */
        public static final SourceLocation NONE = new SourceLocation(
            0,
            (short) 0
        );

        // ----- Instance attributes -----

        /** The line of the source location. */
        public final int line;

        /** The column of the source location. */
        public final short column;

        // ----- Constructors -----

        /**
         * Create a new source location from a line and a column.
         *
         * @param line The line of the source location.
         * @param column The column of the source location.
         */
        SourceLocation(
            final int line,
            final short column
        ) {
            this.line = line;
            this.column = column;
        }

        /**
         * Create a source location from its line and column.
         *
         * @param line The line.
         * @param column The column.
         * @return The newly create source location.
         */
        public static SourceLocation create(
            final int line,
            final short column
        ) {
            return new SourceLocation(
                line,
                column
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given source location in the Java class.
         *
         * @param pointer Pointer to the native source location.
         * @return The newly wrapper source location.
         */
        static SourceLocation wrap(
            final Pointer pointer
        ) {
            return wrap((SourceLocationNative) pointer.readWord(0));
        }

        /**
         * Wrap a source location native value in the Java class.
         *
         * @param sourceLocationNative The source location NI native value.
         * @return The newly wrapped source location.
         */
        static SourceLocation wrap(
            final SourceLocationNative sourceLocationNative
        ) {
            return new SourceLocation(
                sourceLocationNative.get_line(),
                sourceLocationNative.get_column()
            );
        }

        /**
         * Uwrap the source location in the given NI pointer.
         *
         * @param sourceLocationNative The NI pointer to the native structure
         *  to fill.
         */
        public void unwrap(
            final SourceLocationNative sourceLocationNative
        ) {
            sourceLocationNative.set_line(this.line);
            sourceLocationNative.set_column(this.column);
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return this.line + ":" + this.column;
        }

    }

    /**
     * Location of a span of text in a source file.
     */
    public static final class SourceLocationRange {

        // ----- Class attributes -----

        /** Singleton that represents the none source location range. */
        public static final SourceLocationRange NONE =
            new SourceLocationRange(
                SourceLocation.NONE,
                SourceLocation.NONE
            );

        // ----- Instance attributes -----

        /** The start of the range. */
        public final SourceLocation start;

        /** The end of the range. */
        public final SourceLocation end;

        // ----- Constructors -----

        /**
         * Create a source location range from its bounds.
         *
         * @param start The start of the range.
         * @param end The end of the range.
         */
        SourceLocationRange(
            final SourceLocation start,
            final SourceLocation end
        ) {
            this.start = start;
            this.end = end;
        }

        /**
         * Create a new source location range from its bounds.
         *
         * @param start The starting bound.
         * @param end The ending bound.
         * @return The newly created source location range.
         */
        public static SourceLocationRange create(
            final SourceLocation start,
            final SourceLocation end
        ) {
            return new SourceLocationRange(
                start,
                end
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given source location range in the Java class.*
         *
         * @param pointer The pointer to the native source location range.
         * @return The newly wrapped source location range.
         */
        static SourceLocationRange wrap(
            final Pointer pointer
        ) {
            return wrap((SourceLocationRangeNative) pointer.readWord(0));
        }

        /**
         * Wrap a source location range native value in the Java class.
         *
         * @param sourceLocationRangeNative The source location range NI
         * native value.
         * @return The newly wrapped source location range.
         */
        static SourceLocationRange wrap(
            final SourceLocationRangeNative sourceLocationRangeNative
        ) {
            return new SourceLocationRange(
                new SourceLocation(
                    sourceLocationRangeNative.get_start_line(),
                    sourceLocationRangeNative.get_start_column()
                ),
                new SourceLocation(
                    sourceLocationRangeNative.get_end_line(),
                    sourceLocationRangeNative.get_end_column()
                )
            );
        }

        /**
         * Uwrap the source location range in the given NI pointer.
         *
         * @param sourceLocationRangeNative The NI pointer to the native
         * structure to fill.
         */
        void unwrap(
            final SourceLocationRangeNative sourceLocationRangeNative
        ) {
            sourceLocationRangeNative.set_start_line(this.start.line);
            sourceLocationRangeNative.set_start_column(this.start.column);
            sourceLocationRangeNative.set_end_line(this.end.line);
            sourceLocationRangeNative.set_end_column(this.end.column);
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return this.start.toString() + "-" + this.end.toString();
        }

    }

    /**
     * Diagnostic for an analysis unit: cannot open the source file, parsing
     * error, ...
     */
    public static final class Diagnostic {

        // ----- Class attributes -----

        /** Singleton that represents the none diagnositc. */
        public static final Diagnostic NONE = new Diagnostic(
            SourceLocationRange.NONE,
            Text.NONE
        );

        // ----- Instance attributes -----

        /** The source location range of the diagnostic. */
        public final SourceLocationRange sourceLocationRange;

        /** The message of the diagnostic. */
        public final Text message;

        // ----- Constructors -----

        /**
         * Create a diagnostic from its content.
         *
         * @param sourceLocationRange The range of the diagnostic.
         * @param message The message of the diagnostic.
         */
        Diagnostic(
            final SourceLocationRange sourceLocationRange,
            final Text message
        ) {
            this.sourceLocationRange = sourceLocationRange;
            this.message = message;
        }

        /**
         * Create a new diagnostic from its content.
         *
         * @param sourceLocationRange The source location range concerned by
         * this diagnostic.
         * @param message The message of the diagnostic.
         * @return The newly created diagnostic
         */
        public static Diagnostic create(
            final SourceLocationRange sourceLocationRange,
            final Text message
        ) {
            return new Diagnostic(
                sourceLocationRange,
                message
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to a native diagnostic.
         *
         * @param pointer The pointer to the native diagnositc.
         * @return The wrapped diagnositc.
         */
        static Diagnostic wrap(
            final Pointer pointer
        ) {
            return wrap((DiagnosticNative) pointer.readWord(0));
        }

        /**
         * Wrap a diagnostic native value in the Java class.
         *
         * @param diagnosticNative The diagnostic NI native value.
         * @return The newly wrapped diagnositc.
         */
        static Diagnostic wrap(
            final DiagnosticNative diagnosticNative
        ) {
            return new Diagnostic(
                new SourceLocationRange(
                    new SourceLocation(
                        diagnosticNative.get_start_line(),
                        diagnosticNative.get_start_column()
                    ),
                    new SourceLocation(
                        diagnosticNative.get_end_line(),
                        diagnosticNative.get_end_column()
                    )
                ),
                new Text(
                    new PointerWrapper(diagnosticNative.get_message_chars()),
                    diagnosticNative.get_message_length(),
                    diagnosticNative.get_message_is_allocated() != 0
                )
            );
        }

        /**
         * Unwrap the diagnostic in the given NI pointer.
         *
         * @param diagnosticNative The pointer to the native structure to
         * fill.
         */
        public void unwrap(
            final DiagnosticNative diagnosticNative
        ) {
            diagnosticNative.set_start_line(
                this.sourceLocationRange.start.line
            );
            diagnosticNative.set_start_column(
                this.sourceLocationRange.start.column
            );
            diagnosticNative.set_end_line(
                this.sourceLocationRange.end.line
            );
            diagnosticNative.set_end_column(
                this.sourceLocationRange.end.column
            );
            diagnosticNative.set_message_chars(
                this.message.charPointer.ni()
            );
            diagnosticNative.set_message_length(
                this.message.length
            );
            diagnosticNative.set_message_is_allocated(
                this.message.isAllocated ? 1 : 0
            );
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return this.message.toString()
                + " at <"
                + this.sourceLocationRange.toString()
                + ">";
        }

    }

    /**
     * Interface to override how source files are fetched and decoded.
     */
    public static final class FileReader implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none file reader. */
        public static final FileReader NONE = new FileReader(
            PointerWrapper.nullPointer()
        );

        // ----- Instance attributes -----

        /** The reference to the file reader */
        private final PointerWrapper reference;

        // ----- Constructors -----

        /**
         * Create a new file reader from its native reference.
         *
         * @param reference The reference to the native file reader.
         */
        FileReader(
            final PointerWrapper reference
        ) {
            this.reference = reference;
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to a native file reader.
         *
         * @param pointer The pointer to the native file reader.
         * @return The newly wrapped file reader.
         */
        static FileReader wrap(
            final Pointer pointer
        ) {
            return wrap((FileReaderNative) pointer.readWord(0));
        }

        /**
         * Wrap the given file reader in the Java class.
         *
         * @param fileReaderNative The native file reader to wrap.
         * @return The wrapped file reader.
         */
        static FileReader wrap(
            final FileReaderNative fileReaderNative
        ) {
            return new FileReader(
                new PointerWrapper(fileReaderNative)
            );
        }

        /**
         * Unwrap the file reader in the given pointer.
         *
         * @param pointer The pointer to unwrap the file reader in.
         */
        void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Get the unwrapped file reader.
         *
         * @return The unwrapped native file reader.
         */
        FileReaderNative unwrap() {
            return (FileReaderNative) this.reference.ni();
        }

        // ----- Instance methods -----

        /** @see java.lang.AutoCloseable#close() */
        @Override
        public void close() {

            if(ImageInfo.inImageCode()) {
                NI_LIB.ada_dec_ref_file_reader(
                    this.reference.ni()
                );
            } else {
                JNI_LIB.ada_dec_ref_file_reader(this);
            }
            checkException();

        }

    }

    /*
 * Interface to fetch analysis units from a name and a unit kind.
 *
 * The unit provider mechanism provides an abstraction which assumes that to
 * any couple (unit name, unit kind) we can associate at most one source file.
 * This means that several couples can be associated to the same source file,
 * but on the other hand, only one one source file can be associated to a
 * couple.
 *
 * This is used to make the semantic analysis able to switch from one analysis
 * units to another.
 *
 * See the documentation of each unit provider for the exact semantics of the
 * unit name/kind information.
 */
    public static final class UnitProvider implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none unit provider. */
        public static final UnitProvider NONE = new UnitProvider(
            PointerWrapper.nullPointer()
        );

        // ----- Instance attributes -----

        /** The reference to the unit provider */
        private final PointerWrapper reference;

        // ----- Constructors -----

        /**
         * Create a new unit provider with the reference to the native one.
         *
         * @param reference The reference to the native unit provider.
         */
        UnitProvider(
            final PointerWrapper reference
        ) {
            this.reference = reference;
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given pointer to a native unit provider.
         *
         * @param pointer The pointer to the native unit provider.
         * @return The wrapped unit provider.
         */
        static UnitProvider wrap(
            final Pointer pointer
        ) {
            return wrap((UnitProviderNative) pointer.readWord(0));
        }

        /**
         * Wrap the given native unit provider.
         *
         * @param unitProviderNative The native unit provider.
         * @return The wrapped unit provider.
         */
        static UnitProvider wrap(
            final UnitProviderNative unitProviderNative
        ) {
            return new UnitProvider(
                new PointerWrapper(unitProviderNative)
            );
        }

        /**
         * Unwrap the unit provider in the given native pointer.
         *
         * @param pointer The pointer to place the native unit provider in.
         */
        void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Get the native unit provider.
         *
         * @return The native unit provider.
         */
        UnitProviderNative unwrap() {
            return (UnitProviderNative) this.reference.ni();
        }

        // ----- Instance methods -----

        /** @see java.lang.AutoCloseable#close() */
        public void close() {

            if(ImageInfo.inImageCode()) {
                NI_LIB.ada_dec_ref_unit_provider(this.reference.ni());
            } else {
                JNI_LIB.ada_dec_ref_unit_provider(this);
            }
            checkException();

        }

    }

    /**
 * Interface to handle events sent by the analysis context.
 */
    public static final class EventHandler implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none event handler. */
        public static final EventHandler NONE = new EventHandler(
            PointerWrapper.nullPointer(),
            null,
            null
        );

        /** This map contains all created event handlers. */
        private static final Map<PointerWrapper, EventHandler>
            eventHandlerCache = new ConcurrentHashMap<>();

        // ----- Instance attributes -----

        /** The reference to the native event handler. */
        private final PointerWrapper reference;

        /**
         * The Java callback when an analysis unit is requested in the
         * associated context.
         */
        private final UnitRequestedCallback unitRequestedCallback;

        /**
         * The Java callback when an analysis unit is parsed in the
         * associated context.
         */
        private final UnitParsedCallback unitParsedCallback;

        // ----- Constructors -----

        /**
         * Create a new event handler from its native reference.
         *
         * @param reference The reference to the native event handler.
         * @param unitRequestedCallback The callback for unit requests.
         * @param unitParsedCallback The callback for unit parsing.
         */
        EventHandler(
            final PointerWrapper reference,
            final UnitRequestedCallback unitRequestedCallback,
            final UnitParsedCallback unitParsedCallback
        ) {
            this.reference = reference;
            this.unitRequestedCallback = unitRequestedCallback;
            this.unitParsedCallback = unitParsedCallback;
        }

        /**
         * Create a new even handler with its callbacks. Callbacks can be null.
         *
         * @param unitRequestedCallback The callback for analysis unit requests.
         * @param unitParsedCallback The callback for analysis unit parsing.
         */
        public static EventHandler create(
            final UnitRequestedCallback unitRequestedCallback,
            final UnitParsedCallback unitParsedCallback
        ) {
            // Prepare the reference to the native event handler
            final PointerWrapper reference;

            if(ImageInfo.inImageCode()) {
                // Get the current thread
                final IsolateThread thread = CurrentIsolate.getCurrentThread();

                // Create the native event handler
                final EventHandlerNative resNative =
                    NI_LIB.ada_create_event_handler(
                        (VoidPointer) thread,
                        WordFactory.nullPointer(),
                        NI_LIB.unitRequestedFunction.getFunctionPointer(),
                        NI_LIB.unitParsedFunction.getFunctionPointer()
                    );

                // Set the result to the created event handler
                reference = new PointerWrapper(resNative);
            } else {
                reference = JNI_LIB.ada_create_event_handler(
                    unitRequestedCallback,
                    unitParsedCallback
                );
            }

            // Return the new event handler wrapped object
            return new EventHandler(
                reference,
                unitRequestedCallback,
                unitParsedCallback
            );
        }

        /**
         * Get event handler Java object from its native pointer.
         *
         * @param reference The pointer to the native event handler.
         * @return The associated Java event handler.
         */
        static EventHandler fromReference(
            final PointerWrapper reference
        ) {
            if(eventHandlerCache.containsKey(reference)) {
                return eventHandlerCache.get(reference);
            } else {
                throw new ReferenceException(
                    "Cannot get event handler from this reference: " +
                    reference.toString()
                );
            }
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the given pointer to an event handler.
         *
         * @param pointer The pointer to the native event handler.
         * @return The wrapped event handler.
         */
        EventHandler wrap(
            final Pointer pointer
        ) {
            return wrap((EventHandlerNative) pointer.readWord(0));
        }

        /**
         * Wrap the given native event handler.
         *
         * @param eventHandlerNative The native value of the event handler.
         * @return The wrapped event handler.
         */
        EventHandler wrap(
            final EventHandlerNative eventHandlerNative
        ) {
            return fromReference(new PointerWrapper(eventHandlerNative));
        }

        /**
         * Unwrap the event handler in the given native pointer.
         *
         * @param pointer The pointer to place the native event handler.
         */
        void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Unwrap the event handler to a native value.
         *
         * @return The native value of the event handler.
         */
        EventHandlerNative unwrap() {
            return (EventHandlerNative) this.reference.ni();
        }

        // ----- Getters -----

        public UnitRequestedCallback getUnitRequestCallback() {
            return this.unitRequestedCallback;
        }

        public UnitParsedCallback getUnitParsedCallback() {
            return this.unitParsedCallback;
        }

        // ----- Instance methods -----

        /** @see java.lang.AutoCloseable#close() */
        public void close() {

            if(ImageInfo.inImageCode()) {
                NI_LIB.ada_dec_ref_event_handler(this.reference.ni());
            } else {
                JNI_LIB.ada_dec_ref_event_handler(this);
            }
            checkException();

        }

        // ----- Inner classes -----

        /**
         * Callback that will be called when a unit is requested from the
         * context ``Context``.
         *
         * ``Name`` is the name of the requested unit.
         *
         * ``From`` is the unit from which the unit was requested.
         *
         * ``Found`` indicates whether the requested unit was found or not.
         *
         * ``Is_Not_Found_Error`` indicates whether the fact that the unit was
         * not found is an error or not.
         *
         * .. warning:: The interface of this callback is probably subject to
         *    change, so should be treated as experimental.
         */
        @FunctionalInterface
        public interface UnitRequestedCallback {
            void invoke(
                AnalysisContext context,
                String name,
                AnalysisUnit from,
                boolean found,
                boolean isNotFoundError
            );
        }

        /**
         * Callback that will be called when any unit is parsed from the
         * context ``Context``.
         *
         * ``Unit`` is the resulting unit.
         *
         * ``Reparsed`` indicates whether the unit was reparsed, or whether it
         * was the first parse.
         */
        @FunctionalInterface
        public interface UnitParsedCallback {
            void invoke(
                AnalysisContext context,
                AnalysisUnit unit,
                boolean reparsed
            );
        }

    }

    /**
     * Reference to a token in an analysis unit.
     */
    public static class Token {

        // ----- Instance attributes -----

        /**
         * We only store the reference to the context to avoid ref-count
         * problems. To access the token context go throught the
         * analysis unit.
         */
        protected final PointerWrapper contextRef;

        /** The unit of the token. */
        public final AnalysisUnit unit;

        /** The pointer to the token data handler. */
        protected final PointerWrapper tokenDataHandler;

        /** The index of the token. */
        public final int tokenIndex;

        /** The trivia index. */
        public final int triviaIndex;

        /** The kind of the token. */
        public final TokenKind kind;

        /** The text of the token. */
        protected final String text;

        /** The source location range of the token. */
        public final SourceLocationRange sourceLocationRange;

        // ----- Constructors -----

        /**
         * Create a new token from its content.
         *
         * @param contextRef The context of the token.
         * @param unit The unit which owns the token.
         * @param tokenDataHandler The pointer to the token data.
         * @param tokenIndex The index of the token.
         * @param triviaIndex The trivia index of the token.
         * @param kind The kind of the token in an integer.
         * @param text The text of the token.
         * @param sourceLocationRange The location of the token.
         */
        Token(
            final PointerWrapper contextRef,
            final AnalysisUnit unit,
            final PointerWrapper tokenDataHandler,
            final int tokenIndex,
            final int triviaIndex,
            final TokenKind kind,
            final String text,
            final SourceLocationRange sourceLocationRange
        ) {
            this.contextRef = contextRef;
            this.unit = unit;
            this.tokenDataHandler = tokenDataHandler;
            this.tokenIndex = tokenIndex;
            this.triviaIndex = triviaIndex;
            this.kind = kind;
            this.text = text;
            this.sourceLocationRange = sourceLocationRange;
        }

        // ----- Graal C API methods -----

        /**
         * Wrap the pointer to a native token.
         *
         * @param pointer The pointer to the native token.
         * @param unit The analysis unit which owns the token.
         * @return The wrapped token or a none value if the token is a none
         * one.
         */
        static Token wrap(
            final Pointer pointer,
            final AnalysisUnit unit
        ) {
            return wrap(
                (TokenNative) pointer.readWord(0),
                unit
            );
        }

        /**
         * Wrap a native token value in the Java class.
         *
         * @param tokenNative The native NI token value.
         * @param unit The analysis unit that owns the token.
         * @return The wrapped token or a none value if the token data
         * handler is null.
         */
        static Token wrap(
            final TokenNative tokenNative,
            final AnalysisUnit unit
        ) {
            if(tokenNative.get_data().isNull())
                return NONE(unit);

            // Fetch the token kind, source location range and text from the
            // tokenNative reference.
            final TokenKind kind = TokenKind.fromC(
                NI_LIB.ada_token_get_kind(tokenNative)
            );

            final SourceLocationRangeNative slocRangeNative =
                StackValue.get(SourceLocationRangeNative.class);
            NI_LIB.ada_token_sloc_range(
                tokenNative,
                slocRangeNative
            );
            final SourceLocationRange slocRange =
                SourceLocationRange.wrap(slocRangeNative);

            final TextNative textNative = StackValue.get(TextNative.class);
            NI_LIB.ada_token_range_text(
                tokenNative,
                tokenNative,
                textNative
            );
            final String text = Text.wrap(textNative).getContent();
            NI_LIB.ada_destroy_text(textNative);

            // Wrap them in a high-level Token instance
            return new Token(
                new PointerWrapper(tokenNative.get_context()),
                unit,
                new PointerWrapper(tokenNative.get_data()),
                tokenNative.get_token_index(),
                tokenNative.get_trivia_index(),
                kind,
                text,
                slocRange
            );
        }

        /**
         * Unwrap the token in the given NI pointer.
         *
         * @param tokenNative The NI pointer to the native structure to
         * fill.
         */
        public void unwrap(
            final TokenNative tokenNative
        ) {
            tokenNative.set_context(this.contextRef.ni());
            tokenNative.set_data(this.tokenDataHandler.ni());
            tokenNative.set_token_index(this.tokenIndex);
            tokenNative.set_trivia_index(this.triviaIndex);
        }

        // ----- Getters -----

        public String getText() {
            return this.text;
        }

        public boolean isTrivia() {
            return this.triviaIndex != 0;
        }

        public boolean isNone() {
            return false;
        }

        // ----- Class methods -----

        /**
         * Get a none token for the given unit.
         *
         * @param unit The unit to get a none token for.
         * @return The none token for the given unit.
         */
        public static Token NONE(
            final AnalysisUnit unit
        ) {
            return NoToken.getInstance(unit);
        }

        /**
         * Get the text from the start token to the end token.
         *
         * @param start The start token.
         * @param end The end token.
         * @return The text between the two tokens.
         */
        @CompilerDirectives.TruffleBoundary
        public static String textRange(
            final Token start,
            final Token end
        ) {

            if(ImageInfo.inImageCode()) {
                final TokenNative startNative = StackValue.get(
                    TokenNative.class
                );
                start.unwrap(startNative);

                final TokenNative endNative = StackValue.get(
                    TokenNative.class
                );
                end.unwrap(endNative);

                final TextNative textNative = StackValue.get(
                    TextNative.class
                );
                Text.NONE.unwrap(textNative);
                NI_LIB.ada_token_range_text(
                    startNative,
                    endNative,
                    textNative
                );
                try(final Text resText = Text.wrap(textNative)) {
                    return resText.getContent();
                }
            } else {
                try(
                    final Text resText = JNI_LIB.ada_token_range_text(
                        start,
                        end
                    )
                ) {
                    return resText.getContent();
                }
            }

        }

        // ----- Instance methods -----

        /**
         * Get the next token.
         *
         * @return The next token in the source.
         */
        public Token next() {

            if(ImageInfo.inImageCode()) {
                final TokenNative tokenNative = StackValue.get(
                    TokenNative.class
                );
                this.unwrap(tokenNative);

                final TokenNative nextNative = StackValue.get(
                    TokenNative.class
                );
                NI_LIB.ada_token_next(
                    tokenNative,
                    nextNative
                );
                return wrap(nextNative, this.unit);
            } else {
                return JNI_LIB.ada_token_next(this);
            }

        }

        /**
         * Get the previous token.
         *
         * @return The previous token in the source.
         */
        public Token previous() {

            if(ImageInfo.inImageCode()) {
                final TokenNative tokenNative = StackValue.get(
                    TokenNative.class
                );
                this.unwrap(tokenNative);

                final TokenNative previousNative = StackValue.get(
                    TokenNative.class
                );
                NI_LIB.ada_token_previous(
                    tokenNative,
                    previousNative
                );
                return wrap(previousNative, this.unit);
            } else {
                return JNI_LIB.ada_token_previous(this);
            }

        }

        /**
         * Check of the token is equivalent to the other one.
         *
         * @param other The other token to compare with.
         */
        public boolean isEquivalent(
            final Token other
        ) {

            if(ImageInfo.inImageCode()) {
                final TokenNative leftNative = StackValue.get(
                    TokenNative.class
                );
                this.unwrap(leftNative);

                final TokenNative rightNative = StackValue.get(
                    TokenNative.class
                );
                other.unwrap(rightNative);

                return NI_LIB.ada_token_is_equivalent(
                    leftNative,
                    rightNative
                ) != 0;
            } else {
                return JNI_LIB.ada_token_is_equivalent(this, other);
            }

        }

        // ----- Override methods -----

        @Override
        @CompilerDirectives.TruffleBoundary
        public String toString() {
            return "<Token Kind="
                + this.kind.name
                + " Text=\""
                + stringRepresentation(this.getText())
                + "\">";
        }

        @Override
        public boolean equals(
            Object o
        ) {
            if(o == this) return true;
            if(!(o instanceof Token)) return false;
            final Token other = (Token) o;
            return other.tokenDataHandler.equals(this.tokenDataHandler) &&
                    other.tokenIndex == this.tokenIndex &&
                    other.triviaIndex == this.triviaIndex;
        }

        // ----- Inner classes -----

        /**
        * This class represents the absence of token.
        */
        private static final class NoToken extends Token {

            // ----- Class attributes -----

            /** The map of the no token instances */
            private static final Map<AnalysisUnit, NoToken> instances
                = new HashMap<>();

            // ----- Constructors -----

            /**
            * Create a new no token for the given analysis unit.
            * The constructor is private to avoid too many instances.
            *
            * @param unit The analysis unit to create the no token for.
            */
            private NoToken(
                final PointerWrapper contextRef,
                final AnalysisUnit unit
            ) {
                super(
                    contextRef,
                    unit,
                    PointerWrapper.nullPointer(),
                    0,
                    0,
                    TokenKind.fromC(-1),
                    null,
                    SourceLocationRange.NONE
                );
            }

            /**
            * Get the no token instance for the given analysis unit.
            *
            * @param unit The unit to get the no token instance for.
            * @return The no token instance.
            */
            static NoToken getInstance(
                final AnalysisUnit unit
            ) {
                if(!instances.containsKey(unit)) {
                    try(AnalysisContext context = unit.getContext()) {
                        instances.put(
                            unit,
                            new NoToken(context.reference, unit)
                        );
                    }
                }
                return instances.get(unit);
            }

            // ----- Getters -----

            @Override
            public String getText() {
                return "";
            }

            // ----- Instance methods -----

            @Override
            public Token next() {
                return this;
            }

            @Override
            public Token previous() {
                return this;
            }

            @Override
            public boolean isEquivalent(
                final Token other
            ) {
                return other instanceof NoToken;
            }

            @Override
            public boolean isNone() {
                return true;
            }

            @Override
            public void unwrap(
                final TokenNative tokenNative
            ) {
                tokenNative.set_context(this.contextRef.ni());
                tokenNative.set_data(this.tokenDataHandler.ni());
                tokenNative.set_token_index(this.tokenIndex);
                tokenNative.set_trivia_index(this.triviaIndex);
            }

            // ----- Override methods -----

            @Override
            @CompilerDirectives.TruffleBoundary
            public String toString() {
                return "<Token Kind="
                    + this.kind.name
                    + " Text=\"\">";
            }

            @Override
            public boolean equals(
                Object o
            ) {
                return o == this;
            }

        }

    }

    /**
     * This type represents a context for all source analysis. This is the
     * first type you need to create to use Libadalang. It will contain the
     * results of all analysis, and is the main holder for all the data.
     *
     * You can create several analysis contexts if you need to, which enables
     * you, for example to:
     *
     * * analyze several different projects at the same time;
     *
     * * analyze different parts of the same projects in parallel.
     *
     * In the current design, contexts always keep all of their analysis units
     * allocated. If you need to get this memory released, the only option at
     * your disposal is to destroy your analysis context instance.
     */
    public static final class AnalysisContext implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton that represents the none analysis context. */
        public static final AnalysisContext NONE = new AnalysisContext(
            PointerWrapper.nullPointer(),
            null
        );

        /** This map contains all created analysis contexts. */
        private static final Map<PointerWrapper, AnalysisContext> contextCache
            = new ConcurrentHashMap<>();

        // ----- Instance attributes -----

        /** The reference to the native analysis context. */
        private final PointerWrapper reference;

        /** The event handler associated with the context. */
        private EventHandler eventHandler;

        // ----- Constructors -----

        /**
         * Create a new analysis unit from its reference.
         *
         * @param reference The native analysis context.
         * @param eventHandler The associated event handler.
         */
        private AnalysisContext(
            final PointerWrapper reference,
            final EventHandler eventHandler
        ) {
            this.reference = reference;
            this.eventHandler = eventHandler;

            increaseRefCounter(this);
        }

        /**
         * Create a new analysis context from scratch with its configuration.
         *
         * @param charset The charset for the analysis context, it can be null.
         * @param fileReader The file reader for the analysis context, it
         * can be null.
         * @param unitProvider The unit provider for the analysis context,
         * it can be null.
         * @param eventHandler The event handler for the analysis context,
         * it can be null.
         * @param withTrivia If the analysis context should include trivias.
         * @param tabStop The effect of the tabulations on the column number.
         */
        private AnalysisContext(
            final String charset,
            final FileReader fileReader,
            final UnitProvider unitProvider,
            final EventHandler eventHandler,
            final boolean withTrivia,
            final int tabStop
        ) {
            // Call the function to allocate the native analysis context
            final PointerWrapper reference;
            if(ImageInfo.inImageCode()) {
                reference = new PointerWrapper(
                    NI_LIB.ada_allocate_analysis_context()
                );
            } else {
                reference = JNI_LIB.ada_create_analysis_context(
                    charset,
                    (fileReader == null ?
                        FileReader.NONE :
                        fileReader),
                    (unitProvider == null ?
                        UnitProvider.NONE :
                        unitProvider),
                    (eventHandler == null ?
                        EventHandler.NONE :
                        eventHandler),
                    withTrivia,
                    tabStop
                );
            }

            // Place the context in the cache for potention callbacks during
            // context initialization.
            this.reference = reference;
            this.eventHandler = eventHandler;
            contextCache.put(this.reference, this);

            // Perform the context initialization
            if(ImageInfo.inImageCode()) {
                final CCharPointer charsetNative =
                    charset == null ?
                    WordFactory.nullPointer() :
                    toCString(charset);

                NI_LIB.ada_initialize_analysis_context(
                    (AnalysisContextNative) reference.ni(),
                    charsetNative,
                    (fileReader == null ?
                        WordFactory.nullPointer() :
                        fileReader.reference.ni()),
                    (unitProvider == null ?
                        WordFactory.nullPointer() :
                        unitProvider.reference.ni()),
                    (eventHandler == null ?
                        WordFactory.nullPointer() :
                        eventHandler.reference.ni()),
                    (withTrivia ? 1 : 0),
                    tabStop
                );
                if(charset != null) UnmanagedMemory.free(charsetNative);
            }
        }

        /**
         * Create a new analysis context with the default parameters.
         *
         * @return The newly create analysis unit.
         */
        public static AnalysisContext create() {
            return new AnalysisContext(
                null,
                null,
                null,
                null,
                true,
                8
            );
        }

        /**
         * Create an analysis context with its parameters.
         *
         * @param charset The charset for the analysis context, it can be null.
         * @param fileReader The file reader for the analysis context, it
         * can be null.
         * @param unitProvider The unit provider for the analysis context,
         * it can be null.
         * @param eventHandler The event handler for the analysis context,
         * it can be null.
         * @param withTrivia If the analysis context should include trivias.
         * @param tabStop The effect of the tabulations on the column number.
         * @return The newly create analysis unit.
         */
        public static AnalysisContext create(
            final String charset,
            final FileReader fileReader,
            final UnitProvider unitProvider,
            final EventHandler eventHandler,
            final boolean withTrivia,
            final int tabStop
        ) {
            return new AnalysisContext(
                charset,
                fileReader,
                unitProvider,
                eventHandler,
                withTrivia,
                tabStop
            );
        }

        /**
         * Get the analysis context object from its reference.
         *
         * @param reference The native reference to the analysis context.
         * @param eventHandler The corresponding event handler.
         * @param setEventHandler Whether to set the result's eventHandler
         * field to eventHandler when there is already a cached
         * AnalysisContext.
         * @return The Java analysis unit associated with the reference.
         */
        static AnalysisContext fromReference(
            final PointerWrapper reference,
            final EventHandler eventHandler,
            final boolean setEventHandler
        ) {
            if(contextCache.containsKey(reference)) {
                final AnalysisContext res = contextCache.get(reference);
                increaseRefCounter(res);
                if(setEventHandler)
                    res.eventHandler = eventHandler;
                return res;
            } else {
                final AnalysisContext result =
                    new AnalysisContext(reference, eventHandler);
                contextCache.put(reference, result);
                return result;
            }
        }

        /**
         * Get the analysis context object from its reference.
         *
         * @param reference The native reference to the analysis context.
         * @return The Java analysis unit associated with the reference.
         */
        static AnalysisContext fromReference(
            final PointerWrapper reference
        ) {
            return fromReference(reference, null, false);
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a native pointer to the native analysis context in the
         * Java class.
         *
         * @param pointer The pointer to the NI analysis context
         * native value.
         * @return The newly wrapped analysis context.
         */
        static AnalysisContext wrap(
            final Pointer pointer
        ) {
            return wrap((AnalysisContextNative) pointer.readWord(0));
        }

        /**
         * Wrap an analysis context native value in the Java class.
         *
         * @param analysisContextNative The NI analysis context native value.
         * @return The newly wrapped analysis context.
         */
        static AnalysisContext wrap(
            final AnalysisContextNative analysisContextNative
        ) {
            return fromReference(new PointerWrapper(analysisContextNative));
        }

        /**
         * Unwrap the analysis context in the given native pointer.
         *
         * @param pointer The pointer to place the native analysis unit.
         */
        void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Get the native value of the analysis context.
         *
         * @return The native analysis context.
         */
        AnalysisContextNative unwrap() {
            return (AnalysisContextNative) this.reference.ni();
        }

        // ----- Getters -----

        public EventHandler getEventHandler() {
            return this.eventHandler;
        }

        // ----- Instance methods -----

        /**
         * Get an analysis unit from the given file in the current context.
         *
         * @param fileName The file to get the analysis unit from.
         * @return The new analysis unit.
         */
        public AnalysisUnit getUnitFromFile(
            final String fileName
        ) {
            return this.getUnitFromFile(
                fileName,
                null,
                false,
                DEFAULT_GRAMMAR_RULE
            );
        }

        /**
         * Get an analysis unit from the given file in the current context
         * with additional parameters.
         *
         * @param fileName The file to get the analysis unit from.
         * @param charset The charset of the given file.
         * @param reparse If the file should be reparsed.
         * @param rule The grammar rule to parse the source with.
         * @return The new analysis unit.
         */
        public AnalysisUnit getUnitFromFile(
            final String fileName,
            final String charset,
            final boolean reparse,
            final GrammarRule rule
        ) {

            if(ImageInfo.inImageCode()) {
                final CCharPointer fileNameNative = toCString(fileName);
                final CCharPointer charsetNative =
                    charset == null ?
                    WordFactory.nullPointer() :
                    toCString(charset);

                final AnalysisUnitNative resNative =
                    NI_LIB.ada_get_analysis_unit_from_file(
                    this.reference.ni(),
                    fileNameNative,
                    charsetNative,
                    (reparse ? 1 : 0),
                    rule.toC()
                );
                UnmanagedMemory.free(fileNameNative);
                if(charset != null) UnmanagedMemory.free(charsetNative);
                return AnalysisUnit.wrap(resNative);
            } else {
                return JNI_LIB.ada_get_analysis_unit_from_file(
                    this,
                    fileName,
                    charset,
                    reparse,
                    rule.toC()
                );
            }

        }

        /**
         * Get an analysis unit from the given buffer in the current context.
         *
         * @param buffer The buffer to parse.
         * @param name The name of the buffer.
         * @return The new analysis unit.
         */
        public AnalysisUnit getUnitFromBuffer(
            final String buffer,
            final String name
        ) {
            return this.getUnitFromBuffer(
                buffer,
                name,
                null,
                DEFAULT_GRAMMAR_RULE
            );
        }

        /**
         * Get an analysis unit from the given buffer in the current context
         * with additional parameters.
         *
         * @param buffer The buffer to parse.
         * @param name The name of the buffer.
         * @param charset The charset of the buffer.
         * @param rule The rule to parse the buffer with.
         * @return The new analysis unit.
         */
        public AnalysisUnit getUnitFromBuffer(
            final String buffer,
            final String name,
            final String charset,
            final GrammarRule rule
        ) {

            if(ImageInfo.inImageCode()) {
                final CCharPointer bufferNative = toCString(buffer);
                final CCharPointer nameNative = toCString(name);
                final CCharPointer charsetNative =
                    charset == null ?
                    WordFactory.nullPointer() :
                    toCString(charset);

                final AnalysisUnitNative resNative =
                    NI_LIB.ada_get_analysis_unit_from_buffer(
                    this.reference.ni(),
                    nameNative,
                    charsetNative,
                    bufferNative,
                    buffer.length(),
                    rule.toC()
                );
                UnmanagedMemory.free(bufferNative);
                UnmanagedMemory.free(nameNative);
                if(charset != null) UnmanagedMemory.free(charsetNative);
                return AnalysisUnit.wrap(resNative);
            } else {
                return JNI_LIB.ada_get_analysis_unit_from_buffer(
                    this,
                    name,
                    charset,
                    buffer,
                    buffer.length(),
                    rule.toC()
                );
            }

        }

        /**
         * Get an analysis unit from the given unit name and unit kind in the
         * current context with additional parameters.
         *
         * @param name Name of the unit.
         * @param kind Kind of the unit.
         * @param charset The charset of the buffer.
         * @param rule The rule to parse the buffer with.
         * @return The new analysis unit.
         */
        public AnalysisUnit getUnitFromProvider(
            final Text name,
            final AnalysisUnitKind kind,
            final String charset,
            final boolean reparse
        ) {
            if(ImageInfo.inImageCode()) {
                TextNative nameNative = StackValue.get(TextNative.class);
                name.unwrap(nameNative);
                final CCharPointer charsetNative =
                    charset == null ?
                    WordFactory.nullPointer() :
                    toCString(charset);
                final AnalysisUnitNative resNative =
                    NI_LIB.ada_get_analysis_unit_from_provider(
                    this.reference.ni(),
                    nameNative,
                    kind.toC(),
                    charsetNative,
                    (reparse ? 1 : 0)
                );
                if(charset != null) UnmanagedMemory.free(charsetNative);
                return AnalysisUnit.wrap(resNative);
            } else {
                return JNI_LIB.ada_get_analysis_unit_from_provider(
                    this,
                    name,
                    kind.toC(),
                    charset,
                    reparse
                );
            }
        }

        public AnalysisUnit getUnitFromProvider(
            final Text name,
            final AnalysisUnitKind kind
        ) {
            return this.getUnitFromProvider(name, kind, "", false);
        }

        /**
         * Increase the reference counter of the given context.
         *
         * @param context The context to increase the reference counter of.
         */
        private static void increaseRefCounter(
            final AnalysisContext context
        ) {
            // Increase the context reference counter of the context if not null
            if(!context.reference.isNull()) {
                if(ImageInfo.inImageCode()) {
                    NI_LIB.ada_context_incref(context.reference.ni());
                } else {
                    JNI_LIB.ada_context_incref(context.reference.jni());
                }
            }
        }

        /** @see java.lang.AutoCloseable#close() */
        @Override
        public void close() {

            if(ImageInfo.inImageCode()) {
                NI_LIB.ada_context_decref(this.reference.ni());
            } else {
                JNI_LIB.ada_context_decref(this.reference.jni());
            }
            checkException();

        }

        
      

        /**
         * Assign in ``Context`` configuration pragmas files to analysis units
         * as described in ``Global_Pragmas`` (configuration pragmas file that
         * applies to all analysis units, or null) and ``Local_Pragmas``
         * (mapping that associates an analysis unit to the local configuration
         * pragmas file that applies to it).
         *
         * This raises a ``Precondition_Failure`` exception if any analysis
         * unit in ``Mapping`` does not belong to ``Context`` or if an analysis
         * unit appears twice as a key in ``Mapping``.
         */
        public void setConfigPragmasMapping(
            AnalysisUnit globalPragmas,
            Map<AnalysisUnit, AnalysisUnit> localPragmas
        ) {
            // Create the flat array for local pragmas that the C API expects
            AnalysisUnit[] locals;
            int i = 0;
            int localCount;
            if (localPragmas == null) {
                localCount = 1;
                locals = new AnalysisUnit[localCount];
            } else {
                localCount = 2 * localPragmas.size() + 1;
                locals = new AnalysisUnit[localCount];
                for (Map.Entry<AnalysisUnit, AnalysisUnit> entry :
                     localPragmas.entrySet()) {
                    AnalysisUnit key = entry.getKey();
                    AnalysisUnit value = entry.getValue();

                    if (key == null)
                        throw new RuntimeException("localPragmas: null key");
                    if (value == null)
                        throw new RuntimeException("localPragmas: null value");

                    locals[i] = key;
                    locals[i + 1] = value;
                    i += 2;
                }
            }
            locals[i] = null;

            if(ImageInfo.inImageCode()) {
                final int unitSize = SizeOf.get(AnalysisUnitNative.class);
                final Pointer localPragmasNative = UnmanagedMemory.calloc(
                    localCount * unitSize
                );
                for (i = 0; i < localCount; ++i) {
                    final AnalysisUnitNative u =
                        locals[i] == null
                        ? WordFactory.nullPointer()
                        : locals[i].unwrap();
                    localPragmasNative.writeWord(unitSize * i, u);
                }

                final AnalysisUnitNative globalPragmasNative =
                    globalPragmas == null
                    ? WordFactory.nullPointer()
                    : globalPragmas.unwrap();

                NI_LIB.ada_set_config_pragmas_mapping(
                    this.reference.ni(),
                    globalPragmasNative,
                    localPragmasNative
                );
            } else {
                JNI_LIB.ada_set_config_pragmas_mapping(
                    this,
                    globalPragmas,
                    locals
                );
            }
            checkException();
        }



    }

    /**
     * This type represents the analysis of a single file.
     *
     * This type has strong-reference semantics and is ref-counted.
     * Furthermore, a reference to a unit contains an implicit reference to the
     * context that owns it. This means that keeping a reference to a unit will
     * keep the context and all the unit it contains allocated.
     */
    public static final class AnalysisUnit {

        // ----- Class attributes -----

        /** Singleton that represents the none analysis unit. */
        public static final AnalysisUnit NONE = new AnalysisUnit(
            PointerWrapper.nullPointer()
        );

        // ----- Instance attributes -----

        /** The reference to the native analysis unit. */
        private final PointerWrapper reference;

        /** The cache for the unit root. */
        private AdaNode root;

        // ----- Constructors -----

        /**
         * Create a new analysis unit from its value.
         *
         * @param reference The native analysis unit native value in
         * a pointer wrapper.
         */
        AnalysisUnit(
            final PointerWrapper reference
        ) {
            this.reference = reference;
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native analysis unit in the Java class.
         *
         * @param pointer The pointer the native analysis unit value.
         * @return The newly wrapped analysis unit.
         */
        static AnalysisUnit wrap(
            final Pointer pointer
        ) {
            return wrap((AnalysisUnitNative) pointer.readWord(0));
        }

        /**
         * Wrap a NI analysis unit native value in the Java class.
         *
         * @param analysisUnitNative The NI analysis unit native value.
         * @return The newly wrapped analysis unit.
         */
        static AnalysisUnit wrap(
            final AnalysisUnitNative analysisUnitNative
        ) {
            return new AnalysisUnit(new PointerWrapper(analysisUnitNative));
        }

        /**
         * Unwrap the analysis unit in the given pointer.
         *
         * @param pointer The pointer to place the native analysis unit in.
         */
        void unwrap(
            final Pointer pointer
        ) {
            pointer.writeWord(0, this.unwrap());
        }

        /**
         * Unwrap the analysis unit as a native value.
         *
         * @return The native analysis unit.
         */
        AnalysisUnitNative unwrap() {
            return (AnalysisUnitNative) this.reference.ni();
        }

        // ----- Instance methods -----

        /**
         * Get the root node of the analysis unit.
         *
         * @return The root node.
         */
        public AdaNode getRoot() {
            if(this.root == null) {

                if(ImageInfo.inImageCode()) {
                    final EntityNative entityNative = StackValue.get(
                        EntityNative.class
                    );
                    NI_LIB.ada_unit_root(
                        this.reference.ni(),
                        entityNative
                    );
                    this.root = AdaNode.fromEntity(
                        Entity.wrap(entityNative)
                    );
                } else {
                    this.root = AdaNode.fromEntity(
                        JNI_LIB.ada_unit_root(this)
                    );
                }

            }
            return this.root;
        }

        /**
         * Get the analysis unit file name with its full path.
         *
         * @return The unit file name.
         */
        public String getFileName() {
            return this.getFileName(true);
        }

        /**
         * Get the unit's file name.
         *
         * @param fullPath If the method should return the
         * file full absolute path.
         * @return The file name.
         */
        @CompilerDirectives.TruffleBoundary
        public String getFileName(
            final boolean fullPath
        ) {
            final String absoluteFile;

            if(ImageInfo.inImageCode()) {
                final CCharPointer resNative = NI_LIB.ada_unit_filename(
                    this.reference.ni()
                );
                absoluteFile = toJString(resNative);
                NI_LIB.ada_free(resNative);
            } else {
                absoluteFile = JNI_LIB.ada_unit_filename(this);
            }

            if(fullPath) {
                return absoluteFile;
            } else {
                return new File(absoluteFile).getName();
            }
        }

        /**
         * Get the number of tokens in the analysis unit.
         *
         * @return The number of token.
         */
        public int getTokenCount() {

            if(ImageInfo.inImageCode()) {
                return NI_LIB.ada_unit_token_count(this.reference.ni());
            } else {
                return JNI_LIB.ada_unit_token_count(this);
            }

        }

        /**
         * Get the number of trivia in the analysis unit.
         *
         * @return The number of trivia.
         */
        public int getTriviaCount() {

            if(ImageInfo.inImageCode()) {
                return NI_LIB.ada_unit_trivia_count(
                    this.reference.ni()
                );
            } else {
                return JNI_LIB.ada_unit_trivia_count(this);
            }

        }

        /**
         * Return the first token of the analysis unit.
         *
         * @return The first token.
         */
        public Token getFirstToken() {

            if(ImageInfo.inImageCode()) {
                final TokenNative tokenNative = StackValue.get(
                    TokenNative.class
                );
                NI_LIB.ada_unit_first_token(
                    this.reference.ni(),
                    tokenNative
                );
                return Token.wrap(tokenNative, this);
            } else {
                return JNI_LIB.ada_unit_first_token(this);
            }

        }

        /**
         * Return the last token of the analysis unit.
         *
         * @return The last token.
         */
        public Token getLastToken() {

            if(ImageInfo.inImageCode()) {
                final TokenNative tokenNative = StackValue.get(
                    TokenNative.class
                );
                NI_LIB.ada_unit_last_token(
                    this.reference.ni(),
                    tokenNative
                );
                return Token.wrap(tokenNative, this);
            } else {
                return JNI_LIB.ada_unit_last_token(this);
            }

        }

        /**
         * Get the text of the analysis unit in a string.
         *
         * @return The text of the analysis unit source.
         */
        public String getText() {
            return Token.textRange(
                this.getFirstToken(),
                this.getLastToken()
            );
        }

        /**
         * Get the analysis context that owns the unit.
         *
         * @return The owning analysis context.
         */
        public AnalysisContext getContext() {

            if(ImageInfo.inImageCode()) {
                final AnalysisContextNative contextNative =
                    NI_LIB.ada_unit_context(this.reference.ni());
                return AnalysisContext.wrap(contextNative);
            } else {
                return JNI_LIB.ada_unit_context(this);
            }

        }

        /**
         * Get the list of assiated diagnositcs. Those are parsing errors.
         *
         * @return The diagnositcs of the unit.
         */
        public List<Diagnostic> getDiagnostics() {
            final int diagnosticCount;

            if(ImageInfo.inImageCode()) {
                diagnosticCount = NI_LIB.ada_unit_diagnostic_count(
                    this.reference.ni()
                );
            } else {
                diagnosticCount = JNI_LIB.ada_unit_diagnostic_count(
                    this
                );
            }

            final List<Diagnostic> res = new ArrayList<>(diagnosticCount);

            if(ImageInfo.inImageCode()) {
                final DiagnosticNative diagnosticNative = StackValue.get(
                    DiagnosticNative.class
                );
                for(int i = 0 ; i < diagnosticCount ; i++) {
                    NI_LIB.ada_unit_diagnostic(
                        this.reference.ni(),
                        i,
                        diagnosticNative
                    );
                    res.add(Diagnostic.wrap(diagnosticNative));
                }
            } else {
                for(int i = 0 ; i < diagnosticCount ; i++) {
                    res.add(
                        JNI_LIB.ada_unit_diagnostic(this, i)
                    );
                }
            }

            return res;
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            return "<AnalysisUnit \"" + this.getFileName(false) + "\">";
        }

        @Override
        public boolean equals(Object o) {
            if(this == o) return true;
            if(!(o instanceof AnalysisUnit)) return false;
            final AnalysisUnit other = (AnalysisUnit) o;
            return other.reference.equals(other.reference);
        }

    }

    // ===== Generated structure wrapping classes =====

        
    
    

    /**

     */
    public static final class Metadata {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final Metadata NONE = new Metadata(
            false,PointerWrapper.nullPointer(),PointerWrapper.nullPointer()
        );

        // ----- Instance attributes -----

        public final
        boolean
        dottableSubp;
        public final
        PointerWrapper
        primitive;
        public final
        PointerWrapper
        primitiveRealType;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        Metadata(
            final boolean dottableSubp,final PointerWrapper primitive,final PointerWrapper primitiveRealType
        ) {
            this.dottableSubp = dottableSubp;
            this.primitive = primitive;
            this.primitiveRealType = primitiveRealType;
        }

        /**
         * Create a new structure with the field values.
         */
        public static Metadata create(
            final boolean dottableSubp,final PointerWrapper primitive,final PointerWrapper primitiveRealType
        ) {
            return new Metadata(
                dottableSubp,primitive,primitiveRealType
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static Metadata wrap(
            final Pointer niPointer
        ) {
            return wrap((MetadataNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static Metadata wrap(
            final MetadataNative structNative
        ) {
            return new Metadata(
                structNative.get_dottable_subp() != 0,PointerWrapper.wrap(structNative.get_primitive()),PointerWrapper.wrap(structNative.get_primitive_real_type())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final MetadataNative structNative
        ) {
            CCharPointer dottable_subpNative = structNative.address_dottable_subp();dottable_subpNative.write(this.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer primitiveNative = structNative.address_primitive();primitiveNative.writeWord(0, this.primitive.ni());
            Pointer primitive_real_typeNative = structNative.address_primitive_real_type();primitive_real_typeNative.writeWord(0, this.primitiveRealType.ni());
        }


    }

        
    
    

    /**

     */
    public static final class EntityInfo {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final EntityInfo NONE = new EntityInfo(
            Metadata.NONE,PointerWrapper.nullPointer(),false
        );

        // ----- Instance attributes -----

        public final
        Metadata
        md;
        public final
        PointerWrapper
        rebindings;
        public final
        boolean
        fromRebound;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        EntityInfo(
            final Metadata md,final PointerWrapper rebindings,final boolean fromRebound
        ) {
            this.md = md;
            this.rebindings = rebindings;
            this.fromRebound = fromRebound;
        }

        /**
         * Create a new structure with the field values.
         */
        public static EntityInfo create(
            final Metadata md,final PointerWrapper rebindings,final boolean fromRebound
        ) {
            return new EntityInfo(
                md,rebindings,fromRebound
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static EntityInfo wrap(
            final Pointer niPointer
        ) {
            return wrap((EntityInfoNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static EntityInfo wrap(
            final EntityInfoNative structNative
        ) {
            return new EntityInfo(
                new Metadata(structNative.get_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_md_primitive()), PointerWrapper.wrap(structNative.get_md_primitive_real_type())),PointerWrapper.wrap(structNative.get_rebindings()),structNative.get_from_rebound() != 0
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final EntityInfoNative structNative
        ) {
            CCharPointer md_dottable_subpNative = structNative.address_md_dottable_subp();md_dottable_subpNative.write(this.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer md_primitiveNative = structNative.address_md_primitive();md_primitiveNative.writeWord(0, this.md.primitive.ni());
            Pointer md_primitive_real_typeNative = structNative.address_md_primitive_real_type();md_primitive_real_typeNative.writeWord(0, this.md.primitiveRealType.ni());
            Pointer rebindingsNative = structNative.address_rebindings();rebindingsNative.writeWord(0, this.rebindings.ni());
            CCharPointer from_reboundNative = structNative.address_from_rebound();from_reboundNative.write(this.fromRebound ? (byte) 1 : (byte) 0);
        }


    }

    
    

    /**

     */
    public static final class Entity {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final Entity NONE = new Entity(
            PointerWrapper.nullPointer(),EntityInfo.NONE
        );

        // ----- Instance attributes -----

        public final
        PointerWrapper
        node;
        public final
        EntityInfo
        info;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        Entity(
            final PointerWrapper node,final EntityInfo info
        ) {
            this.node = node;
            this.info = info;
        }

        /**
         * Create a new structure with the field values.
         */
        public static Entity create(
            final PointerWrapper node,final EntityInfo info
        ) {
            return new Entity(
                node,info
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static Entity wrap(
            final Pointer niPointer
        ) {
            return wrap((EntityNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static Entity wrap(
            final EntityNative structNative
        ) {
            return new Entity(
                PointerWrapper.wrap(structNative.get_node()),new EntityInfo(new Metadata(structNative.get_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_info_md_primitive()), PointerWrapper.wrap(structNative.get_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_info_rebindings()), structNative.get_info_from_rebound() != 0)
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final EntityNative structNative
        ) {
            Pointer nodeNative = structNative.address_node();nodeNative.writeWord(0, this.node.ni());
            CCharPointer info_md_dottable_subpNative = structNative.address_info_md_dottable_subp();info_md_dottable_subpNative.write(this.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer info_md_primitiveNative = structNative.address_info_md_primitive();info_md_primitiveNative.writeWord(0, this.info.md.primitive.ni());
            Pointer info_md_primitive_real_typeNative = structNative.address_info_md_primitive_real_type();info_md_primitive_real_typeNative.writeWord(0, this.info.md.primitiveRealType.ni());
            Pointer info_rebindingsNative = structNative.address_info_rebindings();info_rebindingsNative.writeWord(0, this.info.rebindings.ni());
            CCharPointer info_from_reboundNative = structNative.address_info_from_rebound();info_from_reboundNative.write(this.info.fromRebound ? (byte) 1 : (byte) 0);
        }


    }

        
    
    

    /**
     * Composite field representing the aspect of an entity (:rmlink:`13`).
     */
    public static final class Aspect {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final Aspect NONE = new Aspect(
            false,Entity.NONE,Entity.NONE,false
        );

        // ----- Instance attributes -----

        public final
        boolean
        exists;
        public final
        Entity
        node;
        public final
        Entity
        value;
        public final
        boolean
        inherited;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        Aspect(
            final boolean exists,final Entity node,final Entity value,final boolean inherited
        ) {
            this.exists = exists;
            this.node = node;
            this.value = value;
            this.inherited = inherited;
        }

        /**
         * Create a new structure with the field values.
         */
        public static Aspect create(
            final boolean exists,final Entity node,final Entity value,final boolean inherited
        ) {
            return new Aspect(
                exists,node,value,inherited
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static Aspect wrap(
            final Pointer niPointer
        ) {
            return wrap((AspectNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static Aspect wrap(
            final AspectNative structNative
        ) {
            return new Aspect(
                structNative.get_exists() != 0,new Entity(PointerWrapper.wrap(structNative.get_node_node()), new EntityInfo(new Metadata(structNative.get_node_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_node_info_md_primitive()), PointerWrapper.wrap(structNative.get_node_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_node_info_rebindings()), structNative.get_node_info_from_rebound() != 0)),new Entity(PointerWrapper.wrap(structNative.get_value_node()), new EntityInfo(new Metadata(structNative.get_value_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_value_info_md_primitive()), PointerWrapper.wrap(structNative.get_value_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_value_info_rebindings()), structNative.get_value_info_from_rebound() != 0)),structNative.get_inherited() != 0
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final AspectNative structNative
        ) {
            CCharPointer existsNative = structNative.address_exists();existsNative.write(this.exists ? (byte) 1 : (byte) 0);
            Pointer node_nodeNative = structNative.address_node_node();node_nodeNative.writeWord(0, this.node.node.ni());
            CCharPointer node_info_md_dottable_subpNative = structNative.address_node_info_md_dottable_subp();node_info_md_dottable_subpNative.write(this.node.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer node_info_md_primitiveNative = structNative.address_node_info_md_primitive();node_info_md_primitiveNative.writeWord(0, this.node.info.md.primitive.ni());
            Pointer node_info_md_primitive_real_typeNative = structNative.address_node_info_md_primitive_real_type();node_info_md_primitive_real_typeNative.writeWord(0, this.node.info.md.primitiveRealType.ni());
            Pointer node_info_rebindingsNative = structNative.address_node_info_rebindings();node_info_rebindingsNative.writeWord(0, this.node.info.rebindings.ni());
            CCharPointer node_info_from_reboundNative = structNative.address_node_info_from_rebound();node_info_from_reboundNative.write(this.node.info.fromRebound ? (byte) 1 : (byte) 0);
            Pointer value_nodeNative = structNative.address_value_node();value_nodeNative.writeWord(0, this.value.node.ni());
            CCharPointer value_info_md_dottable_subpNative = structNative.address_value_info_md_dottable_subp();value_info_md_dottable_subpNative.write(this.value.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer value_info_md_primitiveNative = structNative.address_value_info_md_primitive();value_info_md_primitiveNative.writeWord(0, this.value.info.md.primitive.ni());
            Pointer value_info_md_primitive_real_typeNative = structNative.address_value_info_md_primitive_real_type();value_info_md_primitive_real_typeNative.writeWord(0, this.value.info.md.primitiveRealType.ni());
            Pointer value_info_rebindingsNative = structNative.address_value_info_rebindings();value_info_rebindingsNative.writeWord(0, this.value.info.rebindings.ni());
            CCharPointer value_info_from_reboundNative = structNative.address_value_info_from_rebound();value_info_from_reboundNative.write(this.value.info.fromRebound ? (byte) 1 : (byte) 0);
            CCharPointer inheritedNative = structNative.address_inherited();inheritedNative.write(this.inherited ? (byte) 1 : (byte) 0);
        }


    }

        
    
    

    /**

     */
    public static final class CompletionItem {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final CompletionItem NONE = new CompletionItem(
            Entity.NONE,false,false,0
        );

        // ----- Instance attributes -----

        public final
        Entity
        decl;
        public final
        boolean
        isDotCall;
        public final
        boolean
        isVisible;
        public final
        int
        weight;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        CompletionItem(
            final Entity decl,final boolean isDotCall,final boolean isVisible,final int weight
        ) {
            this.decl = decl;
            this.isDotCall = isDotCall;
            this.isVisible = isVisible;
            this.weight = weight;
        }

        /**
         * Create a new structure with the field values.
         */
        public static CompletionItem create(
            final Entity decl,final boolean isDotCall,final boolean isVisible,final int weight
        ) {
            return new CompletionItem(
                decl,isDotCall,isVisible,weight
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static CompletionItem wrap(
            final Pointer niPointer
        ) {
            return wrap((CompletionItemNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static CompletionItem wrap(
            final CompletionItemNative structNative
        ) {
            return new CompletionItem(
                new Entity(PointerWrapper.wrap(structNative.get_decl_node()), new EntityInfo(new Metadata(structNative.get_decl_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_decl_info_md_primitive()), PointerWrapper.wrap(structNative.get_decl_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_decl_info_rebindings()), structNative.get_decl_info_from_rebound() != 0)),structNative.get_is_dot_call() != 0,structNative.get_is_visible() != 0,structNative.get_weight()
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final CompletionItemNative structNative
        ) {
            Pointer decl_nodeNative = structNative.address_decl_node();decl_nodeNative.writeWord(0, this.decl.node.ni());
            CCharPointer decl_info_md_dottable_subpNative = structNative.address_decl_info_md_dottable_subp();decl_info_md_dottable_subpNative.write(this.decl.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer decl_info_md_primitiveNative = structNative.address_decl_info_md_primitive();decl_info_md_primitiveNative.writeWord(0, this.decl.info.md.primitive.ni());
            Pointer decl_info_md_primitive_real_typeNative = structNative.address_decl_info_md_primitive_real_type();decl_info_md_primitive_real_typeNative.writeWord(0, this.decl.info.md.primitiveRealType.ni());
            Pointer decl_info_rebindingsNative = structNative.address_decl_info_rebindings();decl_info_rebindingsNative.writeWord(0, this.decl.info.rebindings.ni());
            CCharPointer decl_info_from_reboundNative = structNative.address_decl_info_from_rebound();decl_info_from_reboundNative.write(this.decl.info.fromRebound ? (byte) 1 : (byte) 0);
            CCharPointer is_dot_callNative = structNative.address_is_dot_call();is_dot_callNative.write(this.isDotCall ? (byte) 1 : (byte) 0);
            CCharPointer is_visibleNative = structNative.address_is_visible();is_visibleNative.write(this.isVisible ? (byte) 1 : (byte) 0);
            CIntPointer weightNative = structNative.address_weight();weightNative.write(this.weight);
        }


    }

        
        
    
    

    /**
     * Represent the range of a discrete type or subtype. The bounds are not
     * evaluated, you need to call ``eval_as_int`` on them, if they're static,
     * to get their value.
     */
    public static final class DiscreteRange {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final DiscreteRange NONE = new DiscreteRange(
            Entity.NONE,Entity.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        lowBound;
        public final
        Entity
        highBound;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        DiscreteRange(
            final Entity lowBound,final Entity highBound
        ) {
            this.lowBound = lowBound;
            this.highBound = highBound;
        }

        /**
         * Create a new structure with the field values.
         */
        public static DiscreteRange create(
            final Entity lowBound,final Entity highBound
        ) {
            return new DiscreteRange(
                lowBound,highBound
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static DiscreteRange wrap(
            final Pointer niPointer
        ) {
            return wrap((DiscreteRangeNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static DiscreteRange wrap(
            final DiscreteRangeNative structNative
        ) {
            return new DiscreteRange(
                new Entity(PointerWrapper.wrap(structNative.get_low_bound_node()), new EntityInfo(new Metadata(structNative.get_low_bound_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_low_bound_info_md_primitive()), PointerWrapper.wrap(structNative.get_low_bound_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_low_bound_info_rebindings()), structNative.get_low_bound_info_from_rebound() != 0)),new Entity(PointerWrapper.wrap(structNative.get_high_bound_node()), new EntityInfo(new Metadata(structNative.get_high_bound_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_high_bound_info_md_primitive()), PointerWrapper.wrap(structNative.get_high_bound_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_high_bound_info_rebindings()), structNative.get_high_bound_info_from_rebound() != 0))
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final DiscreteRangeNative structNative
        ) {
            Pointer low_bound_nodeNative = structNative.address_low_bound_node();low_bound_nodeNative.writeWord(0, this.lowBound.node.ni());
            CCharPointer low_bound_info_md_dottable_subpNative = structNative.address_low_bound_info_md_dottable_subp();low_bound_info_md_dottable_subpNative.write(this.lowBound.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer low_bound_info_md_primitiveNative = structNative.address_low_bound_info_md_primitive();low_bound_info_md_primitiveNative.writeWord(0, this.lowBound.info.md.primitive.ni());
            Pointer low_bound_info_md_primitive_real_typeNative = structNative.address_low_bound_info_md_primitive_real_type();low_bound_info_md_primitive_real_typeNative.writeWord(0, this.lowBound.info.md.primitiveRealType.ni());
            Pointer low_bound_info_rebindingsNative = structNative.address_low_bound_info_rebindings();low_bound_info_rebindingsNative.writeWord(0, this.lowBound.info.rebindings.ni());
            CCharPointer low_bound_info_from_reboundNative = structNative.address_low_bound_info_from_rebound();low_bound_info_from_reboundNative.write(this.lowBound.info.fromRebound ? (byte) 1 : (byte) 0);
            Pointer high_bound_nodeNative = structNative.address_high_bound_node();high_bound_nodeNative.writeWord(0, this.highBound.node.ni());
            CCharPointer high_bound_info_md_dottable_subpNative = structNative.address_high_bound_info_md_dottable_subp();high_bound_info_md_dottable_subpNative.write(this.highBound.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer high_bound_info_md_primitiveNative = structNative.address_high_bound_info_md_primitive();high_bound_info_md_primitiveNative.writeWord(0, this.highBound.info.md.primitive.ni());
            Pointer high_bound_info_md_primitive_real_typeNative = structNative.address_high_bound_info_md_primitive_real_type();high_bound_info_md_primitive_real_typeNative.writeWord(0, this.highBound.info.md.primitiveRealType.ni());
            Pointer high_bound_info_rebindingsNative = structNative.address_high_bound_info_rebindings();high_bound_info_rebindingsNative.writeWord(0, this.highBound.info.rebindings.ni());
            CCharPointer high_bound_info_from_reboundNative = structNative.address_high_bound_info_from_rebound();high_bound_info_from_reboundNative.write(this.highBound.info.fromRebound ? (byte) 1 : (byte) 0);
        }


    }

        
    
    

    /**
     * Represent a set of values (as a list of choices) on a discriminant.
     */
    public static final class DiscriminantValues {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final DiscriminantValues NONE = new DiscriminantValues(
            Entity.NONE,Entity.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        discriminant;
        public final
        Entity
        values;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        DiscriminantValues(
            final Entity discriminant,final Entity values
        ) {
            this.discriminant = discriminant;
            this.values = values;
        }

        /**
         * Create a new structure with the field values.
         */
        public static DiscriminantValues create(
            final Entity discriminant,final Entity values
        ) {
            return new DiscriminantValues(
                discriminant,values
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static DiscriminantValues wrap(
            final Pointer niPointer
        ) {
            return wrap((DiscriminantValuesNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static DiscriminantValues wrap(
            final DiscriminantValuesNative structNative
        ) {
            return new DiscriminantValues(
                new Entity(PointerWrapper.wrap(structNative.get_discriminant_node()), new EntityInfo(new Metadata(structNative.get_discriminant_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_discriminant_info_md_primitive()), PointerWrapper.wrap(structNative.get_discriminant_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_discriminant_info_rebindings()), structNative.get_discriminant_info_from_rebound() != 0)),new Entity(PointerWrapper.wrap(structNative.get_values_node()), new EntityInfo(new Metadata(structNative.get_values_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_values_info_md_primitive()), PointerWrapper.wrap(structNative.get_values_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_values_info_rebindings()), structNative.get_values_info_from_rebound() != 0))
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final DiscriminantValuesNative structNative
        ) {
            Pointer discriminant_nodeNative = structNative.address_discriminant_node();discriminant_nodeNative.writeWord(0, this.discriminant.node.ni());
            CCharPointer discriminant_info_md_dottable_subpNative = structNative.address_discriminant_info_md_dottable_subp();discriminant_info_md_dottable_subpNative.write(this.discriminant.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer discriminant_info_md_primitiveNative = structNative.address_discriminant_info_md_primitive();discriminant_info_md_primitiveNative.writeWord(0, this.discriminant.info.md.primitive.ni());
            Pointer discriminant_info_md_primitive_real_typeNative = structNative.address_discriminant_info_md_primitive_real_type();discriminant_info_md_primitive_real_typeNative.writeWord(0, this.discriminant.info.md.primitiveRealType.ni());
            Pointer discriminant_info_rebindingsNative = structNative.address_discriminant_info_rebindings();discriminant_info_rebindingsNative.writeWord(0, this.discriminant.info.rebindings.ni());
            CCharPointer discriminant_info_from_reboundNative = structNative.address_discriminant_info_from_rebound();discriminant_info_from_reboundNative.write(this.discriminant.info.fromRebound ? (byte) 1 : (byte) 0);
            Pointer values_nodeNative = structNative.address_values_node();values_nodeNative.writeWord(0, this.values.node.ni());
            CCharPointer values_info_md_dottable_subpNative = structNative.address_values_info_md_dottable_subp();values_info_md_dottable_subpNative.write(this.values.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer values_info_md_primitiveNative = structNative.address_values_info_md_primitive();values_info_md_primitiveNative.writeWord(0, this.values.info.md.primitive.ni());
            Pointer values_info_md_primitive_real_typeNative = structNative.address_values_info_md_primitive_real_type();values_info_md_primitive_real_typeNative.writeWord(0, this.values.info.md.primitiveRealType.ni());
            Pointer values_info_rebindingsNative = structNative.address_values_info_rebindings();values_info_rebindingsNative.writeWord(0, this.values.info.rebindings.ni());
            CCharPointer values_info_from_reboundNative = structNative.address_values_info_from_rebound();values_info_from_reboundNative.write(this.values.info.fromRebound ? (byte) 1 : (byte) 0);
        }


    }

        
    
    

    /**
     * Documentation annotation.
     */
    public static final class DocAnnotation {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final DocAnnotation NONE = new DocAnnotation(
            StringWrapper.NONE,StringWrapper.NONE
        );

        // ----- Instance attributes -----

        public final
        String
        key;
        public final
        String
        value;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        DocAnnotation(
            final String key,final String value
        ) {
            this.key = key;
            this.value = value;
        }

        /**
         * Create a new structure with the field values.
         */
        public static DocAnnotation create(
            final String key,final String value
        ) {
            return new DocAnnotation(
                key,value
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static DocAnnotation wrap(
            final Pointer niPointer
        ) {
            return wrap((DocAnnotationNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static DocAnnotation wrap(
            final DocAnnotationNative structNative
        ) {
            return new DocAnnotation(
                StringWrapper.wrap(structNative.get_key()),StringWrapper.wrap(structNative.get_value())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final DocAnnotationNative structNative
        ) {
            Pointer keyNative = structNative.address_key();StringWrapper.unwrap(this.key, keyNative);
            Pointer valueNative = structNative.address_value();StringWrapper.unwrap(this.value, valueNative);
        }

        /**
         * Release the structure.
         *
         * @param structNative The native structure to release.
         */
        static void release(DocAnnotationNative structNative) {
            NI_LIB.ada_internal_doc_annotation_dec_ref(structNative);
        }

    }

        
        
        
        
        
        
        
    
    

    /**
     * Data structure used by zip_with_params, Name.call_params,
     * GenericInstantiation.inst_params, BaseAggregate.aggregate_params,
     * SubtypeIndication.subtype_constraints, and EnumRepClause.params
     * properties. Associates an expression (the actual) to a formal param
     * declaration (the parameter).
     */
    public static final class ParamActual {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final ParamActual NONE = new ParamActual(
            Entity.NONE,Entity.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        param;
        public final
        Entity
        actual;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        ParamActual(
            final Entity param,final Entity actual
        ) {
            this.param = param;
            this.actual = actual;
        }

        /**
         * Create a new structure with the field values.
         */
        public static ParamActual create(
            final Entity param,final Entity actual
        ) {
            return new ParamActual(
                param,actual
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static ParamActual wrap(
            final Pointer niPointer
        ) {
            return wrap((ParamActualNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static ParamActual wrap(
            final ParamActualNative structNative
        ) {
            return new ParamActual(
                new Entity(PointerWrapper.wrap(structNative.get_param_node()), new EntityInfo(new Metadata(structNative.get_param_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_param_info_md_primitive()), PointerWrapper.wrap(structNative.get_param_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_param_info_rebindings()), structNative.get_param_info_from_rebound() != 0)),new Entity(PointerWrapper.wrap(structNative.get_actual_node()), new EntityInfo(new Metadata(structNative.get_actual_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_actual_info_md_primitive()), PointerWrapper.wrap(structNative.get_actual_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_actual_info_rebindings()), structNative.get_actual_info_from_rebound() != 0))
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final ParamActualNative structNative
        ) {
            Pointer param_nodeNative = structNative.address_param_node();param_nodeNative.writeWord(0, this.param.node.ni());
            CCharPointer param_info_md_dottable_subpNative = structNative.address_param_info_md_dottable_subp();param_info_md_dottable_subpNative.write(this.param.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer param_info_md_primitiveNative = structNative.address_param_info_md_primitive();param_info_md_primitiveNative.writeWord(0, this.param.info.md.primitive.ni());
            Pointer param_info_md_primitive_real_typeNative = structNative.address_param_info_md_primitive_real_type();param_info_md_primitive_real_typeNative.writeWord(0, this.param.info.md.primitiveRealType.ni());
            Pointer param_info_rebindingsNative = structNative.address_param_info_rebindings();param_info_rebindingsNative.writeWord(0, this.param.info.rebindings.ni());
            CCharPointer param_info_from_reboundNative = structNative.address_param_info_from_rebound();param_info_from_reboundNative.write(this.param.info.fromRebound ? (byte) 1 : (byte) 0);
            Pointer actual_nodeNative = structNative.address_actual_node();actual_nodeNative.writeWord(0, this.actual.node.ni());
            CCharPointer actual_info_md_dottable_subpNative = structNative.address_actual_info_md_dottable_subp();actual_info_md_dottable_subpNative.write(this.actual.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer actual_info_md_primitiveNative = structNative.address_actual_info_md_primitive();actual_info_md_primitiveNative.writeWord(0, this.actual.info.md.primitive.ni());
            Pointer actual_info_md_primitive_real_typeNative = structNative.address_actual_info_md_primitive_real_type();actual_info_md_primitive_real_typeNative.writeWord(0, this.actual.info.md.primitiveRealType.ni());
            Pointer actual_info_rebindingsNative = structNative.address_actual_info_rebindings();actual_info_rebindingsNative.writeWord(0, this.actual.info.rebindings.ni());
            CCharPointer actual_info_from_reboundNative = structNative.address_actual_info_from_rebound();actual_info_from_reboundNative.write(this.actual.info.fromRebound ? (byte) 1 : (byte) 0);
        }


    }

        
        
        
    
    

    /**
     * Result for a cross reference query returning a reference.
     */
    public static final class RefResult {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final RefResult NONE = new RefResult(
            Entity.NONE,RefResultKind.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        ref;
        public final
        RefResultKind
        kind;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        RefResult(
            final Entity ref,final RefResultKind kind
        ) {
            this.ref = ref;
            this.kind = kind;
        }

        /**
         * Create a new structure with the field values.
         */
        public static RefResult create(
            final Entity ref,final RefResultKind kind
        ) {
            return new RefResult(
                ref,kind
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefResult wrap(
            final Pointer niPointer
        ) {
            return wrap((RefResultNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefResult wrap(
            final RefResultNative structNative
        ) {
            return new RefResult(
                new Entity(PointerWrapper.wrap(structNative.get_ref_node()), new EntityInfo(new Metadata(structNative.get_ref_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_ref_info_md_primitive()), PointerWrapper.wrap(structNative.get_ref_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_ref_info_rebindings()), structNative.get_ref_info_from_rebound() != 0)),RefResultKind.fromC(structNative.get_kind())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final RefResultNative structNative
        ) {
            Pointer ref_nodeNative = structNative.address_ref_node();ref_nodeNative.writeWord(0, this.ref.node.ni());
            CCharPointer ref_info_md_dottable_subpNative = structNative.address_ref_info_md_dottable_subp();ref_info_md_dottable_subpNative.write(this.ref.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer ref_info_md_primitiveNative = structNative.address_ref_info_md_primitive();ref_info_md_primitiveNative.writeWord(0, this.ref.info.md.primitive.ni());
            Pointer ref_info_md_primitive_real_typeNative = structNative.address_ref_info_md_primitive_real_type();ref_info_md_primitive_real_typeNative.writeWord(0, this.ref.info.md.primitiveRealType.ni());
            Pointer ref_info_rebindingsNative = structNative.address_ref_info_rebindings();ref_info_rebindingsNative.writeWord(0, this.ref.info.rebindings.ni());
            CCharPointer ref_info_from_reboundNative = structNative.address_ref_info_from_rebound();ref_info_from_reboundNative.write(this.ref.info.fromRebound ? (byte) 1 : (byte) 0);
            CIntPointer kindNative = structNative.address_kind();kindNative.write(this.kind.toC());
        }


    }

        
    
    

    /**
     * Result for a cross reference query returning a referenced decl.
     */
    public static final class RefdDecl {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final RefdDecl NONE = new RefdDecl(
            Entity.NONE,RefResultKind.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        decl;
        public final
        RefResultKind
        kind;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        RefdDecl(
            final Entity decl,final RefResultKind kind
        ) {
            this.decl = decl;
            this.kind = kind;
        }

        /**
         * Create a new structure with the field values.
         */
        public static RefdDecl create(
            final Entity decl,final RefResultKind kind
        ) {
            return new RefdDecl(
                decl,kind
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefdDecl wrap(
            final Pointer niPointer
        ) {
            return wrap((RefdDeclNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefdDecl wrap(
            final RefdDeclNative structNative
        ) {
            return new RefdDecl(
                new Entity(PointerWrapper.wrap(structNative.get_decl_node()), new EntityInfo(new Metadata(structNative.get_decl_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_decl_info_md_primitive()), PointerWrapper.wrap(structNative.get_decl_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_decl_info_rebindings()), structNative.get_decl_info_from_rebound() != 0)),RefResultKind.fromC(structNative.get_kind())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final RefdDeclNative structNative
        ) {
            Pointer decl_nodeNative = structNative.address_decl_node();decl_nodeNative.writeWord(0, this.decl.node.ni());
            CCharPointer decl_info_md_dottable_subpNative = structNative.address_decl_info_md_dottable_subp();decl_info_md_dottable_subpNative.write(this.decl.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer decl_info_md_primitiveNative = structNative.address_decl_info_md_primitive();decl_info_md_primitiveNative.writeWord(0, this.decl.info.md.primitive.ni());
            Pointer decl_info_md_primitive_real_typeNative = structNative.address_decl_info_md_primitive_real_type();decl_info_md_primitive_real_typeNative.writeWord(0, this.decl.info.md.primitiveRealType.ni());
            Pointer decl_info_rebindingsNative = structNative.address_decl_info_rebindings();decl_info_rebindingsNative.writeWord(0, this.decl.info.rebindings.ni());
            CCharPointer decl_info_from_reboundNative = structNative.address_decl_info_from_rebound();decl_info_from_reboundNative.write(this.decl.info.fromRebound ? (byte) 1 : (byte) 0);
            CIntPointer kindNative = structNative.address_kind();kindNative.write(this.kind.toC());
        }


    }

        
    
    

    /**
     * Result for a cross reference query returning a referenced defining name.
     */
    public static final class RefdDef {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final RefdDef NONE = new RefdDef(
            Entity.NONE,RefResultKind.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        defName;
        public final
        RefResultKind
        kind;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        RefdDef(
            final Entity defName,final RefResultKind kind
        ) {
            this.defName = defName;
            this.kind = kind;
        }

        /**
         * Create a new structure with the field values.
         */
        public static RefdDef create(
            final Entity defName,final RefResultKind kind
        ) {
            return new RefdDef(
                defName,kind
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefdDef wrap(
            final Pointer niPointer
        ) {
            return wrap((RefdDefNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static RefdDef wrap(
            final RefdDefNative structNative
        ) {
            return new RefdDef(
                new Entity(PointerWrapper.wrap(structNative.get_def_name_node()), new EntityInfo(new Metadata(structNative.get_def_name_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_def_name_info_md_primitive()), PointerWrapper.wrap(structNative.get_def_name_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_def_name_info_rebindings()), structNative.get_def_name_info_from_rebound() != 0)),RefResultKind.fromC(structNative.get_kind())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final RefdDefNative structNative
        ) {
            Pointer def_name_nodeNative = structNative.address_def_name_node();def_name_nodeNative.writeWord(0, this.defName.node.ni());
            CCharPointer def_name_info_md_dottable_subpNative = structNative.address_def_name_info_md_dottable_subp();def_name_info_md_dottable_subpNative.write(this.defName.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer def_name_info_md_primitiveNative = structNative.address_def_name_info_md_primitive();def_name_info_md_primitiveNative.writeWord(0, this.defName.info.md.primitive.ni());
            Pointer def_name_info_md_primitive_real_typeNative = structNative.address_def_name_info_md_primitive_real_type();def_name_info_md_primitive_real_typeNative.writeWord(0, this.defName.info.md.primitiveRealType.ni());
            Pointer def_name_info_rebindingsNative = structNative.address_def_name_info_rebindings();def_name_info_rebindingsNative.writeWord(0, this.defName.info.rebindings.ni());
            CCharPointer def_name_info_from_reboundNative = structNative.address_def_name_info_from_rebound();def_name_info_from_reboundNative.write(this.defName.info.fromRebound ? (byte) 1 : (byte) 0);
            CIntPointer kindNative = structNative.address_kind();kindNative.write(this.kind.toC());
        }


    }

        
    
    

    /**
     * Represent one of the shapes that a variant record can have, as a list of
     * the available components.
     */
    public static final class Shape {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final Shape NONE = new Shape(
            AdaNodeArray.NONE,DiscriminantValuesArray.NONE
        );

        // ----- Instance attributes -----

        public final
        AdaNodeArray
        components;
        public final
        DiscriminantValuesArray
        discriminantsValues;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        Shape(
            final AdaNodeArray components,final DiscriminantValuesArray discriminantsValues
        ) {
            this.components = components;
            this.discriminantsValues = discriminantsValues;
        }

        /**
         * Create a new structure with the field values.
         */
        public static Shape create(
            final AdaNodeArray components,final DiscriminantValuesArray discriminantsValues
        ) {
            return new Shape(
                components,discriminantsValues
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static Shape wrap(
            final Pointer niPointer
        ) {
            return wrap((ShapeNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static Shape wrap(
            final ShapeNative structNative
        ) {
            return new Shape(
                AdaNodeArray.wrap(structNative.get_components()),DiscriminantValuesArray.wrap(structNative.get_discriminants_values())
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final ShapeNative structNative
        ) {
            Pointer componentsNative = structNative.address_components();this.components.unwrap(componentsNative);
            Pointer discriminants_valuesNative = structNative.address_discriminants_values();this.discriminantsValues.unwrap(discriminants_valuesNative);
        }

        /**
         * Release the structure.
         *
         * @param structNative The native structure to release.
         */
        static void release(ShapeNative structNative) {
            NI_LIB.ada_internal_shape_dec_ref(structNative);
        }

    }

        
    
    

    /**
     * Represent a substitution of a BasicDecl by a given value. This can then
     * be used as part of an environment in the eval_as_*_in_env property. See
     * the declaration of those properties for more details.
     */
    public static final class Substitution {

        // ----- Class attributes -----

        /** Singleton that represents the none value for the structure. */
        public static final Substitution NONE = new Substitution(
            Entity.NONE,BigIntegerWrapper.NONE,Entity.NONE
        );

        // ----- Instance attributes -----

        public final
        Entity
        fromDecl;
        public final
        BigInteger
        toValue;
        public final
        Entity
        valueType;

        // ----- Constructors -----

        /**
         * Create a new structure object from the value if its fields.
         */
        Substitution(
            final Entity fromDecl,final BigInteger toValue,final Entity valueType
        ) {
            this.fromDecl = fromDecl;
            this.toValue = toValue;
            this.valueType = valueType;
        }

        /**
         * Create a new structure with the field values.
         */
        public static Substitution create(
            final Entity fromDecl,final BigInteger toValue,final Entity valueType
        ) {
            return new Substitution(
                fromDecl,toValue,valueType
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to the native structure value in the Java class.
         *
         * @param niPointer The pointer to the NI structure native value.
         * @return The newly wrapped structure.
         */
        static Substitution wrap(
            final Pointer niPointer
        ) {
            return wrap((SubstitutionNative) niPointer.readWord(0));
        }

        /**
         * Wrap the given structure native value in the Java class.
         *
         * @param structNative The NI structure native value.
         * @return The newly wrapped structure.
         */
        static Substitution wrap(
            final SubstitutionNative structNative
        ) {
            return new Substitution(
                new Entity(PointerWrapper.wrap(structNative.get_from_decl_node()), new EntityInfo(new Metadata(structNative.get_from_decl_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_from_decl_info_md_primitive()), PointerWrapper.wrap(structNative.get_from_decl_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_from_decl_info_rebindings()), structNative.get_from_decl_info_from_rebound() != 0)),BigIntegerWrapper.wrap(structNative.get_to_value()),new Entity(PointerWrapper.wrap(structNative.get_value_type_node()), new EntityInfo(new Metadata(structNative.get_value_type_info_md_dottable_subp() != 0, PointerWrapper.wrap(structNative.get_value_type_info_md_primitive()), PointerWrapper.wrap(structNative.get_value_type_info_md_primitive_real_type())), PointerWrapper.wrap(structNative.get_value_type_info_rebindings()), structNative.get_value_type_info_from_rebound() != 0))
            );
        }

        /**
         * Unwrap the structure in the given native value.
         *
         * @param structNative The NI structure native value to fill.
         */
        void unwrap(
            final SubstitutionNative structNative
        ) {
            Pointer from_decl_nodeNative = structNative.address_from_decl_node();from_decl_nodeNative.writeWord(0, this.fromDecl.node.ni());
            CCharPointer from_decl_info_md_dottable_subpNative = structNative.address_from_decl_info_md_dottable_subp();from_decl_info_md_dottable_subpNative.write(this.fromDecl.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer from_decl_info_md_primitiveNative = structNative.address_from_decl_info_md_primitive();from_decl_info_md_primitiveNative.writeWord(0, this.fromDecl.info.md.primitive.ni());
            Pointer from_decl_info_md_primitive_real_typeNative = structNative.address_from_decl_info_md_primitive_real_type();from_decl_info_md_primitive_real_typeNative.writeWord(0, this.fromDecl.info.md.primitiveRealType.ni());
            Pointer from_decl_info_rebindingsNative = structNative.address_from_decl_info_rebindings();from_decl_info_rebindingsNative.writeWord(0, this.fromDecl.info.rebindings.ni());
            CCharPointer from_decl_info_from_reboundNative = structNative.address_from_decl_info_from_rebound();from_decl_info_from_reboundNative.write(this.fromDecl.info.fromRebound ? (byte) 1 : (byte) 0);
            Pointer to_valueNative = structNative.address_to_value();BigIntegerWrapper.unwrap(this.toValue, to_valueNative);
            Pointer value_type_nodeNative = structNative.address_value_type_node();value_type_nodeNative.writeWord(0, this.valueType.node.ni());
            CCharPointer value_type_info_md_dottable_subpNative = structNative.address_value_type_info_md_dottable_subp();value_type_info_md_dottable_subpNative.write(this.valueType.info.md.dottableSubp ? (byte) 1 : (byte) 0);
            Pointer value_type_info_md_primitiveNative = structNative.address_value_type_info_md_primitive();value_type_info_md_primitiveNative.writeWord(0, this.valueType.info.md.primitive.ni());
            Pointer value_type_info_md_primitive_real_typeNative = structNative.address_value_type_info_md_primitive_real_type();value_type_info_md_primitive_real_typeNative.writeWord(0, this.valueType.info.md.primitiveRealType.ni());
            Pointer value_type_info_rebindingsNative = structNative.address_value_type_info_rebindings();value_type_info_rebindingsNative.writeWord(0, this.valueType.info.rebindings.ni());
            CCharPointer value_type_info_from_reboundNative = structNative.address_value_type_info_from_rebound();value_type_info_from_reboundNative.write(this.valueType.info.fromRebound ? (byte) 1 : (byte) 0);
        }

        /**
         * Release the structure.
         *
         * @param structNative The native structure to release.
         */
        static void release(SubstitutionNative structNative) {
            NI_LIB.ada_internal_substitution_dec_ref(structNative);
        }

    }


    // ===== Generated array wrapping classes =====

    /**
     * This class is the base of all array wrapping class.
     */
    public static abstract class ArrayBase<T> implements Iterable<T> {

        // ----- Attributes -----

        /** The content of the array. */
        protected final T[] content;

        // ----- Constructors -----

        /**
         * Protected constructor.
         */
        protected ArrayBase(
            final T[] content
        ) {
            this.content = content;
        }

        // ----- Instance methods -----

        /**
         * Get the size of the array.
         *
         * @return The size of the native array.
         */
        public int size() {
            return this.content.length;
        }

        /**
         * Get the element at the given place in the array.
         *
         * @param i The index of the element to get.
         * @return The element at the given index.
         * @throws ArrayIndexOutOfBoundsException If the requested index is
         * out of bounds.
         */
        public T get(
            final int i
        ) {
            return this.content[i];
        }

        /**
         * Set the element at the given index.
         *
         * @param i The index of the element to set.
         * @param elem The element to place in the array.
         * @throws ArrayIndexOutOfBoundsException If the requested index is
         * out of bounds.
         */
        public void set(
            final int i,
            final T elem
        ) {
            this.content[i] = elem;
        }

        /** @see java.lang.Iterable#iterator() */
        @Override
        public Iterator<T> iterator() {
            return new LangkitArrayIterator<T>(this);
        }

        // ----- Override methods -----

        @Override
        public String toString() {
            final StringBuilder res = new StringBuilder("[");
            for(int i = 0 ; i < this.size() ; i++) {
                res.append(this.get(i).toString());
                if(i < this.size() - 1) res.append(", ");
            }
            res.append(']');
            return res.toString();
        }

        // ----- Inner classes -----

        /**
         * The iterator class for the langkit arrays
         */
        protected static class LangkitArrayIterator<U>
        implements Iterator<U> {

            // ----- Attributes -----

            /** The array to iterate on. */
            private final ArrayBase<U> array;

            /** The current index. */
            private int index;

            // ----- Constructors -----

            /**
             * Create a new array iterator.
             *
             * @param array The array to iterate on.
             */
            LangkitArrayIterator(
                final ArrayBase<U> array
            ) {
                this.array = array;
                this.index = 0;
            }

            // ----- Instance methods -----

            /** @see java.util.Iterator#hasNext() */
            @Override
            public boolean hasNext() {
                return this.index < this.array.size();
            }

            /** @see java.util.Iterator#next() */
            @Override
            public U next() {
                return this.array.get(this.index++);
            }

        }

    }

    
    

    /**
     * This class represents the ada_discriminant_values_array Java wrapping class
     */
    public static final class
    DiscriminantValuesArray extends ArrayBase<DiscriminantValues> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final DiscriminantValuesArray NONE = new DiscriminantValuesArray(
            new DiscriminantValues[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        DiscriminantValuesArray(
            final DiscriminantValues[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static DiscriminantValuesArray jniCreate(
            final DiscriminantValues[] jniContent
        ) {
            final DiscriminantValues[] content =
                new DiscriminantValues[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new DiscriminantValuesArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static DiscriminantValuesArray create(
            final int size
        ) {
            return new DiscriminantValuesArray(
                new DiscriminantValues[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static DiscriminantValuesArray wrap(
            final Pointer pointer
        ) {
            return wrap((DiscriminantValuesArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static DiscriminantValuesArray wrap(
            final DiscriminantValuesArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final DiscriminantValues[] content = new DiscriminantValues[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            DiscriminantValuesNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(DiscriminantValuesNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = DiscriminantValues.wrap(toRead);
            }

            // Return the new langkit array
            return new DiscriminantValuesArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final DiscriminantValuesArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        DiscriminantValuesArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final DiscriminantValuesArrayNative res = NI_LIB.ada_discriminant_values_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            DiscriminantValuesNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(DiscriminantValuesNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((DiscriminantValuesArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final DiscriminantValuesArrayNative arrayNative
        ) {
            NI_LIB.ada_discriminant_values_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private DiscriminantValues[] jniContent() {
            final DiscriminantValues[] res =
                new DiscriminantValues[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_doc_annotation_array Java wrapping class
     */
    public static final class
    DocAnnotationArray extends ArrayBase<DocAnnotation> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final DocAnnotationArray NONE = new DocAnnotationArray(
            new DocAnnotation[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        DocAnnotationArray(
            final DocAnnotation[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static DocAnnotationArray jniCreate(
            final DocAnnotation[] jniContent
        ) {
            final DocAnnotation[] content =
                new DocAnnotation[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new DocAnnotationArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static DocAnnotationArray create(
            final int size
        ) {
            return new DocAnnotationArray(
                new DocAnnotation[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static DocAnnotationArray wrap(
            final Pointer pointer
        ) {
            return wrap((DocAnnotationArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static DocAnnotationArray wrap(
            final DocAnnotationArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final DocAnnotation[] content = new DocAnnotation[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            DocAnnotationNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(DocAnnotationNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = DocAnnotation.wrap(toRead);
            }

            // Return the new langkit array
            return new DocAnnotationArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final DocAnnotationArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        DocAnnotationArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final DocAnnotationArrayNative res = NI_LIB.ada_doc_annotation_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            DocAnnotationNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(DocAnnotationNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((DocAnnotationArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final DocAnnotationArrayNative arrayNative
        ) {
            NI_LIB.ada_doc_annotation_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private DocAnnotation[] jniContent() {
            final DocAnnotation[] res =
                new DocAnnotation[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_ada_node_array Java wrapping class
     */
    public static final class
    AdaNodeArray extends ArrayBase<AdaNode> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final AdaNodeArray NONE = new AdaNodeArray(
            new AdaNode[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        AdaNodeArray(
            final AdaNode[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static AdaNodeArray jniCreate(
            final Entity[] jniContent
        ) {
            final AdaNode[] content =
                new AdaNode[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    AdaNode.fromEntity(jniContent[i]);
            }
            return new AdaNodeArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static AdaNodeArray create(
            final int size
        ) {
            return new AdaNodeArray(
                new AdaNode[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static AdaNodeArray wrap(
            final Pointer pointer
        ) {
            return wrap((AdaNodeArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static AdaNodeArray wrap(
            final AdaNodeArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final AdaNode[] content = new AdaNode[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            EntityNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(EntityNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = AdaNode.fromEntity(Entity.wrap(toRead));
            }

            // Return the new langkit array
            return new AdaNodeArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final AdaNodeArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        AdaNodeArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final AdaNodeArrayNative res = NI_LIB.ada_ada_node_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            EntityNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(EntityNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].entity.unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((AdaNodeArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final AdaNodeArrayNative arrayNative
        ) {
            NI_LIB.ada_ada_node_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private Entity[] jniContent() {
            final Entity[] res =
                new Entity[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = (this.content[i] != null ? this.content[i].entity : null);
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_param_actual_array Java wrapping class
     */
    public static final class
    ParamActualArray extends ArrayBase<ParamActual> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final ParamActualArray NONE = new ParamActualArray(
            new ParamActual[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        ParamActualArray(
            final ParamActual[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static ParamActualArray jniCreate(
            final ParamActual[] jniContent
        ) {
            final ParamActual[] content =
                new ParamActual[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new ParamActualArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static ParamActualArray create(
            final int size
        ) {
            return new ParamActualArray(
                new ParamActual[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static ParamActualArray wrap(
            final Pointer pointer
        ) {
            return wrap((ParamActualArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static ParamActualArray wrap(
            final ParamActualArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final ParamActual[] content = new ParamActual[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            ParamActualNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(ParamActualNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = ParamActual.wrap(toRead);
            }

            // Return the new langkit array
            return new ParamActualArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final ParamActualArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        ParamActualArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final ParamActualArrayNative res = NI_LIB.ada_param_actual_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            ParamActualNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(ParamActualNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((ParamActualArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final ParamActualArrayNative arrayNative
        ) {
            NI_LIB.ada_param_actual_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private ParamActual[] jniContent() {
            final ParamActual[] res =
                new ParamActual[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_ref_result_array Java wrapping class
     */
    public static final class
    RefResultArray extends ArrayBase<RefResult> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final RefResultArray NONE = new RefResultArray(
            new RefResult[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        RefResultArray(
            final RefResult[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static RefResultArray jniCreate(
            final RefResult[] jniContent
        ) {
            final RefResult[] content =
                new RefResult[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new RefResultArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static RefResultArray create(
            final int size
        ) {
            return new RefResultArray(
                new RefResult[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static RefResultArray wrap(
            final Pointer pointer
        ) {
            return wrap((RefResultArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static RefResultArray wrap(
            final RefResultArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final RefResult[] content = new RefResult[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            RefResultNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(RefResultNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = RefResult.wrap(toRead);
            }

            // Return the new langkit array
            return new RefResultArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final RefResultArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        RefResultArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final RefResultArrayNative res = NI_LIB.ada_ref_result_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            RefResultNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(RefResultNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((RefResultArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final RefResultArrayNative arrayNative
        ) {
            NI_LIB.ada_ref_result_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private RefResult[] jniContent() {
            final RefResult[] res =
                new RefResult[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_shape_array Java wrapping class
     */
    public static final class
    ShapeArray extends ArrayBase<Shape> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final ShapeArray NONE = new ShapeArray(
            new Shape[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        ShapeArray(
            final Shape[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static ShapeArray jniCreate(
            final Shape[] jniContent
        ) {
            final Shape[] content =
                new Shape[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new ShapeArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static ShapeArray create(
            final int size
        ) {
            return new ShapeArray(
                new Shape[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static ShapeArray wrap(
            final Pointer pointer
        ) {
            return wrap((ShapeArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static ShapeArray wrap(
            final ShapeArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final Shape[] content = new Shape[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            ShapeNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(ShapeNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = Shape.wrap(toRead);
            }

            // Return the new langkit array
            return new ShapeArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final ShapeArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        ShapeArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final ShapeArrayNative res = NI_LIB.ada_shape_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            ShapeNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(ShapeNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((ShapeArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final ShapeArrayNative arrayNative
        ) {
            NI_LIB.ada_shape_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private Shape[] jniContent() {
            final Shape[] res =
                new Shape[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_substitution_array Java wrapping class
     */
    public static final class
    SubstitutionArray extends ArrayBase<Substitution> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final SubstitutionArray NONE = new SubstitutionArray(
            new Substitution[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        SubstitutionArray(
            final Substitution[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static SubstitutionArray jniCreate(
            final Substitution[] jniContent
        ) {
            final Substitution[] content =
                new Substitution[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new SubstitutionArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static SubstitutionArray create(
            final int size
        ) {
            return new SubstitutionArray(
                new Substitution[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static SubstitutionArray wrap(
            final Pointer pointer
        ) {
            return wrap((SubstitutionArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static SubstitutionArray wrap(
            final SubstitutionArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final Substitution[] content = new Substitution[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            SubstitutionNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(SubstitutionNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = Substitution.wrap(toRead);
            }

            // Return the new langkit array
            return new SubstitutionArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final SubstitutionArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        SubstitutionArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final SubstitutionArrayNative res = NI_LIB.ada_substitution_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            SubstitutionNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(SubstitutionNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((SubstitutionArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final SubstitutionArrayNative arrayNative
        ) {
            NI_LIB.ada_substitution_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private Substitution[] jniContent() {
            final Substitution[] res =
                new Substitution[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_analysis_unit_array Java wrapping class
     */
    public static final class
    AnalysisUnitArray extends ArrayBase<AnalysisUnit> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final AnalysisUnitArray NONE = new AnalysisUnitArray(
            new AnalysisUnit[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        AnalysisUnitArray(
            final AnalysisUnit[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static AnalysisUnitArray jniCreate(
            final AnalysisUnit[] jniContent
        ) {
            final AnalysisUnit[] content =
                new AnalysisUnit[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new AnalysisUnitArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static AnalysisUnitArray create(
            final int size
        ) {
            return new AnalysisUnitArray(
                new AnalysisUnit[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static AnalysisUnitArray wrap(
            final Pointer pointer
        ) {
            return wrap((AnalysisUnitArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static AnalysisUnitArray wrap(
            final AnalysisUnitArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final AnalysisUnit[] content = new AnalysisUnit[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            Pointer toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(AnalysisUnitNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = AnalysisUnit.wrap(toRead);
            }

            // Return the new langkit array
            return new AnalysisUnitArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            
        ) {
            // Create a new native array with the size
            final AnalysisUnitArrayNative resNative = this.unwrap(
                
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        AnalysisUnitArrayNative unwrap(
            
        ) {
            // Create a new native array with the size
            final AnalysisUnitArrayNative res = NI_LIB.ada_analysis_unit_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            Pointer toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(AnalysisUnitNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                toWrite.writeWord(0, this.content[i].unwrap());
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((AnalysisUnitArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final AnalysisUnitArrayNative arrayNative
        ) {
            NI_LIB.ada_analysis_unit_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private AnalysisUnit[] jniContent() {
            final AnalysisUnit[] res =
                new AnalysisUnit[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }

    
    

    /**
     * This class represents the ada_unbounded_text_type_array Java wrapping class
     */
    public static final class
    UnboundedTextTypeArray extends ArrayBase<Symbol> {

        // ----- Class attributes -----

        /** Singleton that represents the none array. */
        public static final UnboundedTextTypeArray NONE = new UnboundedTextTypeArray(
            new Symbol[0]
        );

        // ----- Constructors -----

        /**
         * Create a new array with the given content.
         *
         * @param content The content of the array.
         */
        UnboundedTextTypeArray(
            final Symbol[] content
        ) {
            super(content);
        }

        /**
         * Create a new array from the JNI stub.
         *
         * @param content The unwrapped JNI content.
         */
        private static UnboundedTextTypeArray jniCreate(
            final Symbol[] jniContent
        ) {
            final Symbol[] content =
                new Symbol[jniContent.length];
            for(int i = 0 ; i < content.length ; i++) {
                content[i] =
                    jniContent[i];
            }
            return new UnboundedTextTypeArray(content);
        }

        /**
         * Create a sized array.
         *
         * @param size The size of the array you want to create.
         * @return The newly created array.
         */
        public static UnboundedTextTypeArray create(
            final int size
        ) {
            return new UnboundedTextTypeArray(
                new Symbol[size]
            );
        }

        // ----- Graal C API methods -----

        /**
         * Wrap a pointer to an array native value in the Java class.
         *
         * @param pointer The pointer to the array NI native value.
         * @return The newly wrapped array.
         */
        static UnboundedTextTypeArray wrap(
            final Pointer pointer
        ) {
            return wrap((UnboundedTextTypeArrayNative) pointer.readWord(0));
        }

        /**
         * Wrap an array native value in the Java class.
         *
         * @param nativeArray The NI array native value to wrap.
         * @return The newly wrapped array.
         */
        static UnboundedTextTypeArray wrap(
            final UnboundedTextTypeArrayNative nativeArray
        ) {
            // Get the size and prepare the working variables
            final int size = nativeArray.get_n();
            final Symbol[] content = new Symbol[size];
            final Pointer nativeItems = nativeArray.address_items();
            Pointer nativeItem;
            SymbolNative toRead;

            // Iterate over all array elements
            for(int i = 0 ; i < size ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(SymbolNative.class)
                );
                toRead = WordFactory.unsigned(nativeItem.rawValue());
                content[i] = Symbol.wrap(toRead);
            }

            // Return the new langkit array
            return new UnboundedTextTypeArray(content);
        }

        /**
         * Unwrap the array in the given pointer
         *
         * @param pointer The pointer to place the native array pointer
         * in.
         */
        void unwrap(
            final Pointer pointer
            , final AnalysisContext currentContext
        ) {
            // Create a new native array with the size
            final UnboundedTextTypeArrayNative resNative = this.unwrap(
                currentContext
            );

            // Place the result in the pointer
            pointer.writeWord(0, resNative);
        }

        /**
         * Allocate a new native array and unwrap inside.
         *
         * @return The newly allocated unwraped array.
         */
        UnboundedTextTypeArrayNative unwrap(
            final AnalysisContext currentContext
        ) {
            // Create a new native array with the size
            final UnboundedTextTypeArrayNative res = NI_LIB.ada_unbounded_text_type_array_create(
                this.content.length
            );

            // Prepare the working vars
            final Pointer nativeItems = res.address_items();
            Pointer nativeItem;
            SymbolNative toWrite;

            // Place all elements in the native array
            for(int i = 0 ; i < this.content.length ; i++) {
                nativeItem = nativeItems.add(
                    i * SizeOf.get(SymbolNative.class)
                );
                toWrite = WordFactory.unsigned(
                    nativeItem.rawValue()
                );
                this.content[i].unwrap(toWrite, currentContext);
            }

            // Return the result
            return res;
        }

        /**
         * Release native array pointer by the given pointer.
         *
         * @param The pointer to the array to release.
         */
        static void release(
            final Pointer pointer
        ) {
            release((UnboundedTextTypeArrayNative) pointer.readWord(0));
        }

        /**
         * Release the given native array.
         *
         * @param arrayNative The native array to release.
         */
        static void release(
            final UnboundedTextTypeArrayNative arrayNative
        ) {
            NI_LIB.ada_unbounded_text_type_array_dec_ref(arrayNative);
        }

        // ----- Getters -----

        /**
         * Get the content in an array unwrapped for the JNI stubs.
         *
         * @return The content unwrapped.
         */
        private Symbol[] jniContent() {
            final Symbol[] res =
                new Symbol[this.content.length];
            for(int i = 0 ; i < res.length ; i++) {
                res[i] = this.content[i];
            }
            return res;
        }

    }


    // ===== Generated iterator wrapping classes =====

    
    

    /**

     */
    public static final class CompletionItemIterator implements AutoCloseable {

        // ----- Class attributes -----

        /** Singleton for the iterator NONE value. */
        public static final CompletionItemIterator NONE = new CompletionItemIterator(
            PointerWrapper.nullPointer()
        );

        // ----- Attributes -----

        /** The pointer to the native iterator. */
        public final PointerWrapper reference;

        // ----- Constructors -----

        /**
         * Create a new CompletionItemIterator from the reference to its native value.
         *
         * @param reference The reference to the native iterator value.
         */
        private CompletionItemIterator(
            PointerWrapper reference
        ) {
            this.reference = reference;
        }

        /**
         * Wrap a pointer to the iterator native value in the Java class.
         *
         * @param niPointer The pointer to the NI iterator native value.
         * @return The newly created iterator or null if the given pointer
         * is null.
         */
        static CompletionItemIterator wrap(
            Pointer niPointer
        ) {
            if(niPointer.isNull()) return null;
            else return wrap((CompletionItemIteratorNative) niPointer.readWord(0));
        }

        /**
         * Wrap the iterator native value in the Java class.
         *
         * @param iteratorNative The NI iterator native value.
         * @return The newly created iterator or null if the iterator value
         * is null.
         */
        static CompletionItemIterator wrap(
            CompletionItemIteratorNative iteratorNative
        ) {
            if(((PointerBase) iteratorNative).isNull()) return null;
            else return new CompletionItemIterator(
                new PointerWrapper(iteratorNative)
            );
        }

        // ----- Instance methods -----

        /** @see java/lang/AutoCloseable#close() */
        public void close() {
            System.err.println("HANDLE THE ITERATOR CLOSING !!!");
        }

    }


    // ===== Node classes =====

    /**
     * Data type for all nodes. Nodes are assembled to make up a tree.  See the
     * node primitives below to inspect such trees.
     *
     * Unlike for contexts and units, this type has weak-reference semantics:
     * keeping a reference to a node has no effect on the decision to keep the
     * unit that it owns allocated. This means that once all references to the
     * context and units related to a node are dropped, the context and its
     * units are deallocated and the node becomes a stale reference: most
     * operations on it will raise a ``Stale_Reference_Error``.
     *
     * Note that since reparsing an analysis unit deallocates all the nodes it
     * contains, this operation makes all reference to these nodes stale as
     * well.
     */
    public static abstract class AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AdaNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pDeclarativeScope",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_declarative_scope",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pEnclosingCompilationUnit",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_enclosing_compilation_unit",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pGetUninstantiatedNode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_uninstantiated_node",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pComplete",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_complete",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pValidKeywords",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_valid_keywords",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pGenericInstantiations",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_generic_instantiations",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pSemanticParent",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_semantic_parent",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pParentBasicDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_parent_basic_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pFilterIsImportedBy",
                        new Class[]{AnalysisUnitArray.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnitArray.class,
                        "units"
                    ));
                    parameters.add(new Param(
                        boolean.class,
                        "transitive"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_filter_is_imported_by",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pXrefEntryPoint",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_xref_entry_point",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pResolveNames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_resolve_names",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pStandardUnit",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_standard_unit",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pStdEntity",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "sym"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_std_entity",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pBoolType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_bool_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pIntType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_int_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pUniversalIntType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_universal_int_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pUniversalRealType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_universal_real_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pStdCharType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_std_char_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pStdWideCharType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_std_wide_char_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pStdWideWideCharType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_std_wide_wide_char_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pTopLevelDecl",
                        new Class[]{AnalysisUnit.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnit.class,
                        "unit"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_top_level_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pChoiceMatch",
                        new Class[]{BigInteger.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        BigInteger.class,
                        "value"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_choice_match",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "pGnatXref",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_gnat_xref",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "parent",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "parent",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "parents",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "withSelf",
                        true
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "parents",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "children",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "children",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "tokenStart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "token_start",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "tokenEnd",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "token_end",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "childIndex",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "child_index",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "previousSibling",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "previous_sibling",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "nextSibling",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "next_sibling",
                        new LibadalangField(method, parameters)
                    );
                }
                
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "isGhost",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "is_ghost",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AdaNode.class.getMethod(
                        "fullSlocImage",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "full_sloc_image",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AdaNode NONE = new NoneNode();

        /** The entity of the node. */
        public final Entity entity;

        /** The analysis unit that owns the node. */
        protected AnalysisUnit unit;

        /** The cache for the image of the node. */
        protected String image;

        // ----- Constructors -----

        /**
         * Create a new node with its entity.
         *
         * @param entity The node's entity.
         */
        protected AdaNode(
            final Entity entity
        ) {
            this.entity = entity;
            this.unit = null;
            this.image = null;
        }

        /**
         * Get a node from the given entity.
         *
         * @param entity The entity to get the node from.
         * @return The newly created node.
         */
        public static AdaNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AdaNode.NONE :
                dispatchNodeCreation(entity);
        }

        /**
         * Dispatch the node creation to return the valid Java object
         * according to the node kind.
         *
         * @param entity The entity to create the node from.
         * @return The wrapped node in the correct class.
         */
        protected static AdaNode dispatchNodeCreation(
            final Entity entity
        ) {
            int nodeKind = -1;

            if(ImageInfo.inImageCode()) {
                EntityNative entityNative = StackValue.get(
                    EntityNative.class
                );
                entity.unwrap(entityNative);
                nodeKind = NI_LIB.ada_node_kind(entityNative);
            } else {
                nodeKind = JNI_LIB.ada_node_kind(entity);
            }

            switch(nodeKind) {
                case 1:
                    return entity.node.isNull() ?
                        AbortAbsent.NONE :
                        new AbortAbsent(entity);
                case 2:
                    return entity.node.isNull() ?
                        AbortPresent.NONE :
                        new AbortPresent(entity);
                case 3:
                    return entity.node.isNull() ?
                        AbstractAbsent.NONE :
                        new AbstractAbsent(entity);
                case 4:
                    return entity.node.isNull() ?
                        AbstractPresent.NONE :
                        new AbstractPresent(entity);
                case 5:
                    return entity.node.isNull() ?
                        AdaNodeList.NONE :
                        new AdaNodeList(entity);
                case 6:
                    return entity.node.isNull() ?
                        AbstractStateDeclList.NONE :
                        new AbstractStateDeclList(entity);
                case 7:
                    return entity.node.isNull() ?
                        AlternativesList.NONE :
                        new AlternativesList(entity);
                case 8:
                    return entity.node.isNull() ?
                        ConstraintList.NONE :
                        new ConstraintList(entity);
                case 9:
                    return entity.node.isNull() ?
                        DeclList.NONE :
                        new DeclList(entity);
                case 10:
                    return entity.node.isNull() ?
                        StmtList.NONE :
                        new StmtList(entity);
                case 11:
                    return entity.node.isNull() ?
                        AspectAssocList.NONE :
                        new AspectAssocList(entity);
                case 12:
                    return entity.node.isNull() ?
                        BaseAssocList.NONE :
                        new BaseAssocList(entity);
                case 13:
                    return entity.node.isNull() ?
                        AssocList.NONE :
                        new AssocList(entity);
                case 14:
                    return entity.node.isNull() ?
                        BasicDeclList.NONE :
                        new BasicDeclList(entity);
                case 15:
                    return entity.node.isNull() ?
                        CaseExprAlternativeList.NONE :
                        new CaseExprAlternativeList(entity);
                case 16:
                    return entity.node.isNull() ?
                        CaseStmtAlternativeList.NONE :
                        new CaseStmtAlternativeList(entity);
                case 17:
                    return entity.node.isNull() ?
                        CompilationUnitList.NONE :
                        new CompilationUnitList(entity);
                case 18:
                    return entity.node.isNull() ?
                        ConcatOperandList.NONE :
                        new ConcatOperandList(entity);
                case 19:
                    return entity.node.isNull() ?
                        ContractCaseAssocList.NONE :
                        new ContractCaseAssocList(entity);
                case 20:
                    return entity.node.isNull() ?
                        DefiningNameList.NONE :
                        new DefiningNameList(entity);
                case 21:
                    return entity.node.isNull() ?
                        DiscriminantSpecList.NONE :
                        new DiscriminantSpecList(entity);
                case 22:
                    return entity.node.isNull() ?
                        ElsifExprPartList.NONE :
                        new ElsifExprPartList(entity);
                case 23:
                    return entity.node.isNull() ?
                        ElsifStmtPartList.NONE :
                        new ElsifStmtPartList(entity);
                case 24:
                    return entity.node.isNull() ?
                        EnumLiteralDeclList.NONE :
                        new EnumLiteralDeclList(entity);
                case 25:
                    return entity.node.isNull() ?
                        ExprAlternativesList.NONE :
                        new ExprAlternativesList(entity);
                case 26:
                    return entity.node.isNull() ?
                        DiscriminantChoiceList.NONE :
                        new DiscriminantChoiceList(entity);
                case 27:
                    return entity.node.isNull() ?
                        NameList.NONE :
                        new NameList(entity);
                case 28:
                    return entity.node.isNull() ?
                        ParentList.NONE :
                        new ParentList(entity);
                case 29:
                    return entity.node.isNull() ?
                        ParamSpecList.NONE :
                        new ParamSpecList(entity);
                case 30:
                    return entity.node.isNull() ?
                        PragmaNodeList.NONE :
                        new PragmaNodeList(entity);
                case 31:
                    return entity.node.isNull() ?
                        SelectWhenPartList.NONE :
                        new SelectWhenPartList(entity);
                case 32:
                    return entity.node.isNull() ?
                        UnconstrainedArrayIndexList.NONE :
                        new UnconstrainedArrayIndexList(entity);
                case 33:
                    return entity.node.isNull() ?
                        VariantList.NONE :
                        new VariantList(entity);
                case 34:
                    return entity.node.isNull() ?
                        AliasedAbsent.NONE :
                        new AliasedAbsent(entity);
                case 35:
                    return entity.node.isNull() ?
                        AliasedPresent.NONE :
                        new AliasedPresent(entity);
                case 36:
                    return entity.node.isNull() ?
                        AllAbsent.NONE :
                        new AllAbsent(entity);
                case 37:
                    return entity.node.isNull() ?
                        AllPresent.NONE :
                        new AllPresent(entity);
                case 38:
                    return entity.node.isNull() ?
                        ConstrainedArrayIndices.NONE :
                        new ConstrainedArrayIndices(entity);
                case 39:
                    return entity.node.isNull() ?
                        UnconstrainedArrayIndices.NONE :
                        new UnconstrainedArrayIndices(entity);
                case 40:
                    return entity.node.isNull() ?
                        AspectAssoc.NONE :
                        new AspectAssoc(entity);
                case 41:
                    return entity.node.isNull() ?
                        AtClause.NONE :
                        new AtClause(entity);
                case 42:
                    return entity.node.isNull() ?
                        AttributeDefClause.NONE :
                        new AttributeDefClause(entity);
                case 43:
                    return entity.node.isNull() ?
                        EnumRepClause.NONE :
                        new EnumRepClause(entity);
                case 44:
                    return entity.node.isNull() ?
                        RecordRepClause.NONE :
                        new RecordRepClause(entity);
                case 45:
                    return entity.node.isNull() ?
                        AspectSpec.NONE :
                        new AspectSpec(entity);
                case 46:
                    return entity.node.isNull() ?
                        ContractCaseAssoc.NONE :
                        new ContractCaseAssoc(entity);
                case 47:
                    return entity.node.isNull() ?
                        PragmaArgumentAssoc.NONE :
                        new PragmaArgumentAssoc(entity);
                case 48:
                    return entity.node.isNull() ?
                        EntrySpec.NONE :
                        new EntrySpec(entity);
                case 49:
                    return entity.node.isNull() ?
                        EnumSubpSpec.NONE :
                        new EnumSubpSpec(entity);
                case 50:
                    return entity.node.isNull() ?
                        SubpSpec.NONE :
                        new SubpSpec(entity);
                case 51:
                    return entity.node.isNull() ?
                        SyntheticBinarySpec.NONE :
                        new SyntheticBinarySpec(entity);
                case 52:
                    return entity.node.isNull() ?
                        SyntheticUnarySpec.NONE :
                        new SyntheticUnarySpec(entity);
                case 53:
                    return entity.node.isNull() ?
                        ComponentList.NONE :
                        new ComponentList(entity);
                case 54:
                    return entity.node.isNull() ?
                        KnownDiscriminantPart.NONE :
                        new KnownDiscriminantPart(entity);
                case 55:
                    return entity.node.isNull() ?
                        UnknownDiscriminantPart.NONE :
                        new UnknownDiscriminantPart(entity);
                case 56:
                    return entity.node.isNull() ?
                        EntryCompletionFormalParams.NONE :
                        new EntryCompletionFormalParams(entity);
                case 57:
                    return entity.node.isNull() ?
                        GenericFormalPart.NONE :
                        new GenericFormalPart(entity);
                case 58:
                    return entity.node.isNull() ?
                        NullRecordDef.NONE :
                        new NullRecordDef(entity);
                case 59:
                    return entity.node.isNull() ?
                        RecordDef.NONE :
                        new RecordDef(entity);
                case 60:
                    return entity.node.isNull() ?
                        AggregateAssoc.NONE :
                        new AggregateAssoc(entity);
                case 61:
                    return entity.node.isNull() ?
                        MultiDimArrayAssoc.NONE :
                        new MultiDimArrayAssoc(entity);
                case 62:
                    return entity.node.isNull() ?
                        CompositeConstraintAssoc.NONE :
                        new CompositeConstraintAssoc(entity);
                case 63:
                    return entity.node.isNull() ?
                        IteratedAssoc.NONE :
                        new IteratedAssoc(entity);
                case 64:
                    return entity.node.isNull() ?
                        ParamAssoc.NONE :
                        new ParamAssoc(entity);
                case 65:
                    return entity.node.isNull() ?
                        AbstractStateDecl.NONE :
                        new AbstractStateDecl(entity);
                case 66:
                    return entity.node.isNull() ?
                        AnonymousExprDecl.NONE :
                        new AnonymousExprDecl(entity);
                case 67:
                    return entity.node.isNull() ?
                        ComponentDecl.NONE :
                        new ComponentDecl(entity);
                case 68:
                    return entity.node.isNull() ?
                        DiscriminantSpec.NONE :
                        new DiscriminantSpec(entity);
                case 69:
                    return entity.node.isNull() ?
                        GenericFormalObjDecl.NONE :
                        new GenericFormalObjDecl(entity);
                case 70:
                    return entity.node.isNull() ?
                        GenericFormalPackage.NONE :
                        new GenericFormalPackage(entity);
                case 71:
                    return entity.node.isNull() ?
                        GenericFormalSubpDecl.NONE :
                        new GenericFormalSubpDecl(entity);
                case 72:
                    return entity.node.isNull() ?
                        GenericFormalTypeDecl.NONE :
                        new GenericFormalTypeDecl(entity);
                case 73:
                    return entity.node.isNull() ?
                        ParamSpec.NONE :
                        new ParamSpec(entity);
                case 74:
                    return entity.node.isNull() ?
                        SyntheticFormalParamDecl.NONE :
                        new SyntheticFormalParamDecl(entity);
                case 75:
                    return entity.node.isNull() ?
                        GenericPackageInternal.NONE :
                        new GenericPackageInternal(entity);
                case 76:
                    return entity.node.isNull() ?
                        PackageDecl.NONE :
                        new PackageDecl(entity);
                case 77:
                    return entity.node.isNull() ?
                        DiscreteBaseSubtypeDecl.NONE :
                        new DiscreteBaseSubtypeDecl(entity);
                case 78:
                    return entity.node.isNull() ?
                        SubtypeDecl.NONE :
                        new SubtypeDecl(entity);
                case 79:
                    return entity.node.isNull() ?
                        ClasswideTypeDecl.NONE :
                        new ClasswideTypeDecl(entity);
                case 80:
                    return entity.node.isNull() ?
                        IncompleteTypeDecl.NONE :
                        new IncompleteTypeDecl(entity);
                case 81:
                    return entity.node.isNull() ?
                        IncompleteFormalTypeDecl.NONE :
                        new IncompleteFormalTypeDecl(entity);
                case 82:
                    return entity.node.isNull() ?
                        IncompleteTaggedTypeDecl.NONE :
                        new IncompleteTaggedTypeDecl(entity);
                case 83:
                    return entity.node.isNull() ?
                        ProtectedTypeDecl.NONE :
                        new ProtectedTypeDecl(entity);
                case 84:
                    return entity.node.isNull() ?
                        TaskTypeDecl.NONE :
                        new TaskTypeDecl(entity);
                case 85:
                    return entity.node.isNull() ?
                        SingleTaskTypeDecl.NONE :
                        new SingleTaskTypeDecl(entity);
                case 86:
                    return entity.node.isNull() ?
                        AnonymousTypeDecl.NONE :
                        new AnonymousTypeDecl(entity);
                case 87:
                    return entity.node.isNull() ?
                        SynthAnonymousTypeDecl.NONE :
                        new SynthAnonymousTypeDecl(entity);
                case 88:
                    return entity.node.isNull() ?
                        ConcreteTypeDecl.NONE :
                        new ConcreteTypeDecl(entity);
                case 89:
                    return entity.node.isNull() ?
                        FormalTypeDecl.NONE :
                        new FormalTypeDecl(entity);
                case 90:
                    return entity.node.isNull() ?
                        AbstractSubpDecl.NONE :
                        new AbstractSubpDecl(entity);
                case 91:
                    return entity.node.isNull() ?
                        AbstractFormalSubpDecl.NONE :
                        new AbstractFormalSubpDecl(entity);
                case 92:
                    return entity.node.isNull() ?
                        ConcreteFormalSubpDecl.NONE :
                        new ConcreteFormalSubpDecl(entity);
                case 93:
                    return entity.node.isNull() ?
                        SubpDecl.NONE :
                        new SubpDecl(entity);
                case 94:
                    return entity.node.isNull() ?
                        EntryDecl.NONE :
                        new EntryDecl(entity);
                case 95:
                    return entity.node.isNull() ?
                        EnumLiteralDecl.NONE :
                        new EnumLiteralDecl(entity);
                case 96:
                    return entity.node.isNull() ?
                        SyntheticCharEnumLit.NONE :
                        new SyntheticCharEnumLit(entity);
                case 97:
                    return entity.node.isNull() ?
                        GenericSubpInternal.NONE :
                        new GenericSubpInternal(entity);
                case 98:
                    return entity.node.isNull() ?
                        SyntheticSubpDecl.NONE :
                        new SyntheticSubpDecl(entity);
                case 99:
                    return entity.node.isNull() ?
                        AcceptStmtBody.NONE :
                        new AcceptStmtBody(entity);
                case 100:
                    return entity.node.isNull() ?
                        ExprFunction.NONE :
                        new ExprFunction(entity);
                case 101:
                    return entity.node.isNull() ?
                        NullSubpDecl.NONE :
                        new NullSubpDecl(entity);
                case 102:
                    return entity.node.isNull() ?
                        SubpBody.NONE :
                        new SubpBody(entity);
                case 103:
                    return entity.node.isNull() ?
                        SubpRenamingDecl.NONE :
                        new SubpRenamingDecl(entity);
                case 104:
                    return entity.node.isNull() ?
                        PackageBodyStub.NONE :
                        new PackageBodyStub(entity);
                case 105:
                    return entity.node.isNull() ?
                        ProtectedBodyStub.NONE :
                        new ProtectedBodyStub(entity);
                case 106:
                    return entity.node.isNull() ?
                        SubpBodyStub.NONE :
                        new SubpBodyStub(entity);
                case 107:
                    return entity.node.isNull() ?
                        TaskBodyStub.NONE :
                        new TaskBodyStub(entity);
                case 108:
                    return entity.node.isNull() ?
                        EntryBody.NONE :
                        new EntryBody(entity);
                case 109:
                    return entity.node.isNull() ?
                        PackageBody.NONE :
                        new PackageBody(entity);
                case 110:
                    return entity.node.isNull() ?
                        ProtectedBody.NONE :
                        new ProtectedBody(entity);
                case 111:
                    return entity.node.isNull() ?
                        TaskBody.NONE :
                        new TaskBody(entity);
                case 112:
                    return entity.node.isNull() ?
                        EntryIndexSpec.NONE :
                        new EntryIndexSpec(entity);
                case 113:
                    return entity.node.isNull() ?
                        ErrorDecl.NONE :
                        new ErrorDecl(entity);
                case 114:
                    return entity.node.isNull() ?
                        ExceptionDecl.NONE :
                        new ExceptionDecl(entity);
                case 115:
                    return entity.node.isNull() ?
                        ExceptionHandler.NONE :
                        new ExceptionHandler(entity);
                case 116:
                    return entity.node.isNull() ?
                        ForLoopVarDecl.NONE :
                        new ForLoopVarDecl(entity);
                case 117:
                    return entity.node.isNull() ?
                        GenericPackageDecl.NONE :
                        new GenericPackageDecl(entity);
                case 118:
                    return entity.node.isNull() ?
                        GenericSubpDecl.NONE :
                        new GenericSubpDecl(entity);
                case 119:
                    return entity.node.isNull() ?
                        GenericPackageInstantiation.NONE :
                        new GenericPackageInstantiation(entity);
                case 120:
                    return entity.node.isNull() ?
                        GenericSubpInstantiation.NONE :
                        new GenericSubpInstantiation(entity);
                case 121:
                    return entity.node.isNull() ?
                        GenericPackageRenamingDecl.NONE :
                        new GenericPackageRenamingDecl(entity);
                case 122:
                    return entity.node.isNull() ?
                        GenericSubpRenamingDecl.NONE :
                        new GenericSubpRenamingDecl(entity);
                case 123:
                    return entity.node.isNull() ?
                        LabelDecl.NONE :
                        new LabelDecl(entity);
                case 124:
                    return entity.node.isNull() ?
                        NamedStmtDecl.NONE :
                        new NamedStmtDecl(entity);
                case 125:
                    return entity.node.isNull() ?
                        NumberDecl.NONE :
                        new NumberDecl(entity);
                case 126:
                    return entity.node.isNull() ?
                        ObjectDecl.NONE :
                        new ObjectDecl(entity);
                case 127:
                    return entity.node.isNull() ?
                        ExtendedReturnStmtObjectDecl.NONE :
                        new ExtendedReturnStmtObjectDecl(entity);
                case 128:
                    return entity.node.isNull() ?
                        NoTypeObjectRenamingDecl.NONE :
                        new NoTypeObjectRenamingDecl(entity);
                case 129:
                    return entity.node.isNull() ?
                        PackageRenamingDecl.NONE :
                        new PackageRenamingDecl(entity);
                case 130:
                    return entity.node.isNull() ?
                        SingleProtectedDecl.NONE :
                        new SingleProtectedDecl(entity);
                case 131:
                    return entity.node.isNull() ?
                        SingleTaskDecl.NONE :
                        new SingleTaskDecl(entity);
                case 132:
                    return entity.node.isNull() ?
                        CaseStmtAlternative.NONE :
                        new CaseStmtAlternative(entity);
                case 133:
                    return entity.node.isNull() ?
                        CompilationUnit.NONE :
                        new CompilationUnit(entity);
                case 134:
                    return entity.node.isNull() ?
                        ComponentClause.NONE :
                        new ComponentClause(entity);
                case 135:
                    return entity.node.isNull() ?
                        ComponentDef.NONE :
                        new ComponentDef(entity);
                case 136:
                    return entity.node.isNull() ?
                        ConstantAbsent.NONE :
                        new ConstantAbsent(entity);
                case 137:
                    return entity.node.isNull() ?
                        ConstantPresent.NONE :
                        new ConstantPresent(entity);
                case 138:
                    return entity.node.isNull() ?
                        CompositeConstraint.NONE :
                        new CompositeConstraint(entity);
                case 139:
                    return entity.node.isNull() ?
                        DeltaConstraint.NONE :
                        new DeltaConstraint(entity);
                case 140:
                    return entity.node.isNull() ?
                        DigitsConstraint.NONE :
                        new DigitsConstraint(entity);
                case 141:
                    return entity.node.isNull() ?
                        RangeConstraint.NONE :
                        new RangeConstraint(entity);
                case 142:
                    return entity.node.isNull() ?
                        DeclarativePart.NONE :
                        new DeclarativePart(entity);
                case 143:
                    return entity.node.isNull() ?
                        PrivatePart.NONE :
                        new PrivatePart(entity);
                case 144:
                    return entity.node.isNull() ?
                        PublicPart.NONE :
                        new PublicPart(entity);
                case 145:
                    return entity.node.isNull() ?
                        ElsifExprPart.NONE :
                        new ElsifExprPart(entity);
                case 146:
                    return entity.node.isNull() ?
                        ElsifStmtPart.NONE :
                        new ElsifStmtPart(entity);
                case 147:
                    return entity.node.isNull() ?
                        AbstractStateDeclExpr.NONE :
                        new AbstractStateDeclExpr(entity);
                case 148:
                    return entity.node.isNull() ?
                        Allocator.NONE :
                        new Allocator(entity);
                case 149:
                    return entity.node.isNull() ?
                        Aggregate.NONE :
                        new Aggregate(entity);
                case 150:
                    return entity.node.isNull() ?
                        BracketAggregate.NONE :
                        new BracketAggregate(entity);
                case 151:
                    return entity.node.isNull() ?
                        DeltaAggregate.NONE :
                        new DeltaAggregate(entity);
                case 152:
                    return entity.node.isNull() ?
                        BracketDeltaAggregate.NONE :
                        new BracketDeltaAggregate(entity);
                case 153:
                    return entity.node.isNull() ?
                        NullRecordAggregate.NONE :
                        new NullRecordAggregate(entity);
                case 154:
                    return entity.node.isNull() ?
                        BinOp.NONE :
                        new BinOp(entity);
                case 155:
                    return entity.node.isNull() ?
                        RelationOp.NONE :
                        new RelationOp(entity);
                case 156:
                    return entity.node.isNull() ?
                        BoxExpr.NONE :
                        new BoxExpr(entity);
                case 157:
                    return entity.node.isNull() ?
                        CaseExprAlternative.NONE :
                        new CaseExprAlternative(entity);
                case 158:
                    return entity.node.isNull() ?
                        ConcatOp.NONE :
                        new ConcatOp(entity);
                case 159:
                    return entity.node.isNull() ?
                        ConcatOperand.NONE :
                        new ConcatOperand(entity);
                case 160:
                    return entity.node.isNull() ?
                        CaseExpr.NONE :
                        new CaseExpr(entity);
                case 161:
                    return entity.node.isNull() ?
                        IfExpr.NONE :
                        new IfExpr(entity);
                case 162:
                    return entity.node.isNull() ?
                        ContractCases.NONE :
                        new ContractCases(entity);
                case 163:
                    return entity.node.isNull() ?
                        DeclExpr.NONE :
                        new DeclExpr(entity);
                case 164:
                    return entity.node.isNull() ?
                        MembershipExpr.NONE :
                        new MembershipExpr(entity);
                case 165:
                    return entity.node.isNull() ?
                        AttributeRef.NONE :
                        new AttributeRef(entity);
                case 166:
                    return entity.node.isNull() ?
                        CallExpr.NONE :
                        new CallExpr(entity);
                case 167:
                    return entity.node.isNull() ?
                        DefiningName.NONE :
                        new DefiningName(entity);
                case 168:
                    return entity.node.isNull() ?
                        SyntheticDefiningName.NONE :
                        new SyntheticDefiningName(entity);
                case 169:
                    return entity.node.isNull() ?
                        DiscreteSubtypeName.NONE :
                        new DiscreteSubtypeName(entity);
                case 170:
                    return entity.node.isNull() ?
                        DottedName.NONE :
                        new DottedName(entity);
                case 171:
                    return entity.node.isNull() ?
                        EndName.NONE :
                        new EndName(entity);
                case 172:
                    return entity.node.isNull() ?
                        ExplicitDeref.NONE :
                        new ExplicitDeref(entity);
                case 173:
                    return entity.node.isNull() ?
                        QualExpr.NONE :
                        new QualExpr(entity);
                case 174:
                    return entity.node.isNull() ?
                        ReduceAttributeRef.NONE :
                        new ReduceAttributeRef(entity);
                case 175:
                    return entity.node.isNull() ?
                        CharLiteral.NONE :
                        new CharLiteral(entity);
                case 176:
                    return entity.node.isNull() ?
                        Identifier.NONE :
                        new Identifier(entity);
                case 177:
                    return entity.node.isNull() ?
                        OpAbs.NONE :
                        new OpAbs(entity);
                case 178:
                    return entity.node.isNull() ?
                        OpAnd.NONE :
                        new OpAnd(entity);
                case 179:
                    return entity.node.isNull() ?
                        OpAndThen.NONE :
                        new OpAndThen(entity);
                case 180:
                    return entity.node.isNull() ?
                        OpConcat.NONE :
                        new OpConcat(entity);
                case 181:
                    return entity.node.isNull() ?
                        OpDiv.NONE :
                        new OpDiv(entity);
                case 182:
                    return entity.node.isNull() ?
                        OpDoubleDot.NONE :
                        new OpDoubleDot(entity);
                case 183:
                    return entity.node.isNull() ?
                        OpEq.NONE :
                        new OpEq(entity);
                case 184:
                    return entity.node.isNull() ?
                        OpGt.NONE :
                        new OpGt(entity);
                case 185:
                    return entity.node.isNull() ?
                        OpGte.NONE :
                        new OpGte(entity);
                case 186:
                    return entity.node.isNull() ?
                        OpIn.NONE :
                        new OpIn(entity);
                case 187:
                    return entity.node.isNull() ?
                        OpLt.NONE :
                        new OpLt(entity);
                case 188:
                    return entity.node.isNull() ?
                        OpLte.NONE :
                        new OpLte(entity);
                case 189:
                    return entity.node.isNull() ?
                        OpMinus.NONE :
                        new OpMinus(entity);
                case 190:
                    return entity.node.isNull() ?
                        OpMod.NONE :
                        new OpMod(entity);
                case 191:
                    return entity.node.isNull() ?
                        OpMult.NONE :
                        new OpMult(entity);
                case 192:
                    return entity.node.isNull() ?
                        OpNeq.NONE :
                        new OpNeq(entity);
                case 193:
                    return entity.node.isNull() ?
                        OpNot.NONE :
                        new OpNot(entity);
                case 194:
                    return entity.node.isNull() ?
                        OpNotIn.NONE :
                        new OpNotIn(entity);
                case 195:
                    return entity.node.isNull() ?
                        OpOr.NONE :
                        new OpOr(entity);
                case 196:
                    return entity.node.isNull() ?
                        OpOrElse.NONE :
                        new OpOrElse(entity);
                case 197:
                    return entity.node.isNull() ?
                        OpPlus.NONE :
                        new OpPlus(entity);
                case 198:
                    return entity.node.isNull() ?
                        OpPow.NONE :
                        new OpPow(entity);
                case 199:
                    return entity.node.isNull() ?
                        OpRem.NONE :
                        new OpRem(entity);
                case 200:
                    return entity.node.isNull() ?
                        OpXor.NONE :
                        new OpXor(entity);
                case 201:
                    return entity.node.isNull() ?
                        StringLiteral.NONE :
                        new StringLiteral(entity);
                case 202:
                    return entity.node.isNull() ?
                        NullLiteral.NONE :
                        new NullLiteral(entity);
                case 203:
                    return entity.node.isNull() ?
                        IntLiteral.NONE :
                        new IntLiteral(entity);
                case 204:
                    return entity.node.isNull() ?
                        RealLiteral.NONE :
                        new RealLiteral(entity);
                case 205:
                    return entity.node.isNull() ?
                        SyntheticIdentifier.NONE :
                        new SyntheticIdentifier(entity);
                case 206:
                    return entity.node.isNull() ?
                        TargetName.NONE :
                        new TargetName(entity);
                case 207:
                    return entity.node.isNull() ?
                        UpdateAttributeRef.NONE :
                        new UpdateAttributeRef(entity);
                case 208:
                    return entity.node.isNull() ?
                        ParenExpr.NONE :
                        new ParenExpr(entity);
                case 209:
                    return entity.node.isNull() ?
                        QuantifiedExpr.NONE :
                        new QuantifiedExpr(entity);
                case 210:
                    return entity.node.isNull() ?
                        RaiseExpr.NONE :
                        new RaiseExpr(entity);
                case 211:
                    return entity.node.isNull() ?
                        UnOp.NONE :
                        new UnOp(entity);
                case 212:
                    return entity.node.isNull() ?
                        HandledStmts.NONE :
                        new HandledStmts(entity);
                case 213:
                    return entity.node.isNull() ?
                        InterfaceKindLimited.NONE :
                        new InterfaceKindLimited(entity);
                case 214:
                    return entity.node.isNull() ?
                        InterfaceKindProtected.NONE :
                        new InterfaceKindProtected(entity);
                case 215:
                    return entity.node.isNull() ?
                        InterfaceKindSynchronized.NONE :
                        new InterfaceKindSynchronized(entity);
                case 216:
                    return entity.node.isNull() ?
                        InterfaceKindTask.NONE :
                        new InterfaceKindTask(entity);
                case 217:
                    return entity.node.isNull() ?
                        IterTypeIn.NONE :
                        new IterTypeIn(entity);
                case 218:
                    return entity.node.isNull() ?
                        IterTypeOf.NONE :
                        new IterTypeOf(entity);
                case 219:
                    return entity.node.isNull() ?
                        LibraryItem.NONE :
                        new LibraryItem(entity);
                case 220:
                    return entity.node.isNull() ?
                        LimitedAbsent.NONE :
                        new LimitedAbsent(entity);
                case 221:
                    return entity.node.isNull() ?
                        LimitedPresent.NONE :
                        new LimitedPresent(entity);
                case 222:
                    return entity.node.isNull() ?
                        ForLoopSpec.NONE :
                        new ForLoopSpec(entity);
                case 223:
                    return entity.node.isNull() ?
                        WhileLoopSpec.NONE :
                        new WhileLoopSpec(entity);
                case 224:
                    return entity.node.isNull() ?
                        ModeDefault.NONE :
                        new ModeDefault(entity);
                case 225:
                    return entity.node.isNull() ?
                        ModeIn.NONE :
                        new ModeIn(entity);
                case 226:
                    return entity.node.isNull() ?
                        ModeInOut.NONE :
                        new ModeInOut(entity);
                case 227:
                    return entity.node.isNull() ?
                        ModeOut.NONE :
                        new ModeOut(entity);
                case 228:
                    return entity.node.isNull() ?
                        MultiAbstractStateDecl.NONE :
                        new MultiAbstractStateDecl(entity);
                case 229:
                    return entity.node.isNull() ?
                        NotNullAbsent.NONE :
                        new NotNullAbsent(entity);
                case 230:
                    return entity.node.isNull() ?
                        NotNullPresent.NONE :
                        new NotNullPresent(entity);
                case 231:
                    return entity.node.isNull() ?
                        NullComponentDecl.NONE :
                        new NullComponentDecl(entity);
                case 232:
                    return entity.node.isNull() ?
                        OthersDesignator.NONE :
                        new OthersDesignator(entity);
                case 233:
                    return entity.node.isNull() ?
                        OverridingNotOverriding.NONE :
                        new OverridingNotOverriding(entity);
                case 234:
                    return entity.node.isNull() ?
                        OverridingOverriding.NONE :
                        new OverridingOverriding(entity);
                case 235:
                    return entity.node.isNull() ?
                        OverridingUnspecified.NONE :
                        new OverridingUnspecified(entity);
                case 236:
                    return entity.node.isNull() ?
                        Params.NONE :
                        new Params(entity);
                case 237:
                    return entity.node.isNull() ?
                        ParenAbstractStateDecl.NONE :
                        new ParenAbstractStateDecl(entity);
                case 238:
                    return entity.node.isNull() ?
                        PpElseDirective.NONE :
                        new PpElseDirective(entity);
                case 239:
                    return entity.node.isNull() ?
                        PpElsifDirective.NONE :
                        new PpElsifDirective(entity);
                case 240:
                    return entity.node.isNull() ?
                        PpEndIfDirective.NONE :
                        new PpEndIfDirective(entity);
                case 241:
                    return entity.node.isNull() ?
                        PpIfDirective.NONE :
                        new PpIfDirective(entity);
                case 242:
                    return entity.node.isNull() ?
                        PpThenKw.NONE :
                        new PpThenKw(entity);
                case 243:
                    return entity.node.isNull() ?
                        PragmaNode.NONE :
                        new PragmaNode(entity);
                case 244:
                    return entity.node.isNull() ?
                        PrivateAbsent.NONE :
                        new PrivateAbsent(entity);
                case 245:
                    return entity.node.isNull() ?
                        PrivatePresent.NONE :
                        new PrivatePresent(entity);
                case 246:
                    return entity.node.isNull() ?
                        ProtectedDef.NONE :
                        new ProtectedDef(entity);
                case 247:
                    return entity.node.isNull() ?
                        ProtectedAbsent.NONE :
                        new ProtectedAbsent(entity);
                case 248:
                    return entity.node.isNull() ?
                        ProtectedPresent.NONE :
                        new ProtectedPresent(entity);
                case 249:
                    return entity.node.isNull() ?
                        QuantifierAll.NONE :
                        new QuantifierAll(entity);
                case 250:
                    return entity.node.isNull() ?
                        QuantifierSome.NONE :
                        new QuantifierSome(entity);
                case 251:
                    return entity.node.isNull() ?
                        RangeSpec.NONE :
                        new RangeSpec(entity);
                case 252:
                    return entity.node.isNull() ?
                        RenamingClause.NONE :
                        new RenamingClause(entity);
                case 253:
                    return entity.node.isNull() ?
                        SyntheticRenamingClause.NONE :
                        new SyntheticRenamingClause(entity);
                case 254:
                    return entity.node.isNull() ?
                        ReverseAbsent.NONE :
                        new ReverseAbsent(entity);
                case 255:
                    return entity.node.isNull() ?
                        ReversePresent.NONE :
                        new ReversePresent(entity);
                case 256:
                    return entity.node.isNull() ?
                        SelectWhenPart.NONE :
                        new SelectWhenPart(entity);
                case 257:
                    return entity.node.isNull() ?
                        AcceptStmt.NONE :
                        new AcceptStmt(entity);
                case 258:
                    return entity.node.isNull() ?
                        AcceptStmtWithStmts.NONE :
                        new AcceptStmtWithStmts(entity);
                case 259:
                    return entity.node.isNull() ?
                        ForLoopStmt.NONE :
                        new ForLoopStmt(entity);
                case 260:
                    return entity.node.isNull() ?
                        LoopStmt.NONE :
                        new LoopStmt(entity);
                case 261:
                    return entity.node.isNull() ?
                        WhileLoopStmt.NONE :
                        new WhileLoopStmt(entity);
                case 262:
                    return entity.node.isNull() ?
                        BeginBlock.NONE :
                        new BeginBlock(entity);
                case 263:
                    return entity.node.isNull() ?
                        DeclBlock.NONE :
                        new DeclBlock(entity);
                case 264:
                    return entity.node.isNull() ?
                        CaseStmt.NONE :
                        new CaseStmt(entity);
                case 265:
                    return entity.node.isNull() ?
                        ExtendedReturnStmt.NONE :
                        new ExtendedReturnStmt(entity);
                case 266:
                    return entity.node.isNull() ?
                        IfStmt.NONE :
                        new IfStmt(entity);
                case 267:
                    return entity.node.isNull() ?
                        NamedStmt.NONE :
                        new NamedStmt(entity);
                case 268:
                    return entity.node.isNull() ?
                        SelectStmt.NONE :
                        new SelectStmt(entity);
                case 269:
                    return entity.node.isNull() ?
                        ErrorStmt.NONE :
                        new ErrorStmt(entity);
                case 270:
                    return entity.node.isNull() ?
                        AbortStmt.NONE :
                        new AbortStmt(entity);
                case 271:
                    return entity.node.isNull() ?
                        AssignStmt.NONE :
                        new AssignStmt(entity);
                case 272:
                    return entity.node.isNull() ?
                        CallStmt.NONE :
                        new CallStmt(entity);
                case 273:
                    return entity.node.isNull() ?
                        DelayStmt.NONE :
                        new DelayStmt(entity);
                case 274:
                    return entity.node.isNull() ?
                        ExitStmt.NONE :
                        new ExitStmt(entity);
                case 275:
                    return entity.node.isNull() ?
                        GotoStmt.NONE :
                        new GotoStmt(entity);
                case 276:
                    return entity.node.isNull() ?
                        Label.NONE :
                        new Label(entity);
                case 277:
                    return entity.node.isNull() ?
                        NullStmt.NONE :
                        new NullStmt(entity);
                case 278:
                    return entity.node.isNull() ?
                        RaiseStmt.NONE :
                        new RaiseStmt(entity);
                case 279:
                    return entity.node.isNull() ?
                        RequeueStmt.NONE :
                        new RequeueStmt(entity);
                case 280:
                    return entity.node.isNull() ?
                        ReturnStmt.NONE :
                        new ReturnStmt(entity);
                case 281:
                    return entity.node.isNull() ?
                        TerminateAlternative.NONE :
                        new TerminateAlternative(entity);
                case 282:
                    return entity.node.isNull() ?
                        SubpKindFunction.NONE :
                        new SubpKindFunction(entity);
                case 283:
                    return entity.node.isNull() ?
                        SubpKindProcedure.NONE :
                        new SubpKindProcedure(entity);
                case 284:
                    return entity.node.isNull() ?
                        Subunit.NONE :
                        new Subunit(entity);
                case 285:
                    return entity.node.isNull() ?
                        SynchronizedAbsent.NONE :
                        new SynchronizedAbsent(entity);
                case 286:
                    return entity.node.isNull() ?
                        SynchronizedPresent.NONE :
                        new SynchronizedPresent(entity);
                case 287:
                    return entity.node.isNull() ?
                        TaggedAbsent.NONE :
                        new TaggedAbsent(entity);
                case 288:
                    return entity.node.isNull() ?
                        TaggedPresent.NONE :
                        new TaggedPresent(entity);
                case 289:
                    return entity.node.isNull() ?
                        TaskDef.NONE :
                        new TaskDef(entity);
                case 290:
                    return entity.node.isNull() ?
                        TypeAttributesRepository.NONE :
                        new TypeAttributesRepository(entity);
                case 291:
                    return entity.node.isNull() ?
                        AccessToSubpDef.NONE :
                        new AccessToSubpDef(entity);
                case 292:
                    return entity.node.isNull() ?
                        AnonymousTypeAccessDef.NONE :
                        new AnonymousTypeAccessDef(entity);
                case 293:
                    return entity.node.isNull() ?
                        TypeAccessDef.NONE :
                        new TypeAccessDef(entity);
                case 294:
                    return entity.node.isNull() ?
                        ArrayTypeDef.NONE :
                        new ArrayTypeDef(entity);
                case 295:
                    return entity.node.isNull() ?
                        DerivedTypeDef.NONE :
                        new DerivedTypeDef(entity);
                case 296:
                    return entity.node.isNull() ?
                        EnumTypeDef.NONE :
                        new EnumTypeDef(entity);
                case 297:
                    return entity.node.isNull() ?
                        FormalDiscreteTypeDef.NONE :
                        new FormalDiscreteTypeDef(entity);
                case 298:
                    return entity.node.isNull() ?
                        InterfaceTypeDef.NONE :
                        new InterfaceTypeDef(entity);
                case 299:
                    return entity.node.isNull() ?
                        ModIntTypeDef.NONE :
                        new ModIntTypeDef(entity);
                case 300:
                    return entity.node.isNull() ?
                        PrivateTypeDef.NONE :
                        new PrivateTypeDef(entity);
                case 301:
                    return entity.node.isNull() ?
                        DecimalFixedPointDef.NONE :
                        new DecimalFixedPointDef(entity);
                case 302:
                    return entity.node.isNull() ?
                        FloatingPointDef.NONE :
                        new FloatingPointDef(entity);
                case 303:
                    return entity.node.isNull() ?
                        OrdinaryFixedPointDef.NONE :
                        new OrdinaryFixedPointDef(entity);
                case 304:
                    return entity.node.isNull() ?
                        RecordTypeDef.NONE :
                        new RecordTypeDef(entity);
                case 305:
                    return entity.node.isNull() ?
                        SignedIntTypeDef.NONE :
                        new SignedIntTypeDef(entity);
                case 306:
                    return entity.node.isNull() ?
                        AnonymousType.NONE :
                        new AnonymousType(entity);
                case 307:
                    return entity.node.isNull() ?
                        EnumLitSynthTypeExpr.NONE :
                        new EnumLitSynthTypeExpr(entity);
                case 308:
                    return entity.node.isNull() ?
                        SubtypeIndication.NONE :
                        new SubtypeIndication(entity);
                case 309:
                    return entity.node.isNull() ?
                        ConstrainedSubtypeIndication.NONE :
                        new ConstrainedSubtypeIndication(entity);
                case 310:
                    return entity.node.isNull() ?
                        DiscreteSubtypeIndication.NONE :
                        new DiscreteSubtypeIndication(entity);
                case 311:
                    return entity.node.isNull() ?
                        SyntheticTypeExpr.NONE :
                        new SyntheticTypeExpr(entity);
                case 312:
                    return entity.node.isNull() ?
                        UnconstrainedArrayIndex.NONE :
                        new UnconstrainedArrayIndex(entity);
                case 313:
                    return entity.node.isNull() ?
                        UntilAbsent.NONE :
                        new UntilAbsent(entity);
                case 314:
                    return entity.node.isNull() ?
                        UntilPresent.NONE :
                        new UntilPresent(entity);
                case 315:
                    return entity.node.isNull() ?
                        UsePackageClause.NONE :
                        new UsePackageClause(entity);
                case 316:
                    return entity.node.isNull() ?
                        UseTypeClause.NONE :
                        new UseTypeClause(entity);
                case 317:
                    return entity.node.isNull() ?
                        ValueSequence.NONE :
                        new ValueSequence(entity);
                case 318:
                    return entity.node.isNull() ?
                        Variant.NONE :
                        new Variant(entity);
                case 319:
                    return entity.node.isNull() ?
                        VariantPart.NONE :
                        new VariantPart(entity);
                case 320:
                    return entity.node.isNull() ?
                        WithClause.NONE :
                        new WithClause(entity);
                case 321:
                    return entity.node.isNull() ?
                        WithPrivateAbsent.NONE :
                        new WithPrivateAbsent(entity);
                case 322:
                    return entity.node.isNull() ?
                        WithPrivatePresent.NONE :
                        new WithPrivatePresent(entity);
                default:
                    throw new EnumException(
                        "Cannot find the node type from " + nodeKind
                    );
            }
        }

        // ----- Getters -----

        public String getKindName() {
            return AdaNode.kindName;
        }

        public String[] getFieldNames() {
            return AdaNode.fieldNames;
        }

        public boolean isListType() {
            return AdaNode.isListType;
        }

        public boolean isTokenNode() {

            if(ImageInfo.inImageCode()) {
                EntityNative entityNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(entityNative);
                return NI_LIB.ada_node_is_token_node(
                    entityNative
                ) != 0;
            } else {
                return JNI_LIB.ada_node_is_token_node(this.entity);
            }

        }

        public boolean isNone() {
            return this.entity.node.isNull();
        }

        // ----- Instance methods -----

        /**
         * Get the analysis unit of the node.
         *
         * @return The unit which owns the node.
         */
        public AnalysisUnit getUnit() {
            if(this.unit == null) {

                if(ImageInfo.inImageCode()) {
                    final EntityNative entityNative = StackValue.get(
                        EntityNative.class
                    );
                    this.entity.unwrap(entityNative);

                    final AnalysisUnitNative unitNative =
                        NI_LIB.ada_node_unit(entityNative);
                    this.unit = AnalysisUnit.wrap(unitNative);
                } else {
                    this.unit = JNI_LIB.ada_node_unit(this.entity);
                }

            }
            return this.unit;
        }

        /**
         * Get the descritpion of a field from its name.
         *
         * @param name The langkit field name to get the description for.
         * @return The Java description of the langkit field.
         */
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(String name) {
            return AdaNode.fieldDescriptions.get(name);
        }

        /**
         * Get the children count of the node.
         *
         * @return The children count.
         */
        public int getChildrenCount() {

            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                return NI_LIB.ada_node_children_count(thisNative);
            } else {
                return JNI_LIB.ada_node_children_count(this.entity);
            }

        }

        /**
         * Get the child at the given position.
         *
         * @param n The index of the child to get.
         * @return The child at the given index.
         */
        public AdaNode getChild(
            final int n
        ) {

            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                final EntityNative resNative = StackValue.get(
                    EntityNative.class
                );
                NI_LIB.ada_node_child(
                    thisNative,
                    n,
                    resNative
                );

                return fromEntity(Entity.wrap(resNative));
            } else {
                return fromEntity(JNI_LIB.ada_node_child(
                    this.entity,
                    n
                ));
            }

        }

        /**
         * Get the text of the node.
         *
         * @return The text of the node.
         */
        public String getText() {

            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                final TextNative resNative = StackValue.get(TextNative.class);
                Text.NONE.unwrap(resNative);
                NI_LIB.ada_node_text(
                    thisNative,
                    resNative
                );

                try(final Text resText = Text.wrap(resNative)) {
                    return resText.getContent();
                }
            } else {
                try(
                    final Text resText = JNI_LIB.ada_node_text(
                        this.entity
                    );
                ) {
                    return resText.getContent();
                }
            }

        }

        /**
         * Get the image of the node.
         *
         * @return The node's image.
         */
        public String getImage() {
            if(this.image == null) {

                if(ImageInfo.inImageCode()) {
                    final EntityNative thisNative = StackValue.get(
                        EntityNative.class
                    );
                    this.entity.unwrap(thisNative);

                    final TextNative resNative = StackValue.get(
                        TextNative.class
                    );
                    Text.NONE.unwrap(resNative);
                    NI_LIB.ada_node_image(thisNative, resNative);

                    try(final Text resText = Text.wrap(resNative)) {
                        this.image = resText.getContent();
                    }
                } else {
                    try(
                        final Text resText = JNI_LIB.ada_node_image(
                            this.entity
                        )
                    ) {
                        this.image = resText.getContent();
                    }
                }

            }

            return this.image;
        }

        /**
         * Get the source location range of the node.
         *
         * @return The source location range of the node.
         */
        public SourceLocationRange getSourceLocationRange() {

            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                final SourceLocationRangeNative resNative = StackValue.get(
                    SourceLocationRangeNative.class
                );
                NI_LIB.ada_node_sloc_range(
                    thisNative,
                    resNative
                );

                return SourceLocationRange.wrap(resNative);
            } else {
                return JNI_LIB.ada_node_sloc_range(this.entity);
            }

        }

        // ----- Dumping methods -----

        /**
         * Return the AST in a string.
         *
         * @return The string containing the representation of the AST
         * from the node.
         */
        @CompilerDirectives.TruffleBoundary
        public String dumpAST() {
            final StringBuilder builder = new StringBuilder();
            this.dumpAST(builder);
            return builder.toString();
        }

        /**
         * Dump the AST in the given string builder.
         *
         * @param builder The builder to dump the AST in.
         */
        @CompilerDirectives.TruffleBoundary
        public void dumpAST(
            final StringBuilder builder
        ) {
            this.dumpAST(builder, "");
        }

        /**
         * Dump a node field in the given string builder.
         *
         * @param builder The string builder to put the file in.
         * @param indent The current indentation string.
         * @param name The name of the field.
         * @param value The value of the field.
         */
        protected static void dumpField(
            final StringBuilder builder,
            final String indent,
            final String name,
            final AdaNode value
        ) {
            builder.append(indent)
                .append(name)
                .append(":\n");
            value.dumpAST(builder, indent + "  ");
        }

        /**
         * Dump the AST in the given string builder with the indent level.
         *
         * @param builder The builder to dump the AST in.
         * @param indent The starting indent level.
         */
        @CompilerDirectives.TruffleBoundary
        protected void dumpAST(
            final StringBuilder builder,
            String indent
        ) {
            // Prepare the working variables
            String image = this.getImage();
            image = image.substring(1, image.length());
            final int childrenCount = this.getChildrenCount();

            // Print the node
            builder.append(indent)
                .append(image);
            if(this.isTokenNode()) {
                builder.append(": ")
                    .append(this.getText());
            }
            builder.append('\n');

            // Print the field of the node
            indent = indent + "|";
            if(this.isListType()) {
                for(int i = 0 ; i < childrenCount ; i++) {
                    final AdaNode child = this.getChild(i);
                    dumpField(builder, indent, "item_" + i, child);
                }
            } else {
                for(int i = 0 ; i < childrenCount ; i++) {
                    final AdaNode child = this.getChild(i);
                    final String name = this.getFieldNames()[i];
                    dumpField(builder, indent, name, child);
                }
            }
        }

        // ----- Visitor methods -----

        /**
         * Accept the given visitor.
         *
         * @param visitor The visitor to accept.
         * @return The result of the visit.
         */
        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        /**
         * Accept the given visitor.
         *
         * @param visitor The visitor to accept.
         * @param param The parameter of the visit.
         * @return The result of the visit.
         */
        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the scope of definition of this basic declaration.
         */
        public DeclarativePart pDeclarativeScope(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_declarative_scope(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_declarative_scope(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * Return the compilation unit containing this node.
         *
         * .. note:: This returns the ``CompilationUnit`` node, which is
         *    different from the ``AnalysisUnit``. In particular, an analysis
         *    unit can contain multiple compilation units.
         */
        public CompilationUnit pEnclosingCompilationUnit(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_enclosing_compilation_unit(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CompilationUnit res = CompilationUnit.NONE;
                if(propException == null) {
                    res = CompilationUnit.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_enclosing_compilation_unit(
                    this.entity
                );

                // Wrap and return the result
                return CompilationUnit.fromEntity(res);
            }

        }

        
    

        /**
         * Assuming this node comes from an instantiated generic declaration,
         * return its non-instantiated counterpart lying in the generic
         * declaration.
         */
        public AdaNode pGetUninstantiatedNode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_get_uninstantiated_node(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_get_uninstantiated_node(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return possible completions at this point in the file.
         */
        public CompletionItemIterator pComplete(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_complete(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CompletionItemIterator res = CompletionItemIterator.NONE;
                if(propException == null) {
                    res = CompletionItemIterator.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final CompletionItemIterator res = JNI_LIB.ada_ada_node_p_complete(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of keywords that are valid at this point in the
         * file.
         *
         * .. note:: This is work in progress. It will return all keywords for
         *    now, without looking at the context.
         */
        public UnboundedTextTypeArray pValidKeywords(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_valid_keywords(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_ada_node_p_valid_keywords(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the potentially empty list of generic package/subprogram
         * instantiations that led to the creation of this entity. Outer-most
         * instantiations appear last.
         */
        public AdaNodeArray pGenericInstantiations(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_generic_instantiations(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_ada_node_p_generic_instantiations(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the semantic parent for this node, if applicable, null
         * otherwise.
         *
         * .. note:: A node lying outside of a library item's declaration or
         *    subunit's body does not have a parent environment, meaning that
         *    this property will return null.
         */
        public AdaNode pSemanticParent(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_semantic_parent(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_semantic_parent(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the parent basic decl for this node, if applicable, null
         * otherwise.
         *
         * .. note:: If the parent BasicDecl of the given node is a generic
         *    declaration, this call will return the instantiation from which
         *    the node was retrieved instead, if any.
         *
         * .. note:: When called on a subunit's body, this property will return
         *    its corresponding body stub.
         *
         * .. note:: When called on a node lying outside of a library item's
         *    declaration or subunit's body this property will return null.
         */
        public BasicDecl pParentBasicDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_parent_basic_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_parent_basic_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Filters out among the list of given units those that cannot refer to
         * the unit in which this node lies. If transitive is True, the whole
         * transitive closure of imports will be used to find a reference to
         * the unit of this node.
         */
        public AnalysisUnitArray pFilterIsImportedBy(
            final AnalysisUnitArray units,final boolean transitive
        ) {

            // Verify that arguments are not null
            if(units == null) throw new IllegalArgumentException(
                "Argument 'units' of type " +
                "AnalysisUnitArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitArrayNative unitsNative = units.unwrap();
                byte transitiveNative = (transitive ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_filter_is_imported_by(
                    thisNative,
                    unitsNative,
                    transitiveNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AnalysisUnitArray res = AnalysisUnitArray.NONE;
                if(propException == null) {
                    res = AnalysisUnitArray.wrap(resNative);

                    AnalysisUnitArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                AnalysisUnitArray.release(
                    unitsNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AnalysisUnitArray res = JNI_LIB.ada_ada_node_p_filter_is_imported_by(
                    units,
                    transitive,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Designates entities that are entry point for the xref solving
         * infrastructure. If this returns true, then resolve_names can be
         * called on it.
         *
         * .. note:: For convenience, and unlike what is defined in the ARM
         *    wrt. complete contexts for name resolution, ``xref_entry_points``
         *    can be nested.
         */
        public boolean pXrefEntryPoint(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_xref_entry_point(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_ada_node_p_xref_entry_point(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * This will resolve names for this node. If the operation is
         * successful, then type_var and ref_var will be bound on appropriate
         * subnodes of the statement.
         */
        public boolean pResolveNames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_resolve_names(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_ada_node_p_resolve_names(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Static method. Return the analysis unit corresponding to the
         * Standard package.
         */
        public AnalysisUnit pStandardUnit(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_standard_unit(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AnalysisUnit res = AnalysisUnit.NONE;
                if(propException == null) {
                    res = AnalysisUnit.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AnalysisUnit res = JNI_LIB.ada_ada_node_p_standard_unit(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Static property. Return an entity from the standard package with
         * name ``sym``.
         */
        public AdaNode pStdEntity(
            final Symbol sym
        ) {

            // Verify that arguments are not null
            if(sym == null) throw new IllegalArgumentException(
                "Argument 'sym' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative symNative = StackValue.get(SymbolNative.class);sym.unwrap(symNative, currentContext);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_std_entity(
                    thisNative,
                    symNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_std_entity(
                    sym,
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Boolean type.
         */
        public BaseTypeDecl pBoolType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_bool_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_bool_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Integer type.
         */
        public BaseTypeDecl pIntType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_int_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_int_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Universal Integer type.
         */
        public AdaNode pUniversalIntType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_universal_int_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_universal_int_type(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Universal Real type.
         */
        public AdaNode pUniversalRealType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_universal_real_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_universal_real_type(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Character type.
         */
        public BaseTypeDecl pStdCharType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_std_char_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_std_char_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Wide_Character type.
         */
        public BaseTypeDecl pStdWideCharType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_std_wide_char_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_std_wide_char_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Return the standard Wide_Wide_Character type.
         */
        public BaseTypeDecl pStdWideWideCharType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_std_wide_wide_char_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_std_wide_wide_char_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Static method. Get the top-level decl in ``unit``.  This is the body
         * of a Subunit, or the item of a ``LibraryItem``.
         */
        public BasicDecl pTopLevelDecl(
            final AnalysisUnit unit
        ) {

            // Verify that arguments are not null
            if(unit == null) throw new IllegalArgumentException(
                "Argument 'unit' of type " +
                "AnalysisUnit cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitNative unitNative = unit.unwrap();

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_top_level_decl(
                    thisNative,
                    unitNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_top_level_decl(
                    unit,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Assuming that self is a choice expression (such as what can appear
         * in an alternative of a case statement or in the RHS of a membership
         * expression, this property returns whether the given value satisfies
         * it.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public boolean pChoiceMatch(
            final BigInteger value
        ) {

            // Verify that arguments are not null
            if(value == null) throw new IllegalArgumentException(
                "Argument 'value' of type " +
                "BigInteger cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                BigIntegerNative valueNative = StackValue.get(SizeOf.get(VoidPointer.class));BigIntegerWrapper.unwrap(value, valueNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_p_choice_match(
                    thisNative,
                    valueNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters
                BigIntegerWrapper.release(
                    valueNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_ada_node_p_choice_match(
                    value,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a cross reference from this name to a defining identifier,
         * trying to mimic GNAT's xrefs as much as possible.
         */
        public DefiningName pGnatXref(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_p_gnat_xref(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_p_gnat_xref(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return the syntactic parent for this node. Return null for the root
         * node.
         */
        public AdaNode parent(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_parent(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_parent(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return an array that contains the lexical parents, this node
         * included iff ``with_self`` is True. Nearer parents are first in the
         * list.
         */
        public AdaNodeArray parents(
            final boolean withSelf
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte withSelfNative = (withSelf ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_parents(
                    thisNative,
                    withSelfNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_ada_node_parents(
                    withSelf,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return an array that contains the direct lexical children.
         *
         * .. warning:: This constructs a whole array every-time you call it,
         *    and as such is less efficient than calling the ``Child`` built-
         *    in.
         */
        public AdaNodeArray children(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_children(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_ada_node_children(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the first token used to parse this node.
         */
        public Token tokenStart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();


                // Unwrap the arguments

                // Create the result native
                final TokenNative resNative =
                    StackValue.get(TokenNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_token_start(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Token res = Token.NONE(currentUnit);
                if(propException == null) {
                    res = Token.wrap(resNative, currentUnit);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Token res = JNI_LIB.ada_ada_node_token_start(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the last token used to parse this node.
         */
        public Token tokenEnd(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();


                // Unwrap the arguments

                // Create the result native
                final TokenNative resNative =
                    StackValue.get(TokenNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_token_end(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Token res = Token.NONE(currentUnit);
                if(propException == null) {
                    res = Token.wrap(resNative, currentUnit);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Token res = JNI_LIB.ada_ada_node_token_end(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the 0-based index for Node in its parent's children.
         */
        public int childIndex(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_child_index(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                int res = 0;
                if(propException == null) {
                    res = resNative.read();

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final int res = JNI_LIB.ada_ada_node_child_index(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the node's previous sibling, or null if there is no such
         * sibling.
         */
        public AdaNode previousSibling(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_previous_sibling(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_previous_sibling(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the node's next sibling, or null if there is no such sibling.
         */
        public AdaNode nextSibling(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ada_node_next_sibling(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ada_node_next_sibling(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    


        
    

        /**
         * Return whether the node is a ghost.
         *
         * Unlike regular nodes, ghost nodes cover no token in the input
         * source: they are logically located instead between two tokens. Both
         * the ``token_start`` and the ``token_end`` of all ghost nodes is the
         * token right after this logical position.
         */
        public boolean isGhost(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_is_ghost(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_ada_node_is_ghost(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a string containing the filename + the sloc in GNU conformant
         * format. Useful to create diagnostics from a node.
         */
        public String fullSlocImage(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_ada_node_full_sloc_image(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_ada_node_full_sloc_image(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Override methods -----

        @Override
        public String toString() {
            return this.getImage();
        }

        @Override
        public boolean equals(Object o) {
            if(this == o) return true;
            if(!(o instanceof AdaNode)) return false;
            final AdaNode other = (AdaNode) o;

            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                final EntityNative otherNative = StackValue.get(
                    EntityNative.class
                );
                other.entity.unwrap(otherNative);

                return NI_LIB.ada_node_is_equivalent(
                    thisNative,
                    otherNative
                ) != 0;
            } else {
                return JNI_LIB.ada_node_is_equivalent(
                    this.entity, other.entity
                ) != 0;
            }
        }

        @Override
        public int hashCode() {
            if(ImageInfo.inImageCode()) {
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                return NI_LIB.ada_node_hash(thisNative);
            } else {
                return JNI_LIB.ada_node_hash(this.entity);
            }

        }

        // ----- Inner classes -----

        /**
         * This class represents the none node without any concrete type.
         */
        private static final class NoneNode extends AdaNode {
            NoneNode() {super(Entity.NONE);}
        }

    }

    // ===== Generated AST node wrapping classes =====

    
    

    /**
     * Qualifier for the ``abort`` keyword.
     */
    public static abstract
    class AbortNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbortNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AbortNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbortNode NONE =
            new AbortNodeNone();

        // ----- Constructors -----

        protected AbortNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbortNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbortNode.NONE :

                (AbortNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbortNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbortNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbortNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbortNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of AbortPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_abort_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_abort_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AbortNode.
         */
        private static final class AbortNodeNone extends AbortNode {
            AbortNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class AbortAbsent extends AbortNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbortAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AbortNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbortAbsent NONE =
            new AbortAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbortAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbortAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbortAbsent.NONE :

                new AbortAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbortAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbortAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbortAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbortAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class AbortPresent extends AbortNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbortPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AbortNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbortPresent NONE =
            new AbortPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbortPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbortPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbortPresent.NONE :

                new AbortPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbortPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbortPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbortPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbortPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``abstract`` keyword.
     */
    public static abstract
    class AbstractNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AbstractNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractNode NONE =
            new AbstractNodeNone();

        // ----- Constructors -----

        protected AbstractNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractNode.NONE :

                (AbstractNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of AbstractPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_abstract_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_abstract_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AbstractNode.
         */
        private static final class AbstractNodeNone extends AbstractNode {
            AbstractNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class AbstractAbsent extends AbstractNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AbstractNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractAbsent NONE =
            new AbstractAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractAbsent.NONE :

                new AbstractAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class AbstractPresent extends AbstractNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AbstractNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractPresent NONE =
            new AbstractPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractPresent.NONE :

                new AbstractPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static abstract
    class AdaList extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AdaList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AdaList NONE =
            new AdaListNone();

        // ----- Constructors -----

        protected AdaList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AdaList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AdaList.NONE :

                (AdaList) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AdaList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AdaList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AdaList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AdaList.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AdaList.
         */
        private static final class AdaListNone extends AdaList {
            AdaListNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * List of AdaNode.
     *
     * This list node can contain one of the following nodes:
     * ``AbstractStateDecl``, ``AbstractSubpDecl``, ``Allocator``,
     * ``AspectClause``, ``AttributeRef``, ``BaseAggregate``, ``BaseSubpBody``,
     * ``BinOp``, ``BodyStub``, ``CallExpr``, ``CharLiteral``,
     * ``ComponentClause``, ``ComponentDecl``, ``ConcatOp``,
     * ``ConcreteTypeDecl``, ``CondExpr``, ``DeclExpr``, ``DottedName``,
     * ``EntryBody``, ``EntryDecl``, ``ErrorDecl``, ``ExceptionDecl``,
     * ``ExceptionHandler``, ``ExplicitDeref``, ``GenericDecl``,
     * ``GenericFormal``, ``GenericInstantiation``, ``GenericRenamingDecl``,
     * ``Identifier``, ``IncompleteTypeDecl``, ``MembershipExpr``,
     * ``NullComponentDecl``, ``NullLiteral``, ``NumLiteral``, ``NumberDecl``,
     * ``ObjectDecl``, ``OthersDesignator``, ``PackageBody``, ``PackageDecl``,
     * ``PackageRenamingDecl``, ``ParenAbstractStateDecl``, ``ParenExpr``,
     * ``PragmaNode``, ``ProtectedBody``, ``ProtectedTypeDecl``, ``QualExpr``,
     * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
     * ``SingleProtectedDecl``, ``SingleTaskDecl``, ``Stmt``,
     * ``StringLiteral``, ``SubpDecl``, ``SubtypeDecl``, ``SubtypeIndication``,
     * ``TargetName``, ``TaskBody``, ``TaskTypeDecl``, ``UnOp``,
     * ``UpdateAttributeRef``, ``UseClause``, ``WithClause``
     */
    public static 
    class AdaNodeList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AdaNodeList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AdaNodeList NONE =
            new AdaNodeList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AdaNodeList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AdaNodeList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AdaNodeList.NONE :

                new AdaNodeList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AdaNodeList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AdaNodeList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AdaNodeList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AdaNodeList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of AbstractStateDecls.
     *
     * This list node can contain one of the following nodes:
     * ``AbstractStateDecl``, ``ParenAbstractStateDecl``
     */
    public static 
    class AbstractStateDeclList extends AdaNodeList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractStateDeclList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNodeList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractStateDeclList NONE =
            new AbstractStateDeclList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractStateDeclList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractStateDeclList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractStateDeclList.NONE :

                new AbstractStateDeclList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractStateDeclList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractStateDeclList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractStateDeclList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractStateDeclList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of alternatives in a ``when ...`` clause.
     *
     * This list node can contain one of the following nodes: ``Allocator``,
     * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
     * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
     * ``DiscreteSubtypeIndication``, ``DottedName``, ``ExplicitDeref``,
     * ``Identifier``, ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
     * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
     * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
     * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
     */
    public static 
    class AlternativesList extends AdaNodeList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AlternativesList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNodeList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AlternativesList NONE =
            new AlternativesList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AlternativesList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AlternativesList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AlternativesList.NONE :

                new AlternativesList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AlternativesList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AlternativesList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AlternativesList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AlternativesList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of constraints.
     *
     * This list node can contain one of the following nodes: ``AttributeRef``,
     * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``DottedName``,
     * ``ExplicitDeref``, ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
     * ``StringLiteral``, ``SubtypeIndication``, ``TargetName``,
     * ``UpdateAttributeRef``
     */
    public static 
    class ConstraintList extends AdaNodeList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstraintList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNodeList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstraintList NONE =
            new ConstraintList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConstraintList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstraintList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstraintList.NONE :

                new ConstraintList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstraintList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstraintList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstraintList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstraintList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of declarations.
     *
     * This list node can contain one of the following nodes:
     * ``AbstractSubpDecl``, ``AspectClause``, ``ComponentDecl``,
     * ``EntryDecl``, ``ExprFunction``, ``NullSubpDecl``, ``PragmaNode``,
     * ``SubpDecl``, ``SubpRenamingDecl``
     */
    public static 
    class DeclList extends AdaNodeList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeclList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNodeList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeclList NONE =
            new DeclList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeclList(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeclList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeclList.NONE :

                new DeclList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeclList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeclList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeclList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeclList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of statements.
     *
     * This list node can contain one of the following nodes: ``PragmaNode``,
     * ``Stmt``
     */
    public static 
    class StmtList extends AdaNodeList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "StmtList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNodeList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final StmtList NONE =
            new StmtList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected StmtList(
            final Entity entity
        ) {
            super(entity);
        }

        public static StmtList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                StmtList.NONE :

                new StmtList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return StmtList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return StmtList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return StmtList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return StmtList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of AspectAssoc.
     */
    public static 
    class AspectAssocList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AspectAssocList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AspectAssocList NONE =
            new AspectAssocList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AspectAssocList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AspectAssocList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AspectAssocList.NONE :

                new AspectAssocList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AspectAssocList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AspectAssocList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AspectAssocList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AspectAssocList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of BaseAssoc.
     */
    public static 
    class BaseAssocList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseAssocList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseAssocList NONE =
            new BaseAssocList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BaseAssocList(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseAssocList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseAssocList.NONE :

                new BaseAssocList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseAssocList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseAssocList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseAssocList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseAssocList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of BasicAssoc.
     */
    public static abstract
    class BasicAssocList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasicAssocList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasicAssocList NONE =
            new BasicAssocListNone();

        // ----- Constructors -----

        protected BasicAssocList(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasicAssocList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasicAssocList.NONE :

                (BasicAssocList) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasicAssocList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasicAssocList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasicAssocList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasicAssocList.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BasicAssocList.
         */
        private static final class BasicAssocListNone extends BasicAssocList {
            BasicAssocListNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * List of associations.
     */
    public static 
    class AssocList extends BasicAssocList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AssocList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicAssocList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AssocList.class.getMethod(
                        "pZipWithParams",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_zip_with_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AssocList NONE =
            new AssocList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AssocList(
            final Entity entity
        ) {
            super(entity);
        }

        public static AssocList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AssocList.NONE :

                new AssocList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AssocList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AssocList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AssocList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AssocList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Returns an array of pairs, associating formal parameters to actual
         * expressions. The formals to match are retrieved by resolving the
         * call which this AssocList represents the actuals of.
         */
        public ParamActualArray pZipWithParams(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_assoc_list_p_zip_with_params(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_assoc_list_p_zip_with_params(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of BasicDecl.
     *
     * This list node can contain one of the following nodes: ``NumberDecl``,
     * ``ObjectDecl``, ``SingleProtectedDecl``, ``SingleTaskDecl``
     */
    public static 
    class BasicDeclList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasicDeclList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasicDeclList NONE =
            new BasicDeclList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BasicDeclList(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasicDeclList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasicDeclList.NONE :

                new BasicDeclList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasicDeclList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasicDeclList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasicDeclList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasicDeclList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of CaseExprAlternative.
     */
    public static 
    class CaseExprAlternativeList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseExprAlternativeList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseExprAlternativeList NONE =
            new CaseExprAlternativeList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseExprAlternativeList(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseExprAlternativeList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseExprAlternativeList.NONE :

                new CaseExprAlternativeList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseExprAlternativeList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseExprAlternativeList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseExprAlternativeList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseExprAlternativeList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of CaseStmtAlternative.
     */
    public static 
    class CaseStmtAlternativeList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseStmtAlternativeList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseStmtAlternativeList NONE =
            new CaseStmtAlternativeList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseStmtAlternativeList(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseStmtAlternativeList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseStmtAlternativeList.NONE :

                new CaseStmtAlternativeList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseStmtAlternativeList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseStmtAlternativeList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseStmtAlternativeList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseStmtAlternativeList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of CompilationUnit.
     */
    public static 
    class CompilationUnitList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CompilationUnitList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CompilationUnitList NONE =
            new CompilationUnitList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CompilationUnitList(
            final Entity entity
        ) {
            super(entity);
        }

        public static CompilationUnitList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CompilationUnitList.NONE :

                new CompilationUnitList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CompilationUnitList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CompilationUnitList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CompilationUnitList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CompilationUnitList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of ConcatOperand.
     */
    public static 
    class ConcatOperandList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConcatOperandList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConcatOperandList NONE =
            new ConcatOperandList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConcatOperandList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConcatOperandList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConcatOperandList.NONE :

                new ConcatOperandList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConcatOperandList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConcatOperandList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConcatOperandList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConcatOperandList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of ContractCaseAssoc.
     */
    public static 
    class ContractCaseAssocList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ContractCaseAssocList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ContractCaseAssocList NONE =
            new ContractCaseAssocList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ContractCaseAssocList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ContractCaseAssocList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ContractCaseAssocList.NONE :

                new ContractCaseAssocList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ContractCaseAssocList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ContractCaseAssocList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ContractCaseAssocList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ContractCaseAssocList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of DefiningName.
     */
    public static 
    class DefiningNameList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DefiningNameList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DefiningNameList NONE =
            new DefiningNameList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DefiningNameList(
            final Entity entity
        ) {
            super(entity);
        }

        public static DefiningNameList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DefiningNameList.NONE :

                new DefiningNameList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DefiningNameList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DefiningNameList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DefiningNameList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DefiningNameList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of DiscriminantSpec.
     */
    public static 
    class DiscriminantSpecList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscriminantSpecList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscriminantSpecList NONE =
            new DiscriminantSpecList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscriminantSpecList(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscriminantSpecList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscriminantSpecList.NONE :

                new DiscriminantSpecList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscriminantSpecList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscriminantSpecList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscriminantSpecList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscriminantSpecList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of ElsifExprPart.
     */
    public static 
    class ElsifExprPartList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ElsifExprPartList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ElsifExprPartList NONE =
            new ElsifExprPartList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ElsifExprPartList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ElsifExprPartList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ElsifExprPartList.NONE :

                new ElsifExprPartList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ElsifExprPartList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ElsifExprPartList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ElsifExprPartList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ElsifExprPartList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of ElsifStmtPart.
     */
    public static 
    class ElsifStmtPartList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ElsifStmtPartList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ElsifStmtPartList NONE =
            new ElsifStmtPartList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ElsifStmtPartList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ElsifStmtPartList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ElsifStmtPartList.NONE :

                new ElsifStmtPartList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ElsifStmtPartList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ElsifStmtPartList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ElsifStmtPartList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ElsifStmtPartList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of EnumLiteralDecl.
     */
    public static 
    class EnumLiteralDeclList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumLiteralDeclList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumLiteralDeclList NONE =
            new EnumLiteralDeclList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumLiteralDeclList(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumLiteralDeclList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumLiteralDeclList.NONE :

                new EnumLiteralDeclList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumLiteralDeclList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumLiteralDeclList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumLiteralDeclList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumLiteralDeclList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of Expr.
     *
     * This list node can contain one of the following nodes: ``Allocator``,
     * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
     * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
     * ``DiscreteSubtypeName``, ``DottedName``, ``ExplicitDeref``,
     * ``Identifier``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
     * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
     * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
     */
    public static abstract
    class ExprList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExprList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExprList NONE =
            new ExprListNone();

        // ----- Constructors -----

        protected ExprList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExprList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExprList.NONE :

                (ExprList) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExprList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExprList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExprList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExprList.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ExprList.
         */
        private static final class ExprListNone extends ExprList {
            ExprListNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * List of alternatives in a membership test expression.
     *
     * This list node can contain one of the following nodes: ``Allocator``,
     * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
     * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
     * ``DiscreteSubtypeName``, ``DottedName``, ``ExplicitDeref``,
     * ``Identifier``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
     * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
     * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
     */
    public static 
    class ExprAlternativesList extends ExprList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExprAlternativesList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ExprList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExprAlternativesList NONE =
            new ExprAlternativesList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExprAlternativesList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExprAlternativesList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExprAlternativesList.NONE :

                new ExprAlternativesList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExprAlternativesList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExprAlternativesList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExprAlternativesList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExprAlternativesList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of Identifier.
     */
    public static abstract
    class IdentifierList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IdentifierList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IdentifierList NONE =
            new IdentifierListNone();

        // ----- Constructors -----

        protected IdentifierList(
            final Entity entity
        ) {
            super(entity);
        }

        public static IdentifierList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IdentifierList.NONE :

                (IdentifierList) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IdentifierList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IdentifierList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IdentifierList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IdentifierList.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * IdentifierList.
         */
        private static final class IdentifierListNone extends IdentifierList {
            IdentifierListNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * List of discriminant associations.
     */
    public static 
    class DiscriminantChoiceList extends IdentifierList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscriminantChoiceList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            IdentifierList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscriminantChoiceList NONE =
            new DiscriminantChoiceList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscriminantChoiceList(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscriminantChoiceList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscriminantChoiceList.NONE :

                new DiscriminantChoiceList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscriminantChoiceList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscriminantChoiceList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscriminantChoiceList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscriminantChoiceList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of Name.
     *
     * This list node can contain one of the following nodes: ``AttributeRef``,
     * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
     * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
     * ``TargetName``, ``UpdateAttributeRef``
     */
    public static 
    class NameList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NameList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NameList NONE =
            new NameList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NameList(
            final Entity entity
        ) {
            super(entity);
        }

        public static NameList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NameList.NONE :

                new NameList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NameList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NameList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NameList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NameList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of parents in a type declaration.
     *
     * This list node can contain one of the following nodes: ``CharLiteral``,
     * ``DottedName``, ``Identifier``, ``StringLiteral``
     */
    public static 
    class ParentList extends NameList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParentList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            NameList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParentList NONE =
            new ParentList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParentList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParentList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParentList.NONE :

                new ParentList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParentList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParentList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParentList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParentList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of ParamSpec.
     */
    public static 
    class ParamSpecList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParamSpecList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParamSpecList NONE =
            new ParamSpecList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParamSpecList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParamSpecList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParamSpecList.NONE :

                new ParamSpecList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParamSpecList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParamSpecList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParamSpecList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParamSpecList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of Pragma.
     */
    public static 
    class PragmaNodeList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PragmaNodeList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PragmaNodeList NONE =
            new PragmaNodeList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PragmaNodeList(
            final Entity entity
        ) {
            super(entity);
        }

        public static PragmaNodeList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PragmaNodeList.NONE :

                new PragmaNodeList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PragmaNodeList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PragmaNodeList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PragmaNodeList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PragmaNodeList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of SelectWhenPart.
     */
    public static 
    class SelectWhenPartList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SelectWhenPartList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SelectWhenPartList NONE =
            new SelectWhenPartList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SelectWhenPartList(
            final Entity entity
        ) {
            super(entity);
        }

        public static SelectWhenPartList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SelectWhenPartList.NONE :

                new SelectWhenPartList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SelectWhenPartList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SelectWhenPartList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SelectWhenPartList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SelectWhenPartList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of UnconstrainedArrayIndex.
     */
    public static 
    class UnconstrainedArrayIndexList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UnconstrainedArrayIndexList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UnconstrainedArrayIndexList NONE =
            new UnconstrainedArrayIndexList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UnconstrainedArrayIndexList(
            final Entity entity
        ) {
            super(entity);
        }

        public static UnconstrainedArrayIndexList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UnconstrainedArrayIndexList.NONE :

                new UnconstrainedArrayIndexList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UnconstrainedArrayIndexList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UnconstrainedArrayIndexList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UnconstrainedArrayIndexList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UnconstrainedArrayIndexList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of Variant.
     */
    public static 
    class VariantList extends AdaList {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "VariantList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            true;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaList.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final VariantList NONE =
            new VariantList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected VariantList(
            final Entity entity
        ) {
            super(entity);
        }

        public static VariantList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                VariantList.NONE :

                new VariantList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return VariantList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return VariantList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return VariantList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return VariantList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``aliased`` keyword.
     */
    public static abstract
    class AliasedNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AliasedNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AliasedNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AliasedNode NONE =
            new AliasedNodeNone();

        // ----- Constructors -----

        protected AliasedNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static AliasedNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AliasedNode.NONE :

                (AliasedNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AliasedNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AliasedNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AliasedNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AliasedNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of AliasedPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_aliased_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_aliased_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AliasedNode.
         */
        private static final class AliasedNodeNone extends AliasedNode {
            AliasedNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class AliasedAbsent extends AliasedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AliasedAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AliasedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AliasedAbsent NONE =
            new AliasedAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AliasedAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AliasedAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AliasedAbsent.NONE :

                new AliasedAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AliasedAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AliasedAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AliasedAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AliasedAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class AliasedPresent extends AliasedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AliasedPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AliasedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AliasedPresent NONE =
            new AliasedPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AliasedPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AliasedPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AliasedPresent.NONE :

                new AliasedPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AliasedPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AliasedPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AliasedPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AliasedPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``all`` keyword.
     */
    public static abstract
    class AllNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AllNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AllNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AllNode NONE =
            new AllNodeNone();

        // ----- Constructors -----

        protected AllNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static AllNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AllNode.NONE :

                (AllNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AllNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AllNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AllNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AllNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of AllPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_all_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_all_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AllNode.
         */
        private static final class AllNodeNone extends AllNode {
            AllNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class AllAbsent extends AllNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AllAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AllNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AllAbsent NONE =
            new AllAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AllAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AllAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AllAbsent.NONE :

                new AllAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AllAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AllAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AllAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AllAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class AllPresent extends AllNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AllPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AllNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AllPresent NONE =
            new AllPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AllPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static AllPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AllPresent.NONE :

                new AllPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AllPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AllPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AllPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AllPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Specification for array indexes (:rmlink:`3.6`).
     */
    public static abstract
    class ArrayIndices extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ArrayIndices";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ArrayIndices NONE =
            new ArrayIndicesNone();

        // ----- Constructors -----

        protected ArrayIndices(
            final Entity entity
        ) {
            super(entity);
        }

        public static ArrayIndices fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ArrayIndices.NONE :

                (ArrayIndices) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ArrayIndices.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ArrayIndices.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ArrayIndices.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ArrayIndices.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ArrayIndices.
         */
        private static final class ArrayIndicesNone extends ArrayIndices {
            ArrayIndicesNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Constrained specification for array indexes (:rmlink:`3.6`).
     */
    public static 
    class ConstrainedArrayIndices extends ArrayIndices {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstrainedArrayIndices";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fList"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ArrayIndices.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ConstrainedArrayIndices.class.getMethod(
                        "fList",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_list",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstrainedArrayIndices NONE =
            new ConstrainedArrayIndices(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConstrainedArrayIndices(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstrainedArrayIndices fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstrainedArrayIndices.NONE :

                new ConstrainedArrayIndices(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstrainedArrayIndices.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstrainedArrayIndices.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstrainedArrayIndices.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstrainedArrayIndices.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AttributeRef``, ``BinOp``, ``CallExpr``, ``CharLiteral``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``SubtypeIndication``,
         * ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ConstraintList fList(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_constrained_array_indices_f_list(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ConstraintList res = ConstraintList.NONE;
                if(propException == null) {
                    res = ConstraintList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_constrained_array_indices_f_list(
                    this.entity
                );

                // Wrap and return the result
                return ConstraintList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Unconstrained specification for array indexes (:rmlink:`3.6`).
     */
    public static 
    class UnconstrainedArrayIndices extends ArrayIndices {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UnconstrainedArrayIndices";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fTypes"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ArrayIndices.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UnconstrainedArrayIndices.class.getMethod(
                        "fTypes",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_types",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UnconstrainedArrayIndices NONE =
            new UnconstrainedArrayIndices(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UnconstrainedArrayIndices(
            final Entity entity
        ) {
            super(entity);
        }

        public static UnconstrainedArrayIndices fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UnconstrainedArrayIndices.NONE :

                new UnconstrainedArrayIndices(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UnconstrainedArrayIndices.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UnconstrainedArrayIndices.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UnconstrainedArrayIndices.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UnconstrainedArrayIndices.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public UnconstrainedArrayIndexList fTypes(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_unconstrained_array_indices_f_types(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnconstrainedArrayIndexList res = UnconstrainedArrayIndexList.NONE;
                if(propException == null) {
                    res = UnconstrainedArrayIndexList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_unconstrained_array_indices_f_types(
                    this.entity
                );

                // Wrap and return the result
                return UnconstrainedArrayIndexList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Name/expression association in an aspect.
     */
    public static 
    class AspectAssoc extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AspectAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fId","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AspectAssoc.class.getMethod(
                        "fId",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AspectAssoc.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AspectAssoc.class.getMethod(
                        "pIsGhostCode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_ghost_code",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AspectAssoc NONE =
            new AspectAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AspectAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static AspectAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AspectAssoc.NONE :

                new AspectAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AspectAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AspectAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AspectAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AspectAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fId(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_aspect_assoc_f_id(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_aspect_assoc_f_id(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AbstractStateDeclExpr``, ``Allocator``, ``AttributeRef``,
         * ``BaseAggregate``, ``BinOp``, ``CallExpr``, ``CharLiteral``,
         * ``ConcatOp``, ``CondExpr``, ``ContractCases``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_aspect_assoc_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_aspect_assoc_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * Return whether this aspect is ghost code or not. See SPARK RM 6.9.
         */
        public boolean pIsGhostCode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_aspect_assoc_p_is_ghost_code(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_aspect_assoc_p_is_ghost_code(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for aspect clauses.
     */
    public static abstract
    class AspectClause extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AspectClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AspectClause NONE =
            new AspectClauseNone();

        // ----- Constructors -----

        protected AspectClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static AspectClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AspectClause.NONE :

                (AspectClause) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AspectClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AspectClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AspectClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AspectClause.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AspectClause.
         */
        private static final class AspectClauseNone extends AspectClause {
            AspectClauseNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Representation clause (``for .. use at ...;``) (:rmlink:`13.5.1`).
     */
    public static 
    class AtClause extends AspectClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AtClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AspectClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AtClause.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AtClause.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AtClause NONE =
            new AtClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AtClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static AtClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AtClause.NONE :

                new AtClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AtClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AtClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AtClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AtClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BaseId fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_at_clause_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseId res = BaseId.NONE;
                if(propException == null) {
                    res = BaseId.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_at_clause_f_name(
                    this.entity
                );

                // Wrap and return the result
                return BaseId.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_at_clause_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_at_clause_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Clause for an attribute definition (``for ...'Attribute use ...;``)
     * (:rmlink:`13.3`).
     */
    public static 
    class AttributeDefClause extends AspectClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AttributeDefClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAttributeExpr","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AspectClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AttributeDefClause.class.getMethod(
                        "fAttributeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_attribute_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AttributeDefClause.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AttributeDefClause NONE =
            new AttributeDefClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AttributeDefClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static AttributeDefClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AttributeDefClause.NONE :

                new AttributeDefClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AttributeDefClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AttributeDefClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AttributeDefClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AttributeDefClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fAttributeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_attribute_def_clause_f_attribute_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_attribute_def_clause_f_attribute_expr(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_attribute_def_clause_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_attribute_def_clause_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Representation clause for enumeration types (:rmlink:`13.4`).
     */
    public static 
    class EnumRepClause extends AspectClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumRepClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fTypeName","fAggregate"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AspectClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EnumRepClause.class.getMethod(
                        "fTypeName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EnumRepClause.class.getMethod(
                        "fAggregate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_aggregate",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EnumRepClause.class.getMethod(
                        "pParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumRepClause NONE =
            new EnumRepClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumRepClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumRepClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumRepClause.NONE :

                new EnumRepClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumRepClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumRepClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumRepClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumRepClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fTypeName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_enum_rep_clause_f_type_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_enum_rep_clause_f_type_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseAggregate fAggregate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_enum_rep_clause_f_aggregate(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseAggregate res = BaseAggregate.NONE;
                if(propException == null) {
                    res = BaseAggregate.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_enum_rep_clause_f_aggregate(
                    this.entity
                );

                // Wrap and return the result
                return BaseAggregate.fromEntity(res);
            }

        }

        
    

        /**
         * Returns an array of pairs, associating enum literals to
         * representation clause actuals.
         */
        public ParamActualArray pParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_enum_rep_clause_p_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_enum_rep_clause_p_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Representation clause for a record type (:rmlink:`13.5.1`).
     */
    public static 
    class RecordRepClause extends AspectClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RecordRepClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAtExpr","fComponents"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AspectClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RecordRepClause.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RecordRepClause.class.getMethod(
                        "fAtExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_at_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RecordRepClause.class.getMethod(
                        "fComponents",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_components",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RecordRepClause NONE =
            new RecordRepClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RecordRepClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static RecordRepClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RecordRepClause.NONE :

                new RecordRepClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RecordRepClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RecordRepClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RecordRepClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RecordRepClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_rep_clause_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_rep_clause_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``NullLiteral``,
         * ``NumLiteral``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
         * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
         * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fAtExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_rep_clause_f_at_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_rep_clause_f_at_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ComponentClause``, ``PragmaNode``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fComponents(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_rep_clause_f_components(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_rep_clause_f_components(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of aspects in a declaration (:rmlink:`13.1.1`).
     */
    public static 
    class AspectSpec extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AspectSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAspectAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AspectSpec.class.getMethod(
                        "fAspectAssocs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_aspect_assocs",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AspectSpec NONE =
            new AspectSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AspectSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static AspectSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AspectSpec.NONE :

                new AspectSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AspectSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AspectSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AspectSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AspectSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AspectAssocList fAspectAssocs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_aspect_spec_f_aspect_assocs(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AspectAssocList res = AspectAssocList.NONE;
                if(propException == null) {
                    res = AspectAssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_aspect_spec_f_aspect_assocs(
                    this.entity
                );

                // Wrap and return the result
                return AspectAssocList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Abstract class for a key/value association, where the value is an
     * expression.
     */
    public static abstract
    class BaseAssoc extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseAssoc.class.getMethod(
                        "pAssocExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_assoc_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseAssoc NONE =
            new BaseAssocNone();

        // ----- Constructors -----

        protected BaseAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseAssoc.NONE :

                (BaseAssoc) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseAssoc.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Returns the expression side of this assoc node.
         */
        public Expr pAssocExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_assoc_p_assoc_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_assoc_p_assoc_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseAssoc.
         */
        private static final class BaseAssocNone extends BaseAssoc {
            BaseAssocNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Single association for the ``Contract_Case`` aspect.
     */
    public static 
    class ContractCaseAssoc extends BaseAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ContractCaseAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fGuard","fConsequence"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ContractCaseAssoc.class.getMethod(
                        "fGuard",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_guard",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ContractCaseAssoc.class.getMethod(
                        "fConsequence",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_consequence",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ContractCaseAssoc NONE =
            new ContractCaseAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ContractCaseAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static ContractCaseAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ContractCaseAssoc.NONE :

                new ContractCaseAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ContractCaseAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ContractCaseAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ContractCaseAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ContractCaseAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fGuard(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_contract_case_assoc_f_guard(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_contract_case_assoc_f_guard(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fConsequence(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_contract_case_assoc_f_consequence(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_contract_case_assoc_f_consequence(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Argument association in a pragma.
     */
    public static 
    class PragmaArgumentAssoc extends BaseAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PragmaArgumentAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PragmaArgumentAssoc.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PragmaArgumentAssoc.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PragmaArgumentAssoc NONE =
            new PragmaArgumentAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PragmaArgumentAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static PragmaArgumentAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PragmaArgumentAssoc.NONE :

                new PragmaArgumentAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PragmaArgumentAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PragmaArgumentAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PragmaArgumentAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PragmaArgumentAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``Identifier``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pragma_argument_assoc_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pragma_argument_assoc_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pragma_argument_assoc_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pragma_argument_assoc_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for lists of formal parameters. This is used in every case a
     * list of "formals" can be called or instantiated, so in all the following
     * cases:
     *
     * * Subprogram specifications (and subprogram calls).
     *
     * * Component lists (and aggregates).
     *
     * * Generic formals (and generic instantiations).
     *
     * This allows to share the parameter unpacking/matching logic.
     *
     * This is a Libadalang abstraction that has no existence in the Ada
     * reference manual.
     */
    public static abstract
    class BaseFormalParamHolder extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseFormalParamHolder";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamHolder.class.getMethod(
                        "pAbstractFormalParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_abstract_formal_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamHolder.class.getMethod(
                        "pFormalParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_formal_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamHolder.class.getMethod(
                        "pNbMinParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_nb_min_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamHolder.class.getMethod(
                        "pNbMaxParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_nb_max_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamHolder.class.getMethod(
                        "pParamTypes",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_param_types",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseFormalParamHolder NONE =
            new BaseFormalParamHolderNone();

        // ----- Constructors -----

        protected BaseFormalParamHolder(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseFormalParamHolder fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseFormalParamHolder.NONE :

                (BaseFormalParamHolder) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseFormalParamHolder.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseFormalParamHolder.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseFormalParamHolder.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseFormalParamHolder.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the list of abstract formal parameters for this holder.
         */
        public AdaNodeArray pAbstractFormalParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_formal_param_holder_p_abstract_formal_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_formal_param_holder_p_abstract_formal_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return all parameters as a ``DefiningName`` array. This property
         * doesn't return record discriminants nor variants when called on a
         * record component list.
         */
        public AdaNodeArray pFormalParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_formal_param_holder_p_formal_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_formal_param_holder_p_formal_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the minimum number of parameters this subprogram can be
         * called while still being a legal call.
         */
        public int pNbMinParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_formal_param_holder_p_nb_min_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                int res = 0;
                if(propException == null) {
                    res = resNative.read();

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final int res = JNI_LIB.ada_base_formal_param_holder_p_nb_min_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the maximum number of parameters this subprogram can be
         * called while still being a legal call.
         */
        public int pNbMaxParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_formal_param_holder_p_nb_max_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                int res = 0;
                if(propException == null) {
                    res = resNative.read();

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final int res = JNI_LIB.ada_base_formal_param_holder_p_nb_max_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns the type of each parameter of Self.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public AdaNodeArray pParamTypes(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_formal_param_holder_p_param_types(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_formal_param_holder_p_param_types(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseFormalParamHolder.
         */
        private static final class BaseFormalParamHolderNone extends BaseFormalParamHolder {
            BaseFormalParamHolderNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Base class for subprogram specifications (:rmlink:`6.1`).
     */
    public static abstract
    class BaseSubpSpec extends BaseFormalParamHolder {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseSubpSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamHolder.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pReturns",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_returns",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pPrimitiveSubpTypes",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_primitive_subp_types",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pPrimitiveSubpFirstType",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_primitive_subp_first_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pPrimitiveSubpTaggedType",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_primitive_subp_tagged_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpSpec.class.getMethod(
                        "pReturnType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_return_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseSubpSpec NONE =
            new BaseSubpSpecNone();

        // ----- Constructors -----

        protected BaseSubpSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseSubpSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseSubpSpec.NONE :

                (BaseSubpSpec) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseSubpSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseSubpSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseSubpSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseSubpSpec.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Syntax property. Return the type expression node corresponding to
         * the return of this subprogram spec.
         */
        public TypeExpr pReturns(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_returns(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_spec_p_returns(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }

        
    

        /**
         * Returns the array of parameters specification for this subprogram
         * spec.
         */
        public AdaNodeArray pParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_subp_spec_p_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the types of which this subprogram is a primitive of.
         */
        public AdaNodeArray pPrimitiveSubpTypes(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_primitive_subp_types(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_subp_spec_p_primitive_subp_types(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the first type of which this subprogram is a primitive of.
         */
        public BaseTypeDecl pPrimitiveSubpFirstType(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_primitive_subp_first_type(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_spec_p_primitive_subp_first_type(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * If this subprogram is a primitive for a tagged type, then return
         * this type.
         */
        public BaseTypeDecl pPrimitiveSubpTaggedType(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_primitive_subp_tagged_type(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_spec_p_primitive_subp_tagged_type(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns the return type of Self, if applicable (e.g. if Self is a
         * subprogram). Else, returns null.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pReturnType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_spec_p_return_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_spec_p_return_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseSubpSpec.
         */
        private static final class BaseSubpSpecNone extends BaseSubpSpec {
            BaseSubpSpecNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Entry specification.
     *
     * This node does not have ARM existence, because in the RM subprogram
     * specifications don't encompass the ad-hoc specifications that happen in
     * entry declarations. Entry declarations are described in :rmlink:`9.5.2`.
     */
    public static 
    class EntrySpec extends BaseSubpSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EntrySpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fEntryName","fFamilyType","fEntryParams"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EntrySpec.class.getMethod(
                        "fEntryName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_entry_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntrySpec.class.getMethod(
                        "fFamilyType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_family_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntrySpec.class.getMethod(
                        "fEntryParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_entry_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EntrySpec NONE =
            new EntrySpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EntrySpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static EntrySpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EntrySpec.NONE :

                new EntrySpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EntrySpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EntrySpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EntrySpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EntrySpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fEntryName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_spec_f_entry_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_spec_f_entry_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``SubtypeIndication``,
         * ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public AdaNode fFamilyType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_spec_f_family_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_spec_f_family_type(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Params fEntryParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_spec_f_entry_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Params res = Params.NONE;
                if(propException == null) {
                    res = Params.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_spec_f_entry_params(
                    this.entity
                );

                // Wrap and return the result
                return Params.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic node for the abstract subprogram spec of an enum literal.
     *
     * NOTE: This has no existence in the ARM. While enum literals are
     * functions semantically, they're not such syntactically.
     */
    public static 
    class EnumSubpSpec extends BaseSubpSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumSubpSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumSubpSpec NONE =
            new EnumSubpSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumSubpSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumSubpSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumSubpSpec.NONE :

                new EnumSubpSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumSubpSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumSubpSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumSubpSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumSubpSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Subprogram specification (:rmlink:`6.1`).
     */
    public static 
    class SubpSpec extends BaseSubpSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSubpKind","fSubpName","fSubpParams","fSubpReturns"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubpSpec.class.getMethod(
                        "fSubpKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpSpec.class.getMethod(
                        "fSubpName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpSpec.class.getMethod(
                        "fSubpParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpSpec.class.getMethod(
                        "fSubpReturns",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_returns",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpSpec NONE =
            new SubpSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpSpec.NONE :

                new SubpSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpKind fSubpKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_spec_f_subp_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpKind res = SubpKind.NONE;
                if(propException == null) {
                    res = SubpKind.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_spec_f_subp_kind(
                    this.entity
                );

                // Wrap and return the result
                return SubpKind.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DefiningName fSubpName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_spec_f_subp_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_spec_f_subp_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Params fSubpParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_spec_f_subp_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Params res = Params.NONE;
                if(propException == null) {
                    res = Params.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_spec_f_subp_params(
                    this.entity
                );

                // Wrap and return the result
                return Params.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * This field may be null even when there are no parsing errors.
         */
        public TypeExpr fSubpReturns(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_spec_f_subp_returns(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_spec_f_subp_returns(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic subprogram specification for binary operators.
     */
    public static 
    class SyntheticBinarySpec extends BaseSubpSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticBinarySpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fLeftParam","fRightParam","fReturnTypeExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticBinarySpec.class.getMethod(
                        "fLeftParam",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_left_param",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticBinarySpec.class.getMethod(
                        "fRightParam",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_right_param",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticBinarySpec.class.getMethod(
                        "fReturnTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_return_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticBinarySpec NONE =
            new SyntheticBinarySpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticBinarySpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticBinarySpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticBinarySpec.NONE :

                new SyntheticBinarySpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticBinarySpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticBinarySpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticBinarySpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticBinarySpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SyntheticFormalParamDecl fLeftParam(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_binary_spec_f_left_param(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SyntheticFormalParamDecl res = SyntheticFormalParamDecl.NONE;
                if(propException == null) {
                    res = SyntheticFormalParamDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_binary_spec_f_left_param(
                    this.entity
                );

                // Wrap and return the result
                return SyntheticFormalParamDecl.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SyntheticFormalParamDecl fRightParam(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_binary_spec_f_right_param(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SyntheticFormalParamDecl res = SyntheticFormalParamDecl.NONE;
                if(propException == null) {
                    res = SyntheticFormalParamDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_binary_spec_f_right_param(
                    this.entity
                );

                // Wrap and return the result
                return SyntheticFormalParamDecl.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public TypeExpr fReturnTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_binary_spec_f_return_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_binary_spec_f_return_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic subprogram specification for unary operators.
     */
    public static 
    class SyntheticUnarySpec extends BaseSubpSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticUnarySpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRightParam","fReturnTypeExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticUnarySpec.class.getMethod(
                        "fRightParam",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_right_param",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticUnarySpec.class.getMethod(
                        "fReturnTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_return_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticUnarySpec NONE =
            new SyntheticUnarySpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticUnarySpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticUnarySpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticUnarySpec.NONE :

                new SyntheticUnarySpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticUnarySpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticUnarySpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticUnarySpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticUnarySpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SyntheticFormalParamDecl fRightParam(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_unary_spec_f_right_param(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SyntheticFormalParamDecl res = SyntheticFormalParamDecl.NONE;
                if(propException == null) {
                    res = SyntheticFormalParamDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_unary_spec_f_right_param(
                    this.entity
                );

                // Wrap and return the result
                return SyntheticFormalParamDecl.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SyntheticTypeExpr fReturnTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_unary_spec_f_return_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SyntheticTypeExpr res = SyntheticTypeExpr.NONE;
                if(propException == null) {
                    res = SyntheticTypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_unary_spec_f_return_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return SyntheticTypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of component declarations (:rmlink:`3.8`).
     */
    public static 
    class ComponentList extends BaseFormalParamHolder {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ComponentList";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fComponents","fVariantPart"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamHolder.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ComponentList.class.getMethod(
                        "fComponents",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_components",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentList.class.getMethod(
                        "fVariantPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_variant_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ComponentList NONE =
            new ComponentList(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ComponentList(
            final Entity entity
        ) {
            super(entity);
        }

        public static ComponentList fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ComponentList.NONE :

                new ComponentList(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ComponentList.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ComponentList.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ComponentList.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ComponentList.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AspectClause``, ``ComponentDecl``, ``NullComponentDecl``,
         * ``PragmaNode``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fComponents(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_list_f_components(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_list_f_components(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public VariantPart fVariantPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_list_f_variant_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                VariantPart res = VariantPart.NONE;
                if(propException == null) {
                    res = VariantPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_list_f_variant_part(
                    this.entity
                );

                // Wrap and return the result
                return VariantPart.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Specification for discriminants in type declarations.
     */
    public static abstract
    class DiscriminantPart extends BaseFormalParamHolder {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscriminantPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamHolder.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscriminantPart NONE =
            new DiscriminantPartNone();

        // ----- Constructors -----

        protected DiscriminantPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscriminantPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscriminantPart.NONE :

                (DiscriminantPart) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscriminantPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscriminantPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscriminantPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscriminantPart.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * DiscriminantPart.
         */
        private static final class DiscriminantPartNone extends DiscriminantPart {
            DiscriminantPartNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Known list of discriminants in type declarations (:rmlink:`3.7`).
     */
    public static 
    class KnownDiscriminantPart extends DiscriminantPart {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "KnownDiscriminantPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDiscrSpecs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DiscriminantPart.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = KnownDiscriminantPart.class.getMethod(
                        "fDiscrSpecs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discr_specs",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final KnownDiscriminantPart NONE =
            new KnownDiscriminantPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected KnownDiscriminantPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static KnownDiscriminantPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                KnownDiscriminantPart.NONE :

                new KnownDiscriminantPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return KnownDiscriminantPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return KnownDiscriminantPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return KnownDiscriminantPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return KnownDiscriminantPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DiscriminantSpecList fDiscrSpecs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_known_discriminant_part_f_discr_specs(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantSpecList res = DiscriminantSpecList.NONE;
                if(propException == null) {
                    res = DiscriminantSpecList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_known_discriminant_part_f_discr_specs(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantSpecList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Unknown list of discriminants in type declarations (:rmlink:`3.7`).
     */
    public static 
    class UnknownDiscriminantPart extends DiscriminantPart {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UnknownDiscriminantPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DiscriminantPart.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UnknownDiscriminantPart NONE =
            new UnknownDiscriminantPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UnknownDiscriminantPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static UnknownDiscriminantPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UnknownDiscriminantPart.NONE :

                new UnknownDiscriminantPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UnknownDiscriminantPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UnknownDiscriminantPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UnknownDiscriminantPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UnknownDiscriminantPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal parameters for the completion of an ``EntryDecl`` (either an
     * ``EntryBody`` or an ``AcceptStmt``).
     */
    public static 
    class EntryCompletionFormalParams extends BaseFormalParamHolder {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EntryCompletionFormalParams";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fParams"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamHolder.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EntryCompletionFormalParams.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EntryCompletionFormalParams NONE =
            new EntryCompletionFormalParams(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EntryCompletionFormalParams(
            final Entity entity
        ) {
            super(entity);
        }

        public static EntryCompletionFormalParams fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EntryCompletionFormalParams.NONE :

                new EntryCompletionFormalParams(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EntryCompletionFormalParams.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EntryCompletionFormalParams.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EntryCompletionFormalParams.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EntryCompletionFormalParams.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Params fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_completion_formal_params_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Params res = Params.NONE;
                if(propException == null) {
                    res = Params.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_completion_formal_params_f_params(
                    this.entity
                );

                // Wrap and return the result
                return Params.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of declaration for generic formals (:rmlink:`12.1`).
     */
    public static 
    class GenericFormalPart extends BaseFormalParamHolder {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormalPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamHolder.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericFormalPart.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormalPart NONE =
            new GenericFormalPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericFormalPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormalPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormalPart.NONE :

                new GenericFormalPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormalPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormalPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormalPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormalPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``GenericFormal``, ``PragmaNode``, ``UseClause``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_formal_part_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_formal_part_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for record definitions (:rmlink:`3.8`).
     */
    public static abstract
    class BaseRecordDef extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseRecordDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fComponents"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseRecordDef.class.getMethod(
                        "fComponents",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_components",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseRecordDef NONE =
            new BaseRecordDefNone();

        // ----- Constructors -----

        protected BaseRecordDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseRecordDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseRecordDef.NONE :

                (BaseRecordDef) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseRecordDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseRecordDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseRecordDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseRecordDef.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ComponentList fComponents(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_record_def_f_components(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ComponentList res = ComponentList.NONE;
                if(propException == null) {
                    res = ComponentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_record_def_f_components(
                    this.entity
                );

                // Wrap and return the result
                return ComponentList.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseRecordDef.
         */
        private static final class BaseRecordDefNone extends BaseRecordDef {
            BaseRecordDefNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Record definition for ``null record``.
     */
    public static 
    class NullRecordDef extends BaseRecordDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullRecordDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fComponents"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseRecordDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullRecordDef NONE =
            new NullRecordDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullRecordDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullRecordDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullRecordDef.NONE :

                new NullRecordDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullRecordDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullRecordDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullRecordDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullRecordDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Record definition that contains components (``record ... end record``).
     */
    public static 
    class RecordDef extends BaseRecordDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RecordDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fComponents"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseRecordDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RecordDef NONE =
            new RecordDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RecordDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static RecordDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RecordDef.NONE :

                new RecordDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RecordDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RecordDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RecordDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RecordDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Association of one or several names to an expression.
     */
    public static abstract
    class BasicAssoc extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasicAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BasicAssoc.class.getMethod(
                        "pGetParams",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasicAssoc NONE =
            new BasicAssocNone();

        // ----- Constructors -----

        protected BasicAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasicAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasicAssoc.NONE :

                (BasicAssoc) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasicAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasicAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasicAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasicAssoc.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the list of parameters that this association refers to.
         */
        public AdaNodeArray pGetParams(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_assoc_p_get_params(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_assoc_p_get_params(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BasicAssoc.
         */
        private static final class BasicAssocNone extends BasicAssoc {
            BasicAssocNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Association (X => Y) used for aggregates associations (:rmlink:`4.3`).
     */
    public static 
    class AggregateAssoc extends BasicAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AggregateAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDesignators","fRExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AggregateAssoc.class.getMethod(
                        "fDesignators",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_designators",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AggregateAssoc.class.getMethod(
                        "fRExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_r_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AggregateAssoc NONE =
            new AggregateAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AggregateAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static AggregateAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AggregateAssoc.NONE :

                new AggregateAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AggregateAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AggregateAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AggregateAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AggregateAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``Allocator``, ``AttributeRef``, ``BaseAggregate``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``ConcatOp``,
         * ``CondExpr``, ``DeclExpr``, ``DiscreteSubtypeIndication``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AlternativesList fDesignators(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_aggregate_assoc_f_designators(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AlternativesList res = AlternativesList.NONE;
                if(propException == null) {
                    res = AlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_aggregate_assoc_f_designators(
                    this.entity
                );

                // Wrap and return the result
                return AlternativesList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fRExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_aggregate_assoc_f_r_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_aggregate_assoc_f_r_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Association used for multi-dimension array aggregates.
     */
    public static 
    class MultiDimArrayAssoc extends AggregateAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "MultiDimArrayAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDesignators","fRExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AggregateAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final MultiDimArrayAssoc NONE =
            new MultiDimArrayAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected MultiDimArrayAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static MultiDimArrayAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                MultiDimArrayAssoc.NONE :

                new MultiDimArrayAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return MultiDimArrayAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return MultiDimArrayAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return MultiDimArrayAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return MultiDimArrayAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Association of discriminant names to an expression (:rmlink:`3.7.1`).
     */
    public static 
    class CompositeConstraintAssoc extends BasicAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CompositeConstraintAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fConstraintExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CompositeConstraintAssoc.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompositeConstraintAssoc.class.getMethod(
                        "fConstraintExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_constraint_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CompositeConstraintAssoc NONE =
            new CompositeConstraintAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CompositeConstraintAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static CompositeConstraintAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CompositeConstraintAssoc.NONE :

                new CompositeConstraintAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CompositeConstraintAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CompositeConstraintAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CompositeConstraintAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CompositeConstraintAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``IdentifierList``
         *
         * When there are no parsing errors, this field is never null.
         */
        public DiscriminantChoiceList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_composite_constraint_assoc_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantChoiceList res = DiscriminantChoiceList.NONE;
                if(propException == null) {
                    res = DiscriminantChoiceList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_composite_constraint_assoc_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantChoiceList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DiscreteSubtypeIndication``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fConstraintExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_composite_constraint_assoc_f_constraint_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_composite_constraint_assoc_f_constraint_expr(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Iterated association (Ada 2020, :rmlink:`4.3.3`).
     */
    public static 
    class IteratedAssoc extends BasicAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IteratedAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec","fRExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IteratedAssoc.class.getMethod(
                        "fSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IteratedAssoc.class.getMethod(
                        "fRExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_r_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IteratedAssoc NONE =
            new IteratedAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IteratedAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static IteratedAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IteratedAssoc.NONE :

                new IteratedAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IteratedAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IteratedAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IteratedAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IteratedAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ForLoopSpec fSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_iterated_assoc_f_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ForLoopSpec res = ForLoopSpec.NONE;
                if(propException == null) {
                    res = ForLoopSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_iterated_assoc_f_spec(
                    this.entity
                );

                // Wrap and return the result
                return ForLoopSpec.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fRExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_iterated_assoc_f_r_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_iterated_assoc_f_r_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Association (X => Y) used for parameter associations (:rmlink:`6.4`).
     */
    public static 
    class ParamAssoc extends BasicAssoc {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParamAssoc";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDesignator","fRExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicAssoc.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ParamAssoc.class.getMethod(
                        "fDesignator",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_designator",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ParamAssoc.class.getMethod(
                        "fRExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_r_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParamAssoc NONE =
            new ParamAssoc(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParamAssoc(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParamAssoc fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParamAssoc.NONE :

                new ParamAssoc(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParamAssoc.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParamAssoc.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParamAssoc.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParamAssoc.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Identifier``,
         * ``OthersDesignator``, ``StringLiteral``
         *
         * This field may be null even when there are no parsing errors.
         */
        public AdaNode fDesignator(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_assoc_f_designator(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_assoc_f_designator(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fRExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_assoc_f_r_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_assoc_f_r_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Root class for an Ada declaration (:rmlink:`3.1`). A declaration
     * associates a name with a language entity, for example a type or a
     * variable.
     */
    public static abstract
    class BasicDecl extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasicDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsFormal",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_formal",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pDocAnnotations",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_doc_annotations",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pDoc",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_doc",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pPreviousPartForDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_previous_part_for_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pCanonicalPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pAllParts",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_all_parts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsStaticDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_static_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "fAspects",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_aspects",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetAspectAssoc",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_aspect_assoc",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetAspectSpecExpr",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_aspect_spec_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetAspect",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_aspect",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pHasAspect",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_has_aspect",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetPragma",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_pragma",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetRepresentationClause",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_representation_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pGetAtClause",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_at_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsImported",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_imported",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsGhostCode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_ghost_code",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsCompilationUnitRoot",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_compilation_unit_root",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsVisible",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "fromNode"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_visible",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pBaseSubpDeclarations",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_base_subp_declarations",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pRootSubpDeclarations",
                        new Class[]{AdaNode.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_root_subp_declarations",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pFindAllOverrides",
                        new Class[]{AnalysisUnitArray.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnitArray.class,
                        "units"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_all_overrides",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pDefiningNames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_defining_names",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pDefiningName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_defining_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pTypeExpression",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_type_expression",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pSubpSpecOrNull",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "followGeneric",
                        true
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_subp_spec_or_null",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsSubprogram",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_subprogram",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pRelativeName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_relative_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pRelativeNameText",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_relative_name_text",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pNextPartForDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_next_part_for_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pBodyPartForDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part_for_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pMostVisiblePart",
                        new Class[]{AdaNode.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "origin"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_most_visible_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pFullyQualifiedNameArray",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "includeProfile",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_fully_qualified_name_array",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pCanonicalFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pUniqueIdentifyingName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_unique_identifying_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicDecl.class.getMethod(
                        "pIsConstantObject",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_constant_object",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasicDecl NONE =
            new BasicDeclNone();

        // ----- Constructors -----

        protected BasicDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasicDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasicDecl.NONE :

                (BasicDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasicDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasicDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasicDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasicDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Whether this decl is the nested decl of a generic formal
         * declaration.
         */
        public boolean pIsFormal(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_formal(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_formal(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the documentation annotations associated with this decl.
         * Annotations are any comment line of the form:
         *
         * .. code::
         *
         *    --% [annotation_name]: [annotation]
         *
         * Raises a property error if the doc is incorrectly formatted.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public DocAnnotationArray pDocAnnotations(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_doc_annotations(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DocAnnotationArray res = DocAnnotationArray.NONE;
                if(propException == null) {
                    res = DocAnnotationArray.wrap(resNative);

                    DocAnnotationArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final DocAnnotationArray res = JNI_LIB.ada_basic_decl_p_doc_annotations(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the documentation associated with this decl. Raises a
         * property error if the doc is incorrectly formatted.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public String pDoc(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_doc(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_basic_decl_p_doc(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the previous part for this decl, if applicable.
         *
         * .. note:: It is not named previous_part, because BaseTypeDecl has a
         *    more precise version of previous_part that returns a
         *    BaseTypeDecl. Probably, we want to rename the specific versions,
         *    and have the root property be named previous_part. (TODO
         *    R925-008)
         */
        public BasicDecl pPreviousPartForDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_previous_part_for_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_previous_part_for_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the canonical part for this decl. In the case of decls
         * composed of several parts, the canonical part will be the first
         * part.
         */
        public BasicDecl pCanonicalPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_canonical_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_canonical_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return all parts that define this entity, sorted from first part to
         * last part.
         */
        public AdaNodeArray pAllParts(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_all_parts(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_decl_p_all_parts(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this declaration is static.
         */
        public boolean pIsStaticDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_static_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_static_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of aspects that are attached to this node.
         *
         * This field may be null even when there are no parsing errors.
         */
        public AspectSpec fAspects(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_f_aspects(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AspectSpec res = AspectSpec.NONE;
                if(propException == null) {
                    res = AspectSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_f_aspects(
                    this.entity
                );

                // Wrap and return the result
                return AspectSpec.fromEntity(res);
            }

        }

        
    

        /**
         * Return the aspect with name ``name`` for this entity.
         */
        public AspectAssoc pGetAspectAssoc(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_aspect_assoc(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AspectAssoc res = AspectAssoc.NONE;
                if(propException == null) {
                    res = AspectAssoc.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_get_aspect_assoc(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return AspectAssoc.fromEntity(res);
            }

        }

        
    

        /**
         * Return the expression associated to the aspect with name ``name``
         * for this entity.
         */
        public Expr pGetAspectSpecExpr(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_aspect_spec_expr(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_get_aspect_spec_expr(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * Return the aspect with name ``name`` associated to this entity.
         *
         * Aspects are properties of entities that can be specified by the Ada
         * program, either via aspect specifications, pragmas, or attributes.
         *
         * This will return the syntactic node corresponding to attribute
         * directly.
         *
         * Note: for some aspects (e.g. Inline), Libadalang will check if they
         * are defined on any part of the entity.
         */
        public Aspect pGetAspect(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final AspectNative resNative =
                    StackValue.get(AspectNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_aspect(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Aspect res = Aspect.NONE;
                if(propException == null) {
                    res = Aspect.wrap(resNative);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Aspect res = JNI_LIB.ada_basic_decl_p_get_aspect(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns whether the boolean aspect named ``name`` is set on the
         * entity represented by this node.
         *
         * Aspects are properties of entities that can be specified by the Ada
         * program, either via aspect specifications, pragmas, or attributes.
         *
         * "Aspect" is used as in RM terminology (see :rmlink:`13`).
         */
        public boolean pHasAspect(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_has_aspect(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_has_aspect(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the pragma with name ``name`` associated to this entity.
         *
         * Please use the ``p_get_aspect`` property instead if you are
         * interested in aspects, i.e. information that can be represented by
         * either aspect specification nodes, pragma nodes or attribute
         * definition nodes.
         */
        public PragmaNode pGetPragma(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_pragma(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PragmaNode res = PragmaNode.NONE;
                if(propException == null) {
                    res = PragmaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_get_pragma(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return PragmaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the representation clause associated to this type decl that
         * defines the given attribute name.
         */
        public AttributeDefClause pGetRepresentationClause(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_representation_clause(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AttributeDefClause res = AttributeDefClause.NONE;
                if(propException == null) {
                    res = AttributeDefClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_get_representation_clause(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return AttributeDefClause.fromEntity(res);
            }

        }

        
    

        /**
         * Return the at clause associated to this declaration.
         */
        public AtClause pGetAtClause(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_get_at_clause(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AtClause res = AtClause.NONE;
                if(propException == null) {
                    res = AtClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_get_at_clause(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return AtClause.fromEntity(res);
            }

        }

        
    

        /**
         * Whether this declaration is imported from another language.
         */
        public boolean pIsImported(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_imported(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_imported(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this declaration is ghost code or not. See SPARK RM
         * 6.9.
         */
        public boolean pIsGhostCode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_ghost_code(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_ghost_code(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether a BasicDecl is the root decl for its unit.
         */
        public boolean pIsCompilationUnitRoot(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_compilation_unit_root(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_compilation_unit_root(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this declaration is visible from the point of view of
         * the given ``origin`` node.
         *
         * .. attention:: Only package-level (public or private) declarations
         *    are supported for now.
         */
        public boolean pIsVisible(
            final AdaNode fromNode
        ) {

            // Verify that arguments are not null
            if(fromNode == null) throw new IllegalArgumentException(
                "Argument 'fromNode' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative fromNodeNative = StackValue.get(EntityNative.class);fromNode.entity.unwrap(fromNodeNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_visible(
                    thisNative,
                    fromNodeNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_visible(
                    (fromNode != null ? fromNode.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * If Self declares a primitive subprogram of some tagged type T,
         * return the set of all subprogram declarations that it overrides
         * (including itself).
         *
         * .. note:: for the moment this only works for tagged types. Remains
         *    to be seen if we need to extend it.
         */
        public AdaNodeArray pBaseSubpDeclarations(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_base_subp_declarations(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_decl_p_base_subp_declarations(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * If Self declares a primitive subprogram of some tagged type T,
         * return the root subprogram declarations that it overrides. There can
         * be several, as in the following scenario:
         *
         * * package Root defines the root tagged type T and subprogram Foo.
         *
         * * package Itf defines interface I and abstract subprogram Foo.
         *
         * * package D defines "type U is new Root.T and Itf.I" and an
         *   overriding subprogram Foo.
         *
         * Here, root_subp_declarations of Foo defined in package D will return
         * both Foo from package Root and Foo from package Itf.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public AdaNodeArray pRootSubpDeclarations(
            final AdaNode origin,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_root_subp_declarations(
                    thisNative,
                    originNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_decl_p_root_subp_declarations(
                    (origin != null ? origin.entity : null),
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * If Self is the declaration of a primitive of some type T, return the
         * list of all subprogram that override this subprogram among the given
         * units.
         */
        public AdaNodeArray pFindAllOverrides(
            final AnalysisUnitArray units,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(units == null) throw new IllegalArgumentException(
                "Argument 'units' of type " +
                "AnalysisUnitArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitArrayNative unitsNative = units.unwrap();
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_find_all_overrides(
                    thisNative,
                    unitsNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                AnalysisUnitArray.release(
                    unitsNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_decl_p_find_all_overrides(
                    units,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Get all the names of this basic declaration.
         */
        public AdaNodeArray pDefiningNames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_defining_names(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_basic_decl_p_defining_names(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Get the name of this declaration. If this declaration has several
         * names, it will return the first one.
         */
        public DefiningName pDefiningName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_defining_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_defining_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return the type expression for this BasicDecl if applicable, a null
         * otherwise.
         */
        public TypeExpr pTypeExpression(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_type_expression(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_type_expression(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }

        
    

        /**
         * If Self is a Subp, returns the specification of this subprogram.
         *
         * If ``follow_generic`` is True, will also work for instances of
         * ``GenericSubpDecl``.
         */
        public BaseSubpSpec pSubpSpecOrNull(
            final boolean followGeneric
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte followGenericNative = (followGeneric ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_subp_spec_or_null(
                    thisNative,
                    followGenericNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseSubpSpec res = BaseSubpSpec.NONE;
                if(propException == null) {
                    res = BaseSubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_subp_spec_or_null(
                    followGeneric,
                    this.entity
                );

                // Wrap and return the result
                return BaseSubpSpec.fromEntity(res);
            }

        }

        
    

        /**
         * Return True if self is a subprogram node in the general sense (which
         * is, an entity that can be called). This includes separates and
         * entries.
         *
         * .. attention: This is a purely syntactic query and will return True
         *    for everything that is a syntactic entity that can be called like
         *    a subprogram in some contexts, even generic formal subprograms
         *    for example.
         */
        public boolean pIsSubprogram(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_subprogram(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_subprogram(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the relative name for Self. If Self's defining name is
         * ``A.B.C``, return ``C`` as a node.
         */
        public Name pRelativeName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_relative_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_relative_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * Return the relative name for Self, as text.
         */
        public Symbol pRelativeNameText(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final SymbolNative resNative =
                    StackValue.get(SymbolNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_relative_name_text(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Symbol res = Symbol.NONE;
                if(propException == null) {
                    res = Symbol.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Symbol res = JNI_LIB.ada_basic_decl_p_relative_name_text(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the next part of this declaration, if applicable.
         *
         * .. note:: It is not named next_part, because BaseTypeDecl has a more
         *    precise version of next_part that returns a BaseTypeDecl.
         *    Probably, we want to rename the specific versions, and have the
         *    root property be named next_part. (TODO R925-008)
         */
        public BasicDecl pNextPartForDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_next_part_for_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_next_part_for_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the body corresponding to this declaration, if applicable.
         *
         * .. note:: It is not named body_part, subclasses have more precise
         *    versions named body_part and returning a more precise result.
         *    Probably, we want to rename the specific versions, and have the
         *    root property be named body_part. (TODO R925-008)
         */
        public BodyNode pBodyPartForDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_body_part_for_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BodyNode res = BodyNode.NONE;
                if(propException == null) {
                    res = BodyNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_body_part_for_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BodyNode.fromEntity(res);
            }

        }

        
    

        /**
         * Given an origin node and the entity represented by Self, this
         * property returns the most visible completion of Self that can be
         * seen by origin, according to Ada's visibility rules.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BasicDecl pMostVisiblePart(
            final AdaNode origin,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_decl_p_most_visible_part(
                    thisNative,
                    originNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_decl_p_most_visible_part(
                    (origin != null ? origin.entity : null),
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the fully qualified name corresponding to this declaration,
         * as an array of symbols.
         */
        public UnboundedTextTypeArray pFullyQualifiedNameArray(
            final boolean includeProfile
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte includeProfileNative = (includeProfile ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_fully_qualified_name_array(
                    thisNative,
                    includeProfileNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_basic_decl_p_fully_qualified_name_array(
                    includeProfile,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the fully qualified name corresponding to this declaration.
         */
        public String pFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_basic_decl_p_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a canonical representation of the fully qualified name
         * corresponding to this declaration.
         */
        public String pCanonicalFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_canonical_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_basic_decl_p_canonical_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a unique identifying name for this declaration, provided this
         * declaration is a public declaration. In the case of subprograms,
         * this will include the profile.
         *
         * .. attention:: This will only return a unique name for public
         *    declarations. Notably, anything nested in an unnamed declare
         *    block won't be handled correctly.
         */
        public String pUniqueIdentifyingName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_unique_identifying_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_basic_decl_p_unique_identifying_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this object is constant or not.
         */
        public boolean pIsConstantObject(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_decl_p_is_constant_object(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_decl_p_is_constant_object(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BasicDecl.
         */
        private static final class BasicDeclNone extends BasicDecl {
            BasicDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Contained (directly or indirectly) in an AbstractStateDeclExpr, and is
     * used to represent the BasicDecl associated with the abstract state
     * introduced by the Abstract_State aspect. This node is necessary because
     * all of our name resolution routines expect BasicDecls as environments'
     * values.
     *
     * The only purpose of this node is to populate the env with the abstract
     * state declared through this node, so it can be referred in SPARK aspects
     * such as Global, Depends, Refined_State, etc.
     */
    public static 
    class AbstractStateDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractStateDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AbstractStateDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractStateDecl NONE =
            new AbstractStateDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractStateDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractStateDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractStateDecl.NONE :

                new AbstractStateDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractStateDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractStateDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractStateDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractStateDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_abstract_state_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_abstract_state_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Represents a anonymous declaration that holds an expression.
     *
     * This is used to store the results of queries such as ``referenced_decl``
     * called on references to object formals from inside a instantiated
     * generic in order to return the relevant actual.
     *
     * Indeed, ``referenced_decl`` must return a ``BasicDecl``, but actuals of
     * generic instantiations are ``Expr``. This wrapper node is therefore a
     * way to both satisfy the ``BasicDecl`` interface, and provide to the user
     * the expression of the actual through the ``expr`` field.
     */
    public static 
    class AnonymousExprDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AnonymousExprDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AnonymousExprDecl.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AnonymousExprDecl.class.getMethod(
                        "pGetFormal",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_formal",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AnonymousExprDecl NONE =
            new AnonymousExprDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AnonymousExprDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static AnonymousExprDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AnonymousExprDecl.NONE :

                new AnonymousExprDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AnonymousExprDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AnonymousExprDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AnonymousExprDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AnonymousExprDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the expression wrapped by this declaration.
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_anonymous_expr_decl_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_anonymous_expr_decl_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * Return the generic formal object declaration corresponding to this
         * actual.
         */
        public DefiningName pGetFormal(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_anonymous_expr_decl_p_get_formal(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_anonymous_expr_decl_p_get_formal(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for formal parameter declarations. This is used both for
     * records components and for subprogram parameters.
     *
     * This is a Libadalang abstraction, that has no ARM existence.
     */
    public static abstract
    class BaseFormalParamDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseFormalParamDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseFormalParamDecl.class.getMethod(
                        "pFormalType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_formal_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseFormalParamDecl NONE =
            new BaseFormalParamDeclNone();

        // ----- Constructors -----

        protected BaseFormalParamDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseFormalParamDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseFormalParamDecl.NONE :

                (BaseFormalParamDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseFormalParamDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseFormalParamDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseFormalParamDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseFormalParamDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the type for this formal.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pFormalType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_formal_param_decl_p_formal_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_formal_param_decl_p_formal_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseFormalParamDecl.
         */
        private static final class BaseFormalParamDeclNone extends BaseFormalParamDecl {
            BaseFormalParamDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Declaration for a component (:rmlink:`3.8`).
     */
    public static 
    class ComponentDecl extends BaseFormalParamDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ComponentDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fComponentDef","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDecl.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDecl.class.getMethod(
                        "fComponentDef",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_component_def",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDecl.class.getMethod(
                        "fDefaultExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ComponentDecl NONE =
            new ComponentDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ComponentDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ComponentDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ComponentDecl.NONE :

                new ComponentDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ComponentDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ComponentDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ComponentDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ComponentDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_decl_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_decl_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ComponentDef fComponentDef(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_decl_f_component_def(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ComponentDef res = ComponentDef.NONE;
                if(propException == null) {
                    res = ComponentDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_decl_f_component_def(
                    this.entity
                );

                // Wrap and return the result
                return ComponentDef.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fDefaultExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_decl_f_default_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_decl_f_default_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Known list of discriminants in type declarations (:rmlink:`3.7`).
     */
    public static 
    class DiscriminantSpec extends BaseFormalParamDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscriminantSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fTypeExpr","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DiscriminantSpec.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DiscriminantSpec.class.getMethod(
                        "fTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DiscriminantSpec.class.getMethod(
                        "fDefaultExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscriminantSpec NONE =
            new DiscriminantSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscriminantSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscriminantSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscriminantSpec.NONE :

                new DiscriminantSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscriminantSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscriminantSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscriminantSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscriminantSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_discriminant_spec_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_discriminant_spec_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * When there are no parsing errors, this field is never null.
         */
        public TypeExpr fTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_discriminant_spec_f_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_discriminant_spec_f_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fDefaultExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_discriminant_spec_f_default_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_discriminant_spec_f_default_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Enclosing declaration for a generic formal. The real declaration is
     * accessible via the ``decl`` field.
     */
    public static abstract
    class GenericFormal extends BaseFormalParamDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormal";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericFormal.class.getMethod(
                        "fDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormal NONE =
            new GenericFormalNone();

        // ----- Constructors -----

        protected GenericFormal(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormal fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormal.NONE :

                (GenericFormal) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormal.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormal.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormal.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormal.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``FormalSubpDecl``, ``FormalTypeDecl``, ``GenericInstantiation``,
         * ``IncompleteFormalTypeDecl``, ``NumberDecl``, ``ObjectDecl``,
         * ``SingleProtectedDecl``, ``SingleTaskDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BasicDecl fDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_formal_f_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_formal_f_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * GenericFormal.
         */
        private static final class GenericFormalNone extends GenericFormal {
            GenericFormalNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Formal declaration for an object.
     */
    public static 
    class GenericFormalObjDecl extends GenericFormal {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormalObjDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericFormal.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormalObjDecl NONE =
            new GenericFormalObjDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericFormalObjDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormalObjDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormalObjDecl.NONE :

                new GenericFormalObjDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormalObjDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormalObjDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormalObjDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormalObjDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal declaration for a package (:rmlink:`12.1`).
     */
    public static 
    class GenericFormalPackage extends GenericFormal {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormalPackage";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericFormal.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormalPackage NONE =
            new GenericFormalPackage(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericFormalPackage(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormalPackage fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormalPackage.NONE :

                new GenericFormalPackage(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormalPackage.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormalPackage.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormalPackage.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormalPackage.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal declaration for a subprogram (:rmlink:`12.1`).
     */
    public static 
    class GenericFormalSubpDecl extends GenericFormal {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormalSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericFormal.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormalSubpDecl NONE =
            new GenericFormalSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericFormalSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormalSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormalSubpDecl.NONE :

                new GenericFormalSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormalSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormalSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormalSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormalSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal declaration for a type (:rmlink:`12.1`).
     */
    public static 
    class GenericFormalTypeDecl extends GenericFormal {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericFormalTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericFormal.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericFormalTypeDecl NONE =
            new GenericFormalTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericFormalTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericFormalTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericFormalTypeDecl.NONE :

                new GenericFormalTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericFormalTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericFormalTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericFormalTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericFormalTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Specification for a parameter (:rmlink:`6.1`).
     */
    public static 
    class ParamSpec extends BaseFormalParamDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParamSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fHasAliased","fMode","fTypeExpr","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ParamSpec.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ParamSpec.class.getMethod(
                        "fHasAliased",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_aliased",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ParamSpec.class.getMethod(
                        "fMode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_mode",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ParamSpec.class.getMethod(
                        "fTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ParamSpec.class.getMethod(
                        "fDefaultExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParamSpec NONE =
            new ParamSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParamSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParamSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParamSpec.NONE :

                new ParamSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParamSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParamSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParamSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParamSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_spec_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_spec_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AliasedNode fHasAliased(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_spec_f_has_aliased(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AliasedNode res = AliasedNode.NONE;
                if(propException == null) {
                    res = AliasedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_spec_f_has_aliased(
                    this.entity
                );

                // Wrap and return the result
                return AliasedNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Mode fMode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_spec_f_mode(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Mode res = Mode.NONE;
                if(propException == null) {
                    res = Mode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_spec_f_mode(
                    this.entity
                );

                // Wrap and return the result
                return Mode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * When there are no parsing errors, this field is never null.
         */
        public TypeExpr fTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_spec_f_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_spec_f_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fDefaultExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_param_spec_f_default_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_param_spec_f_default_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic parameter declaration.
     */
    public static 
    class SyntheticFormalParamDecl extends BaseFormalParamDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticFormalParamDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fParamType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseFormalParamDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticFormalParamDecl.class.getMethod(
                        "fParamType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_param_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticFormalParamDecl NONE =
            new SyntheticFormalParamDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticFormalParamDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticFormalParamDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticFormalParamDecl.NONE :

                new SyntheticFormalParamDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticFormalParamDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticFormalParamDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticFormalParamDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticFormalParamDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public TypeExpr fParamType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_formal_param_decl_f_param_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_formal_param_decl_f_param_type(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for package declarations. This will be used both for non-
     * generic package declarations (via ``PackageDecl``) and for generic ones
     * (via ``GenericPackageInternal``).
     */
    public static abstract
    class BasePackageDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasePackageDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPackageName","fAspects","fPublicPart","fPrivatePart","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BasePackageDecl.class.getMethod(
                        "fPackageName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_package_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasePackageDecl.class.getMethod(
                        "fPublicPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_public_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasePackageDecl.class.getMethod(
                        "fPrivatePart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_private_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasePackageDecl.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasePackageDecl.class.getMethod(
                        "pBodyPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasePackageDecl NONE =
            new BasePackageDeclNone();

        // ----- Constructors -----

        protected BasePackageDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasePackageDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasePackageDecl.NONE :

                (BasePackageDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasePackageDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasePackageDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasePackageDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasePackageDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fPackageName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_package_decl_f_package_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_package_decl_f_package_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PublicPart fPublicPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_package_decl_f_public_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PublicPart res = PublicPart.NONE;
                if(propException == null) {
                    res = PublicPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_package_decl_f_public_part(
                    this.entity
                );

                // Wrap and return the result
                return PublicPart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public PrivatePart fPrivatePart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_package_decl_f_private_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PrivatePart res = PrivatePart.NONE;
                if(propException == null) {
                    res = PrivatePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_package_decl_f_private_part(
                    this.entity
                );

                // Wrap and return the result
                return PrivatePart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_package_decl_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_package_decl_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }

        
    

        /**
         * Return the PackageBody corresponding to this node.
         */
        public PackageBody pBodyPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_package_decl_p_body_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PackageBody res = PackageBody.NONE;
                if(propException == null) {
                    res = PackageBody.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_package_decl_p_body_part(
                    this.entity
                );

                // Wrap and return the result
                return PackageBody.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BasePackageDecl.
         */
        private static final class BasePackageDeclNone extends BasePackageDecl {
            BasePackageDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * This class denotes the internal package contained by a
     * GenericPackageDecl.
     */
    public static 
    class GenericPackageInternal extends BasePackageDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericPackageInternal";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPackageName","fAspects","fPublicPart","fPrivatePart","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasePackageDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericPackageInternal NONE =
            new GenericPackageInternal(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericPackageInternal(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericPackageInternal fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericPackageInternal.NONE :

                new GenericPackageInternal(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericPackageInternal.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericPackageInternal.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericPackageInternal.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericPackageInternal.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Non-generic package declarations (:rmlink:`7.1`).
     */
    public static 
    class PackageDecl extends BasePackageDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PackageDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPackageName","fAspects","fPublicPart","fPrivatePart","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasePackageDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PackageDecl NONE =
            new PackageDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PackageDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static PackageDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PackageDecl.NONE :

                new PackageDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PackageDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PackageDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PackageDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PackageDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for type declarations. It unifies every kind of type that
     * exists in Ada, including types that have no source existence like
     * classwide types.
     */
    public static abstract
    class BaseTypeDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pBaseSubtype",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_base_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pPrivateCompletion",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_private_completion",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsInheritedPrimitive",
                        new Class[]{BasicDecl.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        BasicDecl.class,
                        "p"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_inherited_primitive",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pGetRecordRepresentationClause",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_record_representation_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pGetEnumRepresentationClause",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_enum_representation_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsRecordType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_record_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsArrayType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_array_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pFindDerivedTypes",
                        new Class[]{AdaNode.class,AdaNode.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "root"
                    ));
                    parameters.add(new Param(
                        AdaNode.class,
                        "origin"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_derived_types",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsRealType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_real_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsFloatType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_float_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsFixedPoint",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_fixed_point",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsScalarType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_scalar_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsEnumType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_enum_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsAccessType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_access_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pSpecificType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_specific_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsCharType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_char_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pClasswideType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_classwide_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pDiscreteRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_discrete_range",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsDiscreteType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_discrete_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsIntType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_int_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pAccessedType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_accessed_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsTaggedType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_tagged_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pBaseType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_base_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pBaseTypes",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_base_types",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pFindAllDerivedTypes",
                        new Class[]{AnalysisUnitArray.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnitArray.class,
                        "units"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_all_derived_types",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pCompType",
                        new Class[]{boolean.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "isSubscript",
                        false
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_comp_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIndexType",
                        new Class[]{int.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        int.class,
                        "dim"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_index_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsDerivedType",
                        new Class[]{BaseTypeDecl.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        BaseTypeDecl.class,
                        "otherType"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_derived_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsInterfaceType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_interface_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pMatchingType",
                        new Class[]{BaseTypeDecl.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        BaseTypeDecl.class,
                        "expectedType"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_matching_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pCanonicalType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pPreviousPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "goToIncomplete",
                        true
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_previous_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pNextPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_next_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pFullView",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_full_view",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsDefiniteSubtype",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_definite_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pIsPrivate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_private",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pDiscriminantsList",
                        new Class[]{BaseTypeDecl.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        BaseTypeDecl.class,
                        "stopRecurseAt",
                        BaseTypeDecl.NONE
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_discriminants_list",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pRootType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_root_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseTypeDecl.class.getMethod(
                        "pShapes",
                        new Class[]{boolean.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "includeDiscriminants",
                        true
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_shapes",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseTypeDecl NONE =
            new BaseTypeDeclNone();

        // ----- Constructors -----

        protected BaseTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseTypeDecl.NONE :

                (BaseTypeDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * If this type decl is a subtype decl, return the base subtype. If
         * not, return ``Self``.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pBaseSubtype(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_base_subtype(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_base_subtype(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the private completion for this type, if there is one.
         */
        public BaseTypeDecl pPrivateCompletion(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_private_completion(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_private_completion(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Assuming that P is a primitive of Self, return whether the given
         * primitive P is inherited from one of Self's parents.
         */
        public boolean pIsInheritedPrimitive(
            final BasicDecl p
        ) {

            // Verify that arguments are not null
            if(p == null) throw new IllegalArgumentException(
                "Argument 'p' of type " +
                "BasicDecl cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative pNative = StackValue.get(EntityNative.class);p.entity.unwrap(pNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_inherited_primitive(
                    thisNative,
                    pNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_inherited_primitive(
                    (p != null ? p.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the record representation clause associated to this type
         * decl, if applicable (i.e. this type decl defines a record type).
         */
        public RecordRepClause pGetRecordRepresentationClause(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_get_record_representation_clause(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RecordRepClause res = RecordRepClause.NONE;
                if(propException == null) {
                    res = RecordRepClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_get_record_representation_clause(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return RecordRepClause.fromEntity(res);
            }

        }

        
    

        /**
         * Return the enum representation clause associated to this type decl,
         * if applicable (i.e. this type decl defines an enum type).
         */
        public EnumRepClause pGetEnumRepresentationClause(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_get_enum_representation_clause(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EnumRepClause res = EnumRepClause.NONE;
                if(propException == null) {
                    res = EnumRepClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_get_enum_representation_clause(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return EnumRepClause.fromEntity(res);
            }

        }

        
    

        /**
         * Return whether this type is a record type.
         *
         * .. attention:: Private tagged types extending public tagged records
         *    are not considered as record types.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsRecordType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_record_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_record_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this type is an array type.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsArrayType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_array_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_array_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Find types derived from self in the given ``root`` and its children.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public AdaNodeArray pFindDerivedTypes(
            final AdaNode root,final AdaNode origin,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(root == null) throw new IllegalArgumentException(
                "Argument 'root' of type " +
                "AdaNode cannot be null"
            );
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative rootNative = StackValue.get(EntityNative.class);root.entity.unwrap(rootNative);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_find_derived_types(
                    thisNative,
                    rootNative,
                    originNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_type_decl_p_find_derived_types(
                    (root != null ? root.entity : null),
                    (origin != null ? origin.entity : null),
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is a real type or not.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsRealType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_real_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_real_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is a float type or not.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsFloatType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_float_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_float_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is a fixed point type or not.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsFixedPoint(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_fixed_point(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_fixed_point(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is a scalar type.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsScalarType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_scalar_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_scalar_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is an enum type
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsEnumType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_enum_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_enum_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether Self is an access type or not
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsAccessType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_access_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_access_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the specific type under a class-wide type. Consider for
         * example:
         *
         * .. code:: ada
         *
         *    subtype S1 is T'Class
         *    subtype S2 is S1'Class
         *
         * Calling this property on ``S2`` will return ``T``.
         */
        public BaseTypeDecl pSpecificType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_specific_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_specific_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Whether type is a character type or not
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsCharType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_char_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_char_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the classwide type for this type, if applicable
         */
        public ClasswideTypeDecl pClasswideType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_classwide_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ClasswideTypeDecl res = ClasswideTypeDecl.NONE;
                if(propException == null) {
                    res = ClasswideTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_classwide_type(
                    this.entity
                );

                // Wrap and return the result
                return ClasswideTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the discrete range for this type decl, if applicable.
         */
        public DiscreteRange pDiscreteRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final DiscreteRangeNative resNative =
                    StackValue.get(DiscreteRangeNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_discrete_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscreteRange res = DiscreteRange.NONE;
                if(propException == null) {
                    res = DiscreteRange.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final DiscreteRange res = JNI_LIB.ada_base_type_decl_p_discrete_range(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is a discrete type or not.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsDiscreteType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_discrete_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_discrete_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether type is an integer type or not.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsIntType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_int_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_int_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * If this type is an access type, or a type with an
         * Implicit_Dereference aspect, return the type of a dereference of an
         * instance of this type.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pAccessedType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_accessed_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_accessed_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Whether type is tagged or not
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsTaggedType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_tagged_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_tagged_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the base type entity for this derived type declaration
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pBaseType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_base_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_base_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the list of base types for Self.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public AdaNodeArray pBaseTypes(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_base_types(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_type_decl_p_base_types(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of all types that inherit (directly or indirectly)
         * from Self among the given units.
         */
        public AdaNodeArray pFindAllDerivedTypes(
            final AnalysisUnitArray units,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(units == null) throw new IllegalArgumentException(
                "Argument 'units' of type " +
                "AnalysisUnitArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitArrayNative unitsNative = units.unwrap();
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_find_all_derived_types(
                    thisNative,
                    unitsNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                AnalysisUnitArray.release(
                    unitsNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_type_decl_p_find_all_derived_types(
                    units,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the component type of ``Self``, if applicable. The component
         * type is the type you'll get if you call a value whose type is
         * ``Self``. So it can either be:
         *
         * 1. The component type for an array.
         *
         * 2. The return type for an access to function.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pCompType(
            final boolean isSubscript,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte isSubscriptNative = (isSubscript ? (byte) 1 : (byte) 0);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_comp_type(
                    thisNative,
                    isSubscriptNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_comp_type(
                    isSubscript,
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the index type for dimension ``dim`` for this type, if
         * applicable.
         *
         * .. warning:: ``dim`` is 0-based, so the first ``index_type`` is at
         *    index 0.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pIndexType(
            final int dim,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                int dimNative = dim;
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_index_type(
                    thisNative,
                    dimNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_index_type(
                    dim,
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Whether Self is derived from other_type.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsDerivedType(
            final BaseTypeDecl otherType,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(otherType == null) throw new IllegalArgumentException(
                "Argument 'otherType' of type " +
                "BaseTypeDecl cannot be null"
            );
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative otherTypeNative = StackValue.get(EntityNative.class);otherType.entity.unwrap(otherTypeNative);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_derived_type(
                    thisNative,
                    otherTypeNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_derived_type(
                    (otherType != null ? otherType.entity : null),
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return True iff this type declaration is an interface definition.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsInterfaceType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_interface_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_interface_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether ``self`` matches ``expected_type``.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pMatchingType(
            final BaseTypeDecl expectedType,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(expectedType == null) throw new IllegalArgumentException(
                "Argument 'expectedType' of type " +
                "BaseTypeDecl cannot be null"
            );
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative expectedTypeNative = StackValue.get(EntityNative.class);expectedType.entity.unwrap(expectedTypeNative);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_matching_type(
                    thisNative,
                    expectedTypeNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_matching_type(
                    (expectedType != null ? expectedType.entity : null),
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the canonical type declaration for this type declaration. For
         * subtypes, it will return the base type declaration.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pCanonicalType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_canonical_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_canonical_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns the previous part for this type decl.
         */
        public BaseTypeDecl pPreviousPart(
            final boolean goToIncomplete
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte goToIncompleteNative = (goToIncomplete ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_previous_part(
                    thisNative,
                    goToIncompleteNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_previous_part(
                    goToIncomplete,
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns the next part for this type decl.
         *
         * .. note:: Since this property returns a ``BaseTypeDecl``, it cannot
         *    be used to retrieve the next part of ``TaskTypeDecl`` and
         *    ``ProtectedTypeDecl`` nodes as their next part is actually a
         *    ``Body``. Use ``BasicDecl.next_part_for_decl`` for those instead.
         */
        public BaseTypeDecl pNextPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_next_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_next_part(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the full completion of this type.
         */
        public BaseTypeDecl pFullView(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_full_view(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_full_view(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns whether this is a definite subtype.
         *
         * For convenience, this will return ``False`` for incomplete types,
         * even though the correct answer is more akin to "non applicable".
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public boolean pIsDefiniteSubtype(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_definite_subtype(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_definite_subtype(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether node is a private view of corresponding type.
         */
        public boolean pIsPrivate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_is_private(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_type_decl_p_is_private(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of all discriminants of this type. If this type has
         * no discriminant or only unknown discriminants, an empty list is
         * returned.
         *
         * In order to obtain all the discriminants of an extended type, this
         * property looks on parents, recursively.
         *
         * Extended aggregates can be build from any intermediate parent of an
         * extended type. In that case, this property shouldn't recurse to the
         * root type, but the one used as the aggregate's ancestor, designated
         * by ``stop_recurse_at``.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public AdaNodeArray pDiscriminantsList(
            final BaseTypeDecl stopRecurseAt,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(stopRecurseAt == null) throw new IllegalArgumentException(
                "Argument 'stopRecurseAt' of type " +
                "BaseTypeDecl cannot be null"
            );
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative stopRecurseAtNative = StackValue.get(EntityNative.class);stopRecurseAt.entity.unwrap(stopRecurseAtNative);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_discriminants_list(
                    thisNative,
                    stopRecurseAtNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_base_type_decl_p_discriminants_list(
                    (stopRecurseAt != null ? stopRecurseAt.entity : null),
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the type that is at the root of the derivation hierarchy
         * (ignoring secondary interfaces derivations for tagged types)
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pRootType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_type_decl_p_root_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_type_decl_p_root_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Must be called on a record (sub-)type declaration. Return all the
         * possible shapes that a value of this record type can take. For
         * example, consider the following record definition:
         *
         * .. code::
         *
         *    type R (A : Integer; B : Integer) is record
         *        X : Integer;
         *        case A is
         *            when 1 .. 10 =>
         *                Y_1 : Integer;
         *                case B is
         *                    when 1 .. 10 =>
         *                        Z_1 : Integer;
         *                    when others => null;
         *                end case;
         *            when 11 .. 20 =>
         *                Y_2 : Integer;
         *                case B is
         *                    when 1 .. 10 =>
         *                        Z_2 : Integer;
         *                    when others => null;
         *                end case;
         *            when others => null;
         *        end case;
         *    end record;
         *
         * For this instance, this property will return the following results:
         *
         * .. code::
         *
         *    [
         *        [X, Y_1, Z_1],
         *        [X, Y_1],
         *        [X, Y_2, Z_2],
         *        [X, Y_2],
         *        [X]
         *    ]
         *
         * .. attention:: This property is inaccurate when called on a record
         *    extension which defines components under a certain condition C,
         *    and this same condition is used to define some components in the
         *    parent record: in that case, any feasible shape will in practice
         *    contain either both the components defined under condition C in
         *    the child record and the parent record, or none of them.However,
         *    due to the simplified algorithm we use here to compute the
         *    feasible shapes, we will also return shapes that include the
         *    components of the child record but not the parent record, and
         *    conversely.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public ShapeArray pShapes(
            final boolean includeDiscriminants,final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte includeDiscriminantsNative = (includeDiscriminants ? (byte) 1 : (byte) 0);
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_type_decl_p_shapes(
                    thisNative,
                    includeDiscriminantsNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ShapeArray res = ShapeArray.NONE;
                if(propException == null) {
                    res = ShapeArray.wrap(resNative);

                    ShapeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ShapeArray res = JNI_LIB.ada_base_type_decl_p_shapes(
                    includeDiscriminants,
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseTypeDecl.
         */
        private static final class BaseTypeDeclNone extends BaseTypeDecl {
            BaseTypeDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Base class for subtype declarations (:rmlink:`3.2.2`).
     */
    public static abstract
    class BaseSubtypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseSubtypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubtypeDecl.class.getMethod(
                        "pGetType",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseSubtypeDecl NONE =
            new BaseSubtypeDeclNone();

        // ----- Constructors -----

        protected BaseSubtypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseSubtypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseSubtypeDecl.NONE :

                (BaseSubtypeDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseSubtypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseSubtypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseSubtypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseSubtypeDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Get the type for this subtype.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public BaseTypeDecl pGetType(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subtype_decl_p_get_type(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subtype_decl_p_get_type(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseSubtypeDecl.
         */
        private static final class BaseSubtypeDeclNone extends BaseSubtypeDecl {
            BaseSubtypeDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Specific ``BaseSubtypeDecl`` synthetic subclass for the base type of
     * scalar types.
     */
    public static 
    class DiscreteBaseSubtypeDecl extends BaseSubtypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscreteBaseSubtypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubtypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscreteBaseSubtypeDecl NONE =
            new DiscreteBaseSubtypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscreteBaseSubtypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscreteBaseSubtypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscreteBaseSubtypeDecl.NONE :

                new DiscreteBaseSubtypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscreteBaseSubtypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscreteBaseSubtypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscreteBaseSubtypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscreteBaseSubtypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Subtype declaration (:rmlink:`3.2.2`).
     */
    public static 
    class SubtypeDecl extends BaseSubtypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubtypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fSubtype","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubtypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeDecl.class.getMethod(
                        "fSubtype",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubtypeDecl NONE =
            new SubtypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubtypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubtypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubtypeDecl.NONE :

                new SubtypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubtypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubtypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubtypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubtypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubtypeIndication fSubtype(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subtype_decl_f_subtype(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubtypeIndication res = SubtypeIndication.NONE;
                if(propException == null) {
                    res = SubtypeIndication.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subtype_decl_f_subtype(
                    this.entity
                );

                // Wrap and return the result
                return SubtypeIndication.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic node (not parsed, generated from a property call). Refers to
     * the classwide type for a given tagged type (:rmlink:`3.4.1`).
     */
    public static 
    class ClasswideTypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ClasswideTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ClasswideTypeDecl NONE =
            new ClasswideTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ClasswideTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ClasswideTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ClasswideTypeDecl.NONE :

                new ClasswideTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ClasswideTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ClasswideTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ClasswideTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ClasswideTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Incomplete declaration for a type (:rmlink:`12.5`).
     */
    public static 
    class IncompleteTypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IncompleteTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IncompleteTypeDecl.class.getMethod(
                        "fDiscriminants",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discriminants",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IncompleteTypeDecl NONE =
            new IncompleteTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IncompleteTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static IncompleteTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IncompleteTypeDecl.NONE :

                new IncompleteTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IncompleteTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IncompleteTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IncompleteTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IncompleteTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DiscriminantPart fDiscriminants(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_incomplete_type_decl_f_discriminants(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantPart res = DiscriminantPart.NONE;
                if(propException == null) {
                    res = DiscriminantPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_incomplete_type_decl_f_discriminants(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantPart.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * A formal incomplete type declaration.
     */
    public static 
    class IncompleteFormalTypeDecl extends IncompleteTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IncompleteFormalTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fIsTagged","fDefaultType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            IncompleteTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IncompleteFormalTypeDecl.class.getMethod(
                        "fIsTagged",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_is_tagged",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IncompleteFormalTypeDecl.class.getMethod(
                        "fDefaultType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IncompleteFormalTypeDecl NONE =
            new IncompleteFormalTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IncompleteFormalTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static IncompleteFormalTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IncompleteFormalTypeDecl.NONE :

                new IncompleteFormalTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IncompleteFormalTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IncompleteFormalTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IncompleteFormalTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IncompleteFormalTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public TaggedNode fIsTagged(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_incomplete_formal_type_decl_f_is_tagged(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TaggedNode res = TaggedNode.NONE;
                if(propException == null) {
                    res = TaggedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_incomplete_formal_type_decl_f_is_tagged(
                    this.entity
                );

                // Wrap and return the result
                return TaggedNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fDefaultType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_incomplete_formal_type_decl_f_default_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_incomplete_formal_type_decl_f_default_type(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Incomplete declaration for a tagged type.
     */
    public static 
    class IncompleteTaggedTypeDecl extends IncompleteTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IncompleteTaggedTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fHasAbstract"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            IncompleteTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IncompleteTaggedTypeDecl.class.getMethod(
                        "fHasAbstract",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_abstract",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IncompleteTaggedTypeDecl NONE =
            new IncompleteTaggedTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IncompleteTaggedTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static IncompleteTaggedTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IncompleteTaggedTypeDecl.NONE :

                new IncompleteTaggedTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IncompleteTaggedTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IncompleteTaggedTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IncompleteTaggedTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IncompleteTaggedTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AbstractNode fHasAbstract(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_incomplete_tagged_type_decl_f_has_abstract(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbstractNode res = AbstractNode.NONE;
                if(propException == null) {
                    res = AbstractNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_incomplete_tagged_type_decl_f_has_abstract(
                    this.entity
                );

                // Wrap and return the result
                return AbstractNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a protected type (:rmlink:`9.4`).
     */
    public static 
    class ProtectedTypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fAspects","fInterfaces","fDefinition"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedTypeDecl.class.getMethod(
                        "fDiscriminants",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discriminants",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedTypeDecl.class.getMethod(
                        "fInterfaces",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interfaces",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedTypeDecl.class.getMethod(
                        "fDefinition",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_definition",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedTypeDecl NONE =
            new ProtectedTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedTypeDecl.NONE :

                new ProtectedTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DiscriminantPart fDiscriminants(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_type_decl_f_discriminants(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantPart res = DiscriminantPart.NONE;
                if(propException == null) {
                    res = DiscriminantPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_type_decl_f_discriminants(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantPart.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ParentList fInterfaces(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_type_decl_f_interfaces(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParentList res = ParentList.NONE;
                if(propException == null) {
                    res = ParentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_type_decl_f_interfaces(
                    this.entity
                );

                // Wrap and return the result
                return ParentList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ProtectedDef fDefinition(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_type_decl_f_definition(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ProtectedDef res = ProtectedDef.NONE;
                if(propException == null) {
                    res = ProtectedDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_type_decl_f_definition(
                    this.entity
                );

                // Wrap and return the result
                return ProtectedDef.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a task type (:rmlink:`9.1`).
     */
    public static 
    class TaskTypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaskTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fAspects","fDefinition"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TaskTypeDecl.class.getMethod(
                        "fDiscriminants",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discriminants",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskTypeDecl.class.getMethod(
                        "fDefinition",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_definition",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaskTypeDecl NONE =
            new TaskTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaskTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaskTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaskTypeDecl.NONE :

                new TaskTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaskTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaskTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaskTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaskTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DiscriminantPart fDiscriminants(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_type_decl_f_discriminants(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantPart res = DiscriminantPart.NONE;
                if(propException == null) {
                    res = DiscriminantPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_type_decl_f_discriminants(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantPart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public TaskDef fDefinition(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_type_decl_f_definition(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TaskDef res = TaskDef.NONE;
                if(propException == null) {
                    res = TaskDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_type_decl_f_definition(
                    this.entity
                );

                // Wrap and return the result
                return TaskDef.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type declaration for a single task (:rmlink:`9.1`).
     */
    public static 
    class SingleTaskTypeDecl extends TaskTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SingleTaskTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fAspects","fDefinition"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TaskTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SingleTaskTypeDecl NONE =
            new SingleTaskTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SingleTaskTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SingleTaskTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SingleTaskTypeDecl.NONE :

                new SingleTaskTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SingleTaskTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SingleTaskTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SingleTaskTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SingleTaskTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Type declarations that embed a type definition node. Corresponds to the
     * ARM's full type declarations (:rmlink:`3.2.1`).
     */
    public static abstract
    class TypeDecl extends BaseTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fTypeDef"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TypeDecl.class.getMethod(
                        "fDiscriminants",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discriminants",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeDecl.class.getMethod(
                        "fTypeDef",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_def",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeDecl.class.getMethod(
                        "pGetPrimitives",
                        new Class[]{boolean.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "onlyInherited",
                        false
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "includePredefinedOperators",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_primitives",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TypeDecl NONE =
            new TypeDeclNone();

        // ----- Constructors -----

        protected TypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static TypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TypeDecl.NONE :

                (TypeDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TypeDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DiscriminantPart fDiscriminants(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_decl_f_discriminants(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscriminantPart res = DiscriminantPart.NONE;
                if(propException == null) {
                    res = DiscriminantPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_decl_f_discriminants(
                    this.entity
                );

                // Wrap and return the result
                return DiscriminantPart.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public TypeDef fTypeDef(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_decl_f_type_def(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeDef res = TypeDef.NONE;
                if(propException == null) {
                    res = TypeDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_decl_f_type_def(
                    this.entity
                );

                // Wrap and return the result
                return TypeDef.fromEntity(res);
            }

        }

        
    

        /**
         * Return the list of all primitive operations that are available on
         * this type. If ``only_inherited`` is True, it will only return the
         * primitives that are implicitly inherited by this type, discarding
         * those explicitly defined on this type. Predefined operators are
         * included in the result iff ``include_predefined_operators`` is True.
         * It defaults to False.
         */
        public AdaNodeArray pGetPrimitives(
            final boolean onlyInherited,final boolean includePredefinedOperators
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte onlyInheritedNative = (onlyInherited ? (byte) 1 : (byte) 0);
                byte includePredefinedOperatorsNative = (includePredefinedOperators ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_type_decl_p_get_primitives(
                    thisNative,
                    onlyInheritedNative,
                    includePredefinedOperatorsNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_type_decl_p_get_primitives(
                    onlyInherited,
                    includePredefinedOperators,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * TypeDecl.
         */
        private static final class TypeDeclNone extends TypeDecl {
            TypeDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Anonymous type declaration (for anonymous array or access types). This
     * class has no RM existence, and anonymous (sub)types are referred to
     * implicitly in the RM.
     */
    public static 
    class AnonymousTypeDecl extends TypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AnonymousTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fTypeDef"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AnonymousTypeDecl NONE =
            new AnonymousTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AnonymousTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static AnonymousTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AnonymousTypeDecl.NONE :

                new AnonymousTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AnonymousTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AnonymousTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AnonymousTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AnonymousTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic anonymous type decl. Used to generate anonymous access types.
     */
    public static 
    class SynthAnonymousTypeDecl extends AnonymousTypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SynthAnonymousTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fTypeDef"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AnonymousTypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SynthAnonymousTypeDecl NONE =
            new SynthAnonymousTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SynthAnonymousTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SynthAnonymousTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SynthAnonymousTypeDecl.NONE :

                new SynthAnonymousTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SynthAnonymousTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SynthAnonymousTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SynthAnonymousTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SynthAnonymousTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * A concrete type declaration.
     */
    public static 
    class ConcreteTypeDecl extends TypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConcreteTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fTypeDef","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConcreteTypeDecl NONE =
            new ConcreteTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConcreteTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConcreteTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConcreteTypeDecl.NONE :

                new ConcreteTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConcreteTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConcreteTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConcreteTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConcreteTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * A formal type declaration.
     */
    public static 
    class FormalTypeDecl extends TypeDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "FormalTypeDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fDiscriminants","fTypeDef","fDefaultType","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = FormalTypeDecl.class.getMethod(
                        "fDefaultType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final FormalTypeDecl NONE =
            new FormalTypeDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected FormalTypeDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static FormalTypeDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                FormalTypeDecl.NONE :

                new FormalTypeDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return FormalTypeDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return FormalTypeDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return FormalTypeDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return FormalTypeDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fDefaultType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_formal_type_decl_f_default_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_formal_type_decl_f_default_type(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for subprogram declarations.
     */
    public static abstract
    class BasicSubpDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BasicSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BasicSubpDecl.class.getMethod(
                        "pHasSparkModeOn",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_has_spark_mode_on",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BasicSubpDecl.class.getMethod(
                        "pSubpDeclSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_subp_decl_spec",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BasicSubpDecl NONE =
            new BasicSubpDeclNone();

        // ----- Constructors -----

        protected BasicSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static BasicSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BasicSubpDecl.NONE :

                (BasicSubpDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BasicSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BasicSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BasicSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BasicSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Returns whether this subprogram has explicitly been set as having
         * ``Spark_Mode`` to ``On``, directly or indirectly.
         *
         * Doesn't include subprograms that can be inferred by GNATprove as
         * being SPARK.
         */
        public boolean pHasSparkModeOn(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_basic_subp_decl_p_has_spark_mode_on(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_basic_subp_decl_p_has_spark_mode_on(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the specification for this subprogram
         */
        public BaseSubpSpec pSubpDeclSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_basic_subp_decl_p_subp_decl_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseSubpSpec res = BaseSubpSpec.NONE;
                if(propException == null) {
                    res = BaseSubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_basic_subp_decl_p_subp_decl_spec(
                    this.entity
                );

                // Wrap and return the result
                return BaseSubpSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BasicSubpDecl.
         */
        private static final class BasicSubpDeclNone extends BasicSubpDecl {
            BasicSubpDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * This is an intermediate abstract class for subprogram declarations with
     * a common structure: overriding indicator, ``SubpSpec``, aspects, <other
     * fields>.
     */
    public static abstract
    class ClassicSubpDecl extends BasicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ClassicSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ClassicSubpDecl.class.getMethod(
                        "fOverriding",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_overriding",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ClassicSubpDecl.class.getMethod(
                        "fSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ClassicSubpDecl.class.getMethod(
                        "pBodyPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ClassicSubpDecl NONE =
            new ClassicSubpDeclNone();

        // ----- Constructors -----

        protected ClassicSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ClassicSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ClassicSubpDecl.NONE :

                (ClassicSubpDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ClassicSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ClassicSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ClassicSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ClassicSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OverridingNode fOverriding(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_classic_subp_decl_f_overriding(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OverridingNode res = OverridingNode.NONE;
                if(propException == null) {
                    res = OverridingNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_classic_subp_decl_f_overriding(
                    this.entity
                );

                // Wrap and return the result
                return OverridingNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpSpec fSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_classic_subp_decl_f_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpSpec res = SubpSpec.NONE;
                if(propException == null) {
                    res = SubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_classic_subp_decl_f_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return SubpSpec.fromEntity(res);
            }

        }

        
    

        /**
         * Return the BaseSubpBody corresponding to this node.
         */
        public BaseSubpBody pBodyPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_classic_subp_decl_p_body_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseSubpBody res = BaseSubpBody.NONE;
                if(propException == null) {
                    res = BaseSubpBody.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_classic_subp_decl_p_body_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BaseSubpBody.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ClassicSubpDecl.
         */
        private static final class ClassicSubpDeclNone extends ClassicSubpDecl {
            ClassicSubpDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Declaration for an abstract subprogram (:rmlink:`3.9.3`).
     */
    public static 
    class AbstractSubpDecl extends ClassicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ClassicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractSubpDecl NONE =
            new AbstractSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractSubpDecl.NONE :

                new AbstractSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal subprogram declarations, in generic declarations formal parts
     * (:rmlink:`12.6`).
     */
    public static abstract
    class FormalSubpDecl extends ClassicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "FormalSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ClassicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = FormalSubpDecl.class.getMethod(
                        "fDefaultExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final FormalSubpDecl NONE =
            new FormalSubpDeclNone();

        // ----- Constructors -----

        protected FormalSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static FormalSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                FormalSubpDecl.NONE :

                (FormalSubpDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return FormalSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return FormalSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return FormalSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return FormalSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BoxExpr``, ``CallExpr``, ``CharLiteral``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``NullLiteral``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fDefaultExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_formal_subp_decl_f_default_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_formal_subp_decl_f_default_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * FormalSubpDecl.
         */
        private static final class FormalSubpDeclNone extends FormalSubpDecl {
            FormalSubpDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Formal declaration for an abstract subprogram (:rmlink:`12.6`).
     */
    public static 
    class AbstractFormalSubpDecl extends FormalSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractFormalSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            FormalSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractFormalSubpDecl NONE =
            new AbstractFormalSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractFormalSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractFormalSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractFormalSubpDecl.NONE :

                new AbstractFormalSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractFormalSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractFormalSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractFormalSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractFormalSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Formal declaration for a concrete subprogram (:rmlink:`12.6`).
     */
    public static 
    class ConcreteFormalSubpDecl extends FormalSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConcreteFormalSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fDefaultExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            FormalSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConcreteFormalSubpDecl NONE =
            new ConcreteFormalSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConcreteFormalSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConcreteFormalSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConcreteFormalSubpDecl.NONE :

                new ConcreteFormalSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConcreteFormalSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConcreteFormalSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConcreteFormalSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConcreteFormalSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Regular subprogram declaration (:rmlink:`6.1`).
     */
    public static 
    class SubpDecl extends ClassicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ClassicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpDecl NONE =
            new SubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpDecl.NONE :

                new SubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Entry declaration (:rmlink:`9.4`).
     */
    public static 
    class EntryDecl extends BasicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EntryDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EntryDecl.class.getMethod(
                        "fOverriding",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_overriding",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryDecl.class.getMethod(
                        "fSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryDecl.class.getMethod(
                        "pBodyPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryDecl.class.getMethod(
                        "pAcceptStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_accept_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EntryDecl NONE =
            new EntryDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EntryDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static EntryDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EntryDecl.NONE :

                new EntryDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EntryDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EntryDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EntryDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EntryDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OverridingNode fOverriding(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_decl_f_overriding(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OverridingNode res = OverridingNode.NONE;
                if(propException == null) {
                    res = OverridingNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_decl_f_overriding(
                    this.entity
                );

                // Wrap and return the result
                return OverridingNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public EntrySpec fSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_decl_f_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EntrySpec res = EntrySpec.NONE;
                if(propException == null) {
                    res = EntrySpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_decl_f_spec(
                    this.entity
                );

                // Wrap and return the result
                return EntrySpec.fromEntity(res);
            }

        }

        
    

        /**
         * Return the entry body associated to this entry declaration.
         */
        public BodyNode pBodyPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_decl_p_body_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BodyNode res = BodyNode.NONE;
                if(propException == null) {
                    res = BodyNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_decl_p_body_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BodyNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return an array of accept statements corresponding to this entry.
         */
        public AdaNodeArray pAcceptStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_entry_decl_p_accept_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_entry_decl_p_accept_stmts(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for an enumeration literal (:rmlink:`3.5.1`).
     */
    public static 
    class EnumLiteralDecl extends BasicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumLiteralDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EnumLiteralDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EnumLiteralDecl.class.getMethod(
                        "pEnumType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_enum_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EnumLiteralDecl.class.getMethod(
                        "pEnumRep",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_enum_rep",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumLiteralDecl NONE =
            new EnumLiteralDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumLiteralDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumLiteralDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumLiteralDecl.NONE :

                new EnumLiteralDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumLiteralDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumLiteralDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumLiteralDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumLiteralDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_enum_literal_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_enum_literal_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return the enum type corresponding to this enum literal.
         */
        public TypeDecl pEnumType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_enum_literal_decl_p_enum_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeDecl res = TypeDecl.NONE;
                if(propException == null) {
                    res = TypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_enum_literal_decl_p_enum_type(
                    this.entity
                );

                // Wrap and return the result
                return TypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the integer used to encode this enum literal.
         *
         * .. note:: This property is equivalent to GNAT's ``Enum_Rep``
         *    attribute.
         */
        public BigInteger pEnumRep(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_enum_literal_decl_p_enum_rep(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BigInteger res = BigIntegerWrapper.NONE;
                if(propException == null) {
                    res = BigIntegerWrapper.wrap(resNative);

                    BigIntegerWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final BigInteger res = JNI_LIB.ada_enum_literal_decl_p_enum_rep(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic character enum literal declaration.
     */
    public static 
    class SyntheticCharEnumLit extends EnumLiteralDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticCharEnumLit";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            EnumLiteralDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticCharEnumLit.class.getMethod(
                        "pExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticCharEnumLit NONE =
            new SyntheticCharEnumLit(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticCharEnumLit(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticCharEnumLit fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticCharEnumLit.NONE :

                new SyntheticCharEnumLit(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticCharEnumLit.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticCharEnumLit.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticCharEnumLit.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticCharEnumLit.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the CharLiteral expression corresponding to this enum
         * literal.
         */
        public DefiningName pExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_char_enum_lit_p_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_char_enum_lit_p_expr(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Internal node for generic subprograms.
     */
    public static 
    class GenericSubpInternal extends BasicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericSubpInternal";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSubpSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInternal.class.getMethod(
                        "fSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericSubpInternal NONE =
            new GenericSubpInternal(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericSubpInternal(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericSubpInternal fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericSubpInternal.NONE :

                new GenericSubpInternal(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericSubpInternal.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericSubpInternal.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericSubpInternal.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericSubpInternal.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpSpec fSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_internal_f_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpSpec res = SubpSpec.NONE;
                if(propException == null) {
                    res = SubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_internal_f_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return SubpSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic subprogram declaration.
     *
     * Is used to represent predefined operators. This should also be usable
     * for synthesizing function attributes.
     */
    public static 
    class SyntheticSubpDecl extends BasicSubpDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicSubpDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticSubpDecl.class.getMethod(
                        "fSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_spec",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticSubpDecl NONE =
            new SyntheticSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticSubpDecl.NONE :

                new SyntheticSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``SyntheticBinarySpec``, ``SyntheticUnarySpec``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BaseSubpSpec fSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_subp_decl_f_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseSubpSpec res = BaseSubpSpec.NONE;
                if(propException == null) {
                    res = BaseSubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_subp_decl_f_spec(
                    this.entity
                );

                // Wrap and return the result
                return BaseSubpSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for an Ada body (:rmlink:`3.11`). A body is the completion of
     * a declaration.
     */
    public static abstract
    class BodyNode extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BodyNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BodyNode.class.getMethod(
                        "pPreviousPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_previous_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BodyNode.class.getMethod(
                        "pDeclPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_decl_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BodyNode.class.getMethod(
                        "pSubunitRoot",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_subunit_root",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BodyNode NONE =
            new BodyNodeNone();

        // ----- Constructors -----

        protected BodyNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static BodyNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BodyNode.NONE :

                (BodyNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BodyNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BodyNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BodyNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BodyNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the previous part for this body. Might be a declaration or a
         * body stub.
         */
        public BasicDecl pPreviousPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_body_node_p_previous_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_body_node_p_previous_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the decl corresponding to this node if applicable.
         */
        public BasicDecl pDeclPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_body_node_p_decl_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_body_node_p_decl_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * If self is a subunit, return the body in which it is rooted.
         */
        public BasicDecl pSubunitRoot(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_body_node_p_subunit_root(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_body_node_p_subunit_root(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BodyNode.
         */
        private static final class BodyNodeNone extends BodyNode {
            BodyNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * BasicDecl that is always the declaration of an AcceptStmt. This is
     * nested *inside* of the accept statement.
     */
    public static 
    class AcceptStmtBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AcceptStmtBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmtBody.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AcceptStmtBody NONE =
            new AcceptStmtBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AcceptStmtBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static AcceptStmtBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AcceptStmtBody.NONE :

                new AcceptStmtBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AcceptStmtBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AcceptStmtBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AcceptStmtBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AcceptStmtBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_body_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_body_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for subprogram bodies (:rmlink:`6.3`).
     */
    public static abstract
    class BaseSubpBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseSubpBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpBody.class.getMethod(
                        "fOverriding",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_overriding",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpBody.class.getMethod(
                        "fSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpBody.class.getMethod(
                        "pIsSubjectToProof",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_subject_to_proof",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseSubpBody.class.getMethod(
                        "pHasSparkModeOn",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_has_spark_mode_on",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseSubpBody NONE =
            new BaseSubpBodyNone();

        // ----- Constructors -----

        protected BaseSubpBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseSubpBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseSubpBody.NONE :

                (BaseSubpBody) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseSubpBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseSubpBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseSubpBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseSubpBody.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OverridingNode fOverriding(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_body_f_overriding(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OverridingNode res = OverridingNode.NONE;
                if(propException == null) {
                    res = OverridingNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_body_f_overriding(
                    this.entity
                );

                // Wrap and return the result
                return OverridingNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpSpec fSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_subp_body_f_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpSpec res = SubpSpec.NONE;
                if(propException == null) {
                    res = SubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_subp_body_f_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return SubpSpec.fromEntity(res);
            }

        }

        
    

        /**
         * Returns whether this subprogram body is subject to proof in the
         * context of the SPARK/GNATprove tools.
         */
        public boolean pIsSubjectToProof(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_subp_body_p_is_subject_to_proof(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_subp_body_p_is_subject_to_proof(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns whether this subprogram has explicitly been set as having
         * ``Spark_Mode`` to ``On``, directly or indirectly.
         *
         * Doesn't include subprograms that can be inferred by GNATprove as
         * being SPARK.
         */
        public boolean pHasSparkModeOn(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_subp_body_p_has_spark_mode_on(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_subp_body_p_has_spark_mode_on(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseSubpBody.
         */
        private static final class BaseSubpBodyNone extends BaseSubpBody {
            BaseSubpBodyNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Expression function (:rmlink:`6.8`).
     */
    public static 
    class ExprFunction extends BaseSubpBody {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExprFunction";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fExpr","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpBody.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExprFunction.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExprFunction NONE =
            new ExprFunction(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExprFunction(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExprFunction fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExprFunction.NONE :

                new ExprFunction(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExprFunction.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExprFunction.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExprFunction.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExprFunction.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``BaseAggregate``, ``ParenExpr``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_expr_function_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_expr_function_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a null subprogram (:rmlink:`6.1`).
     */
    public static 
    class NullSubpDecl extends BaseSubpBody {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpBody.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullSubpDecl NONE =
            new NullSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullSubpDecl.NONE :

                new NullSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Subprogram body(:rmlink:`6.3`) .
     */
    public static 
    class SubpBody extends BaseSubpBody {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fAspects","fDecls","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpBody.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubpBody.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpBody.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpBody.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpBody NONE =
            new SubpBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpBody.NONE :

                new SubpBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_body_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_body_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_body_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_body_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_body_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_body_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a subprogram renaming (:rmlink:`8.5.4`).
     */
    public static 
    class SubpRenamingDecl extends BaseSubpBody {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fRenames","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseSubpBody.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubpRenamingDecl.class.getMethod(
                        "fRenames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renames",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpRenamingDecl NONE =
            new SubpRenamingDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpRenamingDecl.NONE :

                new SubpRenamingDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public RenamingClause fRenames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_renaming_decl_f_renames(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RenamingClause res = RenamingClause.NONE;
                if(propException == null) {
                    res = RenamingClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_renaming_decl_f_renames(
                    this.entity
                );

                // Wrap and return the result
                return RenamingClause.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for a body stub (:rmlink:`10.1.3`). A body stub is meant to
     * be completed by .
     */
    public static abstract
    class BodyStub extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BodyStub";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BodyStub.class.getMethod(
                        "pSyntacticFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_syntactic_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BodyStub NONE =
            new BodyStubNone();

        // ----- Constructors -----

        protected BodyStub(
            final Entity entity
        ) {
            super(entity);
        }

        public static BodyStub fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BodyStub.NONE :

                (BodyStub) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BodyStub.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BodyStub.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BodyStub.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BodyStub.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the syntactic fully qualified name to refer to this body.
         *
         * Note that this can raise a Property_Error when the stub is in an
         * illegal place (too nested, in a declare block, etc.).
         */
        public UnboundedTextTypeArray pSyntacticFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_body_stub_p_syntactic_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_body_stub_p_syntactic_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BodyStub.
         */
        private static final class BodyStubNone extends BodyStub {
            BodyStubNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Stub for a package body (``is separate``) (:rmlink:`10.1.3`).
     */
    public static 
    class PackageBodyStub extends BodyStub {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PackageBodyStub";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyStub.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PackageBodyStub.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PackageBodyStub NONE =
            new PackageBodyStub(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PackageBodyStub(
            final Entity entity
        ) {
            super(entity);
        }

        public static PackageBodyStub fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PackageBodyStub.NONE :

                new PackageBodyStub(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PackageBodyStub.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PackageBodyStub.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PackageBodyStub.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PackageBodyStub.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_body_stub_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_body_stub_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Stub for a protected object body (``is separate``) (:rmlink:`10.1.3`).
     */
    public static 
    class ProtectedBodyStub extends BodyStub {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedBodyStub";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyStub.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedBodyStub.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedBodyStub NONE =
            new ProtectedBodyStub(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedBodyStub(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedBodyStub fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedBodyStub.NONE :

                new ProtectedBodyStub(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedBodyStub.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedBodyStub.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedBodyStub.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedBodyStub.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_body_stub_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_body_stub_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Stub for a subprogram body (``is separate``) (:rmlink:`10.1.3`).
     */
    public static 
    class SubpBodyStub extends BodyStub {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpBodyStub";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fSubpSpec","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyStub.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubpBodyStub.class.getMethod(
                        "fOverriding",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_overriding",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubpBodyStub.class.getMethod(
                        "fSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpBodyStub NONE =
            new SubpBodyStub(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpBodyStub(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpBodyStub fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpBodyStub.NONE :

                new SubpBodyStub(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpBodyStub.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpBodyStub.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpBodyStub.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpBodyStub.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OverridingNode fOverriding(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_body_stub_f_overriding(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OverridingNode res = OverridingNode.NONE;
                if(propException == null) {
                    res = OverridingNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_body_stub_f_overriding(
                    this.entity
                );

                // Wrap and return the result
                return OverridingNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpSpec fSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subp_body_stub_f_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpSpec res = SubpSpec.NONE;
                if(propException == null) {
                    res = SubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subp_body_stub_f_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return SubpSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Stub for a task body (``is separate``) (:rmlink:`10.1.3`).
     */
    public static 
    class TaskBodyStub extends BodyStub {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaskBodyStub";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyStub.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TaskBodyStub.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaskBodyStub NONE =
            new TaskBodyStub(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaskBodyStub(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaskBodyStub fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaskBodyStub.NONE :

                new TaskBodyStub(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaskBodyStub.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaskBodyStub.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaskBodyStub.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaskBodyStub.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_body_stub_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_body_stub_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Entry body (:rmlink:`9.5.2`).
     */
    public static 
    class EntryBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EntryBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fEntryName","fIndexSpec","fParams","fAspects","fBarrier","fDecls","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fEntryName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_entry_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fIndexSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_index_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fBarrier",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_barrier",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryBody.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EntryBody NONE =
            new EntryBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EntryBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static EntryBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EntryBody.NONE :

                new EntryBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EntryBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EntryBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EntryBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EntryBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fEntryName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_entry_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_entry_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EntryIndexSpec fIndexSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_index_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EntryIndexSpec res = EntryIndexSpec.NONE;
                if(propException == null) {
                    res = EntryIndexSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_index_spec(
                    this.entity
                );

                // Wrap and return the result
                return EntryIndexSpec.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public EntryCompletionFormalParams fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EntryCompletionFormalParams res = EntryCompletionFormalParams.NONE;
                if(propException == null) {
                    res = EntryCompletionFormalParams.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_params(
                    this.entity
                );

                // Wrap and return the result
                return EntryCompletionFormalParams.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fBarrier(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_barrier(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_barrier(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_body_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_body_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Package body (:rmlink:`7.2`).
     */
    public static 
    class PackageBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PackageBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPackageName","fAspects","fDecls","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PackageBody.class.getMethod(
                        "fPackageName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_package_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageBody.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageBody.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageBody.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PackageBody NONE =
            new PackageBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PackageBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static PackageBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PackageBody.NONE :

                new PackageBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PackageBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PackageBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PackageBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PackageBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fPackageName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_body_f_package_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_body_f_package_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_body_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_body_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_body_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_body_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_body_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_body_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Protected object body (:rmlink:`9.4`).
     */
    public static 
    class ProtectedBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects","fDecls","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedBody.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedBody.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedBody.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedBody NONE =
            new ProtectedBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedBody.NONE :

                new ProtectedBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_body_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_body_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_body_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_body_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_body_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_body_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Task body (:rmlink:`9.1`).
     */
    public static 
    class TaskBody extends BodyNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaskBody";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects","fDecls","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BodyNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TaskBody.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskBody.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskBody.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskBody.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaskBody NONE =
            new TaskBody(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaskBody(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaskBody fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaskBody.NONE :

                new TaskBody(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaskBody.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaskBody.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaskBody.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaskBody.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_body_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_body_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_body_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_body_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_body_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_body_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_body_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_body_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Index specification for an entry body (:rmlink:`9.5.2`).
     */
    public static 
    class EntryIndexSpec extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EntryIndexSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fId","fSubtype","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EntryIndexSpec.class.getMethod(
                        "fId",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EntryIndexSpec.class.getMethod(
                        "fSubtype",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EntryIndexSpec NONE =
            new EntryIndexSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EntryIndexSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static EntryIndexSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EntryIndexSpec.NONE :

                new EntryIndexSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EntryIndexSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EntryIndexSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EntryIndexSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EntryIndexSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fId(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_index_spec_f_id(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_index_spec_f_id(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``SubtypeIndication``,
         * ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fSubtype(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_entry_index_spec_f_subtype(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_entry_index_spec_f_subtype(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Placeholder node for syntax errors in lists of declarations.
     */
    public static 
    class ErrorDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ErrorDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ErrorDecl NONE =
            new ErrorDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ErrorDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ErrorDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ErrorDecl.NONE :

                new ErrorDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ErrorDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ErrorDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ErrorDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ErrorDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Exception declarations (:rmlink:`11.1`).
     */
    public static 
    class ExceptionDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExceptionDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fRenames","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExceptionDecl.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ExceptionDecl.class.getMethod(
                        "fRenames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renames",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExceptionDecl NONE =
            new ExceptionDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExceptionDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExceptionDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExceptionDecl.NONE :

                new ExceptionDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExceptionDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExceptionDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExceptionDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExceptionDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exception_decl_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exception_decl_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RenamingClause fRenames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exception_decl_f_renames(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RenamingClause res = RenamingClause.NONE;
                if(propException == null) {
                    res = RenamingClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exception_decl_f_renames(
                    this.entity
                );

                // Wrap and return the result
                return RenamingClause.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Exception handler (:rmlink:`11.2`).
     */
    public static 
    class ExceptionHandler extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExceptionHandler";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExceptionName","fHandledExceptions","fStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExceptionHandler.class.getMethod(
                        "fExceptionName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_exception_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ExceptionHandler.class.getMethod(
                        "fHandledExceptions",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_handled_exceptions",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ExceptionHandler.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExceptionHandler NONE =
            new ExceptionHandler(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExceptionHandler(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExceptionHandler fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExceptionHandler.NONE :

                new ExceptionHandler(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExceptionHandler.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExceptionHandler.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExceptionHandler.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExceptionHandler.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public DefiningName fExceptionName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exception_handler_f_exception_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exception_handler_f_exception_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AttributeRef``, ``CallExpr``, ``CharLiteral``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``OthersDesignator``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AlternativesList fHandledExceptions(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exception_handler_f_handled_exceptions(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AlternativesList res = AlternativesList.NONE;
                if(propException == null) {
                    res = AlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exception_handler_f_handled_exceptions(
                    this.entity
                );

                // Wrap and return the result
                return AlternativesList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exception_handler_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exception_handler_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for the controlling variable in a ``for`` loop
     * (:rmlink:`5.5`).
     */
    public static 
    class ForLoopVarDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ForLoopVarDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fId","fIdType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopVarDecl.class.getMethod(
                        "fId",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopVarDecl.class.getMethod(
                        "fIdType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ForLoopVarDecl NONE =
            new ForLoopVarDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ForLoopVarDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ForLoopVarDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ForLoopVarDecl.NONE :

                new ForLoopVarDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ForLoopVarDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ForLoopVarDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ForLoopVarDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ForLoopVarDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fId(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_var_decl_f_id(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_var_decl_f_id(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * This field may be null even when there are no parsing errors.
         */
        public TypeExpr fIdType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_var_decl_f_id_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_var_decl_f_id_type(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for generic declarations (:rmlink:`12.1`).
     */
    public static abstract
    class GenericDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fFormalPart"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericDecl.class.getMethod(
                        "fFormalPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_formal_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericDecl NONE =
            new GenericDeclNone();

        // ----- Constructors -----

        protected GenericDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericDecl.NONE :

                (GenericDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public GenericFormalPart fFormalPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_decl_f_formal_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                GenericFormalPart res = GenericFormalPart.NONE;
                if(propException == null) {
                    res = GenericFormalPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_decl_f_formal_part(
                    this.entity
                );

                // Wrap and return the result
                return GenericFormalPart.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * GenericDecl.
         */
        private static final class GenericDeclNone extends GenericDecl {
            GenericDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Generic package declaration (:rmlink:`12.1`).
     */
    public static 
    class GenericPackageDecl extends GenericDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericPackageDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fFormalPart","fPackageDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageDecl.class.getMethod(
                        "fPackageDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_package_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageDecl.class.getMethod(
                        "pBodyPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericPackageDecl NONE =
            new GenericPackageDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericPackageDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericPackageDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericPackageDecl.NONE :

                new GenericPackageDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericPackageDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericPackageDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericPackageDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericPackageDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public GenericPackageInternal fPackageDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_decl_f_package_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                GenericPackageInternal res = GenericPackageInternal.NONE;
                if(propException == null) {
                    res = GenericPackageInternal.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_decl_f_package_decl(
                    this.entity
                );

                // Wrap and return the result
                return GenericPackageInternal.fromEntity(res);
            }

        }

        
    

        /**
         * Return the PackageBody corresponding to this node, or null if there
         * is none.
         */
        public PackageBody pBodyPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_decl_p_body_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PackageBody res = PackageBody.NONE;
                if(propException == null) {
                    res = PackageBody.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_decl_p_body_part(
                    this.entity
                );

                // Wrap and return the result
                return PackageBody.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Generic subprogram declaration (:rmlink:`12.1`).
     */
    public static 
    class GenericSubpDecl extends GenericDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericSubpDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fFormalPart","fSubpDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpDecl.class.getMethod(
                        "fSubpDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpDecl.class.getMethod(
                        "pBodyPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_part",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericSubpDecl NONE =
            new GenericSubpDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericSubpDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericSubpDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericSubpDecl.NONE :

                new GenericSubpDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericSubpDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericSubpDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericSubpDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericSubpDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public GenericSubpInternal fSubpDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_decl_f_subp_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                GenericSubpInternal res = GenericSubpInternal.NONE;
                if(propException == null) {
                    res = GenericSubpInternal.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_decl_f_subp_decl(
                    this.entity
                );

                // Wrap and return the result
                return GenericSubpInternal.fromEntity(res);
            }

        }

        
    

        /**
         * Return the BaseSubpBody corresponding to this node.
         */
        public BaseSubpBody pBodyPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_decl_p_body_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseSubpBody res = BaseSubpBody.NONE;
                if(propException == null) {
                    res = BaseSubpBody.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_decl_p_body_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BaseSubpBody.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Instantiations of generics (:rmlink:`12.3`).
     */
    public static abstract
    class GenericInstantiation extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericInstantiation";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericInstantiation.class.getMethod(
                        "pDesignatedGenericDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_designated_generic_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericInstantiation.class.getMethod(
                        "pInstParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_inst_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericInstantiation NONE =
            new GenericInstantiationNone();

        // ----- Constructors -----

        protected GenericInstantiation(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericInstantiation fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericInstantiation.NONE :

                (GenericInstantiation) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericInstantiation.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericInstantiation.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericInstantiation.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericInstantiation.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the generic decl entity designated by this instantiation,
         * containing the generic context. This is equivalent to the expanded
         * generic unit in GNAT.
         */
        public BasicDecl pDesignatedGenericDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_instantiation_p_designated_generic_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_instantiation_p_designated_generic_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns an array of pairs, associating formal parameters to actual
         * or default expressions.
         */
        public ParamActualArray pInstParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_generic_instantiation_p_inst_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_generic_instantiation_p_inst_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * GenericInstantiation.
         */
        private static final class GenericInstantiationNone extends GenericInstantiation {
            GenericInstantiationNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Instantiations of a generic package.
     */
    public static 
    class GenericPackageInstantiation extends GenericInstantiation {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericPackageInstantiation";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fGenericPkgName","fParams","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericInstantiation.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageInstantiation.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageInstantiation.class.getMethod(
                        "fGenericPkgName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_generic_pkg_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageInstantiation.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericPackageInstantiation NONE =
            new GenericPackageInstantiation(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericPackageInstantiation(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericPackageInstantiation fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericPackageInstantiation.NONE :

                new GenericPackageInstantiation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericPackageInstantiation.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericPackageInstantiation.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericPackageInstantiation.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericPackageInstantiation.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_instantiation_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_instantiation_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fGenericPkgName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_instantiation_f_generic_pkg_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_instantiation_f_generic_pkg_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ParamAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_instantiation_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_instantiation_f_params(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Instantiations of a generic subprogram .
     */
    public static 
    class GenericSubpInstantiation extends GenericInstantiation {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericSubpInstantiation";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOverriding","fKind","fSubpName","fGenericSubpName","fParams","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericInstantiation.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "fOverriding",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_overriding",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "fKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "fSubpName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "fGenericSubpName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_generic_subp_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpInstantiation.class.getMethod(
                        "pDesignatedSubp",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_designated_subp",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericSubpInstantiation NONE =
            new GenericSubpInstantiation(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericSubpInstantiation(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericSubpInstantiation fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericSubpInstantiation.NONE :

                new GenericSubpInstantiation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericSubpInstantiation.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericSubpInstantiation.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericSubpInstantiation.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericSubpInstantiation.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OverridingNode fOverriding(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_f_overriding(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OverridingNode res = OverridingNode.NONE;
                if(propException == null) {
                    res = OverridingNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_f_overriding(
                    this.entity
                );

                // Wrap and return the result
                return OverridingNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpKind fKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_f_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpKind res = SubpKind.NONE;
                if(propException == null) {
                    res = SubpKind.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_f_kind(
                    this.entity
                );

                // Wrap and return the result
                return SubpKind.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fSubpName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_f_subp_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_f_subp_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fGenericSubpName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_f_generic_subp_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_f_generic_subp_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ParamAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_f_params(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }

        
    

        /**
         * Return the subprogram decl designated by this instantiation.
         */
        public BasicSubpDecl pDesignatedSubp(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_instantiation_p_designated_subp(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicSubpDecl res = BasicSubpDecl.NONE;
                if(propException == null) {
                    res = BasicSubpDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_instantiation_p_designated_subp(
                    this.entity
                );

                // Wrap and return the result
                return BasicSubpDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base node for all generic renaming declarations (:rmlink:`8.5.5`).
     */
    public static abstract
    class GenericRenamingDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericRenamingDecl NONE =
            new GenericRenamingDeclNone();

        // ----- Constructors -----

        protected GenericRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericRenamingDecl.NONE :

                (GenericRenamingDecl) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * GenericRenamingDecl.
         */
        private static final class GenericRenamingDeclNone extends GenericRenamingDecl {
            GenericRenamingDeclNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Declaration for a generic package renaming (:rmlink:`8.5.5`).
     */
    public static 
    class GenericPackageRenamingDecl extends GenericRenamingDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericPackageRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fRenames","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericRenamingDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageRenamingDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericPackageRenamingDecl.class.getMethod(
                        "fRenames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renames",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericPackageRenamingDecl NONE =
            new GenericPackageRenamingDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericPackageRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericPackageRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericPackageRenamingDecl.NONE :

                new GenericPackageRenamingDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericPackageRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericPackageRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericPackageRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericPackageRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_renaming_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_renaming_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fRenames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_package_renaming_decl_f_renames(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_package_renaming_decl_f_renames(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a generic subprogram renaming.
     */
    public static 
    class GenericSubpRenamingDecl extends GenericRenamingDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GenericSubpRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fKind","fName","fRenames","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            GenericRenamingDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpRenamingDecl.class.getMethod(
                        "fKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpRenamingDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = GenericSubpRenamingDecl.class.getMethod(
                        "fRenames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renames",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GenericSubpRenamingDecl NONE =
            new GenericSubpRenamingDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GenericSubpRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static GenericSubpRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GenericSubpRenamingDecl.NONE :

                new GenericSubpRenamingDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GenericSubpRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GenericSubpRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GenericSubpRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GenericSubpRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpKind fKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_renaming_decl_f_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpKind res = SubpKind.NONE;
                if(propException == null) {
                    res = SubpKind.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_renaming_decl_f_kind(
                    this.entity
                );

                // Wrap and return the result
                return SubpKind.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_renaming_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_renaming_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fRenames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_generic_subp_renaming_decl_f_renames(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_generic_subp_renaming_decl_f_renames(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a code label (:rmlink:`5.1`).
     */
    public static 
    class LabelDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LabelDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = LabelDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LabelDecl NONE =
            new LabelDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected LabelDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static LabelDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LabelDecl.NONE :

                new LabelDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LabelDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LabelDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LabelDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LabelDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_label_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_label_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * BasicDecl that is always the declaration inside a named statement.
     */
    public static 
    class NamedStmtDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NamedStmtDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = NamedStmtDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NamedStmtDecl NONE =
            new NamedStmtDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NamedStmtDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static NamedStmtDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NamedStmtDecl.NONE :

                new NamedStmtDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NamedStmtDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NamedStmtDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NamedStmtDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NamedStmtDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_named_stmt_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_named_stmt_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a static constant number (:rmlink:`3.3.2`).
     */
    public static 
    class NumberDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NumberDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = NumberDecl.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = NumberDecl.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NumberDecl NONE =
            new NumberDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NumberDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static NumberDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NumberDecl.NONE :

                new NumberDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NumberDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NumberDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NumberDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NumberDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_number_decl_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_number_decl_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``NullLiteral``,
         * ``NumLiteral``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
         * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
         * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_number_decl_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_number_decl_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for Ada object declarations (:rmlink:`3.3.1`). Ada object
     * declarations are variables/constants declarations that can be declared
     * in any declarative scope.
     */
    public static 
    class ObjectDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ObjectDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fHasAliased","fHasConstant","fMode","fTypeExpr","fDefaultExpr","fRenamingClause","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fIds",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ids",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fHasAliased",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_aliased",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fHasConstant",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_constant",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fMode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_mode",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fDefaultExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_default_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "fRenamingClause",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renaming_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "pPrivatePartDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_private_part_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ObjectDecl.class.getMethod(
                        "pPublicPartDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_public_part_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ObjectDecl NONE =
            new ObjectDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ObjectDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ObjectDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ObjectDecl.NONE :

                new ObjectDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ObjectDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ObjectDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ObjectDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ObjectDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningNameList fIds(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_ids(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningNameList res = DefiningNameList.NONE;
                if(propException == null) {
                    res = DefiningNameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_ids(
                    this.entity
                );

                // Wrap and return the result
                return DefiningNameList.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public AliasedNode fHasAliased(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_has_aliased(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AliasedNode res = AliasedNode.NONE;
                if(propException == null) {
                    res = AliasedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_has_aliased(
                    this.entity
                );

                // Wrap and return the result
                return AliasedNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public ConstantNode fHasConstant(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_has_constant(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ConstantNode res = ConstantNode.NONE;
                if(propException == null) {
                    res = ConstantNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_has_constant(
                    this.entity
                );

                // Wrap and return the result
                return ConstantNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Mode fMode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_mode(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Mode res = Mode.NONE;
                if(propException == null) {
                    res = Mode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_mode(
                    this.entity
                );

                // Wrap and return the result
                return Mode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * This field may be null even when there are no parsing errors.
         */
        public TypeExpr fTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fDefaultExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_default_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_default_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RenamingClause fRenamingClause(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_f_renaming_clause(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RenamingClause res = RenamingClause.NONE;
                if(propException == null) {
                    res = RenamingClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_f_renaming_clause(
                    this.entity
                );

                // Wrap and return the result
                return RenamingClause.fromEntity(res);
            }

        }

        
    

        /**
         * If this object decl is the constant completion of an object decl in
         * the public part, return the object decl from the public part.
         */
        public BasicDecl pPrivatePartDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_p_private_part_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_p_private_part_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * If this object decl is the incomplete declaration of a constant in a
         * public part, return its completion in the private part.
         */
        public BasicDecl pPublicPartDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_object_decl_p_public_part_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_object_decl_p_public_part_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Object declaration that is part of an extended return statement
     * (:rmlink:`6.5`).
     */
    public static 
    class ExtendedReturnStmtObjectDecl extends ObjectDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExtendedReturnStmtObjectDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fHasAliased","fHasConstant","fMode","fTypeExpr","fDefaultExpr","fRenamingClause","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ObjectDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExtendedReturnStmtObjectDecl NONE =
            new ExtendedReturnStmtObjectDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExtendedReturnStmtObjectDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExtendedReturnStmtObjectDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExtendedReturnStmtObjectDecl.NONE :

                new ExtendedReturnStmtObjectDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExtendedReturnStmtObjectDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExtendedReturnStmtObjectDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExtendedReturnStmtObjectDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExtendedReturnStmtObjectDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Object declaration without subtype indication. This node has been
     * introduced to cover a special case for ``ObjectDecl``, where
     * ``type_expr`` is made optional (AI12-0275), and therefore cannot fit in
     * an ``ObjectDecl``.
     */
    public static 
    class NoTypeObjectRenamingDecl extends ObjectDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NoTypeObjectRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIds","fHasAliased","fHasConstant","fMode","fTypeExpr","fDefaultExpr","fRenamingClause","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ObjectDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NoTypeObjectRenamingDecl NONE =
            new NoTypeObjectRenamingDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NoTypeObjectRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static NoTypeObjectRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NoTypeObjectRenamingDecl.NONE :

                new NoTypeObjectRenamingDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NoTypeObjectRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NoTypeObjectRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NoTypeObjectRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NoTypeObjectRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a package renaming (:rmlink:`8.5.3`).
     */
    public static 
    class PackageRenamingDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PackageRenamingDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fRenames","fAspects"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PackageRenamingDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageRenamingDecl.class.getMethod(
                        "fRenames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renames",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageRenamingDecl.class.getMethod(
                        "pRenamedPackage",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_renamed_package",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PackageRenamingDecl.class.getMethod(
                        "pFinalRenamedPackage",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_final_renamed_package",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PackageRenamingDecl NONE =
            new PackageRenamingDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PackageRenamingDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static PackageRenamingDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PackageRenamingDecl.NONE :

                new PackageRenamingDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PackageRenamingDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PackageRenamingDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PackageRenamingDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PackageRenamingDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_renaming_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_renaming_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public RenamingClause fRenames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_renaming_decl_f_renames(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RenamingClause res = RenamingClause.NONE;
                if(propException == null) {
                    res = RenamingClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_renaming_decl_f_renames(
                    this.entity
                );

                // Wrap and return the result
                return RenamingClause.fromEntity(res);
            }

        }

        
    

        /**
         * Return the declaration of the package that is renamed by Self.
         */
        public BasicDecl pRenamedPackage(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_renaming_decl_p_renamed_package(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_renaming_decl_p_renamed_package(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the declaration of the package that is ultimately renamed by
         * Self, skipping through all intermediate package renamings.
         */
        public BasicDecl pFinalRenamedPackage(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_package_renaming_decl_p_final_renamed_package(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_package_renaming_decl_p_final_renamed_package(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a single protected object (:rmlink:`9.4`).
     */
    public static 
    class SingleProtectedDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SingleProtectedDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fAspects","fInterfaces","fDefinition"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SingleProtectedDecl.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SingleProtectedDecl.class.getMethod(
                        "fInterfaces",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interfaces",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SingleProtectedDecl.class.getMethod(
                        "fDefinition",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_definition",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SingleProtectedDecl NONE =
            new SingleProtectedDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SingleProtectedDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SingleProtectedDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SingleProtectedDecl.NONE :

                new SingleProtectedDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SingleProtectedDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SingleProtectedDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SingleProtectedDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SingleProtectedDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DefiningName fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_single_protected_decl_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_single_protected_decl_f_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ParentList fInterfaces(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_single_protected_decl_f_interfaces(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParentList res = ParentList.NONE;
                if(propException == null) {
                    res = ParentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_single_protected_decl_f_interfaces(
                    this.entity
                );

                // Wrap and return the result
                return ParentList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ProtectedDef fDefinition(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_single_protected_decl_f_definition(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ProtectedDef res = ProtectedDef.NONE;
                if(propException == null) {
                    res = ProtectedDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_single_protected_decl_f_definition(
                    this.entity
                );

                // Wrap and return the result
                return ProtectedDef.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declaration for a single task (:rmlink:`9.1`).
     */
    public static 
    class SingleTaskDecl extends BasicDecl {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SingleTaskDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fTaskType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BasicDecl.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SingleTaskDecl.class.getMethod(
                        "fTaskType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_task_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SingleTaskDecl NONE =
            new SingleTaskDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SingleTaskDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static SingleTaskDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SingleTaskDecl.NONE :

                new SingleTaskDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SingleTaskDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SingleTaskDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SingleTaskDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SingleTaskDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SingleTaskTypeDecl fTaskType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_single_task_decl_f_task_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SingleTaskTypeDecl res = SingleTaskTypeDecl.NONE;
                if(propException == null) {
                    res = SingleTaskTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_single_task_decl_f_task_type(
                    this.entity
                );

                // Wrap and return the result
                return SingleTaskTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Alternative in a ``case`` statement (``when ... => ...``).
     */
    public static 
    class CaseStmtAlternative extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseStmtAlternative";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fChoices","fStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CaseStmtAlternative.class.getMethod(
                        "fChoices",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_choices",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CaseStmtAlternative.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseStmtAlternative NONE =
            new CaseStmtAlternative(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseStmtAlternative(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseStmtAlternative fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseStmtAlternative.NONE :

                new CaseStmtAlternative(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseStmtAlternative.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseStmtAlternative.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseStmtAlternative.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseStmtAlternative.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``Allocator``, ``AttributeRef``, ``BaseAggregate``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``ConcatOp``,
         * ``CondExpr``, ``DeclExpr``, ``DiscreteSubtypeIndication``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AlternativesList fChoices(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_stmt_alternative_f_choices(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AlternativesList res = AlternativesList.NONE;
                if(propException == null) {
                    res = AlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_stmt_alternative_f_choices(
                    this.entity
                );

                // Wrap and return the result
                return AlternativesList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_stmt_alternative_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_stmt_alternative_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Root node for all Ada analysis units (:rmlink:`10.1.1`).
     */
    public static 
    class CompilationUnit extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CompilationUnit";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrelude","fBody","fPragmas"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "fPrelude",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prelude",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "fBody",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_body",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "fPragmas",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_pragmas",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pSyntacticFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_syntactic_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pUnitKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_unit_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pWithedUnits",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "includePrivates",
                        true
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_withed_units",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pImportedUnits",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "includePrivates",
                        true
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_imported_units",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pUnitDependencies",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_unit_dependencies",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pIsPreelaborable",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_preelaborable",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pOtherPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_other_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pHasRestriction",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_has_restriction",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pAllConfigPragmas",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_all_config_pragmas",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompilationUnit.class.getMethod(
                        "pConfigPragmas",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_config_pragmas",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CompilationUnit NONE =
            new CompilationUnit(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CompilationUnit(
            final Entity entity
        ) {
            super(entity);
        }

        public static CompilationUnit fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CompilationUnit.NONE :

                new CompilationUnit(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CompilationUnit.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CompilationUnit.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CompilationUnit.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CompilationUnit.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * ``with``, ``use`` or ``pragma`` statements.
         *
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``UseClause``, ``WithClause``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fPrelude(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_compilation_unit_f_prelude(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_compilation_unit_f_prelude(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``LibraryItem``,
         * ``Subunit``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fBody(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_compilation_unit_f_body(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_compilation_unit_f_body(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PragmaNodeList fPragmas(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_compilation_unit_f_pragmas(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PragmaNodeList res = PragmaNodeList.NONE;
                if(propException == null) {
                    res = PragmaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_compilation_unit_f_pragmas(
                    this.entity
                );

                // Wrap and return the result
                return PragmaNodeList.fromEntity(res);
            }

        }

        
    

        /**
         * Return the syntactic fully qualified name of this compilation unit.
         */
        public UnboundedTextTypeArray pSyntacticFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_syntactic_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_compilation_unit_p_syntactic_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the kind corresponding to this analysis unit.
         */
        public AnalysisUnitKind pUnitKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_unit_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AnalysisUnitKind res = AnalysisUnitKind.NONE;
                if(propException == null) {
                    res = AnalysisUnitKind.fromC(resNative.read());

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AnalysisUnitKind res = JNI_LIB.ada_compilation_unit_p_unit_kind(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Look for all "with" clauses at the top of this compilation unit and
         * return all the compilation units designated by them. For the
         * complete dependencies list of compilation units, see the
         * ``unit_dependencies`` property. Units imported with a "private with"
         * are included in this list only if ``include_privates`` is True.
         */
        public AdaNodeArray pWithedUnits(
            final boolean includePrivates
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte includePrivatesNative = (includePrivates ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_withed_units(
                    thisNative,
                    includePrivatesNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_compilation_unit_p_withed_units(
                    includePrivates,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return all the compilation units that are directly imported by this
         * one. This includes "with"ed units as well as the direct parent unit.
         * Units imported with a "private with" are included in this list only
         * if ``include_privates`` is True.
         */
        public AdaNodeArray pImportedUnits(
            final boolean includePrivates
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte includePrivatesNative = (includePrivates ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_imported_units(
                    thisNative,
                    includePrivatesNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_compilation_unit_p_imported_units(
                    includePrivates,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of all the compilation units that are (direct and
         * indirect) dependencies of this one. See the
         * ``withed_units``/``imported_units`` properties to only get the
         * direct dependencies of this unit.
         */
        public AdaNodeArray pUnitDependencies(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_unit_dependencies(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_compilation_unit_p_unit_dependencies(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Get the root basic decl defined in this compilation unit.
         */
        public BasicDecl pDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_compilation_unit_p_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_compilation_unit_p_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Whether this compilation unit is preelaborable or not.
         */
        public boolean pIsPreelaborable(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_is_preelaborable(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_compilation_unit_p_is_preelaborable(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * If this compilation unit is of kind UnitSpecification, return its
         * corresponding body unit, and conversely.
         */
        public CompilationUnit pOtherPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_compilation_unit_p_other_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CompilationUnit res = CompilationUnit.NONE;
                if(propException == null) {
                    res = CompilationUnit.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_compilation_unit_p_other_part(
                    this.entity
                );

                // Wrap and return the result
                return CompilationUnit.fromEntity(res);
            }

        }

        
    

        /**
         * Whether this compilation unit is affected by the restriction with
         * the given name.
         *
         * .. warning:: This property only supports the ``No_Elaboration_Code``
         *    restriction for now.
         */
        public boolean pHasRestriction(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_has_restriction(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_compilation_unit_p_has_restriction(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of configuration pragmas that apply to the current
         * unit.
         */
        public AdaNodeArray pAllConfigPragmas(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_all_config_pragmas(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_compilation_unit_p_all_config_pragmas(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of configuration pragmas with the given name that
         * apply to the current unit.
         */
        public AdaNodeArray pConfigPragmas(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_compilation_unit_p_config_pragmas(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_compilation_unit_p_config_pragmas(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Representation clause for a single component (:rmlink:`13.5.1`).
     */
    public static 
    class ComponentClause extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ComponentClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fId","fPosition","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ComponentClause.class.getMethod(
                        "fId",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentClause.class.getMethod(
                        "fPosition",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_position",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentClause.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ComponentClause NONE =
            new ComponentClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ComponentClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static ComponentClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ComponentClause.NONE :

                new ComponentClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ComponentClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ComponentClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ComponentClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ComponentClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fId(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_clause_f_id(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_clause_f_id(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``NullLiteral``,
         * ``NumLiteral``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
         * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
         * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fPosition(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_clause_f_position(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_clause_f_position(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_clause_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_clause_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Definition for a component (:rmlink:`3.6`).
     */
    public static 
    class ComponentDef extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ComponentDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasAliased","fHasConstant","fTypeExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDef.class.getMethod(
                        "fHasAliased",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_aliased",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDef.class.getMethod(
                        "fHasConstant",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_constant",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ComponentDef.class.getMethod(
                        "fTypeExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ComponentDef NONE =
            new ComponentDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ComponentDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static ComponentDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ComponentDef.NONE :

                new ComponentDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ComponentDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ComponentDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ComponentDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ComponentDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AliasedNode fHasAliased(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_def_f_has_aliased(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AliasedNode res = AliasedNode.NONE;
                if(propException == null) {
                    res = AliasedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_def_f_has_aliased(
                    this.entity
                );

                // Wrap and return the result
                return AliasedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ConstantNode fHasConstant(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_def_f_has_constant(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ConstantNode res = ConstantNode.NONE;
                if(propException == null) {
                    res = ConstantNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_def_f_has_constant(
                    this.entity
                );

                // Wrap and return the result
                return ConstantNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AnonymousType``, ``SubtypeIndication``
         *
         * When there are no parsing errors, this field is never null.
         */
        public TypeExpr fTypeExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_component_def_f_type_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TypeExpr res = TypeExpr.NONE;
                if(propException == null) {
                    res = TypeExpr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_component_def_f_type_expr(
                    this.entity
                );

                // Wrap and return the result
                return TypeExpr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``constant`` keyword.
     */
    public static abstract
    class ConstantNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstantNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ConstantNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstantNode NONE =
            new ConstantNodeNone();

        // ----- Constructors -----

        protected ConstantNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstantNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstantNode.NONE :

                (ConstantNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstantNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstantNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstantNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstantNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of ConstantPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_constant_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_constant_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ConstantNode.
         */
        private static final class ConstantNodeNone extends ConstantNode {
            ConstantNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class ConstantAbsent extends ConstantNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstantAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ConstantNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstantAbsent NONE =
            new ConstantAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConstantAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstantAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstantAbsent.NONE :

                new ConstantAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstantAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstantAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstantAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstantAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ConstantPresent extends ConstantNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstantPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ConstantNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstantPresent NONE =
            new ConstantPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConstantPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstantPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstantPresent.NONE :

                new ConstantPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstantPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstantPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstantPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstantPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for type constraints (:rmlink:`3.2.2`).
     */
    public static abstract
    class Constraint extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Constraint";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Constraint NONE =
            new ConstraintNone();

        // ----- Constructors -----

        protected Constraint(
            final Entity entity
        ) {
            super(entity);
        }

        public static Constraint fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Constraint.NONE :

                (Constraint) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Constraint.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Constraint.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Constraint.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Constraint.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Constraint.
         */
        private static final class ConstraintNone extends Constraint {
            ConstraintNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Constraint for a composite type (:rmlink:`3.6.1`). Due to ambiguities in
     * the Ada grammar, this could be either a list of index constraints, if
     * the owning type is an array type, or a list of discriminant constraints,
     * if the owning type is a discriminated record type.
     */
    public static 
    class CompositeConstraint extends Constraint {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CompositeConstraint";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fConstraints"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Constraint.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CompositeConstraint.class.getMethod(
                        "fConstraints",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_constraints",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompositeConstraint.class.getMethod(
                        "pIsIndexConstraint",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_index_constraint",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CompositeConstraint.class.getMethod(
                        "pIsDiscriminantConstraint",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_discriminant_constraint",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CompositeConstraint NONE =
            new CompositeConstraint(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CompositeConstraint(
            final Entity entity
        ) {
            super(entity);
        }

        public static CompositeConstraint fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CompositeConstraint.NONE :

                new CompositeConstraint(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CompositeConstraint.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CompositeConstraint.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CompositeConstraint.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CompositeConstraint.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CompositeConstraintAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fConstraints(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_composite_constraint_f_constraints(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_composite_constraint_f_constraints(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }

        
    

        /**
         * Whether this composite constraint is an index constraint.
         */
        public boolean pIsIndexConstraint(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_composite_constraint_p_is_index_constraint(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_composite_constraint_p_is_index_constraint(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether this composite constraint is a discriminant constraint.
         */
        public boolean pIsDiscriminantConstraint(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_composite_constraint_p_is_discriminant_constraint(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_composite_constraint_p_is_discriminant_constraint(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Delta and range type constraint (:rmlink:`J.3`).
     */
    public static 
    class DeltaConstraint extends Constraint {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeltaConstraint";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDigits","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Constraint.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DeltaConstraint.class.getMethod(
                        "fDigits",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_digits",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DeltaConstraint.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeltaConstraint NONE =
            new DeltaConstraint(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeltaConstraint(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeltaConstraint fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeltaConstraint.NONE :

                new DeltaConstraint(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeltaConstraint.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeltaConstraint.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeltaConstraint.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeltaConstraint.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``NullLiteral``,
         * ``NumLiteral``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
         * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
         * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fDigits(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_delta_constraint_f_digits(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_delta_constraint_f_digits(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_delta_constraint_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_delta_constraint_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Digits and range type constraint (:rmlink:`3.5.9`).
     */
    public static 
    class DigitsConstraint extends Constraint {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DigitsConstraint";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDigits","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Constraint.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DigitsConstraint.class.getMethod(
                        "fDigits",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_digits",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DigitsConstraint.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DigitsConstraint NONE =
            new DigitsConstraint(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DigitsConstraint(
            final Entity entity
        ) {
            super(entity);
        }

        public static DigitsConstraint fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DigitsConstraint.NONE :

                new DigitsConstraint(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DigitsConstraint.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DigitsConstraint.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DigitsConstraint.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DigitsConstraint.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``NullLiteral``,
         * ``NumLiteral``, ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``,
         * ``RaiseExpr``, ``ReduceAttributeRef``, ``StringLiteral``,
         * ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fDigits(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_digits_constraint_f_digits(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_digits_constraint_f_digits(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_digits_constraint_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_digits_constraint_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Range-based type constraint (:rmlink:`3.5`).
     */
    public static 
    class RangeConstraint extends Constraint {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RangeConstraint";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Constraint.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RangeConstraint.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RangeConstraint NONE =
            new RangeConstraint(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RangeConstraint(
            final Entity entity
        ) {
            super(entity);
        }

        public static RangeConstraint fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RangeConstraint.NONE :

                new RangeConstraint(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RangeConstraint.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RangeConstraint.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RangeConstraint.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RangeConstraint.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_range_constraint_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_range_constraint_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of declarations (:rmlink:`3.11`).
     */
    public static 
    class DeclarativePart extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeclarativePart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DeclarativePart.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeclarativePart NONE =
            new DeclarativePart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeclarativePart(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeclarativePart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeclarativePart.NONE :

                new DeclarativePart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeclarativePart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeclarativePart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeclarativePart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeclarativePart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AbstractSubpDecl``, ``AspectClause``, ``BaseSubpBody``,
         * ``BodyStub``, ``ComponentDecl``, ``ConcreteTypeDecl``,
         * ``EntryBody``, ``EntryDecl``, ``ErrorDecl``, ``ExceptionDecl``,
         * ``GenericDecl``, ``GenericInstantiation``, ``GenericRenamingDecl``,
         * ``IncompleteTypeDecl``, ``NumberDecl``, ``ObjectDecl``,
         * ``PackageBody``, ``PackageDecl``, ``PackageRenamingDecl``,
         * ``PragmaNode``, ``ProtectedBody``, ``ProtectedTypeDecl``,
         * ``SingleProtectedDecl``, ``SingleTaskDecl``, ``SubpDecl``,
         * ``SubtypeDecl``, ``TaskBody``, ``TaskTypeDecl``, ``UseClause``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_declarative_part_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_declarative_part_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of declarations in a private part.
     */
    public static 
    class PrivatePart extends DeclarativePart {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PrivatePart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DeclarativePart.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PrivatePart NONE =
            new PrivatePart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PrivatePart(
            final Entity entity
        ) {
            super(entity);
        }

        public static PrivatePart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PrivatePart.NONE :

                new PrivatePart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PrivatePart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PrivatePart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PrivatePart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PrivatePart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of declarations in a public part.
     */
    public static 
    class PublicPart extends DeclarativePart {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PublicPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DeclarativePart.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PublicPart NONE =
            new PublicPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PublicPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static PublicPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PublicPart.NONE :

                new PublicPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PublicPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PublicPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PublicPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PublicPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * ``elsif`` block, part of an ``if`` expression.
     */
    public static 
    class ElsifExprPart extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ElsifExprPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCondExpr","fThenExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ElsifExprPart.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ElsifExprPart.class.getMethod(
                        "fThenExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_then_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ElsifExprPart NONE =
            new ElsifExprPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ElsifExprPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static ElsifExprPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ElsifExprPart.NONE :

                new ElsifExprPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ElsifExprPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ElsifExprPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ElsifExprPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ElsifExprPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_elsif_expr_part_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_elsif_expr_part_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fThenExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_elsif_expr_part_f_then_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_elsif_expr_part_f_then_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``elsif`` part in an ``if`` statement block.
     */
    public static 
    class ElsifStmtPart extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ElsifStmtPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCondExpr","fStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ElsifStmtPart.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ElsifStmtPart.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ElsifStmtPart NONE =
            new ElsifStmtPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ElsifStmtPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static ElsifStmtPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ElsifStmtPart.NONE :

                new ElsifStmtPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ElsifStmtPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ElsifStmtPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ElsifStmtPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ElsifStmtPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_elsif_stmt_part_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_elsif_stmt_part_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_elsif_stmt_part_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_elsif_stmt_part_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for expressions (:rmlink:`4.4`).
     */
    public static abstract
    class Expr extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Expr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pExpressionType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_expression_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pExpectedExpressionType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_expected_expression_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pIsDynamicallyTagged",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_dynamically_tagged",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pIsDispatchingCall",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_dispatching_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pIsStaticExpr",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_static_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pFirstCorrespondingDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_first_corresponding_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pEvalAsInt",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_eval_as_int",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pEvalAsIntInEnv",
                        new Class[]{SubstitutionArray.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        SubstitutionArray.class,
                        "env"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_eval_as_int_in_env",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pEvalAsString",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_eval_as_string",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pEvalAsStringInEnv",
                        new Class[]{SubstitutionArray.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        SubstitutionArray.class,
                        "env"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_eval_as_string_in_env",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Expr.class.getMethod(
                        "pMatchingNodes",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_matching_nodes",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Expr NONE =
            new ExprNone();

        // ----- Constructors -----

        protected Expr(
            final Entity entity
        ) {
            super(entity);
        }

        public static Expr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Expr.NONE :

                (Expr) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Expr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Expr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Expr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Expr.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the declaration corresponding to the type of this expression
         * after name resolution.
         */
        public BaseTypeDecl pExpressionType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_expr_p_expression_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_expr_p_expression_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the declaration corresponding to the expected type of this
         * expression after name resolution.
         */
        public BaseTypeDecl pExpectedExpressionType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_expr_p_expected_expression_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_expr_p_expected_expression_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns whether this expression is dynamically tagged (See
         * :rmlink:`3.9.2`).
         */
        public boolean pIsDynamicallyTagged(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_is_dynamically_tagged(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_expr_p_is_dynamically_tagged(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns True if this ``Name`` corresponds to a dispatching call,
         * including:
         *
         * * Calls done through subprogram access types.
         *
         * * Calls to dispatching subprograms, in the object-oriented sense.
         *
         * .. note:: This is an experimental feature. There might be some
         *    discrepancy with the GNAT concept of "dispatching call".
         *
         * .. note:: This should only be called on a ``Name`` and ``UnOp`` or a
         *    ``BinOp``.
         *
         * .. attention:: There is a known bug, where the ConcatOp node is not
         *    supported, so calling is_dispatching_call on operators nested
         *    inside of a concat operator will always return false. (Internal
         *    TN: VC08-029)
         */
        public boolean pIsDispatchingCall(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_is_dispatching_call(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_expr_p_is_dispatching_call(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this expression is static according to the ARM
         * definition of static. See :rmlink:`4.9`.
         */
        public boolean pIsStaticExpr(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_is_static_expr(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_expr_p_is_static_expr(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the first decl that is lexically named like self in self's
         * scope.
         */
        public BasicDecl pFirstCorrespondingDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_expr_p_first_corresponding_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_expr_p_first_corresponding_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Statically evaluates self, and returns the value of the evaluation
         * as an integer.
         *
         * .. note:: In order for a call to this not to raise, the expression
         *    needs to be a static expression, as specified in :rmlink:`4.9`.
         *    You can verify whether an expression is static with the
         *    ``is_static_expr`` property.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public BigInteger pEvalAsInt(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_eval_as_int(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BigInteger res = BigIntegerWrapper.NONE;
                if(propException == null) {
                    res = BigIntegerWrapper.wrap(resNative);

                    BigIntegerWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final BigInteger res = JNI_LIB.ada_expr_p_eval_as_int(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Statically evaluates self, and returns the value of the evaluation
         * as an integer. The given environment is used to substitute
         * references to declarations by actual values.
         *
         * .. note:: In order for a call to this not to raise, the expression
         *    needs to be a static expression, as specified in :rmlink:`4.9`.
         *    You can verify whether an expression is static with the
         *    ``is_static_expr`` property.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public BigInteger pEvalAsIntInEnv(
            final SubstitutionArray env
        ) {

            // Verify that arguments are not null
            if(env == null) throw new IllegalArgumentException(
                "Argument 'env' of type " +
                "SubstitutionArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                SubstitutionArrayNative envNative = env.unwrap();

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_eval_as_int_in_env(
                    thisNative,
                    envNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BigInteger res = BigIntegerWrapper.NONE;
                if(propException == null) {
                    res = BigIntegerWrapper.wrap(resNative);

                    BigIntegerWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                SubstitutionArray.release(
                    envNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final BigInteger res = JNI_LIB.ada_expr_p_eval_as_int_in_env(
                    env,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Statically evaluates self, and returns the value of the evaluation
         * as a string.
         *
         * .. note:: In order for a call to this not to raise, the expression
         *    needs to be a static expression, as specified in :rmlink:`4.9`.
         *    You can verify whether an expression is static with the
         *    ``is_static_expr`` property.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public String pEvalAsString(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_eval_as_string(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_expr_p_eval_as_string(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Statically evaluates self, and returns the value of the evaluation
         * as a string. The given environment is used to substitute references
         * to declarations by actual values.
         *
         * .. note:: In order for a call to this not to raise, the expression
         *    needs to be a static expression, as specified in :rmlink:`4.9`.
         *    You can verify whether an expression is static with the
         *    ``is_static_expr`` property.
         *
         * .. attention:: This is an experimental feature, so even if it is
         *    exposed to allow experiments, it is totally unsupported and the
         *    API and behavior are very likely to change in the future.
         */
        public String pEvalAsStringInEnv(
            final SubstitutionArray env
        ) {

            // Verify that arguments are not null
            if(env == null) throw new IllegalArgumentException(
                "Argument 'env' of type " +
                "SubstitutionArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                SubstitutionArrayNative envNative = env.unwrap();

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_eval_as_string_in_env(
                    thisNative,
                    envNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                SubstitutionArray.release(
                    envNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_expr_p_eval_as_string_in_env(
                    env,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of AST nodes that can be a match for this expression
         * before overloading analysis.
         */
        public AdaNodeArray pMatchingNodes(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_expr_p_matching_nodes(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_expr_p_matching_nodes(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Expr.
         */
        private static final class ExprNone extends Expr {
            ExprNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Directly corresponds to the right-hand side of the Abstract_State
     * aspect. Only exists because the RHS of an AspectAssoc must be an
     * expression: the actual logic is in AbstractStateDecl.
     */
    public static 
    class AbstractStateDeclExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbstractStateDeclExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fStateDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AbstractStateDeclExpr.class.getMethod(
                        "fStateDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_state_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbstractStateDeclExpr NONE =
            new AbstractStateDeclExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbstractStateDeclExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbstractStateDeclExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbstractStateDeclExpr.NONE :

                new AbstractStateDeclExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbstractStateDeclExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbstractStateDeclExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbstractStateDeclExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbstractStateDeclExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AbstractStateDecl``, ``MultiAbstractStateDecl``,
         * ``ParenAbstractStateDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fStateDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_abstract_state_decl_expr_f_state_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_abstract_state_decl_expr_f_state_decl(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Allocator expression (``new ...``) (:rmlink:`4.8`).
     */
    public static 
    class Allocator extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Allocator";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSubpool","fTypeOrExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Allocator.class.getMethod(
                        "fSubpool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subpool",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Allocator.class.getMethod(
                        "fTypeOrExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_or_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Allocator.class.getMethod(
                        "pGetAllocatedType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_allocated_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Allocator NONE =
            new Allocator(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Allocator(
            final Entity entity
        ) {
            super(entity);
        }

        public static Allocator fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Allocator.NONE :

                new Allocator(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Allocator.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Allocator.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Allocator.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Allocator.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fSubpool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_allocator_f_subpool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_allocator_f_subpool(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``QualExpr``,
         * ``SubtypeIndication``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fTypeOrExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_allocator_f_type_or_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_allocator_f_type_or_expr(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the allocated type for this allocator.
         */
        public BaseTypeDecl pGetAllocatedType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_allocator_p_get_allocated_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_allocator_p_get_allocated_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for aggregates (:rmlink:`4.3`).
     */
    public static abstract
    class BaseAggregate extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseAggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseAggregate.class.getMethod(
                        "fAncestorExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_ancestor_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseAggregate.class.getMethod(
                        "fAssocs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_assocs",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseAggregate.class.getMethod(
                        "pAggregateParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_aggregate_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseAggregate.class.getMethod(
                        "pIsSubaggregate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_subaggregate",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseAggregate NONE =
            new BaseAggregateNone();

        // ----- Constructors -----

        protected BaseAggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseAggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseAggregate.NONE :

                (BaseAggregate) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseAggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseAggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseAggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseAggregate.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fAncestorExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_aggregate_f_ancestor_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_aggregate_f_ancestor_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AggregateAssoc``, ``IteratedAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fAssocs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_aggregate_f_assocs(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_aggregate_f_assocs(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }

        
    

        /**
         * Returns an array of pairs, associating formal parameters to actual
         * expressions. See ``zip_with_params``.
         */
        public ParamActualArray pAggregateParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_aggregate_p_aggregate_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_base_aggregate_p_aggregate_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this aggregate is actually a subaggregate of a
         * multidimensional array aggregate, as described in :rmlink:`4.3.3`.
         */
        public boolean pIsSubaggregate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_base_aggregate_p_is_subaggregate(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_base_aggregate_p_is_subaggregate(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseAggregate.
         */
        private static final class BaseAggregateNone extends BaseAggregate {
            BaseAggregateNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Aggregate that is not a ``null record`` aggregate (:rmlink:`4.3`).
     */
    public static 
    class Aggregate extends BaseAggregate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Aggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseAggregate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Aggregate NONE =
            new Aggregate(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Aggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static Aggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Aggregate.NONE :

                new Aggregate(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Aggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Aggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Aggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Aggregate.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Bracket array or container aggregate (Ada 2020, :rmlink:`4.3`).
     */
    public static 
    class BracketAggregate extends Aggregate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BracketAggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Aggregate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BracketAggregate NONE =
            new BracketAggregate(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BracketAggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static BracketAggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BracketAggregate.NONE :

                new BracketAggregate(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BracketAggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BracketAggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BracketAggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BracketAggregate.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Aggregate for delta aggregate (Ada 2022, :rmlink:`4.3`).
     */
    public static 
    class DeltaAggregate extends BaseAggregate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeltaAggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseAggregate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeltaAggregate NONE =
            new DeltaAggregate(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeltaAggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeltaAggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeltaAggregate.NONE :

                new DeltaAggregate(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeltaAggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeltaAggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeltaAggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeltaAggregate.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Bracket delta aggregate (Ada 2020, :rmlink:`4.3`).
     */
    public static 
    class BracketDeltaAggregate extends DeltaAggregate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BracketDeltaAggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DeltaAggregate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BracketDeltaAggregate NONE =
            new BracketDeltaAggregate(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BracketDeltaAggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static BracketDeltaAggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BracketDeltaAggregate.NONE :

                new BracketDeltaAggregate(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BracketDeltaAggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BracketDeltaAggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BracketDeltaAggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BracketDeltaAggregate.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Aggregate for ``null record`` (:rmlink:`4.3`).
     */
    public static 
    class NullRecordAggregate extends BaseAggregate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullRecordAggregate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fAncestorExpr","fAssocs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseAggregate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullRecordAggregate NONE =
            new NullRecordAggregate(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullRecordAggregate(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullRecordAggregate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullRecordAggregate.NONE :

                new NullRecordAggregate(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullRecordAggregate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullRecordAggregate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullRecordAggregate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullRecordAggregate.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Binary expression.
     *
     * This encompasses several ARM expressions, because it is used for every
     * binary expression in Ada, all documented in ::rmlink:`4.4`.
     */
    public static 
    class BinOp extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BinOp";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fLeft","fOp","fRight"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BinOp.class.getMethod(
                        "fLeft",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_left",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BinOp.class.getMethod(
                        "fOp",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_op",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BinOp.class.getMethod(
                        "fRight",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_right",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BinOp NONE =
            new BinOp(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BinOp(
            final Entity entity
        ) {
            super(entity);
        }

        public static BinOp fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BinOp.NONE :

                new BinOp(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BinOp.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BinOp.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BinOp.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BinOp.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fLeft(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_bin_op_f_left(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_bin_op_f_left(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``OpAndThen``,
         * ``OpAnd``, ``OpDiv``, ``OpDoubleDot``, ``OpEq``, ``OpGt``,
         * ``OpGte``, ``OpLt``, ``OpLte``, ``OpMinus``, ``OpMod``, ``OpMult``,
         * ``OpNeq``, ``OpOrElse``, ``OpOr``, ``OpPlus``, ``OpPow``, ``OpRem``,
         * ``OpXor``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Op fOp(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_bin_op_f_op(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Op res = Op.NONE;
                if(propException == null) {
                    res = Op.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_bin_op_f_op(
                    this.entity
                );

                // Wrap and return the result
                return Op.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fRight(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_bin_op_f_right(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_bin_op_f_right(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Binary operation that compares two value, producing a boolean
     * (:rmlink:`4.4`).
     */
    public static 
    class RelationOp extends BinOp {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RelationOp";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fLeft","fOp","fRight"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BinOp.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RelationOp NONE =
            new RelationOp(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RelationOp(
            final Entity entity
        ) {
            super(entity);
        }

        public static RelationOp fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RelationOp.NONE :

                new RelationOp(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RelationOp.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RelationOp.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RelationOp.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RelationOp.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Box expression (``<>``).
     *
     * This is not an expression per-se in Ada, but treating it as one helps us
     * keep coherent types in some cases, like aggregates expressions.
     */
    public static 
    class BoxExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BoxExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BoxExpr NONE =
            new BoxExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BoxExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static BoxExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BoxExpr.NONE :

                new BoxExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BoxExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BoxExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BoxExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BoxExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Alternative in a ``case`` expression (``when ... => ...``).
     */
    public static 
    class CaseExprAlternative extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseExprAlternative";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fChoices","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CaseExprAlternative.class.getMethod(
                        "fChoices",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_choices",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CaseExprAlternative.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseExprAlternative NONE =
            new CaseExprAlternative(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseExprAlternative(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseExprAlternative fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseExprAlternative.NONE :

                new CaseExprAlternative(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseExprAlternative.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseExprAlternative.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseExprAlternative.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseExprAlternative.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``Allocator``, ``AttributeRef``, ``BaseAggregate``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``ConcatOp``,
         * ``CondExpr``, ``DeclExpr``, ``DiscreteSubtypeIndication``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AlternativesList fChoices(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_expr_alternative_f_choices(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AlternativesList res = AlternativesList.NONE;
                if(propException == null) {
                    res = AlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_expr_alternative_f_choices(
                    this.entity
                );

                // Wrap and return the result
                return AlternativesList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_expr_alternative_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_expr_alternative_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Concatenation expression.
     *
     * Since concatenation expression can be huge in practice, this node
     * handles them as a list of operands rather than a deep tree of binary
     * operators, in order to avoid crashes while parsing of running name
     * resolution on such huge expression.
     *
     * The purpose of this node is to replace the arbitrarily too deep tree of
     * binary operators (which can lead to a stack overflow), as for example
     * with ``"A & B & C & D & E"``:
     *
     * .. code::
     *
     *    BinOp(
     *      Binop(
     *        BinOp(
     *          BinOp(A, &, B), & , C), &, D), &, E)
     *
     * by a single operator, handling a list of operands that can be processed
     * without having to perform deep recursions:
     *
     * .. code::
     *
     *    ConcatOp(A,
     *      ConcatOperand(&, B),
     *      ConcatOperand(&, C),
     *      ConcatOperand(&, D),
     *      ConcatOperand(&, E))
     */
    public static 
    class ConcatOp extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConcatOp";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fFirstOperand","fOtherOperands"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ConcatOp.class.getMethod(
                        "fFirstOperand",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_first_operand",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ConcatOp.class.getMethod(
                        "fOtherOperands",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_other_operands",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ConcatOp.class.getMethod(
                        "pOperands",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_operands",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConcatOp NONE =
            new ConcatOp(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConcatOp(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConcatOp fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConcatOp.NONE :

                new ConcatOp(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConcatOp.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConcatOp.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConcatOp.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConcatOp.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``CondExpr``, ``DeclExpr``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``NullLiteral``, ``NumLiteral``,
         * ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fFirstOperand(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_concat_op_f_first_operand(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_concat_op_f_first_operand(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ConcatOperandList fOtherOperands(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_concat_op_f_other_operands(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ConcatOperandList res = ConcatOperandList.NONE;
                if(propException == null) {
                    res = ConcatOperandList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_concat_op_f_other_operands(
                    this.entity
                );

                // Wrap and return the result
                return ConcatOperandList.fromEntity(res);
            }

        }

        
    

        /**
         * Return the operands of this concatenation expression
         */
        public AdaNodeArray pOperands(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_concat_op_p_operands(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_concat_op_p_operands(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * A concatenation operator and its RHS operand.
     *
     * This node is used to represent the tuple ("&", operand) used by the
     * ``ConcatOp`` node to store its ``other_operands`` list.
     */
    public static 
    class ConcatOperand extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConcatOperand";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOperator","fOperand"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ConcatOperand.class.getMethod(
                        "fOperator",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_operator",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ConcatOperand.class.getMethod(
                        "fOperand",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_operand",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConcatOperand NONE =
            new ConcatOperand(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConcatOperand(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConcatOperand fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConcatOperand.NONE :

                new ConcatOperand(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConcatOperand.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConcatOperand.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConcatOperand.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConcatOperand.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public OpConcat fOperator(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_concat_operand_f_operator(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                OpConcat res = OpConcat.NONE;
                if(propException == null) {
                    res = OpConcat.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_concat_operand_f_operator(
                    this.entity
                );

                // Wrap and return the result
                return OpConcat.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``CondExpr``, ``DeclExpr``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``NullLiteral``, ``NumLiteral``,
         * ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fOperand(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_concat_operand_f_operand(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_concat_operand_f_operand(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for a conditional expressions (:rmlink:`4.5.7`).
     */
    public static abstract
    class CondExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CondExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CondExpr.class.getMethod(
                        "pDependentExprs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_dependent_exprs",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CondExpr NONE =
            new CondExprNone();

        // ----- Constructors -----

        protected CondExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static CondExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CondExpr.NONE :

                (CondExpr) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CondExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CondExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CondExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CondExpr.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the dependent expressions for this conditional expression.
         */
        public AdaNodeArray pDependentExprs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_cond_expr_p_dependent_exprs(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_cond_expr_p_dependent_exprs(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * CondExpr.
         */
        private static final class CondExprNone extends CondExpr {
            CondExprNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * ``case`` expression (:rmlink:`4.5.7`).
     */
    public static 
    class CaseExpr extends CondExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr","fCases"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CondExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CaseExpr.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CaseExpr.class.getMethod(
                        "fCases",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cases",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseExpr NONE =
            new CaseExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseExpr.NONE :

                new CaseExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_expr_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_expr_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public CaseExprAlternativeList fCases(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_expr_f_cases(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CaseExprAlternativeList res = CaseExprAlternativeList.NONE;
                if(propException == null) {
                    res = CaseExprAlternativeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_expr_f_cases(
                    this.entity
                );

                // Wrap and return the result
                return CaseExprAlternativeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``if`` expression (:rmlink`4.5.7`).
     */
    public static 
    class IfExpr extends CondExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IfExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCondExpr","fThenExpr","fAlternatives","fElseExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CondExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IfExpr.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfExpr.class.getMethod(
                        "fThenExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_then_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfExpr.class.getMethod(
                        "fAlternatives",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_alternatives",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfExpr.class.getMethod(
                        "fElseExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_else_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IfExpr NONE =
            new IfExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IfExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static IfExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IfExpr.NONE :

                new IfExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IfExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IfExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IfExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IfExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_expr_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_expr_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fThenExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_expr_f_then_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_expr_f_then_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ElsifExprPartList fAlternatives(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_expr_f_alternatives(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ElsifExprPartList res = ElsifExprPartList.NONE;
                if(propException == null) {
                    res = ElsifExprPartList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_expr_f_alternatives(
                    this.entity
                );

                // Wrap and return the result
                return ElsifExprPartList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fElseExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_expr_f_else_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_expr_f_else_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of associations for the ``Contract_Case`` aspect.
     *
     * Contract cases is a non standard Ada extension that's mainly useful in
     * SPARK. See the SPARK RM for more details.
     */
    public static 
    class ContractCases extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ContractCases";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fContractCases"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ContractCases.class.getMethod(
                        "fContractCases",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_contract_cases",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ContractCases NONE =
            new ContractCases(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ContractCases(
            final Entity entity
        ) {
            super(entity);
        }

        public static ContractCases fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ContractCases.NONE :

                new ContractCases(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ContractCases.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ContractCases.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ContractCases.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ContractCases.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ContractCaseAssocList fContractCases(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_contract_cases_f_contract_cases(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ContractCaseAssocList res = ContractCaseAssocList.NONE;
                if(propException == null) {
                    res = ContractCaseAssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_contract_cases_f_contract_cases(
                    this.entity
                );

                // Wrap and return the result
                return ContractCaseAssocList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Declare expression (Ada 2020, :rmlink:`4.5.9`).
     */
    public static 
    class DeclExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeclExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DeclExpr.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DeclExpr.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeclExpr NONE =
            new DeclExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeclExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeclExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeclExpr.NONE :

                new DeclExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeclExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeclExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeclExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeclExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``NumberDecl``, ``ObjectDecl``, ``SingleProtectedDecl``,
         * ``SingleTaskDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BasicDeclList fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decl_expr_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDeclList res = BasicDeclList.NONE;
                if(propException == null) {
                    res = BasicDeclList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decl_expr_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return BasicDeclList.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decl_expr_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decl_expr_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Represent a membership test (in/not in operators) (:rmlink:`4.4`).
     *
     * Note that we don't consider them as binary operators since multiple
     * expressions on the right hand side are allowed.
     */
    public static 
    class MembershipExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "MembershipExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr","fOp","fMembershipExprs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = MembershipExpr.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = MembershipExpr.class.getMethod(
                        "fOp",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_op",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = MembershipExpr.class.getMethod(
                        "fMembershipExprs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_membership_exprs",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final MembershipExpr NONE =
            new MembershipExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected MembershipExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static MembershipExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                MembershipExpr.NONE :

                new MembershipExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return MembershipExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return MembershipExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return MembershipExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return MembershipExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_membership_expr_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_membership_expr_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``OpIn``,
         * ``OpNotIn``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Op fOp(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_membership_expr_f_op(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Op res = Op.NONE;
                if(propException == null) {
                    res = Op.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_membership_expr_f_op(
                    this.entity
                );

                // Wrap and return the result
                return Op.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``Allocator``, ``AttributeRef``, ``BaseAggregate``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``ConcatOp``,
         * ``CondExpr``, ``DeclExpr``, ``DiscreteSubtypeName``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``NullLiteral``, ``NumLiteral``,
         * ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ExprAlternativesList fMembershipExprs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_membership_expr_f_membership_exprs(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ExprAlternativesList res = ExprAlternativesList.NONE;
                if(propException == null) {
                    res = ExprAlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_membership_expr_f_membership_exprs(
                    this.entity
                );

                // Wrap and return the result
                return ExprAlternativesList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for names (:rmlink:`4.1`).
     */
    public static abstract
    class Name extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Name";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pEnclosingDefiningName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_enclosing_defining_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsDefining",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_defining",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pNameIs",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "sym"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_name_is",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsDirectCall",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_direct_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsAccessCall",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_access_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsCall",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsDotCall",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_dot_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pFailsafeReferencedDefName",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_failsafe_referenced_def_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pReferencedDefiningName",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_referenced_defining_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pAllEnvElements",
                        new Class[]{boolean.class,AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "seq",
                        true
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "seqFrom",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_all_env_elements",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pCalledSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_called_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pReferencedDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_referenced_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pFailsafeReferencedDecl",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_failsafe_referenced_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pNameDesignatedType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_name_designated_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsStaticSubtype",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_static_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pNameMatches",
                        new Class[]{Name.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Name.class,
                        "n"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_name_matches",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pRelativeName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_relative_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsOperatorName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_operator_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsWriteReference",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_write_reference",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsStaticCall",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_static_call",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pAsSymbolArray",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_symbol_array",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pCanonicalText",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_text",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pIsConstant",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_constant",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Name.class.getMethod(
                        "pCallParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_call_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Name NONE =
            new NameNone();

        // ----- Constructors -----

        protected Name(
            final Entity entity
        ) {
            super(entity);
        }

        public static Name fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Name.NONE :

                (Name) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Name.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Name.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Name.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Name.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * If this name is part of a defining name, return the enclosing
         * defining name node.
         */
        public DefiningName pEnclosingDefiningName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_enclosing_defining_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_enclosing_defining_name(
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return True if this name is part of a defining name.
         */
        public boolean pIsDefining(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_defining(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_defining(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Helper. Check that this name matches ``sym``.
         */
        public boolean pNameIs(
            final Symbol sym
        ) {

            // Verify that arguments are not null
            if(sym == null) throw new IllegalArgumentException(
                "Argument 'sym' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative symNative = StackValue.get(SymbolNative.class);sym.unwrap(symNative, currentContext);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_name_is(
                    thisNative,
                    symNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_name_is(
                    sym,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return True iff this name represents a call to a subprogram which is
         * referred by its defining name. (i.e. not through a subprogram
         * access).
         */
        public boolean pIsDirectCall(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_direct_call(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_direct_call(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return True iff this name represents a call to subprogram through an
         * access type.
         */
        public boolean pIsAccessCall(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_access_call(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_access_call(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns True if this Name corresponds to a call.
         */
        public boolean pIsCall(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_call(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_call(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns True if this Name corresponds to a dot notation call.
         */
        public boolean pIsDotCall(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_dot_call(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_dot_call(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Failsafe version of ``referenced_defining_name``. Returns a
         * ``RefdDef``, which can be precise, imprecise, or error.
         */
        public RefdDef pFailsafeReferencedDefName(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final RefdDefNative resNative =
                    StackValue.get(RefdDefNative.class);

                // Call the native function
                NI_LIB.ada_name_p_failsafe_referenced_def_name(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RefdDef res = RefdDef.NONE;
                if(propException == null) {
                    res = RefdDef.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final RefdDef res = JNI_LIB.ada_name_p_failsafe_referenced_def_name(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Like ``referenced_decl``, but will return the defining identifier
         * for the decl, rather than the basic declaration node itself.
         */
        public DefiningName pReferencedDefiningName(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_referenced_defining_name(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_referenced_defining_name(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return all elements in self's scope that are lexically named like
         * Self.
         */
        public AdaNodeArray pAllEnvElements(
            final boolean seq,final AdaNode seqFrom
        ) {

            // Verify that arguments are not null
            if(seqFrom == null) throw new IllegalArgumentException(
                "Argument 'seqFrom' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte seqNative = (seq ? (byte) 1 : (byte) 0);
                EntityNative seqFromNative = StackValue.get(EntityNative.class);seqFrom.entity.unwrap(seqFromNative);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_all_env_elements(
                    thisNative,
                    seqNative,
                    seqFromNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_name_p_all_env_elements(
                    seq,
                    (seqFrom != null ? seqFrom.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the subprogram specification of the subprogram or subprogram
         * access that is being called by this exact Name, if relevant.
         */
        public BaseFormalParamHolder pCalledSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_called_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseFormalParamHolder res = BaseFormalParamHolder.NONE;
                if(propException == null) {
                    res = BaseFormalParamHolder.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_called_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return BaseFormalParamHolder.fromEntity(res);
            }

        }

        
    

        /**
         * Return the declaration this node references after name resolution.
         * If imprecise_fallback is True, errors raised during resolution of
         * the xref equation are caught and a fallback mechanism is triggered,
         * which tries to find the referenced declaration in an ad-hoc way.
         */
        public BasicDecl pReferencedDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_referenced_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_referenced_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Failsafe version of ``referenced_decl``. Returns a ``RefdDecl``,
         * which can be precise, imprecise, or error.
         */
        public RefdDecl pFailsafeReferencedDecl(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final RefdDeclNative resNative =
                    StackValue.get(RefdDeclNative.class);

                // Call the native function
                NI_LIB.ada_name_p_failsafe_referenced_decl(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RefdDecl res = RefdDecl.NONE;
                if(propException == null) {
                    res = RefdDecl.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final RefdDecl res = JNI_LIB.ada_name_p_failsafe_referenced_decl(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Like SubtypeIndication.designated_type, but on names, since because
         * of Ada's ambiguous grammar, some subtype indications will be parsed
         * as names.
         */
        public BaseTypeDecl pNameDesignatedType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_name_designated_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_name_designated_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Returns whether Self denotes a static subtype or not.
         */
        public boolean pIsStaticSubtype(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_static_subtype(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_static_subtype(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether two names match each other.
         *
         * This compares the symbol for Identifier and StringLiteral nodes. We
         * consider that there is no match for all other node kinds.
         */
        public boolean pNameMatches(
            final Name n
        ) {

            // Verify that arguments are not null
            if(n == null) throw new IllegalArgumentException(
                "Argument 'n' of type " +
                "Name cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative nNative = StackValue.get(EntityNative.class);n.entity.unwrap(nNative);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_name_matches(
                    thisNative,
                    nNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_name_matches(
                    (n != null ? n.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns the relative name of this instance. For example, for a
         * prefix ``A.B.C``, this will return ``C``.
         */
        public Name pRelativeName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_name_p_relative_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_name_p_relative_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * Return whether the name that Self designates is an operator.
         */
        public boolean pIsOperatorName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_operator_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_operator_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Whether this name is a write reference.
         *
         * For example, ``X`` is a write reference in the following cases:
         *
         * 1. ``X := 2;``
         *
         * 2. ``X (2) := 2;``
         *
         * 3. ``P(F => X)`` where F is declared ``out`` or ``in out``.
         *
         * 4. ``P(F => T (X))`` where F is declared ``out`` or ``in out``
         *
         * 5. ``X'Access``.
         *
         * 6. ``X.C := 2``, ``R.X := 2``
         *
         * 7. ``X.P`` where the formal for X is declared ``out`` or ``in out``.
         *
         * .. note:: This is an experimental feature. There might be some
         *    discrepancy with the GNAT concept of "write reference".
         */
        public boolean pIsWriteReference(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_write_reference(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_write_reference(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns True if this Name corresponds to a static non-dispatching
         * call. In other words, this will return True if and only if the
         * target of the call is known statically.
         *
         * .. note:: This is an experimental feature. There might be some
         *    discrepancy with the GNAT concept of "static call".
         */
        public boolean pIsStaticCall(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_static_call(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_static_call(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Turn this name into an array of symbols.
         *
         * For instance, a node with name ``A.B.C`` is turned into ``['A', 'B',
         * 'C']``.
         *
         * Only simple name kinds are allowed: Identifier, DottedName and
         * DefiningName. Any other kind will trigger a PropertyError.
         */
        public UnboundedTextTypeArray pAsSymbolArray(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_as_symbol_array(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_name_p_as_symbol_array(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a canonicalized version of this name's text.
         *
         * Only simple name kinds are allowed: Identifier, DottedName and
         * DefiningName. Any other kind will trigger a PropertyError.
         */
        public Symbol pCanonicalText(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final SymbolNative resNative =
                    StackValue.get(SymbolNative.class);

                // Call the native function
                NI_LIB.ada_name_p_canonical_text(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Symbol res = Symbol.NONE;
                if(propException == null) {
                    res = Symbol.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Symbol res = JNI_LIB.ada_name_p_canonical_text(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this name denotes a constant value.
         */
        public boolean pIsConstant(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_is_constant(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_name_p_is_constant(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns an array of pairs, associating formal parameters to actual
         * or default expressions.
         */
        public ParamActualArray pCallParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_name_p_call_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_name_p_call_params(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Name.
         */
        private static final class NameNone extends Name {
            NameNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Expression to reference an attribute (:rmlink:`4.1.4`).
     */
    public static 
    class AttributeRef extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AttributeRef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix","fAttribute","fArgs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AttributeRef.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AttributeRef.class.getMethod(
                        "fAttribute",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_attribute",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AttributeRef.class.getMethod(
                        "fArgs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_args",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AttributeRef NONE =
            new AttributeRef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AttributeRef(
            final Entity entity
        ) {
            super(entity);
        }

        public static AttributeRef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AttributeRef.NONE :

                new AttributeRef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AttributeRef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AttributeRef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AttributeRef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AttributeRef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_attribute_ref_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_attribute_ref_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fAttribute(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_attribute_ref_f_attribute(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_attribute_ref_f_attribute(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ParamAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fArgs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_attribute_ref_f_args(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_attribute_ref_f_args(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Represent a syntactic call expression.
     *
     * At the semantic level, this can be either a subprogram call, an array
     * subcomponent access expression, an array slice or a type conversion, all
     * described in :rmlink:`4.1`, except for subprogram call statements,
     * described in :rmlink:`6.4`.
     */
    public static 
    class CallExpr extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CallExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fSuffix"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CallExpr.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CallExpr.class.getMethod(
                        "fSuffix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_suffix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CallExpr.class.getMethod(
                        "pKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CallExpr.class.getMethod(
                        "pIsArraySlice",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_array_slice",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CallExpr NONE =
            new CallExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CallExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static CallExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CallExpr.NONE :

                new CallExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CallExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CallExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CallExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CallExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_call_expr_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_call_expr_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BasicAssocList``, ``BinOp``, ``CallExpr``, ``CharLiteral``,
         * ``DiscreteSubtypeIndication``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fSuffix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_call_expr_f_suffix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_call_expr_f_suffix(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return whether this expression is a subprogram call, an array
         * subcomponent access expression, an array slice or a type conversion.
         */
        public CallExprKind pKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_call_expr_p_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CallExprKind res = CallExprKind.NONE;
                if(propException == null) {
                    res = CallExprKind.fromC(resNative.read());

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final CallExprKind res = JNI_LIB.ada_call_expr_p_kind(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether this CallExpr is actually an access to a slice of the
         * array denoted by the prefix of this CallExpr.
         */
        public boolean pIsArraySlice(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_call_expr_p_is_array_slice(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_call_expr_p_is_array_slice(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Name that defines an entity (:rmlink:`3.1`).
     */
    public static 
    class DefiningName extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DefiningName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pCanonicalFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pUniqueIdentifyingName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_unique_identifying_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pFullyQualifiedNameArray",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_fully_qualified_name_array",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pFullyQualifiedName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_fully_qualified_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pBasicDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_basic_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pFindRefs",
                        new Class[]{AdaNode.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "root"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_refs",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pFindAllReferences",
                        new Class[]{AnalysisUnitArray.class,boolean.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnitArray.class,
                        "units"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "followRenamings",
                        false
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_all_references",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pFindAllCalls",
                        new Class[]{AnalysisUnitArray.class,boolean.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AnalysisUnitArray.class,
                        "units"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "followRenamings",
                        false
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_find_all_calls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pNextPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_next_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pPreviousPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_previous_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pCanonicalPart",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_canonical_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pMostVisiblePart",
                        new Class[]{AdaNode.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "origin"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_most_visible_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pAllParts",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_all_parts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pGetAspect",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_aspect",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pHasAspect",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_has_aspect",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pGetPragma",
                        new Class[]{Symbol.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_pragma",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pGetRepresentationClause",
                        new Class[]{Symbol.class,boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        Symbol.class,
                        "name"
                    ));
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_representation_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pGetAtClause",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_get_at_clause",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pIsImported",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_imported",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DefiningName.class.getMethod(
                        "pIsGhostCode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_ghost_code",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DefiningName NONE =
            new DefiningName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DefiningName(
            final Entity entity
        ) {
            super(entity);
        }

        public static DefiningName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DefiningName.NONE :

                new DefiningName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DefiningName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DefiningName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DefiningName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DefiningName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``,
         * ``SyntheticIdentifier``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * Return a canonical representation of the fully qualified name
         * corresponding to this defining name.
         */
        public String pCanonicalFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_canonical_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_defining_name_p_canonical_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return a unique identifying name for this defining name, provided
         * this declaration is a public declaration. In the case of
         * subprograms, this will include the profile.
         *
         * .. attention:: This will only return a unique name for public
         *    declarations. Notably, anything nested in an unnamed declare
         *    block won't be handled correctly.
         */
        public String pUniqueIdentifyingName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_unique_identifying_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_defining_name_p_unique_identifying_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the fully qualified name corresponding to this defining name,
         * as an array of symbols.
         */
        public UnboundedTextTypeArray pFullyQualifiedNameArray(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_fully_qualified_name_array(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UnboundedTextTypeArray res = UnboundedTextTypeArray.NONE;
                if(propException == null) {
                    res = UnboundedTextTypeArray.wrap(resNative);

                    UnboundedTextTypeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final UnboundedTextTypeArray res = JNI_LIB.ada_defining_name_p_fully_qualified_name_array(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the fully qualified name corresponding to this defining name.
         */
        public String pFullyQualifiedName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_fully_qualified_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_defining_name_p_fully_qualified_name(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns this DefiningName's basic declaration
         */
        public BasicDecl pBasicDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_basic_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_basic_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Find all references to this defining name in the given ``root`` and
         * its children.
         */
        public RefResultArray pFindRefs(
            final AdaNode root,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(root == null) throw new IllegalArgumentException(
                "Argument 'root' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative rootNative = StackValue.get(EntityNative.class);root.entity.unwrap(rootNative);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_find_refs(
                    thisNative,
                    rootNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RefResultArray res = RefResultArray.NONE;
                if(propException == null) {
                    res = RefResultArray.wrap(resNative);

                    RefResultArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final RefResultArray res = JNI_LIB.ada_defining_name_p_find_refs(
                    (root != null ? root.entity : null),
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Searches all references to this defining name in the given list of
         * units.
         *
         * If ``follow_renamings`` is True, also this also includes references
         * that ultimately refer to this defining name, by unwinding renaming
         * clauses.
         */
        public RefResultArray pFindAllReferences(
            final AnalysisUnitArray units,final boolean followRenamings,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(units == null) throw new IllegalArgumentException(
                "Argument 'units' of type " +
                "AnalysisUnitArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitArrayNative unitsNative = units.unwrap();
                byte followRenamingsNative = (followRenamings ? (byte) 1 : (byte) 0);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_find_all_references(
                    thisNative,
                    unitsNative,
                    followRenamingsNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RefResultArray res = RefResultArray.NONE;
                if(propException == null) {
                    res = RefResultArray.wrap(resNative);

                    RefResultArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                AnalysisUnitArray.release(
                    unitsNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final RefResultArray res = JNI_LIB.ada_defining_name_p_find_all_references(
                    units,
                    followRenamings,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the list of all possible calls to the subprogram which Self
         * is the defining name of.
         *
         * This will return the name corresponding to the call, excluding the
         * parameters if there are any. For instance, it will return ``A`` for
         * the ``A (B)`` call.
         *
         * .. note:: This does not yet support calls done inside generics.
         */
        public RefResultArray pFindAllCalls(
            final AnalysisUnitArray units,final boolean followRenamings,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(units == null) throw new IllegalArgumentException(
                "Argument 'units' of type " +
                "AnalysisUnitArray cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                AnalysisUnitArrayNative unitsNative = units.unwrap();
                byte followRenamingsNative = (followRenamings ? (byte) 1 : (byte) 0);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_find_all_calls(
                    thisNative,
                    unitsNative,
                    followRenamingsNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RefResultArray res = RefResultArray.NONE;
                if(propException == null) {
                    res = RefResultArray.wrap(resNative);

                    RefResultArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters
                AnalysisUnitArray.release(
                    unitsNative
                );


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final RefResultArray res = JNI_LIB.ada_defining_name_p_find_all_calls(
                    units,
                    followRenamings,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Like ``BasicDecl.next_part_for_decl`` on a defining name
         */
        public DefiningName pNextPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_next_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_next_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Like ``BasicDecl.previous_part_for_decl`` on a defining name
         */
        public DefiningName pPreviousPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_previous_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_previous_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Like ``BasicDecl.canonical_part`` on a defining name
         */
        public DefiningName pCanonicalPart(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_canonical_part(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_canonical_part(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Given an origin node and the entity represented by Self, this
         * property returns the most visible completion of Self that can be
         * seen by origin, according to Ada's visibility rules.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public DefiningName pMostVisiblePart(
            final AdaNode origin,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_most_visible_part(
                    thisNative,
                    originNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DefiningName res = DefiningName.NONE;
                if(propException == null) {
                    res = DefiningName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_most_visible_part(
                    (origin != null ? origin.entity : null),
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return DefiningName.fromEntity(res);
            }

        }

        
    

        /**
         * Return all parts that define this entity, sorted from first part to
         * last part.
         */
        public AdaNodeArray pAllParts(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_all_parts(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_defining_name_p_all_parts(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the aspect with name ``name`` associated to entity that this
         * name defines.
         *
         * Aspects are properties of entities that can be specified by the Ada
         * program, either via aspect specifications, pragmas, or attributes.
         *
         * This will return the syntactic node corresponding to attribute
         * directly.
         *
         * Note: for some aspects (e.g. ``Inline``), Libadalang will check if
         * they are defined on any part of the entity.
         */
        public Aspect pGetAspect(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final AspectNative resNative =
                    StackValue.get(AspectNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_get_aspect(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Aspect res = Aspect.NONE;
                if(propException == null) {
                    res = Aspect.wrap(resNative);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Aspect res = JNI_LIB.ada_defining_name_p_get_aspect(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns whether the boolean aspect named ``name`` is set on the
         * entity represented by this node.
         *
         * Aspects are properties of entities that can be specified by the Ada
         * program, either via aspect specifications, pragmas, or attributes.
         *
         * "Aspect" is used as in RM terminology (see :rmlink:`13.1`).
         */
        public boolean pHasAspect(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_has_aspect(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_defining_name_p_has_aspect(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return the pragma with name ``name`` associated to this entity.
         *
         * Please use the ``p_get_aspect`` property instead if you are
         * interested in aspects, i.e. information that can be represented by
         * either aspect specification nodes, pragma nodes or attribute
         * definition nodes.
         */
        public PragmaNode pGetPragma(
            final Symbol name
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_get_pragma(
                    thisNative,
                    nameNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PragmaNode res = PragmaNode.NONE;
                if(propException == null) {
                    res = PragmaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_get_pragma(
                    name,
                    this.entity
                );

                // Wrap and return the result
                return PragmaNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the representation clause associated to this entity that
         * defines the given attribute name.
         */
        public AttributeDefClause pGetRepresentationClause(
            final Symbol name,final boolean impreciseFallback
        ) {

            // Verify that arguments are not null
            if(name == null) throw new IllegalArgumentException(
                "Argument 'name' of type " +
                "Symbol cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);

                // Get the node unit
                final AnalysisUnit currentUnit = this.getUnit();

                // Get the node context
                final AnalysisContext currentContext =
                    currentUnit.getContext();

                // Unwrap the arguments
                SymbolNative nameNative = StackValue.get(SymbolNative.class);name.unwrap(nameNative, currentContext);
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_get_representation_clause(
                    thisNative,
                    nameNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AttributeDefClause res = AttributeDefClause.NONE;
                if(propException == null) {
                    res = AttributeDefClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters

                // Close the context
                currentContext.close();

                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_get_representation_clause(
                    name,
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return AttributeDefClause.fromEntity(res);
            }

        }

        
    

        /**
         * Return the at clause associated to this entity.
         */
        public AtClause pGetAtClause(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_defining_name_p_get_at_clause(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AtClause res = AtClause.NONE;
                if(propException == null) {
                    res = AtClause.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_defining_name_p_get_at_clause(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return AtClause.fromEntity(res);
            }

        }

        
    

        /**
         * Whether this entity defined by this name is imported from another
         * language.
         */
        public boolean pIsImported(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_is_imported(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_defining_name_p_is_imported(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return whether the entity defined by this name is ghost or not. See
         * SPARK RM 6.9.
         */
        public boolean pIsGhostCode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_defining_name_p_is_ghost_code(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_defining_name_p_is_ghost_code(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic DefiningName.
     */
    public static 
    class SyntheticDefiningName extends DefiningName {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticDefiningName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            DefiningName.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticDefiningName NONE =
            new SyntheticDefiningName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticDefiningName(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticDefiningName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticDefiningName.NONE :

                new SyntheticDefiningName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticDefiningName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticDefiningName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticDefiningName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticDefiningName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Subtype name for membership test expressions (:rmlink:`3.6`).
     */
    public static 
    class DiscreteSubtypeName extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscreteSubtypeName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSubtype"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DiscreteSubtypeName.class.getMethod(
                        "fSubtype",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscreteSubtypeName NONE =
            new DiscreteSubtypeName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscreteSubtypeName(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscreteSubtypeName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscreteSubtypeName.NONE :

                new DiscreteSubtypeName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscreteSubtypeName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscreteSubtypeName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscreteSubtypeName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscreteSubtypeName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DiscreteSubtypeIndication fSubtype(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_discrete_subtype_name_f_subtype(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DiscreteSubtypeIndication res = DiscreteSubtypeIndication.NONE;
                if(propException == null) {
                    res = DiscreteSubtypeIndication.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_discrete_subtype_name_f_subtype(
                    this.entity
                );

                // Wrap and return the result
                return DiscreteSubtypeIndication.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Name to select a suffix in a prefix (:rmlink:`4.1.3`).
     */
    public static 
    class DottedName extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DottedName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix","fSuffix"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DottedName.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DottedName.class.getMethod(
                        "fSuffix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_suffix",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DottedName NONE =
            new DottedName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DottedName(
            final Entity entity
        ) {
            super(entity);
        }

        public static DottedName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DottedName.NONE :

                new DottedName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DottedName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DottedName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DottedName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DottedName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_dotted_name_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_dotted_name_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BaseId fSuffix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_dotted_name_f_suffix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseId res = BaseId.NONE;
                if(propException == null) {
                    res = BaseId.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_dotted_name_f_suffix(
                    this.entity
                );

                // Wrap and return the result
                return BaseId.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Entity name in ``end ...;`` syntactic constructs.
     */
    public static 
    class EndName extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EndName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EndName.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = EndName.class.getMethod(
                        "pBasicDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_basic_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EndName NONE =
            new EndName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EndName(
            final Entity entity
        ) {
            super(entity);
        }

        public static EndName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EndName.NONE :

                new EndName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EndName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EndName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EndName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EndName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_end_name_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_end_name_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * Returns this EndName's basic declaration
         */
        public BasicDecl pBasicDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_end_name_p_basic_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_end_name_p_basic_decl(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Explicit dereference expression (``.all``) (:rmlink:`4.1`).
     */
    public static 
    class ExplicitDeref extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExplicitDeref";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExplicitDeref.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExplicitDeref NONE =
            new ExplicitDeref(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExplicitDeref(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExplicitDeref fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExplicitDeref.NONE :

                new ExplicitDeref(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExplicitDeref.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExplicitDeref.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExplicitDeref.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExplicitDeref.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_explicit_deref_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_explicit_deref_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualified expression (``...'(...)``) .(:rmlink:`4.7`).
     */
    public static 
    class QualExpr extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "QualExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix","fSuffix"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = QualExpr.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = QualExpr.class.getMethod(
                        "fSuffix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_suffix",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final QualExpr NONE =
            new QualExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected QualExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static QualExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                QualExpr.NONE :

                new QualExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return QualExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return QualExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return QualExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return QualExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_qual_expr_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_qual_expr_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``BaseAggregate``, ``ParenExpr``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fSuffix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_qual_expr_f_suffix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_qual_expr_f_suffix(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Reduction expression (``Reduce`` attribute). Ada 2022, RM 4.5.10.
     */
    public static 
    class ReduceAttributeRef extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ReduceAttributeRef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix","fAttribute","fArgs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ReduceAttributeRef.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ReduceAttributeRef.class.getMethod(
                        "fAttribute",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_attribute",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ReduceAttributeRef.class.getMethod(
                        "fArgs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_args",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ReduceAttributeRef NONE =
            new ReduceAttributeRef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ReduceAttributeRef(
            final Entity entity
        ) {
            super(entity);
        }

        public static ReduceAttributeRef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ReduceAttributeRef.NONE :

                new ReduceAttributeRef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ReduceAttributeRef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ReduceAttributeRef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ReduceAttributeRef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ReduceAttributeRef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``,
         * ``ValueSequence``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_reduce_attribute_ref_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_reduce_attribute_ref_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fAttribute(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_reduce_attribute_ref_f_attribute(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_reduce_attribute_ref_f_attribute(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ParamAssoc``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AssocList fArgs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_reduce_attribute_ref_f_args(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AssocList res = AssocList.NONE;
                if(propException == null) {
                    res = AssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_reduce_attribute_ref_f_args(
                    this.entity
                );

                // Wrap and return the result
                return AssocList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for nodes that are made up of a single token.
     */
    public static abstract
    class SingleTokNode extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SingleTokNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SingleTokNode NONE =
            new SingleTokNodeNone();

        // ----- Constructors -----

        protected SingleTokNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static SingleTokNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SingleTokNode.NONE :

                (SingleTokNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SingleTokNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SingleTokNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SingleTokNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SingleTokNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * SingleTokNode.
         */
        private static final class SingleTokNodeNone extends SingleTokNode {
            SingleTokNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Base class for identifiers.
     */
    public static abstract
    class BaseId extends SingleTokNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseId";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SingleTokNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseId NONE =
            new BaseIdNone();

        // ----- Constructors -----

        protected BaseId(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseId fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseId.NONE :

                (BaseId) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseId.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseId.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseId.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseId.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseId.
         */
        private static final class BaseIdNone extends BaseId {
            BaseIdNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Character literal (:rmlink:`4.1`).
     */
    public static 
    class CharLiteral extends BaseId {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CharLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseId.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CharLiteral.class.getMethod(
                        "pDenotedValue",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_denoted_value",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CharLiteral NONE =
            new CharLiteral(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CharLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static CharLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CharLiteral.NONE :

                new CharLiteral(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CharLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CharLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CharLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CharLiteral.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the value that this literal denotes.
         */
        public Char pDenotedValue(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CIntPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_char_literal_p_denoted_value(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Char res = Char.NONE;
                if(propException == null) {
                    res = Char.wrap(resNative);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Char res = JNI_LIB.ada_char_literal_p_denoted_value(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Regular identifier (:rmlink:`2.3`).
     */
    public static 
    class Identifier extends BaseId {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Identifier";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseId.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Identifier NONE =
            new Identifier(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Identifier(
            final Entity entity
        ) {
            super(entity);
        }

        public static Identifier fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Identifier.NONE :

                new Identifier(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Identifier.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Identifier.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Identifier.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Identifier.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Operation in a binary expression.
     *
     * Note that the ARM does not consider "double_dot" ("..") as a binary
     * operator, but we process it this way here anyway to keep things simple.
     */
    public static abstract
    class Op extends BaseId {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Op";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseId.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Op NONE =
            new OpNone();

        // ----- Constructors -----

        protected Op(
            final Entity entity
        ) {
            super(entity);
        }

        public static Op fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Op.NONE :

                (Op) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Op.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Op.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Op.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Op.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Op.
         */
        private static final class OpNone extends Op {
            OpNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class OpAbs extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpAbs";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpAbs NONE =
            new OpAbs(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpAbs(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpAbs fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpAbs.NONE :

                new OpAbs(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpAbs.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpAbs.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpAbs.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpAbs.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpAnd extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpAnd";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpAnd NONE =
            new OpAnd(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpAnd(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpAnd fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpAnd.NONE :

                new OpAnd(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpAnd.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpAnd.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpAnd.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpAnd.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpAndThen extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpAndThen";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpAndThen NONE =
            new OpAndThen(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpAndThen(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpAndThen fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpAndThen.NONE :

                new OpAndThen(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpAndThen.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpAndThen.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpAndThen.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpAndThen.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpConcat extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpConcat";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpConcat NONE =
            new OpConcat(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpConcat(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpConcat fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpConcat.NONE :

                new OpConcat(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpConcat.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpConcat.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpConcat.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpConcat.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpDiv extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpDiv";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpDiv NONE =
            new OpDiv(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpDiv(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpDiv fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpDiv.NONE :

                new OpDiv(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpDiv.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpDiv.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpDiv.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpDiv.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpDoubleDot extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpDoubleDot";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpDoubleDot NONE =
            new OpDoubleDot(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpDoubleDot(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpDoubleDot fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpDoubleDot.NONE :

                new OpDoubleDot(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpDoubleDot.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpDoubleDot.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpDoubleDot.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpDoubleDot.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpEq extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpEq";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpEq NONE =
            new OpEq(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpEq(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpEq fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpEq.NONE :

                new OpEq(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpEq.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpEq.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpEq.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpEq.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpGt extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpGt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpGt NONE =
            new OpGt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpGt(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpGt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpGt.NONE :

                new OpGt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpGt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpGt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpGt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpGt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpGte extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpGte";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpGte NONE =
            new OpGte(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpGte(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpGte fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpGte.NONE :

                new OpGte(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpGte.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpGte.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpGte.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpGte.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpIn extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpIn";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpIn NONE =
            new OpIn(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpIn(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpIn fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpIn.NONE :

                new OpIn(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpIn.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpIn.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpIn.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpIn.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpLt extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpLt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpLt NONE =
            new OpLt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpLt(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpLt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpLt.NONE :

                new OpLt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpLt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpLt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpLt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpLt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpLte extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpLte";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpLte NONE =
            new OpLte(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpLte(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpLte fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpLte.NONE :

                new OpLte(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpLte.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpLte.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpLte.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpLte.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpMinus extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpMinus";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpMinus NONE =
            new OpMinus(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpMinus(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpMinus fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpMinus.NONE :

                new OpMinus(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpMinus.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpMinus.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpMinus.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpMinus.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpMod extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpMod";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpMod NONE =
            new OpMod(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpMod(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpMod fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpMod.NONE :

                new OpMod(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpMod.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpMod.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpMod.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpMod.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpMult extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpMult";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpMult NONE =
            new OpMult(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpMult(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpMult fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpMult.NONE :

                new OpMult(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpMult.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpMult.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpMult.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpMult.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpNeq extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpNeq";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpNeq NONE =
            new OpNeq(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpNeq(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpNeq fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpNeq.NONE :

                new OpNeq(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpNeq.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpNeq.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpNeq.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpNeq.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpNot extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpNot";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpNot NONE =
            new OpNot(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpNot(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpNot fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpNot.NONE :

                new OpNot(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpNot.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpNot.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpNot.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpNot.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpNotIn extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpNotIn";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpNotIn NONE =
            new OpNotIn(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpNotIn(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpNotIn fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpNotIn.NONE :

                new OpNotIn(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpNotIn.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpNotIn.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpNotIn.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpNotIn.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpOr extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpOr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpOr NONE =
            new OpOr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpOr(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpOr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpOr.NONE :

                new OpOr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpOr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpOr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpOr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpOr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpOrElse extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpOrElse";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpOrElse NONE =
            new OpOrElse(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpOrElse(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpOrElse fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpOrElse.NONE :

                new OpOrElse(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpOrElse.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpOrElse.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpOrElse.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpOrElse.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpPlus extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpPlus";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpPlus NONE =
            new OpPlus(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpPlus(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpPlus fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpPlus.NONE :

                new OpPlus(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpPlus.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpPlus.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpPlus.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpPlus.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpPow extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpPow";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpPow NONE =
            new OpPow(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpPow(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpPow fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpPow.NONE :

                new OpPow(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpPow.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpPow.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpPow.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpPow.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpRem extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpRem";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpRem NONE =
            new OpRem(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpRem(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpRem fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpRem.NONE :

                new OpRem(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpRem.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpRem.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpRem.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpRem.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OpXor extends Op {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OpXor";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Op.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OpXor NONE =
            new OpXor(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OpXor(
            final Entity entity
        ) {
            super(entity);
        }

        public static OpXor fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OpXor.NONE :

                new OpXor(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OpXor.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OpXor.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OpXor.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OpXor.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * String literal (:rmlink:`2.6`).
     */
    public static 
    class StringLiteral extends BaseId {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "StringLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseId.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = StringLiteral.class.getMethod(
                        "pDenotedValue",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_denoted_value",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final StringLiteral NONE =
            new StringLiteral(
                Entity.NONE
            );

        // ----- Constructors -----

        protected StringLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static StringLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                StringLiteral.NONE :

                new StringLiteral(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return StringLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return StringLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return StringLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return StringLiteral.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the value that this literal denotes.
         */
        public String pDenotedValue(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_string_literal_p_denoted_value(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                String res = StringWrapper.NONE;
                if(propException == null) {
                    res = StringWrapper.wrap(resNative);

                    StringWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final String res = JNI_LIB.ada_string_literal_p_denoted_value(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * The ``null`` literal (:rmlink:`4.4`).
     */
    public static 
    class NullLiteral extends SingleTokNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SingleTokNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullLiteral NONE =
            new NullLiteral(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullLiteral.NONE :

                new NullLiteral(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullLiteral.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for number literals (:rmlink:`2.4`).
     */
    public static abstract
    class NumLiteral extends SingleTokNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NumLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SingleTokNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NumLiteral NONE =
            new NumLiteralNone();

        // ----- Constructors -----

        protected NumLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static NumLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NumLiteral.NONE :

                (NumLiteral) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NumLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NumLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NumLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NumLiteral.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * NumLiteral.
         */
        private static final class NumLiteralNone extends NumLiteral {
            NumLiteralNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Literal for an integer (:rmlink:`2.4`).
     */
    public static 
    class IntLiteral extends NumLiteral {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IntLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            NumLiteral.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IntLiteral.class.getMethod(
                        "pDenotedValue",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_denoted_value",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IntLiteral NONE =
            new IntLiteral(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IntLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static IntLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IntLiteral.NONE :

                new IntLiteral(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IntLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IntLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IntLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IntLiteral.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * Return the value that this literal denotes.
         */
        public BigInteger pDenotedValue(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_int_literal_p_denoted_value(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BigInteger res = BigIntegerWrapper.NONE;
                if(propException == null) {
                    res = BigIntegerWrapper.wrap(resNative);

                    BigIntegerWrapper.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final BigInteger res = JNI_LIB.ada_int_literal_p_denoted_value(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Literal for a real number (:rmlink:`2.4`).
     */
    public static 
    class RealLiteral extends NumLiteral {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RealLiteral";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            NumLiteral.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RealLiteral NONE =
            new RealLiteral(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RealLiteral(
            final Entity entity
        ) {
            super(entity);
        }

        public static RealLiteral fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RealLiteral.NONE :

                new RealLiteral(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RealLiteral.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RealLiteral.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RealLiteral.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RealLiteral.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic identifier.
     */
    public static 
    class SyntheticIdentifier extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticIdentifier";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticIdentifier NONE =
            new SyntheticIdentifier(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticIdentifier(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticIdentifier fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticIdentifier.NONE :

                new SyntheticIdentifier(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticIdentifier.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticIdentifier.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticIdentifier.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticIdentifier.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Name for Ada 2020 ``@`` (:rmlink:`5.2.1`).
     */
    public static 
    class TargetName extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TargetName";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TargetName NONE =
            new TargetName(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TargetName(
            final Entity entity
        ) {
            super(entity);
        }

        public static TargetName fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TargetName.NONE :

                new TargetName(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TargetName.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TargetName.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TargetName.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TargetName.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Reference to the ``Update`` attribute, which is a non standard GNAT
     * attribute.
     */
    public static 
    class UpdateAttributeRef extends Name {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UpdateAttributeRef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPrefix","fAttribute","fValues"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Name.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UpdateAttributeRef.class.getMethod(
                        "fPrefix",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_prefix",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = UpdateAttributeRef.class.getMethod(
                        "fAttribute",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_attribute",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = UpdateAttributeRef.class.getMethod(
                        "fValues",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_values",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UpdateAttributeRef NONE =
            new UpdateAttributeRef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UpdateAttributeRef(
            final Entity entity
        ) {
            super(entity);
        }

        public static UpdateAttributeRef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UpdateAttributeRef.NONE :

                new UpdateAttributeRef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UpdateAttributeRef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UpdateAttributeRef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UpdateAttributeRef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UpdateAttributeRef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fPrefix(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_update_attribute_ref_f_prefix(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_update_attribute_ref_f_prefix(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fAttribute(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_update_attribute_ref_f_attribute(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_update_attribute_ref_f_attribute(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseAggregate fValues(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_update_attribute_ref_f_values(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseAggregate res = BaseAggregate.NONE;
                if(propException == null) {
                    res = BaseAggregate.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_update_attribute_ref_f_values(
                    this.entity
                );

                // Wrap and return the result
                return BaseAggregate.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Parenthesized expression.
     */
    public static 
    class ParenExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParenExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ParenExpr.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParenExpr NONE =
            new ParenExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParenExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParenExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParenExpr.NONE :

                new ParenExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParenExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParenExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParenExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParenExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_paren_expr_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_paren_expr_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Quantified expression (:rmlink:`4.5.8`).
     */
    public static 
    class QuantifiedExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "QuantifiedExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fQuantifier","fLoopSpec","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = QuantifiedExpr.class.getMethod(
                        "fQuantifier",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_quantifier",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = QuantifiedExpr.class.getMethod(
                        "fLoopSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_loop_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = QuantifiedExpr.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final QuantifiedExpr NONE =
            new QuantifiedExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected QuantifiedExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static QuantifiedExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                QuantifiedExpr.NONE :

                new QuantifiedExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return QuantifiedExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return QuantifiedExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return QuantifiedExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return QuantifiedExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Quantifier fQuantifier(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_quantified_expr_f_quantifier(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Quantifier res = Quantifier.NONE;
                if(propException == null) {
                    res = Quantifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_quantified_expr_f_quantifier(
                    this.entity
                );

                // Wrap and return the result
                return Quantifier.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ForLoopSpec fLoopSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_quantified_expr_f_loop_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ForLoopSpec res = ForLoopSpec.NONE;
                if(propException == null) {
                    res = ForLoopSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_quantified_expr_f_loop_spec(
                    this.entity
                );

                // Wrap and return the result
                return ForLoopSpec.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_quantified_expr_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_quantified_expr_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Expression to raise an exception (:rmlink:`4.4`).
     */
    public static 
    class RaiseExpr extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RaiseExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExceptionName","fErrorMessage"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RaiseExpr.class.getMethod(
                        "fExceptionName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_exception_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RaiseExpr.class.getMethod(
                        "fErrorMessage",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_error_message",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RaiseExpr NONE =
            new RaiseExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RaiseExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static RaiseExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RaiseExpr.NONE :

                new RaiseExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RaiseExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RaiseExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RaiseExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RaiseExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fExceptionName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_raise_expr_f_exception_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_raise_expr_f_exception_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fErrorMessage(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_raise_expr_f_error_message(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_raise_expr_f_error_message(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Unary expression.
     *
     * This encompasses several ARM expressions, because it is used for every
     * unary operator in Ada. Those expressions are all documented in
     * :rmlink:`4.4`.
     */
    public static 
    class UnOp extends Expr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UnOp";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fOp","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Expr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UnOp.class.getMethod(
                        "fOp",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_op",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = UnOp.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UnOp NONE =
            new UnOp(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UnOp(
            final Entity entity
        ) {
            super(entity);
        }

        public static UnOp fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UnOp.NONE :

                new UnOp(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UnOp.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UnOp.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UnOp.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UnOp.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``OpAbs``,
         * ``OpMinus``, ``OpNot``, ``OpPlus``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Op fOp(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_un_op_f_op(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Op res = Op.NONE;
                if(propException == null) {
                    res = Op.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_un_op_f_op(
                    this.entity
                );

                // Wrap and return the result
                return Op.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``CondExpr``, ``DeclExpr``, ``DottedName``,
         * ``ExplicitDeref``, ``Identifier``, ``NullLiteral``, ``NumLiteral``,
         * ``ParenExpr``, ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_un_op_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_un_op_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of statements, with optional exception handlers (:rmlink:`11.2`).
     */
    public static 
    class HandledStmts extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "HandledStmts";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fStmts","fExceptions"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = HandledStmts.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = HandledStmts.class.getMethod(
                        "fExceptions",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_exceptions",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final HandledStmts NONE =
            new HandledStmts(
                Entity.NONE
            );

        // ----- Constructors -----

        protected HandledStmts(
            final Entity entity
        ) {
            super(entity);
        }

        public static HandledStmts fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                HandledStmts.NONE :

                new HandledStmts(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return HandledStmts.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return HandledStmts.fieldNames;
        }

        @Override
        public boolean isListType() {
            return HandledStmts.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return HandledStmts.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_handled_stmts_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_handled_stmts_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``ExceptionHandler``, ``PragmaNode``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNodeList fExceptions(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_handled_stmts_f_exceptions(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeList res = AdaNodeList.NONE;
                if(propException == null) {
                    res = AdaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_handled_stmts_f_exceptions(
                    this.entity
                );

                // Wrap and return the result
                return AdaNodeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Kind of interface type.
     */
    public static abstract
    class InterfaceKind extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceKind";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceKind NONE =
            new InterfaceKindNone();

        // ----- Constructors -----

        protected InterfaceKind(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceKind fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceKind.NONE :

                (InterfaceKind) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceKind.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceKind.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceKind.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceKind.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * InterfaceKind.
         */
        private static final class InterfaceKindNone extends InterfaceKind {
            InterfaceKindNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class InterfaceKindLimited extends InterfaceKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceKindLimited";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            InterfaceKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceKindLimited NONE =
            new InterfaceKindLimited(
                Entity.NONE
            );

        // ----- Constructors -----

        protected InterfaceKindLimited(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceKindLimited fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceKindLimited.NONE :

                new InterfaceKindLimited(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceKindLimited.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceKindLimited.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceKindLimited.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceKindLimited.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class InterfaceKindProtected extends InterfaceKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceKindProtected";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            InterfaceKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceKindProtected NONE =
            new InterfaceKindProtected(
                Entity.NONE
            );

        // ----- Constructors -----

        protected InterfaceKindProtected(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceKindProtected fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceKindProtected.NONE :

                new InterfaceKindProtected(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceKindProtected.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceKindProtected.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceKindProtected.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceKindProtected.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class InterfaceKindSynchronized extends InterfaceKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceKindSynchronized";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            InterfaceKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceKindSynchronized NONE =
            new InterfaceKindSynchronized(
                Entity.NONE
            );

        // ----- Constructors -----

        protected InterfaceKindSynchronized(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceKindSynchronized fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceKindSynchronized.NONE :

                new InterfaceKindSynchronized(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceKindSynchronized.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceKindSynchronized.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceKindSynchronized.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceKindSynchronized.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class InterfaceKindTask extends InterfaceKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceKindTask";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            InterfaceKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceKindTask NONE =
            new InterfaceKindTask(
                Entity.NONE
            );

        // ----- Constructors -----

        protected InterfaceKindTask(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceKindTask fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceKindTask.NONE :

                new InterfaceKindTask(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceKindTask.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceKindTask.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceKindTask.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceKindTask.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Iteration type for ``for`` loops.
     */
    public static abstract
    class IterType extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IterType";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IterType NONE =
            new IterTypeNone();

        // ----- Constructors -----

        protected IterType(
            final Entity entity
        ) {
            super(entity);
        }

        public static IterType fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IterType.NONE :

                (IterType) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IterType.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IterType.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IterType.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IterType.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * IterType.
         */
        private static final class IterTypeNone extends IterType {
            IterTypeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class IterTypeIn extends IterType {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IterTypeIn";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            IterType.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IterTypeIn NONE =
            new IterTypeIn(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IterTypeIn(
            final Entity entity
        ) {
            super(entity);
        }

        public static IterTypeIn fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IterTypeIn.NONE :

                new IterTypeIn(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IterTypeIn.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IterTypeIn.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IterTypeIn.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IterTypeIn.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class IterTypeOf extends IterType {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IterTypeOf";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            IterType.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IterTypeOf NONE =
            new IterTypeOf(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IterTypeOf(
            final Entity entity
        ) {
            super(entity);
        }

        public static IterTypeOf fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IterTypeOf.NONE :

                new IterTypeOf(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IterTypeOf.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IterTypeOf.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IterTypeOf.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IterTypeOf.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Library item in a compilation unit (:rmlink:`10.1.1`).
     */
    public static 
    class LibraryItem extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LibraryItem";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasPrivate","fItem"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = LibraryItem.class.getMethod(
                        "fHasPrivate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_private",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = LibraryItem.class.getMethod(
                        "fItem",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_item",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LibraryItem NONE =
            new LibraryItem(
                Entity.NONE
            );

        // ----- Constructors -----

        protected LibraryItem(
            final Entity entity
        ) {
            super(entity);
        }

        public static LibraryItem fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LibraryItem.NONE :

                new LibraryItem(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LibraryItem.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LibraryItem.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LibraryItem.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LibraryItem.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PrivateNode fHasPrivate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_library_item_f_has_private(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PrivateNode res = PrivateNode.NONE;
                if(propException == null) {
                    res = PrivateNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_library_item_f_has_private(
                    this.entity
                );

                // Wrap and return the result
                return PrivateNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AbstractSubpDecl``, ``BaseSubpBody``, ``ErrorDecl``,
         * ``GenericDecl``, ``GenericInstantiation``, ``GenericRenamingDecl``,
         * ``PackageBody``, ``PackageDecl``, ``PackageRenamingDecl``,
         * ``SubpDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BasicDecl fItem(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_library_item_f_item(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_library_item_f_item(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``limited`` keyword.
     */
    public static abstract
    class LimitedNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LimitedNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = LimitedNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LimitedNode NONE =
            new LimitedNodeNone();

        // ----- Constructors -----

        protected LimitedNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static LimitedNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LimitedNode.NONE :

                (LimitedNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LimitedNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LimitedNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LimitedNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LimitedNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of LimitedPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_limited_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_limited_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * LimitedNode.
         */
        private static final class LimitedNodeNone extends LimitedNode {
            LimitedNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class LimitedAbsent extends LimitedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LimitedAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            LimitedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LimitedAbsent NONE =
            new LimitedAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected LimitedAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static LimitedAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LimitedAbsent.NONE :

                new LimitedAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LimitedAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LimitedAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LimitedAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LimitedAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class LimitedPresent extends LimitedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LimitedPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            LimitedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LimitedPresent NONE =
            new LimitedPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected LimitedPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static LimitedPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LimitedPresent.NONE :

                new LimitedPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LimitedPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LimitedPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LimitedPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LimitedPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for loop specifications (:rmlink:`5.5`).
     */
    public static abstract
    class LoopSpec extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LoopSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LoopSpec NONE =
            new LoopSpecNone();

        // ----- Constructors -----

        protected LoopSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static LoopSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LoopSpec.NONE :

                (LoopSpec) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LoopSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LoopSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LoopSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LoopSpec.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * LoopSpec.
         */
        private static final class LoopSpecNone extends LoopSpec {
            LoopSpecNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Specification for a ``for`` loop (:rmlink:`5.5`).
     */
    public static 
    class ForLoopSpec extends LoopSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ForLoopSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fVarDecl","fLoopType","fHasReverse","fIterExpr","fIterFilter"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            LoopSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopSpec.class.getMethod(
                        "fVarDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_var_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopSpec.class.getMethod(
                        "fLoopType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_loop_type",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopSpec.class.getMethod(
                        "fHasReverse",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_reverse",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopSpec.class.getMethod(
                        "fIterExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_iter_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ForLoopSpec.class.getMethod(
                        "fIterFilter",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_iter_filter",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ForLoopSpec NONE =
            new ForLoopSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ForLoopSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static ForLoopSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ForLoopSpec.NONE :

                new ForLoopSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ForLoopSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ForLoopSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ForLoopSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ForLoopSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ForLoopVarDecl fVarDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_spec_f_var_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ForLoopVarDecl res = ForLoopVarDecl.NONE;
                if(propException == null) {
                    res = ForLoopVarDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_spec_f_var_decl(
                    this.entity
                );

                // Wrap and return the result
                return ForLoopVarDecl.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public IterType fLoopType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_spec_f_loop_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                IterType res = IterType.NONE;
                if(propException == null) {
                    res = IterType.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_spec_f_loop_type(
                    this.entity
                );

                // Wrap and return the result
                return IterType.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ReverseNode fHasReverse(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_spec_f_has_reverse(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ReverseNode res = ReverseNode.NONE;
                if(propException == null) {
                    res = ReverseNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_spec_f_has_reverse(
                    this.entity
                );

                // Wrap and return the result
                return ReverseNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``,
         * ``DiscreteSubtypeIndication``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fIterExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_spec_f_iter_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_spec_f_iter_expr(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fIterFilter(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_for_loop_spec_f_iter_filter(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_for_loop_spec_f_iter_filter(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Specification for a ``while`` loop (:rmlink:`5.5`).
     */
    public static 
    class WhileLoopSpec extends LoopSpec {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WhileLoopSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            LoopSpec.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = WhileLoopSpec.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WhileLoopSpec NONE =
            new WhileLoopSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected WhileLoopSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static WhileLoopSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WhileLoopSpec.NONE :

                new WhileLoopSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WhileLoopSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WhileLoopSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WhileLoopSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WhileLoopSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_while_loop_spec_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_while_loop_spec_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Syntactic indicators for passing modes in formals (:rmlink:`6.1`).
     */
    public static abstract
    class Mode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Mode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Mode NONE =
            new ModeNone();

        // ----- Constructors -----

        protected Mode(
            final Entity entity
        ) {
            super(entity);
        }

        public static Mode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Mode.NONE :

                (Mode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Mode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Mode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Mode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Mode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Mode.
         */
        private static final class ModeNone extends Mode {
            ModeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class ModeDefault extends Mode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ModeDefault";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Mode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ModeDefault NONE =
            new ModeDefault(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ModeDefault(
            final Entity entity
        ) {
            super(entity);
        }

        public static ModeDefault fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ModeDefault.NONE :

                new ModeDefault(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ModeDefault.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ModeDefault.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ModeDefault.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ModeDefault.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ModeIn extends Mode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ModeIn";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Mode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ModeIn NONE =
            new ModeIn(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ModeIn(
            final Entity entity
        ) {
            super(entity);
        }

        public static ModeIn fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ModeIn.NONE :

                new ModeIn(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ModeIn.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ModeIn.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ModeIn.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ModeIn.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ModeInOut extends Mode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ModeInOut";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Mode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ModeInOut NONE =
            new ModeInOut(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ModeInOut(
            final Entity entity
        ) {
            super(entity);
        }

        public static ModeInOut fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ModeInOut.NONE :

                new ModeInOut(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ModeInOut.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ModeInOut.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ModeInOut.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ModeInOut.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ModeOut extends Mode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ModeOut";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Mode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ModeOut NONE =
            new ModeOut(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ModeOut(
            final Entity entity
        ) {
            super(entity);
        }

        public static ModeOut fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ModeOut.NONE :

                new ModeOut(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ModeOut.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ModeOut.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ModeOut.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ModeOut.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Node that holds several AbstractStateDecl nodes, which is necessary when
     * the Abstract_State aspect is associated with an aggregate in order to
     * declare a list of abstract states.
     */
    public static 
    class MultiAbstractStateDecl extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "MultiAbstractStateDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = MultiAbstractStateDecl.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final MultiAbstractStateDecl NONE =
            new MultiAbstractStateDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected MultiAbstractStateDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static MultiAbstractStateDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                MultiAbstractStateDecl.NONE :

                new MultiAbstractStateDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return MultiAbstractStateDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return MultiAbstractStateDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return MultiAbstractStateDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return MultiAbstractStateDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AbstractStateDecl``, ``ParenAbstractStateDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AbstractStateDeclList fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_multi_abstract_state_decl_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbstractStateDeclList res = AbstractStateDeclList.NONE;
                if(propException == null) {
                    res = AbstractStateDeclList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_multi_abstract_state_decl_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return AbstractStateDeclList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``not null`` keywords.
     */
    public static abstract
    class NotNull extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NotNull";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = NotNull.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NotNull NONE =
            new NotNullNone();

        // ----- Constructors -----

        protected NotNull(
            final Entity entity
        ) {
            super(entity);
        }

        public static NotNull fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NotNull.NONE :

                (NotNull) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NotNull.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NotNull.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NotNull.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NotNull.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of NotNullPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_not_null_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_not_null_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * NotNull.
         */
        private static final class NotNullNone extends NotNull {
            NotNullNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class NotNullAbsent extends NotNull {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NotNullAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            NotNull.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NotNullAbsent NONE =
            new NotNullAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NotNullAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static NotNullAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NotNullAbsent.NONE :

                new NotNullAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NotNullAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NotNullAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NotNullAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NotNullAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class NotNullPresent extends NotNull {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NotNullPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            NotNull.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NotNullPresent NONE =
            new NotNullPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NotNullPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static NotNullPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NotNullPresent.NONE :

                new NotNullPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NotNullPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NotNullPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NotNullPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NotNullPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Placeholder for the ``null`` in lists of components (:rmlink:`3.8`).
     */
    public static 
    class NullComponentDecl extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullComponentDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullComponentDecl NONE =
            new NullComponentDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullComponentDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullComponentDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullComponentDecl.NONE :

                new NullComponentDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullComponentDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullComponentDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullComponentDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullComponentDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * ``other`` designator.
     */
    public static 
    class OthersDesignator extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OthersDesignator";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OthersDesignator NONE =
            new OthersDesignator(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OthersDesignator(
            final Entity entity
        ) {
            super(entity);
        }

        public static OthersDesignator fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OthersDesignator.NONE :

                new OthersDesignator(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OthersDesignator.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OthersDesignator.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OthersDesignator.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OthersDesignator.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Syntactic indicators for subprogram overriding modes.
     */
    public static abstract
    class OverridingNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OverridingNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OverridingNode NONE =
            new OverridingNodeNone();

        // ----- Constructors -----

        protected OverridingNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static OverridingNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OverridingNode.NONE :

                (OverridingNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OverridingNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OverridingNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OverridingNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OverridingNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * OverridingNode.
         */
        private static final class OverridingNodeNone extends OverridingNode {
            OverridingNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class OverridingNotOverriding extends OverridingNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OverridingNotOverriding";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            OverridingNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OverridingNotOverriding NONE =
            new OverridingNotOverriding(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OverridingNotOverriding(
            final Entity entity
        ) {
            super(entity);
        }

        public static OverridingNotOverriding fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OverridingNotOverriding.NONE :

                new OverridingNotOverriding(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OverridingNotOverriding.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OverridingNotOverriding.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OverridingNotOverriding.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OverridingNotOverriding.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OverridingOverriding extends OverridingNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OverridingOverriding";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            OverridingNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OverridingOverriding NONE =
            new OverridingOverriding(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OverridingOverriding(
            final Entity entity
        ) {
            super(entity);
        }

        public static OverridingOverriding fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OverridingOverriding.NONE :

                new OverridingOverriding(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OverridingOverriding.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OverridingOverriding.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OverridingOverriding.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OverridingOverriding.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class OverridingUnspecified extends OverridingNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OverridingUnspecified";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            OverridingNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OverridingUnspecified NONE =
            new OverridingUnspecified(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OverridingUnspecified(
            final Entity entity
        ) {
            super(entity);
        }

        public static OverridingUnspecified fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OverridingUnspecified.NONE :

                new OverridingUnspecified(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OverridingUnspecified.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OverridingUnspecified.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OverridingUnspecified.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OverridingUnspecified.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * List of parameter specifications.
     */
    public static 
    class Params extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Params";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fParams"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Params.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Params NONE =
            new Params(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Params(
            final Entity entity
        ) {
            super(entity);
        }

        public static Params fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Params.NONE :

                new Params(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Params.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Params.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Params.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Params.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ParamSpecList fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_params_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamSpecList res = ParamSpecList.NONE;
                if(propException == null) {
                    res = ParamSpecList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_params_f_params(
                    this.entity
                );

                // Wrap and return the result
                return ParamSpecList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Holds an AbstractStateDecl between parentheses. Needed to support the
     * syntax:
     *
     * .. code:: ada
     *
     *    package Pkg
     *        with Abstract_State => (A, (B with Some_Aspect))
     */
    public static 
    class ParenAbstractStateDecl extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ParenAbstractStateDecl";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ParenAbstractStateDecl.class.getMethod(
                        "fDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ParenAbstractStateDecl NONE =
            new ParenAbstractStateDecl(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ParenAbstractStateDecl(
            final Entity entity
        ) {
            super(entity);
        }

        public static ParenAbstractStateDecl fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ParenAbstractStateDecl.NONE :

                new ParenAbstractStateDecl(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ParenAbstractStateDecl.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ParenAbstractStateDecl.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ParenAbstractStateDecl.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ParenAbstractStateDecl.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes:
         * ``AbstractStateDecl``, ``ParenAbstractStateDecl``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AdaNode fDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_paren_abstract_state_decl_f_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNode res = AdaNode.NONE;
                if(propException == null) {
                    res = AdaNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_paren_abstract_state_decl_f_decl(
                    this.entity
                );

                // Wrap and return the result
                return AdaNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base node for all preprocessor directives.
     */
    public static abstract
    class PpDirective extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpDirective";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpDirective NONE =
            new PpDirectiveNone();

        // ----- Constructors -----

        protected PpDirective(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpDirective fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpDirective.NONE :

                (PpDirective) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpDirective.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpDirective.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpDirective.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpDirective.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * PpDirective.
         */
        private static final class PpDirectiveNone extends PpDirective {
            PpDirectiveNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * ``else`` preprocessor directive.
     */
    public static 
    class PpElseDirective extends PpDirective {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpElseDirective";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PpDirective.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpElseDirective NONE =
            new PpElseDirective(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PpElseDirective(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpElseDirective fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpElseDirective.NONE :

                new PpElseDirective(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpElseDirective.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpElseDirective.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpElseDirective.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpElseDirective.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * ``elsif ... [then]`` preprocessor directive.
     */
    public static 
    class PpElsifDirective extends PpDirective {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpElsifDirective";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr","fThenKw"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PpDirective.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PpElsifDirective.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PpElsifDirective.class.getMethod(
                        "fThenKw",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_then_kw",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpElsifDirective NONE =
            new PpElsifDirective(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PpElsifDirective(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpElsifDirective fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpElsifDirective.NONE :

                new PpElsifDirective(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpElsifDirective.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpElsifDirective.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpElsifDirective.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpElsifDirective.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``Identifier``, ``ParenExpr``, ``UnOp``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pp_elsif_directive_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pp_elsif_directive_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public PpThenKw fThenKw(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pp_elsif_directive_f_then_kw(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PpThenKw res = PpThenKw.NONE;
                if(propException == null) {
                    res = PpThenKw.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pp_elsif_directive_f_then_kw(
                    this.entity
                );

                // Wrap and return the result
                return PpThenKw.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``end if;`` preprocessor directive.
     */
    public static 
    class PpEndIfDirective extends PpDirective {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpEndIfDirective";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PpDirective.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpEndIfDirective NONE =
            new PpEndIfDirective(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PpEndIfDirective(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpEndIfDirective fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpEndIfDirective.NONE :

                new PpEndIfDirective(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpEndIfDirective.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpEndIfDirective.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpEndIfDirective.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpEndIfDirective.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * ``if ... [then]`` preprocessor directive.
     */
    public static 
    class PpIfDirective extends PpDirective {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpIfDirective";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr","fThenKw"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PpDirective.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PpIfDirective.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PpIfDirective.class.getMethod(
                        "fThenKw",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_then_kw",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpIfDirective NONE =
            new PpIfDirective(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PpIfDirective(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpIfDirective fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpIfDirective.NONE :

                new PpIfDirective(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpIfDirective.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpIfDirective.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpIfDirective.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpIfDirective.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``Identifier``, ``ParenExpr``, ``UnOp``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pp_if_directive_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pp_if_directive_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public PpThenKw fThenKw(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pp_if_directive_f_then_kw(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PpThenKw res = PpThenKw.NONE;
                if(propException == null) {
                    res = PpThenKw.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pp_if_directive_f_then_kw(
                    this.entity
                );

                // Wrap and return the result
                return PpThenKw.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``then`` keyword in preprocessor directives.
     */
    public static 
    class PpThenKw extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PpThenKw";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PpThenKw NONE =
            new PpThenKw(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PpThenKw(
            final Entity entity
        ) {
            super(entity);
        }

        public static PpThenKw fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PpThenKw.NONE :

                new PpThenKw(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PpThenKw.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PpThenKw.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PpThenKw.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PpThenKw.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Class for pragmas (:rmlink:`2.8`). Pragmas are compiler directives, that
     * can be language or compiler defined.
     */
    public static 
    class PragmaNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PragmaNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fId","fArgs"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PragmaNode.class.getMethod(
                        "fId",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_id",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PragmaNode.class.getMethod(
                        "fArgs",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_args",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PragmaNode.class.getMethod(
                        "pIsGhostCode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_ghost_code",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PragmaNode.class.getMethod(
                        "pAssociatedEntities",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_associated_entities",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PragmaNode NONE =
            new PragmaNode(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PragmaNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static PragmaNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PragmaNode.NONE :

                new PragmaNode(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PragmaNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PragmaNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PragmaNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PragmaNode.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fId(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pragma_node_f_id(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pragma_node_f_id(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseAssocList fArgs(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_pragma_node_f_args(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseAssocList res = BaseAssocList.NONE;
                if(propException == null) {
                    res = BaseAssocList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_pragma_node_f_args(
                    this.entity
                );

                // Wrap and return the result
                return BaseAssocList.fromEntity(res);
            }

        }

        
    

        /**
         * Return whether this pragma is ghost code or not. See SPARK RM 6.9.
         */
        public boolean pIsGhostCode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_pragma_node_p_is_ghost_code(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_pragma_node_p_is_ghost_code(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Return an array of ``BasicDecl`` instances associated with this
         * pragma, or an empty array if non applicable.
         */
        public AdaNodeArray pAssociatedEntities(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_pragma_node_p_associated_entities(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AdaNodeArray res = AdaNodeArray.NONE;
                if(propException == null) {
                    res = AdaNodeArray.wrap(resNative);

                    AdaNodeArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final AdaNodeArray res = JNI_LIB.ada_pragma_node_p_associated_entities(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``private`` keyword.
     */
    public static abstract
    class PrivateNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PrivateNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PrivateNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PrivateNode NONE =
            new PrivateNodeNone();

        // ----- Constructors -----

        protected PrivateNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static PrivateNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PrivateNode.NONE :

                (PrivateNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PrivateNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PrivateNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PrivateNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PrivateNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of PrivatePresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_private_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_private_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * PrivateNode.
         */
        private static final class PrivateNodeNone extends PrivateNode {
            PrivateNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class PrivateAbsent extends PrivateNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PrivateAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PrivateNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PrivateAbsent NONE =
            new PrivateAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PrivateAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static PrivateAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PrivateAbsent.NONE :

                new PrivateAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PrivateAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PrivateAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PrivateAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PrivateAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class PrivatePresent extends PrivateNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PrivatePresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            PrivateNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PrivatePresent NONE =
            new PrivatePresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PrivatePresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static PrivatePresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PrivatePresent.NONE :

                new PrivatePresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PrivatePresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PrivatePresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PrivatePresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PrivatePresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a protected object (:rmlink:`9.4`).
     */
    public static 
    class ProtectedDef extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPublicPart","fPrivatePart","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedDef.class.getMethod(
                        "fPublicPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_public_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedDef.class.getMethod(
                        "fPrivatePart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_private_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedDef.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedDef NONE =
            new ProtectedDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedDef.NONE :

                new ProtectedDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PublicPart fPublicPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_def_f_public_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PublicPart res = PublicPart.NONE;
                if(propException == null) {
                    res = PublicPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_def_f_public_part(
                    this.entity
                );

                // Wrap and return the result
                return PublicPart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public PrivatePart fPrivatePart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_def_f_private_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PrivatePart res = PrivatePart.NONE;
                if(propException == null) {
                    res = PrivatePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_def_f_private_part(
                    this.entity
                );

                // Wrap and return the result
                return PrivatePart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_protected_def_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_protected_def_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``protected`` keyword.
     */
    public static abstract
    class ProtectedNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ProtectedNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedNode NONE =
            new ProtectedNodeNone();

        // ----- Constructors -----

        protected ProtectedNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedNode.NONE :

                (ProtectedNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of ProtectedPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_protected_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_protected_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ProtectedNode.
         */
        private static final class ProtectedNodeNone extends ProtectedNode {
            ProtectedNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class ProtectedAbsent extends ProtectedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ProtectedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedAbsent NONE =
            new ProtectedAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedAbsent.NONE :

                new ProtectedAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ProtectedPresent extends ProtectedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ProtectedPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ProtectedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ProtectedPresent NONE =
            new ProtectedPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ProtectedPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ProtectedPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ProtectedPresent.NONE :

                new ProtectedPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ProtectedPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ProtectedPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ProtectedPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ProtectedPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Type for quantified expressions.
     */
    public static abstract
    class Quantifier extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Quantifier";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Quantifier NONE =
            new QuantifierNone();

        // ----- Constructors -----

        protected Quantifier(
            final Entity entity
        ) {
            super(entity);
        }

        public static Quantifier fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Quantifier.NONE :

                (Quantifier) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Quantifier.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Quantifier.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Quantifier.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Quantifier.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Quantifier.
         */
        private static final class QuantifierNone extends Quantifier {
            QuantifierNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class QuantifierAll extends Quantifier {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "QuantifierAll";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Quantifier.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final QuantifierAll NONE =
            new QuantifierAll(
                Entity.NONE
            );

        // ----- Constructors -----

        protected QuantifierAll(
            final Entity entity
        ) {
            super(entity);
        }

        public static QuantifierAll fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                QuantifierAll.NONE :

                new QuantifierAll(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return QuantifierAll.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return QuantifierAll.fieldNames;
        }

        @Override
        public boolean isListType() {
            return QuantifierAll.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return QuantifierAll.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class QuantifierSome extends Quantifier {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "QuantifierSome";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Quantifier.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final QuantifierSome NONE =
            new QuantifierSome(
                Entity.NONE
            );

        // ----- Constructors -----

        protected QuantifierSome(
            final Entity entity
        ) {
            super(entity);
        }

        public static QuantifierSome fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                QuantifierSome.NONE :

                new QuantifierSome(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return QuantifierSome.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return QuantifierSome.fieldNames;
        }

        @Override
        public boolean isListType() {
            return QuantifierSome.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return QuantifierSome.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Range specification (:rmlink:`3.5.7`).
     */
    public static 
    class RangeSpec extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RangeSpec";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RangeSpec.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RangeSpec NONE =
            new RangeSpec(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RangeSpec(
            final Entity entity
        ) {
            super(entity);
        }

        public static RangeSpec fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RangeSpec.NONE :

                new RangeSpec(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RangeSpec.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RangeSpec.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RangeSpec.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RangeSpec.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``BinOp``, ``BoxExpr``, ``CallExpr``, ``CharLiteral``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_range_spec_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_range_spec_f_range(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Renaming clause, used everywhere renamings are valid.
     */
    public static 
    class RenamingClause extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RenamingClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRenamedObject"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RenamingClause.class.getMethod(
                        "fRenamedObject",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_renamed_object",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RenamingClause NONE =
            new RenamingClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RenamingClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static RenamingClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RenamingClause.NONE :

                new RenamingClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RenamingClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RenamingClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RenamingClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RenamingClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fRenamedObject(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_renaming_clause_f_renamed_object(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_renaming_clause_f_renamed_object(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic renaming clause. Used to synthesize object decls with
     * renamings. (See to_anonymous_object_decl).
     */
    public static 
    class SyntheticRenamingClause extends RenamingClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticRenamingClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRenamedObject"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            RenamingClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticRenamingClause NONE =
            new SyntheticRenamingClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticRenamingClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticRenamingClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticRenamingClause.NONE :

                new SyntheticRenamingClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticRenamingClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticRenamingClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticRenamingClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticRenamingClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``reverse`` keyword.
     */
    public static abstract
    class ReverseNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ReverseNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ReverseNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ReverseNode NONE =
            new ReverseNodeNone();

        // ----- Constructors -----

        protected ReverseNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static ReverseNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ReverseNode.NONE :

                (ReverseNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ReverseNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ReverseNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ReverseNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ReverseNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of ReversePresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_reverse_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_reverse_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * ReverseNode.
         */
        private static final class ReverseNodeNone extends ReverseNode {
            ReverseNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class ReverseAbsent extends ReverseNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ReverseAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ReverseNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ReverseAbsent NONE =
            new ReverseAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ReverseAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ReverseAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ReverseAbsent.NONE :

                new ReverseAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ReverseAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ReverseAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ReverseAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ReverseAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class ReversePresent extends ReverseNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ReversePresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            ReverseNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ReversePresent NONE =
            new ReversePresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ReversePresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static ReversePresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ReversePresent.NONE :

                new ReversePresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ReversePresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ReversePresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ReversePresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ReversePresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Alternative part in a ``select`` statements block (:rmlink:`9.7`).
     */
    public static 
    class SelectWhenPart extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SelectWhenPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCondExpr","fStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SelectWhenPart.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SelectWhenPart.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SelectWhenPart NONE =
            new SelectWhenPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SelectWhenPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static SelectWhenPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SelectWhenPart.NONE :

                new SelectWhenPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SelectWhenPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SelectWhenPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SelectWhenPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SelectWhenPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_select_when_part_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_select_when_part_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_select_when_part_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_select_when_part_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Bass class for statements (:rmlink:`5.1`).
     */
    public static abstract
    class Stmt extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Stmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Stmt.class.getMethod(
                        "pIsGhostCode",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_ghost_code",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Stmt NONE =
            new StmtNone();

        // ----- Constructors -----

        protected Stmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static Stmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Stmt.NONE :

                (Stmt) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Stmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Stmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Stmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Stmt.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this statement is ghost code or not. See SPARK RM
         * 6.9.
         */
        public boolean pIsGhostCode(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_stmt_p_is_ghost_code(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_stmt_p_is_ghost_code(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * Stmt.
         */
        private static final class StmtNone extends Stmt {
            StmtNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Base class for composite statements (:rmlink:`5.1`).
     */
    public static abstract
    class CompositeStmt extends Stmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CompositeStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Stmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CompositeStmt NONE =
            new CompositeStmtNone();

        // ----- Constructors -----

        protected CompositeStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static CompositeStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CompositeStmt.NONE :

                (CompositeStmt) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CompositeStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CompositeStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CompositeStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CompositeStmt.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * CompositeStmt.
         */
        private static final class CompositeStmtNone extends CompositeStmt {
            CompositeStmtNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * ``accept`` statement (:rmlink:`9.5.2`).
     */
    public static 
    class AcceptStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AcceptStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fBodyDecl","fEntryIndexExpr","fParams"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmt.class.getMethod(
                        "fBodyDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_body_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmt.class.getMethod(
                        "fEntryIndexExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_entry_index_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmt.class.getMethod(
                        "fParams",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_params",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmt.class.getMethod(
                        "pCorrespondingEntry",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        AdaNode.class,
                        "origin",
                        AdaNode.NONE
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_corresponding_entry",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AcceptStmt NONE =
            new AcceptStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AcceptStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static AcceptStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AcceptStmt.NONE :

                new AcceptStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AcceptStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AcceptStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AcceptStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AcceptStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AcceptStmtBody fBodyDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_f_body_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AcceptStmtBody res = AcceptStmtBody.NONE;
                if(propException == null) {
                    res = AcceptStmtBody.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_f_body_decl(
                    this.entity
                );

                // Wrap and return the result
                return AcceptStmtBody.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fEntryIndexExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_f_entry_index_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_f_entry_index_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public EntryCompletionFormalParams fParams(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_f_params(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EntryCompletionFormalParams res = EntryCompletionFormalParams.NONE;
                if(propException == null) {
                    res = EntryCompletionFormalParams.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_f_params(
                    this.entity
                );

                // Wrap and return the result
                return EntryCompletionFormalParams.fromEntity(res);
            }

        }

        
    

        /**
         * Return the entry which corresponds to this accept statement.
         *
         * ``Origin``: Origin for this property's request. See :ref:`The origin
         * parameter` for more details
         */
        public EntryDecl pCorrespondingEntry(
            final AdaNode origin
        ) {

            // Verify that arguments are not null
            if(origin == null) throw new IllegalArgumentException(
                "Argument 'origin' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNative = StackValue.get(EntityNative.class);origin.entity.unwrap(originNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_p_corresponding_entry(
                    thisNative,
                    originNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EntryDecl res = EntryDecl.NONE;
                if(propException == null) {
                    res = EntryDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_p_corresponding_entry(
                    (origin != null ? origin.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return EntryDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Extended ``accept`` statement (:rmlink:`9.5.2`).
     */
    public static 
    class AcceptStmtWithStmts extends AcceptStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AcceptStmtWithStmts";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fBodyDecl","fEntryIndexExpr","fParams","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AcceptStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmtWithStmts.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AcceptStmtWithStmts.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AcceptStmtWithStmts NONE =
            new AcceptStmtWithStmts(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AcceptStmtWithStmts(
            final Entity entity
        ) {
            super(entity);
        }

        public static AcceptStmtWithStmts fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AcceptStmtWithStmts.NONE :

                new AcceptStmtWithStmts(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AcceptStmtWithStmts.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AcceptStmtWithStmts.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AcceptStmtWithStmts.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AcceptStmtWithStmts.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_with_stmts_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_with_stmts_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_accept_stmt_with_stmts_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_accept_stmt_with_stmts_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for loop statements (:rmlink:`5.5`).
     */
    public static abstract
    class BaseLoopStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseLoopStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BaseLoopStmt.class.getMethod(
                        "fSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_spec",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseLoopStmt.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BaseLoopStmt.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseLoopStmt NONE =
            new BaseLoopStmtNone();

        // ----- Constructors -----

        protected BaseLoopStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseLoopStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseLoopStmt.NONE :

                (BaseLoopStmt) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseLoopStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseLoopStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseLoopStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseLoopStmt.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public LoopSpec fSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_loop_stmt_f_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LoopSpec res = LoopSpec.NONE;
                if(propException == null) {
                    res = LoopSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_loop_stmt_f_spec(
                    this.entity
                );

                // Wrap and return the result
                return LoopSpec.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_loop_stmt_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_loop_stmt_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_base_loop_stmt_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_base_loop_stmt_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseLoopStmt.
         */
        private static final class BaseLoopStmtNone extends BaseLoopStmt {
            BaseLoopStmtNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Statement for ``for`` loops (``for ... loop ... end loop;``)
     * (:rmlink:`5.5`).
     */
    public static 
    class ForLoopStmt extends BaseLoopStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ForLoopStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseLoopStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ForLoopStmt NONE =
            new ForLoopStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ForLoopStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static ForLoopStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ForLoopStmt.NONE :

                new ForLoopStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ForLoopStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ForLoopStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ForLoopStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ForLoopStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement for simple loops (``loop ... end loop;``) (:rmlink:`5.5`).
     */
    public static 
    class LoopStmt extends BaseLoopStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "LoopStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseLoopStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final LoopStmt NONE =
            new LoopStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected LoopStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static LoopStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                LoopStmt.NONE :

                new LoopStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return LoopStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return LoopStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return LoopStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return LoopStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement for ``while`` loops (``while ... loop ... end loop;``)
     * (:rmlink:`5.5`).
     */
    public static 
    class WhileLoopStmt extends BaseLoopStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WhileLoopStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSpec","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseLoopStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WhileLoopStmt NONE =
            new WhileLoopStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected WhileLoopStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static WhileLoopStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WhileLoopStmt.NONE :

                new WhileLoopStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WhileLoopStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WhileLoopStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WhileLoopStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WhileLoopStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for statement blocks (:rmlink:`5.6`).
     */
    public static abstract
    class BlockStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BlockStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BlockStmt NONE =
            new BlockStmtNone();

        // ----- Constructors -----

        protected BlockStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static BlockStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BlockStmt.NONE :

                (BlockStmt) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BlockStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BlockStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BlockStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BlockStmt.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BlockStmt.
         */
        private static final class BlockStmtNone extends BlockStmt {
            BlockStmtNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Statement block with no declarative part (:rmlink:`5.6`).
     */
    public static 
    class BeginBlock extends BlockStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BeginBlock";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BlockStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = BeginBlock.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = BeginBlock.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BeginBlock NONE =
            new BeginBlock(
                Entity.NONE
            );

        // ----- Constructors -----

        protected BeginBlock(
            final Entity entity
        ) {
            super(entity);
        }

        public static BeginBlock fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BeginBlock.NONE :

                new BeginBlock(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BeginBlock.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BeginBlock.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BeginBlock.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BeginBlock.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_begin_block_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_begin_block_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_begin_block_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_begin_block_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement block with a declarative part (:rmlink:`5.6`).
     */
    public static 
    class DeclBlock extends BlockStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DeclBlock";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecls","fStmts","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BlockStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DeclBlock.class.getMethod(
                        "fDecls",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decls",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DeclBlock.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DeclBlock.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DeclBlock NONE =
            new DeclBlock(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DeclBlock(
            final Entity entity
        ) {
            super(entity);
        }

        public static DeclBlock fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DeclBlock.NONE :

                new DeclBlock(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DeclBlock.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DeclBlock.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DeclBlock.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DeclBlock.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public DeclarativePart fDecls(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decl_block_f_decls(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                DeclarativePart res = DeclarativePart.NONE;
                if(propException == null) {
                    res = DeclarativePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decl_block_f_decls(
                    this.entity
                );

                // Wrap and return the result
                return DeclarativePart.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decl_block_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decl_block_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decl_block_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decl_block_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``case`` statement (:rmlink:`5.4`).
     */
    public static 
    class CaseStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CaseStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr","fPragmas","fAlternatives"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CaseStmt.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CaseStmt.class.getMethod(
                        "fPragmas",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_pragmas",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = CaseStmt.class.getMethod(
                        "fAlternatives",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_alternatives",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CaseStmt NONE =
            new CaseStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CaseStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static CaseStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CaseStmt.NONE :

                new CaseStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CaseStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CaseStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CaseStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CaseStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_stmt_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_stmt_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PragmaNodeList fPragmas(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_stmt_f_pragmas(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PragmaNodeList res = PragmaNodeList.NONE;
                if(propException == null) {
                    res = PragmaNodeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_stmt_f_pragmas(
                    this.entity
                );

                // Wrap and return the result
                return PragmaNodeList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public CaseStmtAlternativeList fAlternatives(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_case_stmt_f_alternatives(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CaseStmtAlternativeList res = CaseStmtAlternativeList.NONE;
                if(propException == null) {
                    res = CaseStmtAlternativeList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_case_stmt_f_alternatives(
                    this.entity
                );

                // Wrap and return the result
                return CaseStmtAlternativeList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Extended ``return`` statement (:rmlink:`6.5`).
     */
    public static 
    class ExtendedReturnStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExtendedReturnStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl","fStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExtendedReturnStmt.class.getMethod(
                        "fDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ExtendedReturnStmt.class.getMethod(
                        "fStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExtendedReturnStmt NONE =
            new ExtendedReturnStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExtendedReturnStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExtendedReturnStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExtendedReturnStmt.NONE :

                new ExtendedReturnStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExtendedReturnStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExtendedReturnStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExtendedReturnStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExtendedReturnStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ExtendedReturnStmtObjectDecl fDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_extended_return_stmt_f_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ExtendedReturnStmtObjectDecl res = ExtendedReturnStmtObjectDecl.NONE;
                if(propException == null) {
                    res = ExtendedReturnStmtObjectDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_extended_return_stmt_f_decl(
                    this.entity
                );

                // Wrap and return the result
                return ExtendedReturnStmtObjectDecl.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public HandledStmts fStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_extended_return_stmt_f_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                HandledStmts res = HandledStmts.NONE;
                if(propException == null) {
                    res = HandledStmts.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_extended_return_stmt_f_stmts(
                    this.entity
                );

                // Wrap and return the result
                return HandledStmts.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``if`` statement block (:rmlink:`5.3`).
     */
    public static 
    class IfStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "IfStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCondExpr","fThenStmts","fAlternatives","fElseStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = IfStmt.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfStmt.class.getMethod(
                        "fThenStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_then_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfStmt.class.getMethod(
                        "fAlternatives",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_alternatives",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = IfStmt.class.getMethod(
                        "fElseStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_else_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final IfStmt NONE =
            new IfStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected IfStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static IfStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                IfStmt.NONE :

                new IfStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return IfStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return IfStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return IfStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return IfStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_stmt_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_stmt_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fThenStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_stmt_f_then_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_stmt_f_then_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ElsifStmtPartList fAlternatives(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_stmt_f_alternatives(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ElsifStmtPartList res = ElsifStmtPartList.NONE;
                if(propException == null) {
                    res = ElsifStmtPartList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_stmt_f_alternatives(
                    this.entity
                );

                // Wrap and return the result
                return ElsifStmtPartList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fElseStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_if_stmt_f_else_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_if_stmt_f_else_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Wrapper class, used for composite statements that can be named (declare
     * blocks, loops). This allows to both have a BasicDecl for the named
     * entity declared, and a CompositeStmt for the statement hierarchy.
     */
    public static 
    class NamedStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NamedStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl","fStmt"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = NamedStmt.class.getMethod(
                        "fDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = NamedStmt.class.getMethod(
                        "fStmt",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_stmt",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NamedStmt NONE =
            new NamedStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NamedStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static NamedStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NamedStmt.NONE :

                new NamedStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NamedStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NamedStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NamedStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NamedStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public NamedStmtDecl fDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_named_stmt_f_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NamedStmtDecl res = NamedStmtDecl.NONE;
                if(propException == null) {
                    res = NamedStmtDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_named_stmt_f_decl(
                    this.entity
                );

                // Wrap and return the result
                return NamedStmtDecl.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``BaseLoopStmt``,
         * ``BlockStmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public CompositeStmt fStmt(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_named_stmt_f_stmt(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                CompositeStmt res = CompositeStmt.NONE;
                if(propException == null) {
                    res = CompositeStmt.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_named_stmt_f_stmt(
                    this.entity
                );

                // Wrap and return the result
                return CompositeStmt.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``select`` statements block (:rmlink:`9.7`).
     */
    public static 
    class SelectStmt extends CompositeStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SelectStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fGuards","fElseStmts","fAbortStmts"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            CompositeStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SelectStmt.class.getMethod(
                        "fGuards",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_guards",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SelectStmt.class.getMethod(
                        "fElseStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_else_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SelectStmt.class.getMethod(
                        "fAbortStmts",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_abort_stmts",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SelectStmt NONE =
            new SelectStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SelectStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static SelectStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SelectStmt.NONE :

                new SelectStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SelectStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SelectStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SelectStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SelectStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SelectWhenPartList fGuards(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_select_stmt_f_guards(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SelectWhenPartList res = SelectWhenPartList.NONE;
                if(propException == null) {
                    res = SelectWhenPartList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_select_stmt_f_guards(
                    this.entity
                );

                // Wrap and return the result
                return SelectWhenPartList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fElseStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_select_stmt_f_else_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_select_stmt_f_else_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``PragmaNode``, ``Stmt``
         *
         * When there are no parsing errors, this field is never null.
         */
        public StmtList fAbortStmts(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_select_stmt_f_abort_stmts(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                StmtList res = StmtList.NONE;
                if(propException == null) {
                    res = StmtList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_select_stmt_f_abort_stmts(
                    this.entity
                );

                // Wrap and return the result
                return StmtList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Placeholder node for syntax errors in lists of statements.
     */
    public static 
    class ErrorStmt extends Stmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ErrorStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Stmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ErrorStmt NONE =
            new ErrorStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ErrorStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static ErrorStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ErrorStmt.NONE :

                new ErrorStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ErrorStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ErrorStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ErrorStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ErrorStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for simple statements (:rmlink:`5.1`).
     */
    public static abstract
    class SimpleStmt extends Stmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SimpleStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            Stmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SimpleStmt NONE =
            new SimpleStmtNone();

        // ----- Constructors -----

        protected SimpleStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static SimpleStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SimpleStmt.NONE :

                (SimpleStmt) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SimpleStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SimpleStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SimpleStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SimpleStmt.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * SimpleStmt.
         */
        private static final class SimpleStmtNone extends SimpleStmt {
            SimpleStmtNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * ``abort`` statement (:rmlink:`9.8`).
     */
    public static 
    class AbortStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AbortStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fNames"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AbortStmt.class.getMethod(
                        "fNames",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_names",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AbortStmt NONE =
            new AbortStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AbortStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static AbortStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AbortStmt.NONE :

                new AbortStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AbortStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AbortStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AbortStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AbortStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AttributeRef``, ``CallExpr``, ``CharLiteral``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public NameList fNames(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_abort_stmt_f_names(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NameList res = NameList.NONE;
                if(propException == null) {
                    res = NameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_abort_stmt_f_names(
                    this.entity
                );

                // Wrap and return the result
                return NameList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement for assignments (:rmlink:`5.2`).
     */
    public static 
    class AssignStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AssignStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDest","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AssignStmt.class.getMethod(
                        "fDest",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_dest",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AssignStmt.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AssignStmt NONE =
            new AssignStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AssignStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static AssignStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AssignStmt.NONE :

                new AssignStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AssignStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AssignStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AssignStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AssignStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fDest(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_assign_stmt_f_dest(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_assign_stmt_f_dest(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_assign_stmt_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_assign_stmt_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement for entry or procedure calls (:rmlink:`6.4`).
     */
    public static 
    class CallStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "CallStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCall"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = CallStmt.class.getMethod(
                        "fCall",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_call",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final CallStmt NONE =
            new CallStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected CallStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static CallStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                CallStmt.NONE :

                new CallStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return CallStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return CallStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return CallStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return CallStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fCall(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_call_stmt_f_call(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_call_stmt_f_call(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``delay`` statement (:rmlink:`9.6`).
     */
    public static 
    class DelayStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DelayStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasUntil","fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DelayStmt.class.getMethod(
                        "fHasUntil",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_until",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DelayStmt.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DelayStmt NONE =
            new DelayStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DelayStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static DelayStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DelayStmt.NONE :

                new DelayStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DelayStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DelayStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DelayStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DelayStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public UntilNode fHasUntil(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_delay_stmt_f_has_until(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                UntilNode res = UntilNode.NONE;
                if(propException == null) {
                    res = UntilNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_delay_stmt_f_has_until(
                    this.entity
                );

                // Wrap and return the result
                return UntilNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_delay_stmt_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_delay_stmt_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``exit`` statement (:rmlink:`5.7`).
     */
    public static 
    class ExitStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ExitStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fLoopName","fCondExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ExitStmt.class.getMethod(
                        "fLoopName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_loop_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ExitStmt.class.getMethod(
                        "fCondExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_cond_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ExitStmt NONE =
            new ExitStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ExitStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static ExitStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ExitStmt.NONE :

                new ExitStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ExitStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ExitStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ExitStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ExitStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fLoopName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exit_stmt_f_loop_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exit_stmt_f_loop_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fCondExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_exit_stmt_f_cond_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_exit_stmt_f_cond_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``goto`` statement (:rmlink:`5.8`).
     */
    public static 
    class GotoStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "GotoStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fLabelName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = GotoStmt.class.getMethod(
                        "fLabelName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_label_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final GotoStmt NONE =
            new GotoStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected GotoStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static GotoStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                GotoStmt.NONE :

                new GotoStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return GotoStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return GotoStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return GotoStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return GotoStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fLabelName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_goto_stmt_f_label_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_goto_stmt_f_label_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Statement to declare a code label (:rmlink:`5.1`).
     */
    public static 
    class Label extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Label";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Label.class.getMethod(
                        "fDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Label NONE =
            new Label(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Label(
            final Entity entity
        ) {
            super(entity);
        }

        public static Label fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Label.NONE :

                new Label(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Label.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Label.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Label.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Label.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public LabelDecl fDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_label_f_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LabelDecl res = LabelDecl.NONE;
                if(propException == null) {
                    res = LabelDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_label_f_decl(
                    this.entity
                );

                // Wrap and return the result
                return LabelDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``null;`` statement (:rmlink:`5.1`).
     */
    public static 
    class NullStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "NullStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final NullStmt NONE =
            new NullStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected NullStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static NullStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                NullStmt.NONE :

                new NullStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return NullStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return NullStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return NullStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return NullStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * ``raise`` statement (:rmlink:`11.3`).
     */
    public static 
    class RaiseStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RaiseStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExceptionName","fErrorMessage"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RaiseStmt.class.getMethod(
                        "fExceptionName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_exception_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RaiseStmt.class.getMethod(
                        "fErrorMessage",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_error_message",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RaiseStmt NONE =
            new RaiseStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RaiseStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static RaiseStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RaiseStmt.NONE :

                new RaiseStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RaiseStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RaiseStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RaiseStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RaiseStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Name fExceptionName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_raise_stmt_f_exception_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_raise_stmt_f_exception_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fErrorMessage(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_raise_stmt_f_error_message(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_raise_stmt_f_error_message(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``requeue`` statement (:rmlink:`9.5.4`).
     */
    public static 
    class RequeueStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RequeueStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fCallName","fHasAbort"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RequeueStmt.class.getMethod(
                        "fCallName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_call_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RequeueStmt.class.getMethod(
                        "fHasAbort",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_abort",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RequeueStmt NONE =
            new RequeueStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RequeueStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static RequeueStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RequeueStmt.NONE :

                new RequeueStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RequeueStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RequeueStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RequeueStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RequeueStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CallExpr``, ``CharLiteral``, ``DottedName``, ``ExplicitDeref``,
         * ``Identifier``, ``QualExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fCallName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_requeue_stmt_f_call_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_requeue_stmt_f_call_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AbortNode fHasAbort(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_requeue_stmt_f_has_abort(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbortNode res = AbortNode.NONE;
                if(propException == null) {
                    res = AbortNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_requeue_stmt_f_has_abort(
                    this.entity
                );

                // Wrap and return the result
                return AbortNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``return`` statement (:rmlink:`6.5`).
     */
    public static 
    class ReturnStmt extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ReturnStmt";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fReturnExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ReturnStmt.class.getMethod(
                        "fReturnExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_return_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ReturnStmt NONE =
            new ReturnStmt(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ReturnStmt(
            final Entity entity
        ) {
            super(entity);
        }

        public static ReturnStmt fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ReturnStmt.NONE :

                new ReturnStmt(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ReturnStmt.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ReturnStmt.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ReturnStmt.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ReturnStmt.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``CallExpr``,
         * ``CharLiteral``, ``ConcatOp``, ``CondExpr``, ``DeclExpr``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``, ``ParenExpr``,
         * ``QualExpr``, ``QuantifiedExpr``, ``RaiseExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``, ``UnOp``,
         * ``UpdateAttributeRef``
         *
         * This field may be null even when there are no parsing errors.
         */
        public Expr fReturnExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_return_stmt_f_return_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_return_stmt_f_return_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * ``terminate`` alternative in a ``select`` statement (:rmlink:`9.7`).
     */
    public static 
    class TerminateAlternative extends SimpleStmt {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TerminateAlternative";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SimpleStmt.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TerminateAlternative NONE =
            new TerminateAlternative(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TerminateAlternative(
            final Entity entity
        ) {
            super(entity);
        }

        public static TerminateAlternative fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TerminateAlternative.NONE :

                new TerminateAlternative(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TerminateAlternative.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TerminateAlternative.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TerminateAlternative.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TerminateAlternative.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for a subprogram kind.
     */
    public static abstract
    class SubpKind extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpKind";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpKind NONE =
            new SubpKindNone();

        // ----- Constructors -----

        protected SubpKind(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpKind fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpKind.NONE :

                (SubpKind) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpKind.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpKind.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpKind.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpKind.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * SubpKind.
         */
        private static final class SubpKindNone extends SubpKind {
            SubpKindNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class SubpKindFunction extends SubpKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpKindFunction";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SubpKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpKindFunction NONE =
            new SubpKindFunction(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpKindFunction(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpKindFunction fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpKindFunction.NONE :

                new SubpKindFunction(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpKindFunction.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpKindFunction.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpKindFunction.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpKindFunction.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class SubpKindProcedure extends SubpKind {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubpKindProcedure";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SubpKind.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubpKindProcedure NONE =
            new SubpKindProcedure(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubpKindProcedure(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubpKindProcedure fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubpKindProcedure.NONE :

                new SubpKindProcedure(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubpKindProcedure.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubpKindProcedure.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubpKindProcedure.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubpKindProcedure.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Subunit (``separate``) (:rmlink:`10.1.3`).
     */
    public static 
    class Subunit extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Subunit";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fName","fBody"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Subunit.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Subunit.class.getMethod(
                        "fBody",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_body",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Subunit.class.getMethod(
                        "pBodyRoot",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_body_root",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Subunit NONE =
            new Subunit(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Subunit(
            final Entity entity
        ) {
            super(entity);
        }

        public static Subunit fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Subunit.NONE :

                new Subunit(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Subunit.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Subunit.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Subunit.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Subunit.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``CharLiteral``,
         * ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subunit_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subunit_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``PackageBody``,
         * ``ProtectedBody``, ``SubpBody``, ``TaskBody``
         *
         * When there are no parsing errors, this field is never null.
         */
        public BodyNode fBody(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subunit_f_body(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BodyNode res = BodyNode.NONE;
                if(propException == null) {
                    res = BodyNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subunit_f_body(
                    this.entity
                );

                // Wrap and return the result
                return BodyNode.fromEntity(res);
            }

        }

        
    

        /**
         * Return the body in which this subunit is rooted.
         */
        public BasicDecl pBodyRoot(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subunit_p_body_root(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BasicDecl res = BasicDecl.NONE;
                if(propException == null) {
                    res = BasicDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subunit_p_body_root(
                    this.entity
                );

                // Wrap and return the result
                return BasicDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``synchronized`` keyword.
     */
    public static abstract
    class SynchronizedNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SynchronizedNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SynchronizedNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SynchronizedNode NONE =
            new SynchronizedNodeNone();

        // ----- Constructors -----

        protected SynchronizedNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static SynchronizedNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SynchronizedNode.NONE :

                (SynchronizedNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SynchronizedNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SynchronizedNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SynchronizedNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SynchronizedNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of SynchronizedPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_synchronized_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_synchronized_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * SynchronizedNode.
         */
        private static final class SynchronizedNodeNone extends SynchronizedNode {
            SynchronizedNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class SynchronizedAbsent extends SynchronizedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SynchronizedAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SynchronizedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SynchronizedAbsent NONE =
            new SynchronizedAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SynchronizedAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static SynchronizedAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SynchronizedAbsent.NONE :

                new SynchronizedAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SynchronizedAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SynchronizedAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SynchronizedAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SynchronizedAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class SynchronizedPresent extends SynchronizedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SynchronizedPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SynchronizedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SynchronizedPresent NONE =
            new SynchronizedPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SynchronizedPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static SynchronizedPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SynchronizedPresent.NONE :

                new SynchronizedPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SynchronizedPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SynchronizedPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SynchronizedPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SynchronizedPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``tagged`` keyword.
     */
    public static abstract
    class TaggedNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaggedNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TaggedNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaggedNode NONE =
            new TaggedNodeNone();

        // ----- Constructors -----

        protected TaggedNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaggedNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaggedNode.NONE :

                (TaggedNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaggedNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaggedNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaggedNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaggedNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of TaggedPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_tagged_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_tagged_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * TaggedNode.
         */
        private static final class TaggedNodeNone extends TaggedNode {
            TaggedNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class TaggedAbsent extends TaggedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaggedAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TaggedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaggedAbsent NONE =
            new TaggedAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaggedAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaggedAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaggedAbsent.NONE :

                new TaggedAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaggedAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaggedAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaggedAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaggedAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class TaggedPresent extends TaggedNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaggedPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TaggedNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaggedPresent NONE =
            new TaggedPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaggedPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaggedPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaggedPresent.NONE :

                new TaggedPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaggedPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaggedPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaggedPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaggedPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a task type (:rmlink:`9.1`).
     */
    public static 
    class TaskDef extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TaskDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fInterfaces","fPublicPart","fPrivatePart","fEndName"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TaskDef.class.getMethod(
                        "fInterfaces",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interfaces",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskDef.class.getMethod(
                        "fPublicPart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_public_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskDef.class.getMethod(
                        "fPrivatePart",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_private_part",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TaskDef.class.getMethod(
                        "fEndName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_end_name",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TaskDef NONE =
            new TaskDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TaskDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static TaskDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TaskDef.NONE :

                new TaskDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TaskDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TaskDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TaskDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TaskDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ParentList fInterfaces(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_def_f_interfaces(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParentList res = ParentList.NONE;
                if(propException == null) {
                    res = ParentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_def_f_interfaces(
                    this.entity
                );

                // Wrap and return the result
                return ParentList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PublicPart fPublicPart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_def_f_public_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PublicPart res = PublicPart.NONE;
                if(propException == null) {
                    res = PublicPart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_def_f_public_part(
                    this.entity
                );

                // Wrap and return the result
                return PublicPart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public PrivatePart fPrivatePart(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_def_f_private_part(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PrivatePart res = PrivatePart.NONE;
                if(propException == null) {
                    res = PrivatePart.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_def_f_private_part(
                    this.entity
                );

                // Wrap and return the result
                return PrivatePart.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public EndName fEndName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_task_def_f_end_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EndName res = EndName.NONE;
                if(propException == null) {
                    res = EndName.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_task_def_f_end_name(
                    this.entity
                );

                // Wrap and return the result
                return EndName.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic node that contains the lazy fields for the attribute
     * subprograms of a given type. The lazy fields are not directly on the
     * BaseTypeDecl node itself to minimize its size in memory: with this
     * indirection, a type for which no function attribute is ever synthesized
     * will not waste any memory.
     */
    public static 
    class TypeAttributesRepository extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TypeAttributesRepository";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TypeAttributesRepository NONE =
            new TypeAttributesRepository(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TypeAttributesRepository(
            final Entity entity
        ) {
            super(entity);
        }

        public static TypeAttributesRepository fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TypeAttributesRepository.NONE :

                new TypeAttributesRepository(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TypeAttributesRepository.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TypeAttributesRepository.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TypeAttributesRepository.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TypeAttributesRepository.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for type definitions (:rmlink:`3.2.1`).
     */
    public static abstract
    class TypeDef extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TypeDef NONE =
            new TypeDefNone();

        // ----- Constructors -----

        protected TypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static TypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TypeDef.NONE :

                (TypeDef) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TypeDef.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * TypeDef.
         */
        private static final class TypeDefNone extends TypeDef {
            TypeDefNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Base class for access type definitions (:rmlink:`3.10`).
     */
    public static abstract
    class AccessDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AccessDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AccessDef.class.getMethod(
                        "fHasNotNull",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_not_null",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AccessDef NONE =
            new AccessDefNone();

        // ----- Constructors -----

        protected AccessDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static AccessDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AccessDef.NONE :

                (AccessDef) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AccessDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AccessDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AccessDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AccessDef.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public NotNull fHasNotNull(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_access_def_f_has_not_null(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NotNull res = NotNull.NONE;
                if(propException == null) {
                    res = NotNull.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_access_def_f_has_not_null(
                    this.entity
                );

                // Wrap and return the result
                return NotNull.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * AccessDef.
         */
        private static final class AccessDefNone extends AccessDef {
            AccessDefNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Type definition for accesses to subprograms (:rmlink:`3.10`).
     */
    public static 
    class AccessToSubpDef extends AccessDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AccessToSubpDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fHasProtected","fSubpSpec"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AccessDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AccessToSubpDef.class.getMethod(
                        "fHasProtected",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_protected",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = AccessToSubpDef.class.getMethod(
                        "fSubpSpec",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subp_spec",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AccessToSubpDef NONE =
            new AccessToSubpDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AccessToSubpDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static AccessToSubpDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AccessToSubpDef.NONE :

                new AccessToSubpDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AccessToSubpDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AccessToSubpDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AccessToSubpDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AccessToSubpDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ProtectedNode fHasProtected(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_access_to_subp_def_f_has_protected(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ProtectedNode res = ProtectedNode.NONE;
                if(propException == null) {
                    res = ProtectedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_access_to_subp_def_f_has_protected(
                    this.entity
                );

                // Wrap and return the result
                return ProtectedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubpSpec fSubpSpec(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_access_to_subp_def_f_subp_spec(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubpSpec res = SubpSpec.NONE;
                if(propException == null) {
                    res = SubpSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_access_to_subp_def_f_subp_spec(
                    this.entity
                );

                // Wrap and return the result
                return SubpSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for access type definitions (:rmlink:`3.10`).
     */
    public static abstract
    class BaseTypeAccessDef extends AccessDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "BaseTypeAccessDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AccessDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final BaseTypeAccessDef NONE =
            new BaseTypeAccessDefNone();

        // ----- Constructors -----

        protected BaseTypeAccessDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static BaseTypeAccessDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                BaseTypeAccessDef.NONE :

                (BaseTypeAccessDef) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return BaseTypeAccessDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return BaseTypeAccessDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return BaseTypeAccessDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return BaseTypeAccessDef.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * BaseTypeAccessDef.
         */
        private static final class BaseTypeAccessDefNone extends BaseTypeAccessDef {
            BaseTypeAccessDefNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Synthetic type access, that will directly reference a type decl. It is
     * used to generate synthetic anonymous access types.
     */
    public static 
    class AnonymousTypeAccessDef extends BaseTypeAccessDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AnonymousTypeAccessDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fTypeDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeAccessDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AnonymousTypeAccessDef.class.getMethod(
                        "fTypeDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AnonymousTypeAccessDef NONE =
            new AnonymousTypeAccessDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AnonymousTypeAccessDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static AnonymousTypeAccessDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AnonymousTypeAccessDef.NONE :

                new AnonymousTypeAccessDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AnonymousTypeAccessDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AnonymousTypeAccessDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AnonymousTypeAccessDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AnonymousTypeAccessDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseTypeDecl fTypeDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_anonymous_type_access_def_f_type_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_anonymous_type_access_def_f_type_decl(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Syntactic type definition for accesses.
     */
    public static 
    class TypeAccessDef extends BaseTypeAccessDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TypeAccessDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fHasAll","fHasConstant","fSubtypeIndication"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            BaseTypeAccessDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TypeAccessDef.class.getMethod(
                        "fHasAll",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_all",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeAccessDef.class.getMethod(
                        "fHasConstant",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_constant",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeAccessDef.class.getMethod(
                        "fSubtypeIndication",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype_indication",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TypeAccessDef NONE =
            new TypeAccessDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected TypeAccessDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static TypeAccessDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TypeAccessDef.NONE :

                new TypeAccessDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TypeAccessDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TypeAccessDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TypeAccessDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TypeAccessDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AllNode fHasAll(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_access_def_f_has_all(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AllNode res = AllNode.NONE;
                if(propException == null) {
                    res = AllNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_access_def_f_has_all(
                    this.entity
                );

                // Wrap and return the result
                return AllNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ConstantNode fHasConstant(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_access_def_f_has_constant(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ConstantNode res = ConstantNode.NONE;
                if(propException == null) {
                    res = ConstantNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_access_def_f_has_constant(
                    this.entity
                );

                // Wrap and return the result
                return ConstantNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubtypeIndication fSubtypeIndication(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_access_def_f_subtype_indication(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubtypeIndication res = SubtypeIndication.NONE;
                if(propException == null) {
                    res = SubtypeIndication.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_access_def_f_subtype_indication(
                    this.entity
                );

                // Wrap and return the result
                return SubtypeIndication.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for an array (:rmlink:`3.6`).
     */
    public static 
    class ArrayTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ArrayTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIndices","fComponentType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ArrayTypeDef.class.getMethod(
                        "fIndices",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_indices",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = ArrayTypeDef.class.getMethod(
                        "fComponentType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_component_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ArrayTypeDef NONE =
            new ArrayTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ArrayTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static ArrayTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ArrayTypeDef.NONE :

                new ArrayTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ArrayTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ArrayTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ArrayTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ArrayTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ArrayIndices fIndices(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_array_type_def_f_indices(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ArrayIndices res = ArrayIndices.NONE;
                if(propException == null) {
                    res = ArrayIndices.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_array_type_def_f_indices(
                    this.entity
                );

                // Wrap and return the result
                return ArrayIndices.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ComponentDef fComponentType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_array_type_def_f_component_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ComponentDef res = ComponentDef.NONE;
                if(propException == null) {
                    res = ComponentDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_array_type_def_f_component_type(
                    this.entity
                );

                // Wrap and return the result
                return ComponentDef.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a derived type (:rmlink:`3.4`).
     */
    public static 
    class DerivedTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DerivedTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasAbstract","fHasLimited","fHasSynchronized","fSubtypeIndication","fInterfaces","fRecordExtension","fHasWithPrivate"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fHasAbstract",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_abstract",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fHasLimited",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_limited",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fHasSynchronized",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_synchronized",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fSubtypeIndication",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype_indication",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fInterfaces",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interfaces",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fRecordExtension",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_record_extension",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DerivedTypeDef.class.getMethod(
                        "fHasWithPrivate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_with_private",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DerivedTypeDef NONE =
            new DerivedTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DerivedTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static DerivedTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DerivedTypeDef.NONE :

                new DerivedTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DerivedTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DerivedTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DerivedTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DerivedTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AbstractNode fHasAbstract(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_has_abstract(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbstractNode res = AbstractNode.NONE;
                if(propException == null) {
                    res = AbstractNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_has_abstract(
                    this.entity
                );

                // Wrap and return the result
                return AbstractNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public LimitedNode fHasLimited(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_has_limited(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LimitedNode res = LimitedNode.NONE;
                if(propException == null) {
                    res = LimitedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_has_limited(
                    this.entity
                );

                // Wrap and return the result
                return LimitedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SynchronizedNode fHasSynchronized(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_has_synchronized(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SynchronizedNode res = SynchronizedNode.NONE;
                if(propException == null) {
                    res = SynchronizedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_has_synchronized(
                    this.entity
                );

                // Wrap and return the result
                return SynchronizedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubtypeIndication fSubtypeIndication(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_subtype_indication(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubtypeIndication res = SubtypeIndication.NONE;
                if(propException == null) {
                    res = SubtypeIndication.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_subtype_indication(
                    this.entity
                );

                // Wrap and return the result
                return SubtypeIndication.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ParentList fInterfaces(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_interfaces(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParentList res = ParentList.NONE;
                if(propException == null) {
                    res = ParentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_interfaces(
                    this.entity
                );

                // Wrap and return the result
                return ParentList.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public BaseRecordDef fRecordExtension(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_record_extension(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseRecordDef res = BaseRecordDef.NONE;
                if(propException == null) {
                    res = BaseRecordDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_record_extension(
                    this.entity
                );

                // Wrap and return the result
                return BaseRecordDef.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public WithPrivate fHasWithPrivate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_derived_type_def_f_has_with_private(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                WithPrivate res = WithPrivate.NONE;
                if(propException == null) {
                    res = WithPrivate.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_derived_type_def_f_has_with_private(
                    this.entity
                );

                // Wrap and return the result
                return WithPrivate.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for enumerations (:rmlink:`3.5.1`).
     */
    public static 
    class EnumTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fEnumLiterals"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = EnumTypeDef.class.getMethod(
                        "fEnumLiterals",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_enum_literals",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumTypeDef NONE =
            new EnumTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumTypeDef.NONE :

                new EnumTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public EnumLiteralDeclList fEnumLiterals(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_enum_type_def_f_enum_literals(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                EnumLiteralDeclList res = EnumLiteralDeclList.NONE;
                if(propException == null) {
                    res = EnumLiteralDeclList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_enum_type_def_f_enum_literals(
                    this.entity
                );

                // Wrap and return the result
                return EnumLiteralDeclList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for discrete types in generic formals
     * (:rmlink:`12.5.2`).
     */
    public static 
    class FormalDiscreteTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "FormalDiscreteTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final FormalDiscreteTypeDef NONE =
            new FormalDiscreteTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected FormalDiscreteTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static FormalDiscreteTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                FormalDiscreteTypeDef.NONE :

                new FormalDiscreteTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return FormalDiscreteTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return FormalDiscreteTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return FormalDiscreteTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return FormalDiscreteTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for an interface (:rmlink:`3.9.4`).
     */
    public static 
    class InterfaceTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "InterfaceTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fInterfaceKind","fInterfaces"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = InterfaceTypeDef.class.getMethod(
                        "fInterfaceKind",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interface_kind",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = InterfaceTypeDef.class.getMethod(
                        "fInterfaces",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_interfaces",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final InterfaceTypeDef NONE =
            new InterfaceTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected InterfaceTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static InterfaceTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                InterfaceTypeDef.NONE :

                new InterfaceTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return InterfaceTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return InterfaceTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return InterfaceTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return InterfaceTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public InterfaceKind fInterfaceKind(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_interface_type_def_f_interface_kind(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                InterfaceKind res = InterfaceKind.NONE;
                if(propException == null) {
                    res = InterfaceKind.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_interface_type_def_f_interface_kind(
                    this.entity
                );

                // Wrap and return the result
                return InterfaceKind.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public ParentList fInterfaces(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_interface_type_def_f_interfaces(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParentList res = ParentList.NONE;
                if(propException == null) {
                    res = ParentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_interface_type_def_f_interfaces(
                    this.entity
                );

                // Wrap and return the result
                return ParentList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a modular integer type (:rmlink:`3.5.4`).
     */
    public static 
    class ModIntTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ModIntTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fExpr"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ModIntTypeDef.class.getMethod(
                        "fExpr",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_expr",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ModIntTypeDef NONE =
            new ModIntTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ModIntTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static ModIntTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ModIntTypeDef.NONE :

                new ModIntTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ModIntTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ModIntTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ModIntTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ModIntTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``NullLiteral``, ``NumLiteral``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fExpr(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_mod_int_type_def_f_expr(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_mod_int_type_def_f_expr(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a private type.
     *
     * Libadalang diverges from the ARM here, treating private types like
     * regular type declarations that have an embedded type definition. This
     * type definition hence corresponds to :rmlink:`7.3`.
     */
    public static 
    class PrivateTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "PrivateTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasAbstract","fHasTagged","fHasLimited"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = PrivateTypeDef.class.getMethod(
                        "fHasAbstract",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_abstract",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PrivateTypeDef.class.getMethod(
                        "fHasTagged",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_tagged",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = PrivateTypeDef.class.getMethod(
                        "fHasLimited",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_limited",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final PrivateTypeDef NONE =
            new PrivateTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected PrivateTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static PrivateTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                PrivateTypeDef.NONE :

                new PrivateTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return PrivateTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return PrivateTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return PrivateTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return PrivateTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AbstractNode fHasAbstract(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_private_type_def_f_has_abstract(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbstractNode res = AbstractNode.NONE;
                if(propException == null) {
                    res = AbstractNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_private_type_def_f_has_abstract(
                    this.entity
                );

                // Wrap and return the result
                return AbstractNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public TaggedNode fHasTagged(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_private_type_def_f_has_tagged(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TaggedNode res = TaggedNode.NONE;
                if(propException == null) {
                    res = TaggedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_private_type_def_f_has_tagged(
                    this.entity
                );

                // Wrap and return the result
                return TaggedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public LimitedNode fHasLimited(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_private_type_def_f_has_limited(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LimitedNode res = LimitedNode.NONE;
                if(propException == null) {
                    res = LimitedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_private_type_def_f_has_limited(
                    this.entity
                );

                // Wrap and return the result
                return LimitedNode.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for real numbers (:rmlink:`3.5.6`).
     */
    public static abstract
    class RealTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RealTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RealTypeDef NONE =
            new RealTypeDefNone();

        // ----- Constructors -----

        protected RealTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static RealTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RealTypeDef.NONE :

                (RealTypeDef) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RealTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RealTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RealTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RealTypeDef.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * RealTypeDef.
         */
        private static final class RealTypeDefNone extends RealTypeDef {
            RealTypeDefNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Type definition for decimal fixed-point numbers (:rmlink:`3.5.9`).
     */
    public static 
    class DecimalFixedPointDef extends RealTypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DecimalFixedPointDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDelta","fDigits","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            RealTypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = DecimalFixedPointDef.class.getMethod(
                        "fDelta",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_delta",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DecimalFixedPointDef.class.getMethod(
                        "fDigits",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_digits",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = DecimalFixedPointDef.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DecimalFixedPointDef NONE =
            new DecimalFixedPointDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DecimalFixedPointDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static DecimalFixedPointDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DecimalFixedPointDef.NONE :

                new DecimalFixedPointDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DecimalFixedPointDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DecimalFixedPointDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DecimalFixedPointDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DecimalFixedPointDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``NullLiteral``, ``NumLiteral``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fDelta(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decimal_fixed_point_def_f_delta(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decimal_fixed_point_def_f_delta(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``NullLiteral``, ``NumLiteral``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fDigits(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decimal_fixed_point_def_f_digits(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decimal_fixed_point_def_f_digits(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_decimal_fixed_point_def_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_decimal_fixed_point_def_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for floating-point numbers (:rmlink:`3.5.7`).
     */
    public static 
    class FloatingPointDef extends RealTypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "FloatingPointDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fNumDigits","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            RealTypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = FloatingPointDef.class.getMethod(
                        "fNumDigits",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_num_digits",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = FloatingPointDef.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final FloatingPointDef NONE =
            new FloatingPointDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected FloatingPointDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static FloatingPointDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                FloatingPointDef.NONE :

                new FloatingPointDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return FloatingPointDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return FloatingPointDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return FloatingPointDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return FloatingPointDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``NullLiteral``, ``NumLiteral``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fNumDigits(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_floating_point_def_f_num_digits(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_floating_point_def_f_num_digits(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_floating_point_def_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_floating_point_def_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for ordinary fixed-point numbers (:rmlink:`3.5.9`).
     */
    public static 
    class OrdinaryFixedPointDef extends RealTypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "OrdinaryFixedPointDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDelta","fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            RealTypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = OrdinaryFixedPointDef.class.getMethod(
                        "fDelta",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_delta",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = OrdinaryFixedPointDef.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final OrdinaryFixedPointDef NONE =
            new OrdinaryFixedPointDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected OrdinaryFixedPointDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static OrdinaryFixedPointDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                OrdinaryFixedPointDef.NONE :

                new OrdinaryFixedPointDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return OrdinaryFixedPointDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return OrdinaryFixedPointDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return OrdinaryFixedPointDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return OrdinaryFixedPointDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field can contain one of the following nodes: ``Allocator``,
         * ``AttributeRef``, ``BaseAggregate``, ``BinOp``, ``BoxExpr``,
         * ``CallExpr``, ``CharLiteral``, ``ConcatOp``, ``CondExpr``,
         * ``DeclExpr``, ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``NullLiteral``, ``NumLiteral``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Expr fDelta(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ordinary_fixed_point_def_f_delta(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Expr res = Expr.NONE;
                if(propException == null) {
                    res = Expr.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ordinary_fixed_point_def_f_delta(
                    this.entity
                );

                // Wrap and return the result
                return Expr.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_ordinary_fixed_point_def_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_ordinary_fixed_point_def_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a record (:rmlink:`3.8`).
     */
    public static 
    class RecordTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "RecordTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasAbstract","fHasTagged","fHasLimited","fRecordDef"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = RecordTypeDef.class.getMethod(
                        "fHasAbstract",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_abstract",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RecordTypeDef.class.getMethod(
                        "fHasTagged",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_tagged",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RecordTypeDef.class.getMethod(
                        "fHasLimited",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_limited",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = RecordTypeDef.class.getMethod(
                        "fRecordDef",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_record_def",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final RecordTypeDef NONE =
            new RecordTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected RecordTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static RecordTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                RecordTypeDef.NONE :

                new RecordTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return RecordTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return RecordTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return RecordTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return RecordTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AbstractNode fHasAbstract(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_type_def_f_has_abstract(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AbstractNode res = AbstractNode.NONE;
                if(propException == null) {
                    res = AbstractNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_type_def_f_has_abstract(
                    this.entity
                );

                // Wrap and return the result
                return AbstractNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public TaggedNode fHasTagged(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_type_def_f_has_tagged(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                TaggedNode res = TaggedNode.NONE;
                if(propException == null) {
                    res = TaggedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_type_def_f_has_tagged(
                    this.entity
                );

                // Wrap and return the result
                return TaggedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public LimitedNode fHasLimited(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_type_def_f_has_limited(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LimitedNode res = LimitedNode.NONE;
                if(propException == null) {
                    res = LimitedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_type_def_f_has_limited(
                    this.entity
                );

                // Wrap and return the result
                return LimitedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseRecordDef fRecordDef(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_record_type_def_f_record_def(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseRecordDef res = BaseRecordDef.NONE;
                if(propException == null) {
                    res = BaseRecordDef.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_record_type_def_f_record_def(
                    this.entity
                );

                // Wrap and return the result
                return BaseRecordDef.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Type definition for a signed integer type (:rmlink:`3.5.4`).
     */
    public static 
    class SignedIntTypeDef extends TypeDef {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SignedIntTypeDef";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fRange"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeDef.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SignedIntTypeDef.class.getMethod(
                        "fRange",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_range",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SignedIntTypeDef NONE =
            new SignedIntTypeDef(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SignedIntTypeDef(
            final Entity entity
        ) {
            super(entity);
        }

        public static SignedIntTypeDef fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SignedIntTypeDef.NONE :

                new SignedIntTypeDef(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SignedIntTypeDef.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SignedIntTypeDef.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SignedIntTypeDef.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SignedIntTypeDef.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public RangeSpec fRange(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_signed_int_type_def_f_range(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                RangeSpec res = RangeSpec.NONE;
                if(propException == null) {
                    res = RangeSpec.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_signed_int_type_def_f_range(
                    this.entity
                );

                // Wrap and return the result
                return RangeSpec.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * A type expression is an abstract node that embodies the concept of a
     * reference to a type.
     *
     * Since Ada has both subtype_indications and anonymous (inline) type
     * declarations, a type expression contains one or the other.
     *
     * This node has no ARM correspondence.
     */
    public static abstract
    class TypeExpr extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "TypeExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = TypeExpr.class.getMethod(
                        "pTypeName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_type_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeExpr.class.getMethod(
                        "pDesignatedTypeDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_designated_type_decl",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = TypeExpr.class.getMethod(
                        "pDesignatedTypeDeclFrom",
                        new Class[]{AdaNode.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new Param(
                        AdaNode.class,
                        "originNode"
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_designated_type_decl_from",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final TypeExpr NONE =
            new TypeExprNone();

        // ----- Constructors -----

        protected TypeExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static TypeExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                TypeExpr.NONE :

                (TypeExpr) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return TypeExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return TypeExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return TypeExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return TypeExpr.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return the name node for this type expression, if applicable, else
         * null
         */
        public Name pTypeName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_expr_p_type_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_expr_p_type_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * Returns the type declaration designated by this type expression.
         */
        public BaseTypeDecl pDesignatedTypeDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_expr_p_designated_type_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_expr_p_designated_type_decl(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }

        
    

        /**
         * Return the type declaration designated by this type expression as
         * viewed from the node given by origin_node.
         */
        public BaseTypeDecl pDesignatedTypeDeclFrom(
            final AdaNode originNode
        ) {

            // Verify that arguments are not null
            if(originNode == null) throw new IllegalArgumentException(
                "Argument 'originNode' of type " +
                "AdaNode cannot be null"
            );

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                EntityNative originNodeNative = StackValue.get(EntityNative.class);originNode.entity.unwrap(originNodeNative);

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_type_expr_p_designated_type_decl_from(
                    thisNative,
                    originNodeNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_type_expr_p_designated_type_decl_from(
                    (originNode != null ? originNode.entity : null),
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * TypeExpr.
         */
        private static final class TypeExprNone extends TypeExpr {
            TypeExprNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Container for inline anonymous array and access types declarations.
     */
    public static 
    class AnonymousType extends TypeExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "AnonymousType";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fTypeDecl"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = AnonymousType.class.getMethod(
                        "fTypeDecl",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_type_decl",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final AnonymousType NONE =
            new AnonymousType(
                Entity.NONE
            );

        // ----- Constructors -----

        protected AnonymousType(
            final Entity entity
        ) {
            super(entity);
        }

        public static AnonymousType fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                AnonymousType.NONE :

                new AnonymousType(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return AnonymousType.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return AnonymousType.fieldNames;
        }

        @Override
        public boolean isListType() {
            return AnonymousType.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return AnonymousType.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AnonymousTypeDecl fTypeDecl(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_anonymous_type_f_type_decl(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AnonymousTypeDecl res = AnonymousTypeDecl.NONE;
                if(propException == null) {
                    res = AnonymousTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_anonymous_type_f_type_decl(
                    this.entity
                );

                // Wrap and return the result
                return AnonymousTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic node. Represents the type expression for an enum literal.
     */
    public static 
    class EnumLitSynthTypeExpr extends TypeExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "EnumLitSynthTypeExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final EnumLitSynthTypeExpr NONE =
            new EnumLitSynthTypeExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected EnumLitSynthTypeExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static EnumLitSynthTypeExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                EnumLitSynthTypeExpr.NONE :

                new EnumLitSynthTypeExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return EnumLitSynthTypeExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return EnumLitSynthTypeExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return EnumLitSynthTypeExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return EnumLitSynthTypeExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Reference to a type by name (:rmlink:`3.2.2`).
     */
    public static 
    class SubtypeIndication extends TypeExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SubtypeIndication";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fName","fConstraint"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeIndication.class.getMethod(
                        "fHasNotNull",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_not_null",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeIndication.class.getMethod(
                        "fName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeIndication.class.getMethod(
                        "fConstraint",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_constraint",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeIndication.class.getMethod(
                        "pSubtypeConstraints",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_subtype_constraints",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = SubtypeIndication.class.getMethod(
                        "pIsStaticSubtype",
                        new Class[]{boolean.class}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();
                    parameters.add(new ParamWithDefaultValue(
                        boolean.class,
                        "impreciseFallback",
                        false
                    ));

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_is_static_subtype",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SubtypeIndication NONE =
            new SubtypeIndication(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SubtypeIndication(
            final Entity entity
        ) {
            super(entity);
        }

        public static SubtypeIndication fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SubtypeIndication.NONE :

                new SubtypeIndication(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SubtypeIndication.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SubtypeIndication.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SubtypeIndication.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SubtypeIndication.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public NotNull fHasNotNull(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subtype_indication_f_has_not_null(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NotNull res = NotNull.NONE;
                if(propException == null) {
                    res = NotNull.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subtype_indication_f_has_not_null(
                    this.entity
                );

                // Wrap and return the result
                return NotNull.fromEntity(res);
            }

        }

        
    

        /**
         * This field can contain one of the following nodes: ``AttributeRef``,
         * ``CharLiteral``, ``DottedName``, ``Identifier``, ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public Name fName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subtype_indication_f_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Name res = Name.NONE;
                if(propException == null) {
                    res = Name.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subtype_indication_f_name(
                    this.entity
                );

                // Wrap and return the result
                return Name.fromEntity(res);
            }

        }

        
    

        /**
         * This field may be null even when there are no parsing errors.
         */
        public Constraint fConstraint(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_subtype_indication_f_constraint(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Constraint res = Constraint.NONE;
                if(propException == null) {
                    res = Constraint.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_subtype_indication_f_constraint(
                    this.entity
                );

                // Wrap and return the result
                return Constraint.fromEntity(res);
            }

        }

        
    

        /**
         * Returns an array of pairs, associating formal parameters to actual
         * or default expressions.
         */
        public ParamActualArray pSubtypeConstraints(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final Pointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_subtype_indication_p_subtype_constraints(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ParamActualArray res = ParamActualArray.NONE;
                if(propException == null) {
                    res = ParamActualArray.wrap(resNative);

                    ParamActualArray.release(
                        resNative
                    );
                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final ParamActualArray res = JNI_LIB.ada_subtype_indication_p_subtype_constraints(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }

        
    

        /**
         * Returns whether Self denotes a static subtype or not.
         */
        public boolean pIsStaticSubtype(
            final boolean impreciseFallback
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments
                byte impreciseFallbackNative = (impreciseFallback ? (byte) 1 : (byte) 0);

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_subtype_indication_p_is_static_subtype(
                    thisNative,
                    impreciseFallbackNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_subtype_indication_p_is_static_subtype(
                    impreciseFallback,
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Reference to a type with a range constraint.
     */
    public static 
    class ConstrainedSubtypeIndication extends SubtypeIndication {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ConstrainedSubtypeIndication";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fName","fConstraint"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SubtypeIndication.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ConstrainedSubtypeIndication NONE =
            new ConstrainedSubtypeIndication(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ConstrainedSubtypeIndication(
            final Entity entity
        ) {
            super(entity);
        }

        public static ConstrainedSubtypeIndication fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ConstrainedSubtypeIndication.NONE :

                new ConstrainedSubtypeIndication(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ConstrainedSubtypeIndication.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ConstrainedSubtypeIndication.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ConstrainedSubtypeIndication.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ConstrainedSubtypeIndication.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Reference to a type with a general constraint.
     */
    public static 
    class DiscreteSubtypeIndication extends SubtypeIndication {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "DiscreteSubtypeIndication";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasNotNull","fName","fConstraint"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            SubtypeIndication.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final DiscreteSubtypeIndication NONE =
            new DiscreteSubtypeIndication(
                Entity.NONE
            );

        // ----- Constructors -----

        protected DiscreteSubtypeIndication(
            final Entity entity
        ) {
            super(entity);
        }

        public static DiscreteSubtypeIndication fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                DiscreteSubtypeIndication.NONE :

                new DiscreteSubtypeIndication(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return DiscreteSubtypeIndication.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return DiscreteSubtypeIndication.fieldNames;
        }

        @Override
        public boolean isListType() {
            return DiscreteSubtypeIndication.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return DiscreteSubtypeIndication.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Synthetic type expression. The designated type is already known at
     * instantiation time and is to be given in the ``target_type`` field.
     */
    public static 
    class SyntheticTypeExpr extends TypeExpr {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "SyntheticTypeExpr";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fTargetType"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            TypeExpr.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = SyntheticTypeExpr.class.getMethod(
                        "fTargetType",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_target_type",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final SyntheticTypeExpr NONE =
            new SyntheticTypeExpr(
                Entity.NONE
            );

        // ----- Constructors -----

        protected SyntheticTypeExpr(
            final Entity entity
        ) {
            super(entity);
        }

        public static SyntheticTypeExpr fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                SyntheticTypeExpr.NONE :

                new SyntheticTypeExpr(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return SyntheticTypeExpr.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return SyntheticTypeExpr.fieldNames;
        }

        @Override
        public boolean isListType() {
            return SyntheticTypeExpr.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return SyntheticTypeExpr.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public BaseTypeDecl fTargetType(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_synthetic_type_expr_f_target_type(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                BaseTypeDecl res = BaseTypeDecl.NONE;
                if(propException == null) {
                    res = BaseTypeDecl.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_synthetic_type_expr_f_target_type(
                    this.entity
                );

                // Wrap and return the result
                return BaseTypeDecl.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * List of unconstrained array indexes.
     */
    public static 
    class UnconstrainedArrayIndex extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UnconstrainedArrayIndex";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fSubtypeIndication"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UnconstrainedArrayIndex.class.getMethod(
                        "fSubtypeIndication",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_subtype_indication",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UnconstrainedArrayIndex NONE =
            new UnconstrainedArrayIndex(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UnconstrainedArrayIndex(
            final Entity entity
        ) {
            super(entity);
        }

        public static UnconstrainedArrayIndex fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UnconstrainedArrayIndex.NONE :

                new UnconstrainedArrayIndex(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UnconstrainedArrayIndex.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UnconstrainedArrayIndex.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UnconstrainedArrayIndex.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UnconstrainedArrayIndex.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public SubtypeIndication fSubtypeIndication(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_unconstrained_array_index_f_subtype_indication(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                SubtypeIndication res = SubtypeIndication.NONE;
                if(propException == null) {
                    res = SubtypeIndication.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_unconstrained_array_index_f_subtype_indication(
                    this.entity
                );

                // Wrap and return the result
                return SubtypeIndication.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``until`` keyword.
     */
    public static abstract
    class UntilNode extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UntilNode";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UntilNode.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UntilNode NONE =
            new UntilNodeNone();

        // ----- Constructors -----

        protected UntilNode(
            final Entity entity
        ) {
            super(entity);
        }

        public static UntilNode fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UntilNode.NONE :

                (UntilNode) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UntilNode.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UntilNode.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UntilNode.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UntilNode.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of UntilPresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_until_node_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_until_node_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * UntilNode.
         */
        private static final class UntilNodeNone extends UntilNode {
            UntilNodeNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class UntilAbsent extends UntilNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UntilAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            UntilNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UntilAbsent NONE =
            new UntilAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UntilAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static UntilAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UntilAbsent.NONE :

                new UntilAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UntilAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UntilAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UntilAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UntilAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class UntilPresent extends UntilNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UntilPresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            UntilNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UntilPresent NONE =
            new UntilPresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UntilPresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static UntilPresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UntilPresent.NONE :

                new UntilPresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UntilPresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UntilPresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UntilPresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UntilPresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**
     * Base class for use clauses (:rmlink:`10.1.2`).
     */
    public static abstract
    class UseClause extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UseClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UseClause NONE =
            new UseClauseNone();

        // ----- Constructors -----

        protected UseClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static UseClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UseClause.NONE :

                (UseClause) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UseClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UseClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UseClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UseClause.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * UseClause.
         */
        private static final class UseClauseNone extends UseClause {
            UseClauseNone() {super(Entity.NONE);}
        }

    }


    
    

    /**
     * Use clause for packages (:rmlink:`8.4`).
     */
    public static 
    class UsePackageClause extends UseClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UsePackageClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fPackages"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            UseClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UsePackageClause.class.getMethod(
                        "fPackages",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_packages",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UsePackageClause NONE =
            new UsePackageClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UsePackageClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static UsePackageClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UsePackageClause.NONE :

                new UsePackageClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UsePackageClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UsePackageClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UsePackageClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UsePackageClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public NameList fPackages(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_use_package_clause_f_packages(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NameList res = NameList.NONE;
                if(propException == null) {
                    res = NameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_use_package_clause_f_packages(
                    this.entity
                );

                // Wrap and return the result
                return NameList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Use clause for types (:rmlink:`8.4`).
     */
    public static 
    class UseTypeClause extends UseClause {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "UseTypeClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasAll","fTypes"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            UseClause.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = UseTypeClause.class.getMethod(
                        "fHasAll",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_all",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = UseTypeClause.class.getMethod(
                        "fTypes",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_types",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final UseTypeClause NONE =
            new UseTypeClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected UseTypeClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static UseTypeClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                UseTypeClause.NONE :

                new UseTypeClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return UseTypeClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return UseTypeClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return UseTypeClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return UseTypeClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public AllNode fHasAll(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_use_type_clause_f_has_all(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AllNode res = AllNode.NONE;
                if(propException == null) {
                    res = AllNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_use_type_clause_f_has_all(
                    this.entity
                );

                // Wrap and return the result
                return AllNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``AttributeRef``, ``CallExpr``, ``CharLiteral``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``, ``QualExpr``,
         * ``ReduceAttributeRef``, ``StringLiteral``, ``TargetName``,
         * ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public NameList fTypes(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_use_type_clause_f_types(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NameList res = NameList.NONE;
                if(propException == null) {
                    res = NameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_use_type_clause_f_types(
                    this.entity
                );

                // Wrap and return the result
                return NameList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * The value sequence of a reduction expression (see
     * ``ReduceAttributeRef``). Ada 2022, RM 4.5.10.
     */
    public static 
    class ValueSequence extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "ValueSequence";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fIterAssoc"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = ValueSequence.class.getMethod(
                        "fIterAssoc",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_iter_assoc",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final ValueSequence NONE =
            new ValueSequence(
                Entity.NONE
            );

        // ----- Constructors -----

        protected ValueSequence(
            final Entity entity
        ) {
            super(entity);
        }

        public static ValueSequence fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                ValueSequence.NONE :

                new ValueSequence(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return ValueSequence.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return ValueSequence.fieldNames;
        }

        @Override
        public boolean isListType() {
            return ValueSequence.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return ValueSequence.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public IteratedAssoc fIterAssoc(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_value_sequence_f_iter_assoc(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                IteratedAssoc res = IteratedAssoc.NONE;
                if(propException == null) {
                    res = IteratedAssoc.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_value_sequence_f_iter_assoc(
                    this.entity
                );

                // Wrap and return the result
                return IteratedAssoc.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Single variant in a discriminated type record declaration.
     *
     * This corresponds to a ``when ... => ...`` section in a variant part.
     */
    public static 
    class Variant extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "Variant";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fChoices","fComponents"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = Variant.class.getMethod(
                        "fChoices",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_choices",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = Variant.class.getMethod(
                        "fComponents",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_components",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final Variant NONE =
            new Variant(
                Entity.NONE
            );

        // ----- Constructors -----

        protected Variant(
            final Entity entity
        ) {
            super(entity);
        }

        public static Variant fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                Variant.NONE :

                new Variant(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return Variant.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return Variant.fieldNames;
        }

        @Override
        public boolean isListType() {
            return Variant.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return Variant.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``Allocator``, ``AttributeRef``, ``BaseAggregate``,
         * ``BinOp``, ``CallExpr``, ``CharLiteral``, ``ConcatOp``,
         * ``CondExpr``, ``DeclExpr``, ``DiscreteSubtypeIndication``,
         * ``DottedName``, ``ExplicitDeref``, ``Identifier``,
         * ``MembershipExpr``, ``NullLiteral``, ``NumLiteral``,
         * ``OthersDesignator``, ``ParenExpr``, ``QualExpr``,
         * ``QuantifiedExpr``, ``RaiseExpr``, ``ReduceAttributeRef``,
         * ``StringLiteral``, ``TargetName``, ``UnOp``, ``UpdateAttributeRef``
         *
         * When there are no parsing errors, this field is never null.
         */
        public AlternativesList fChoices(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_variant_f_choices(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                AlternativesList res = AlternativesList.NONE;
                if(propException == null) {
                    res = AlternativesList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_variant_f_choices(
                    this.entity
                );

                // Wrap and return the result
                return AlternativesList.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public ComponentList fComponents(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_variant_f_components(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                ComponentList res = ComponentList.NONE;
                if(propException == null) {
                    res = ComponentList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_variant_f_components(
                    this.entity
                );

                // Wrap and return the result
                return ComponentList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Variant part in a discriminated type record declaration
     * (:rmlink:`3.8.1`).
     *
     * This corresponds to the whole ``case ... is ... end case;`` block.
     */
    public static 
    class VariantPart extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "VariantPart";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fDiscrName","fVariant"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = VariantPart.class.getMethod(
                        "fDiscrName",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_discr_name",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = VariantPart.class.getMethod(
                        "fVariant",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_variant",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final VariantPart NONE =
            new VariantPart(
                Entity.NONE
            );

        // ----- Constructors -----

        protected VariantPart(
            final Entity entity
        ) {
            super(entity);
        }

        public static VariantPart fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                VariantPart.NONE :

                new VariantPart(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return VariantPart.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return VariantPart.fieldNames;
        }

        @Override
        public boolean isListType() {
            return VariantPart.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return VariantPart.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public Identifier fDiscrName(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_variant_part_f_discr_name(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                Identifier res = Identifier.NONE;
                if(propException == null) {
                    res = Identifier.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_variant_part_f_discr_name(
                    this.entity
                );

                // Wrap and return the result
                return Identifier.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public VariantList fVariant(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_variant_part_f_variant(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                VariantList res = VariantList.NONE;
                if(propException == null) {
                    res = VariantList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_variant_part_f_variant(
                    this.entity
                );

                // Wrap and return the result
                return VariantList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * With clause (:rmlink:`10.1.2`).
     */
    public static 
    class WithClause extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WithClause";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            "fHasLimited","fHasPrivate","fPackages"
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = WithClause.class.getMethod(
                        "fHasLimited",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_limited",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = WithClause.class.getMethod(
                        "fHasPrivate",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_has_private",
                        new LibadalangField(method, parameters)
                    );
                }
                
                {
                    // Get the Java method of the field
                    Method method = WithClause.class.getMethod(
                        "fPackages",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "f_packages",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WithClause NONE =
            new WithClause(
                Entity.NONE
            );

        // ----- Constructors -----

        protected WithClause(
            final Entity entity
        ) {
            super(entity);
        }

        public static WithClause fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WithClause.NONE :

                new WithClause(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WithClause.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WithClause.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WithClause.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WithClause.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public LimitedNode fHasLimited(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_with_clause_f_has_limited(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                LimitedNode res = LimitedNode.NONE;
                if(propException == null) {
                    res = LimitedNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_with_clause_f_has_limited(
                    this.entity
                );

                // Wrap and return the result
                return LimitedNode.fromEntity(res);
            }

        }

        
    

        /**
         * When there are no parsing errors, this field is never null.
         */
        public PrivateNode fHasPrivate(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_with_clause_f_has_private(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                PrivateNode res = PrivateNode.NONE;
                if(propException == null) {
                    res = PrivateNode.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_with_clause_f_has_private(
                    this.entity
                );

                // Wrap and return the result
                return PrivateNode.fromEntity(res);
            }

        }

        
    

        /**
         * This field contains a list that itself contains one of the following
         * nodes: ``CharLiteral``, ``DottedName``, ``Identifier``,
         * ``StringLiteral``
         *
         * When there are no parsing errors, this field is never null.
         */
        public NameList fPackages(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final EntityNative resNative =
                    StackValue.get(EntityNative.class);

                // Call the native function
                NI_LIB.ada_with_clause_f_packages(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                NameList res = NameList.NONE;
                if(propException == null) {
                    res = NameList.fromEntity(Entity.wrap(resNative));

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final Entity res = JNI_LIB.ada_with_clause_f_packages(
                    this.entity
                );

                // Wrap and return the result
                return NameList.fromEntity(res);
            }

        }


        // ----- Inner classes -----


    }


    
    

    /**
     * Qualifier for the ``private`` keyword in ``with private`` record
     * clauses.
     */
    public static abstract
    class WithPrivate extends AdaNode {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WithPrivate";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            AdaNode.fieldDescriptions
        );

        // Initialisation of the method map
        static {
            try {
                
                {
                    // Get the Java method of the field
                    Method method = WithPrivate.class.getMethod(
                        "pAsBool",
                        new Class[]{}
                    );

                    // Create the parameter list
                    List<Param> parameters = new ArrayList<>();

                    // Add the method and the parameters in maps
                    fieldDescriptions.put(
                        "p_as_bool",
                        new LibadalangField(method, parameters)
                    );
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WithPrivate NONE =
            new WithPrivateNone();

        // ----- Constructors -----

        protected WithPrivate(
            final Entity entity
        ) {
            super(entity);
        }

        public static WithPrivate fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WithPrivate.NONE :

                (WithPrivate) AdaNode.dispatchNodeCreation(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WithPrivate.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WithPrivate.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WithPrivate.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WithPrivate.fieldDescriptions.getOrDefault(name, null);
        }


        // ----- Field accessors -----

        
    

        /**
         * Return whether this is an instance of WithPrivatePresent
         */
        public boolean pAsBool(
            
        ) {

            // Verify that arguments are not null

            if(ImageInfo.inImageCode()) {
                // Unwrap the current node
                final EntityNative thisNative = StackValue.get(
                    EntityNative.class
                );
                this.entity.unwrap(thisNative);



                // Unwrap the arguments

                // Create the result native
                final CCharPointer resNative =
                    StackValue.get(SizeOf.get(VoidPointer.class));

                // Call the native function
                NI_LIB.ada_with_private_p_as_bool(
                    thisNative,
                    resNative
                );

                // Get the potential exception of the native property call
                final LangkitException propException = getLastException();

                // Wrap the result if the property returned normally
                boolean res = false;
                if(propException == null) {
                    res = (resNative.read() != 0);

                }

                // Release the unwrapped parameters


                // If the property exception is not null throw it
                if(propException != null) {
                    throw propException;
                }

                // Return the result
                return res;
            } else {
                // Call the native function
                final boolean res = JNI_LIB.ada_with_private_p_as_bool(
                    this.entity
                );

                // Wrap and return the result
                return res;
            }

        }


        // ----- Inner classes -----

        /**
         * This class represents the none value of the abstract node
         * WithPrivate.
         */
        private static final class WithPrivateNone extends WithPrivate {
            WithPrivateNone() {super(Entity.NONE);}
        }

    }


    
    

    /**

     */
    public static 
    class WithPrivateAbsent extends WithPrivate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WithPrivateAbsent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            WithPrivate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WithPrivateAbsent NONE =
            new WithPrivateAbsent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected WithPrivateAbsent(
            final Entity entity
        ) {
            super(entity);
        }

        public static WithPrivateAbsent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WithPrivateAbsent.NONE :

                new WithPrivateAbsent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WithPrivateAbsent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WithPrivateAbsent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WithPrivateAbsent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WithPrivateAbsent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }


    
    

    /**

     */
    public static 
    class WithPrivatePresent extends WithPrivate {

        // ----- Static -----

        
    

        /** The name of the node kind */
        public static final String kindName = "WithPrivatePresent";

        /** The names of the fields associated to the node */
        public static final String[] fieldNames = {
            
        };

        /** If the node is a list node */
        public static final boolean isListType =
            false;

        /** The map containing the node's fields description. */
        public static final Map<String, LibadalangField>
        fieldDescriptions = new HashMap<>(
            WithPrivate.fieldDescriptions
        );

        // Initialisation of the method map
        static {
        }


        // ----- Attributes -----

        /** Singleton that represents the none node. */
        public static final WithPrivatePresent NONE =
            new WithPrivatePresent(
                Entity.NONE
            );

        // ----- Constructors -----

        protected WithPrivatePresent(
            final Entity entity
        ) {
            super(entity);
        }

        public static WithPrivatePresent fromEntity(
            final Entity entity
        ) {
            return entity.node.isNull() ?
                WithPrivatePresent.NONE :

                new WithPrivatePresent(entity);
        }

        // ----- Instance methods -----

        @Override
        public String getKindName() {
            return WithPrivatePresent.kindName;
        }

        @Override
        public String[] getFieldNames() {
            return WithPrivatePresent.fieldNames;
        }

        @Override
        public boolean isListType() {
            return WithPrivatePresent.isListType;
        }

        @Override
        @CompilerDirectives.TruffleBoundary
        public LibadalangField getFieldDescription(
            final String name
        ) {
            return WithPrivatePresent.fieldDescriptions.getOrDefault(name, null);
        }

        // ----- Visitor methods -----

        public <T> T accept(BasicVisitor<T> visitor) {
            return visitor.visit(this);
        }

        public <T, P> T accept(ParamVisitor<T, P> visitor, P param) {
            return visitor.visit(this, param);
        }

        // ----- Field accessors -----


        // ----- Inner classes -----


    }



}
